@@ -29,10 +29,12 @@
 #include "amiga/context_menu.h"
 #include "amiga/clipboard.h"
 #include "amiga/bitmap.h"
+#include "amiga/filetype.h"
 #include "amiga/gui.h"
 #include "amiga/history_local.h"
 #include "amiga/iff_dr2d.h"
 #include "amiga/options.h"
+#include "amiga/plugin_hack.h"
 #include "amiga/theme.h"
 #include "amiga/utf8.h"
 #include "desktop/textinput.h"
@@ -48,13 +50,10 @@
 #include <string.h>
 
 static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved);
-static bool ami_context_menu_copy_selection(const char *text, size_t length,
-	struct box *box, void *handle, const char *whitespace_text,
-	size_t whitespace_length);
 static bool ami_context_menu_history(const struct history *history, int x0, int y0,
 	int x1, int y1, const struct history_entry *entry, void *user_data);
 
-uint32 ami_popup_hook(struct Hook *hook,Object *item,APTR reserved);
+static uint32 ami_popup_hook(struct Hook *hook,Object *item,APTR reserved);
 
 enum {
 	CMID_SELECTFILE,
@@ -73,6 +72,7 @@ enum {
 	CMID_SELCOPY,
 	CMID_SELPASTE,
 	CMID_SELSEARCH,
+	CMID_PLUGINCMD,
 	CMSUB_OBJECT,
 	CMSUB_URL,
 	CMSUB_SEL,
@@ -82,13 +82,9 @@ enum {
 
 struct Library  *PopupMenuBase = NULL;
 struct PopupMenuIFace *IPopupMenu = NULL;
-char *ctxmenulab[CMID_LAST];
-
-struct ami_context_menu_selection
-{
-	char text[1024];
-	int length;
-};
+static char *ctxmenulab[CMID_LAST];
+static Object *ctxmenuobj = NULL;
+static struct Hook ctxmenuhook;
 
 void ami_context_menu_init(void)
 {
@@ -116,6 +112,8 @@ void ami_context_menu_init(void)
 	ctxmenulab[CMID_SELCLEAR] = ami_utf8_easy((char *)messages_get("ClearNS"));
 	ctxmenulab[CMID_SELSEARCH] = ami_utf8_easy((char *)messages_get("SearchWeb"));
 
+	ctxmenulab[CMID_PLUGINCMD] = ami_utf8_easy((char *)messages_get("ExternalApp"));
+
 	ctxmenulab[CMSUB_OBJECT] = ami_utf8_easy((char *)messages_get("Object"));
 	ctxmenulab[CMSUB_URL] = ami_utf8_easy((char *)messages_get("Link"));
 	ctxmenulab[CMSUB_SEL] = ami_utf8_easy((char *)messages_get("Selection"));
@@ -128,6 +126,8 @@ void ami_context_menu_free(void)
 {
 	int i;
 
+	if(ctxmenuobj) DisposeObject(ctxmenuobj);
+
 	for(i=0;i<CMID_LAST;i++)
 	{
 		ami_utf8_free(ctxmenulab[i]);
@@ -143,8 +143,7 @@ BOOL ami_context_menu_mouse_trap(struct gui_window_2 *gwin, BOOL trap)
 
 	if(option_context_menu == false) return FALSE;
 
-	if((option_kiosk_mode == false) && (trap == FALSE) &&
-		(gwin->bw->browser_window_type == BROWSER_WINDOW_NORMAL))
+	if((option_kiosk_mode == false) && (trap == FALSE))
 	{
 		if(browser_window_back_available(gwin->bw) &&
 				ami_gadget_hit(gwin->objects[GID_BACK],
@@ -159,7 +158,7 @@ BOOL ami_context_menu_mouse_trap(struct gui_window_2 *gwin, BOOL trap)
 
 	if(gwin->rmbtrapped == trap) return trap;
 
-	SetWindowAttr(gwin->win, WA_RMBTrap, trap, 1);
+	SetWindowAttr(gwin->win, WA_RMBTrap, (APTR)(ULONG)trap, sizeof(BOOL));
 	gwin->rmbtrapped = trap;
 
 	return trap;
@@ -173,17 +172,20 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 	int box_y=0;
 	bool menuhascontent = false;
 	bool no_url = true, no_obj = true, no_sel = true;
+	ULONG ret = 0;
+	struct contextual_content ccdata;
 
 	if(!cc) return;
 	if(content_get_type(cc) != CONTENT_HTML) return;
 
-	if(gwin->objects[OID_MENU]) DisposeObject(gwin->objects[OID_MENU]);
+	if(ctxmenuobj) DisposeObject(ctxmenuobj);
 
-	gwin->popuphook.h_Entry = ami_context_menu_hook;
-	gwin->popuphook.h_Data = gwin;
+	ctxmenuhook.h_Entry = ami_context_menu_hook;
+	ctxmenuhook.h_SubEntry = NULL;
+	ctxmenuhook.h_Data = gwin;
 
-    gwin->objects[OID_MENU] = NewObject( POPUPMENU_GetClass(), NULL,
-                        PMA_MenuHandler, &gwin->popuphook,
+    ctxmenuobj = NewObject( POPUPMENU_GetClass(), NULL,
+                        PMA_MenuHandler, &ctxmenuhook,
 						TAG_DONE);
 
 	if(gwin->bw && gwin->bw->history &&
@@ -193,13 +195,13 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 		gwin->temp = 0;
 		history_enumerate_back(gwin->bw->history, ami_context_menu_history, gwin);
 
-		IDoMethod(gwin->objects[OID_MENU], PM_INSERT,
+		IDoMethod(ctxmenuobj, PM_INSERT,
 			NewObject(POPUPMENU_GetItemClass(), NULL,
 				PMIA_Title, ~0,
 			TAG_DONE),
 		~0);
 
-		IDoMethod(gwin->objects[OID_MENU], PM_INSERT,
+		IDoMethod(ctxmenuobj, PM_INSERT,
 			NewObject(POPUPMENU_GetItemClass(), NULL,
 				PMIA_Title, (ULONG)ctxmenulab[CMID_HISTORY],
 				PMIA_ID, CMID_HISTORY,
@@ -216,13 +218,13 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 		gwin->temp = 0;
 		history_enumerate_forward(gwin->bw->history, ami_context_menu_history, gwin);
 
-		IDoMethod(gwin->objects[OID_MENU], PM_INSERT,
+		IDoMethod(ctxmenuobj, PM_INSERT,
 			NewObject(POPUPMENU_GetItemClass(), NULL,
 				PMIA_Title, ~0,
 			TAG_DONE),
 		~0);
 
-		IDoMethod(gwin->objects[OID_MENU], PM_INSERT,
+		IDoMethod(ctxmenuobj, PM_INSERT,
 			NewObject(POPUPMENU_GetItemClass(), NULL,
 				PMIA_Title, (ULONG)ctxmenulab[CMID_HISTORY],
 				PMIA_ID, CMID_HISTORY,
@@ -234,6 +236,82 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 	}
 	else
 	{
+		browser_window_get_contextual_content(gwin->bw, x, y, &ccdata);
+
+		if(no_url && ccdata.link_url)
+		{
+			IDoMethod(ctxmenuobj,PM_INSERT,
+				NewObject(POPUPMENU_GetItemClass(), NULL,
+					PMIA_Title, (ULONG)ctxmenulab[CMSUB_URL],
+					PMSIMPLESUB,
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_URLOPENWIN],
+							PMIA_ID, CMID_URLOPENWIN,
+							PMIA_UserData, ccdata.link_url,
+						TAG_DONE),
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_URLOPENTAB],
+							PMIA_ID, CMID_URLOPENTAB,
+							PMIA_UserData, ccdata.link_url,
+						TAG_DONE),
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_COPYURL],
+							PMIA_ID, CMID_COPYURL,
+							PMIA_UserData, ccdata.link_url,
+						TAG_DONE),
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_SAVEURL],
+							PMIA_ID, CMID_SAVEURL,
+							PMIA_UserData, ccdata.link_url,
+						TAG_DONE),
+					TAG_DONE),
+				TAG_DONE),
+			~0);
+
+			no_url = false;
+			menuhascontent = true;
+		}
+
+		if(no_obj && ccdata.object &&
+			(content_get_type(ccdata.object) == CONTENT_IMAGE))
+		{
+			IDoMethod(ctxmenuobj, PM_INSERT,
+				NewObject(POPUPMENU_GetItemClass(), NULL,
+					PMIA_Title, (ULONG)ctxmenulab[CMSUB_OBJECT],
+					PMSIMPLESUB,
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_SHOWOBJ],
+							PMIA_ID, CMID_SHOWOBJ,
+							PMIA_UserData, content_get_url(ccdata.object),
+						TAG_DONE),
+						PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_COPYOBJ],
+							PMIA_ID, CMID_COPYOBJ,
+							PMIA_UserData, content_get_url(ccdata.object),
+						TAG_DONE),
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_CLIPOBJ],
+							PMIA_ID, CMID_CLIPOBJ,
+							PMIA_UserData, ccdata.object,
+						TAG_DONE),
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_SAVEOBJ],
+							PMIA_ID, CMID_SAVEOBJ,
+							PMIA_UserData, ccdata.object,
+						TAG_DONE),
+						PMA_AddItem, NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_SAVEIFFOBJ],
+							PMIA_ID, CMID_SAVEIFFOBJ,
+							PMIA_UserData, ccdata.object,
+						TAG_DONE),
+					TAG_DONE),
+				TAG_DONE),
+			~0);
+
+			no_obj = false;
+			menuhascontent = true;
+		}
+
 		curbox = html_get_box_tree(gwin->bw->current_content);
 
 		while(curbox = box_at_point(curbox,x,y,&box_x,&box_y,&cc))
@@ -242,88 +320,15 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 				css_computed_visibility(curbox->style) == CSS_VISIBILITY_HIDDEN)
 			continue;
 
-			if(no_url && curbox->href)
-			{
-				IDoMethod(gwin->objects[OID_MENU],PM_INSERT,
-					NewObject(POPUPMENU_GetItemClass(), NULL,
-						PMIA_Title, (ULONG)ctxmenulab[CMSUB_URL],
-						PMSIMPLESUB,
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_URLOPENWIN],
-								PMIA_ID,CMID_URLOPENWIN,
-								PMIA_UserData,curbox->href,
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_URLOPENTAB],
-								PMIA_ID,CMID_URLOPENTAB,
-								PMIA_UserData,curbox->href,
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_COPYURL],
-								PMIA_ID,CMID_COPYURL,
-								PMIA_UserData,curbox->href,
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_SAVEURL],
-								PMIA_ID,CMID_SAVEURL,
-								PMIA_UserData,curbox->href,
-							TAG_DONE),
-						TAG_DONE),
-					TAG_DONE),
-				~0);
-
-				no_url = false;
-				menuhascontent = true;
-			}
-
-			if(no_obj && curbox->object)
-			{
-				IDoMethod(gwin->objects[OID_MENU],PM_INSERT,
-					NewObject(POPUPMENU_GetItemClass(), NULL,
-						PMIA_Title, (ULONG)ctxmenulab[CMSUB_OBJECT],
-						PMSIMPLESUB,
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_SHOWOBJ],
-								PMIA_ID,CMID_SHOWOBJ,
-								PMIA_UserData, content_get_url(curbox->object),
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_COPYOBJ],
-								PMIA_ID,CMID_COPYOBJ,
-								PMIA_UserData, content_get_url(curbox->object),
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_CLIPOBJ],
-								PMIA_ID,CMID_CLIPOBJ,
-								PMIA_UserData,curbox->object,
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_SAVEOBJ],
-								PMIA_ID,CMID_SAVEOBJ,
-								PMIA_UserData,curbox->object,
-							TAG_DONE),
-							PMA_AddItem,NewObject(POPUPMENU_GetItemClass(), NULL,
-								PMIA_Title, (ULONG)ctxmenulab[CMID_SAVEIFFOBJ],
-								PMIA_ID,CMID_SAVEIFFOBJ,
-								PMIA_UserData,curbox->object,
-							TAG_DONE),
-						TAG_DONE),
-					TAG_DONE),
-				~0);
-
-				no_obj = false;
-				menuhascontent = true;
-			}
-
 			if(no_sel && (curbox->text) ||
 				(curbox->gadget && ((curbox->gadget->type == GADGET_TEXTBOX) ||
 				(curbox->gadget->type == GADGET_TEXTAREA) ||
 				(curbox->gadget->type == GADGET_PASSWORD))))
 			{
-				BOOL disabled_readonly = selection_read_only(gwin->bw->sel);
-				BOOL disabled_noselection = !selection_defined(gwin->bw->sel);
+				BOOL disabled_readonly = selection_read_only(browser_window_get_selection(gwin->bw));
+				BOOL disabled_noselection = !browser_window_has_selection(gwin->bw);
 
-				IDoMethod(gwin->objects[OID_MENU],PM_INSERT,
+				IDoMethod(ctxmenuobj,PM_INSERT,
 					NewObject(POPUPMENU_GetItemClass(), NULL,
 						PMIA_Title, (ULONG)ctxmenulab[CMSUB_SEL],
 						PMIA_SubMenu, NewObject(POPUPMENU_GetClass(), NULL,
@@ -367,13 +372,28 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 				no_sel = false;
 				menuhascontent = true;
 			}
+			if(curbox->object &&
+				(content_get_type(curbox->object) == CONTENT_PLUGIN))
+			{
+				if(ami_mime_content_to_cmd(curbox->object))
+				{
+					IDoMethod(ctxmenuobj,PM_INSERT,
+						NewObject(POPUPMENU_GetItemClass(), NULL,
+							PMIA_Title, (ULONG)ctxmenulab[CMID_PLUGINCMD],
+							PMIA_ID, CMID_PLUGINCMD,
+							PMIA_UserData, curbox->object,
+							TAG_DONE),
+						~0);
 
+					menuhascontent = true;
+				}
+			}
 			if (curbox->gadget)
 			{
 				switch (curbox->gadget->type)
 				{
 					case GADGET_FILE:
-						IDoMethod(gwin->objects[OID_MENU],PM_INSERT,
+						IDoMethod(ctxmenuobj,PM_INSERT,
 							NewObject(POPUPMENU_GetItemClass(), NULL,
 								PMIA_Title, (ULONG)ctxmenulab[CMID_SELECTFILE],
 								PMIA_ID,CMID_SELECTFILE,
@@ -391,7 +411,8 @@ void ami_context_menu_show(struct gui_window_2 *gwin,int x,int y)
 	if(!menuhascontent) return;
 
 	gui_window_set_pointer(gwin->bw->window,GUI_POINTER_DEFAULT);
-	IDoMethod(gwin->objects[OID_MENU],PM_OPEN,gwin->win);
+
+	IDoMethod(ctxmenuobj,PM_OPEN,gwin->win);
 }
 
 static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved)
@@ -401,7 +422,7 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 	APTR userdata = NULL;
 	struct browser_window *bw;
 	struct hlcache_handle *object;
-	char *source_data;
+	const char *source_data;
 	ULONG source_size;
 	struct bitmap *bm;
 
@@ -423,7 +444,7 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 					char fname[1024];
 					int x,y;
 
-					strlcpy(&fname,filereq->fr_Drawer,1024);
+					strlcpy(fname,filereq->fr_Drawer,1024);
 					AddPart(fname,filereq->fr_File,1024);
 
 					if(utf8_from_local_encoding(fname,0,&utf8_fn) != UTF8_CONVERT_OK)
@@ -473,12 +494,12 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 				object = (struct hlcache_handle *)userdata;
 				if((bm = content_get_bitmap(object)))
 				{
-					bm->url = content_get_url(object);
-					bm->title = content_get_title(object);
+					bm->url = (char *)content_get_url(object);
+					bm->title = (char *)content_get_title(object);
 					ami_easy_clipboard_bitmap(bm);
 				}
 #ifdef WITH_NS_SVG
-				else if(content_get_type(object) == CONTENT_SVG)
+				else if(ami_mime_compare(object, "svg") == true)
 				{
 					ami_easy_clipboard_svg(object);
 				}
@@ -496,11 +517,11 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 				{
 					BPTR fh = 0;
 					char fname[1024];
-					strlcpy(&fname,savereq->fr_Drawer,1024);
+					strlcpy(fname,savereq->fr_Drawer,1024);
 					AddPart(fname,savereq->fr_File,1024);
 					ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 
-					if(ami_download_check_overwrite(fname, gwin->win))
+					if(ami_download_check_overwrite(fname, gwin->win, 0))
 					{
 						if(fh = FOpen(fname,MODE_NEWFILE,0))
 						{
@@ -528,17 +549,17 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 					BPTR fh = 0;
 					char fname[1024];
 
-					strlcpy(&fname,savereq->fr_Drawer,1024);
+					strlcpy(fname,savereq->fr_Drawer,1024);
 					AddPart(fname,savereq->fr_File,1024);
 					if((bm = content_get_bitmap(object)))
 					{
-						bm->url = content_get_url(object);
-						bm->title = content_get_title(object);
+						bm->url = (char *)content_get_url(object);
+						bm->title = (char *)content_get_title(object);
 						if(bitmap_save(bm, fname, 0))
 							SetComment(fname, content_get_url(object));
 					}
 #ifdef WITH_NS_SVG
-					else if(content_get_type(object) == CONTENT_SVG)
+					else if(ami_mime_compare(object, "svg") == true)
 					{
 						if(ami_save_svg(object,fname))
 							SetComment(fname, content_get_url(object));
@@ -548,6 +569,10 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 				}
 			break;
 
+			case CMID_PLUGINCMD:
+				amiga_plugin_hack_execute((struct hlcache_handle *)userdata);
+			break;
+
 			case CMID_HISTORY:
 				if(userdata == NULL)
 				{
@@ -584,16 +609,11 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
 
 			case CMID_SELSEARCH:
 			{
-				struct ami_context_menu_selection *sel;
+				struct ami_text_selection *sel;
 				char *url;
 
-				sel = AllocVec(sizeof(struct ami_context_menu_selection),
-					MEMF_PRIVATE | MEMF_CLEAR);
-
-				if(sel)
+				if(sel = ami_selection_to_text(gwin))
 				{
-					selection_traverse(gwin->bw->sel, ami_context_menu_copy_selection,
-						sel);
 					url = search_web_from_term(sel->text);
 					browser_window_go(gwin->bw, url, NULL, true);
 
@@ -607,26 +627,6 @@ static uint32 ami_context_menu_hook(struct Hook *hook,Object *item,APTR reserved
     return itemid;
 }
 
-static bool ami_context_menu_copy_selection(const char *text, size_t length,
-	struct box *box, void *handle, const char *whitespace_text,
-	size_t whitespace_length)
-{
-	struct ami_context_menu_selection *sel = handle;
-	int len = length;
-
-	if((length + (sel->length)) > (sizeof(sel->text)))
-		len = sizeof(sel->text) - (sel->length);
-
-	if(len <= 0) return false;
-
-	memcpy((sel->text) + (sel->length), text, len);
-	sel->length += len;
-
-	sel->text[sel->length] = '\0';
-
-	return true;
-}
-
 static bool ami_context_menu_history(const struct history *history, int x0, int y0,
 	int x1, int y1, const struct history_entry *entry, void *user_data)
 {
@@ -635,7 +635,7 @@ static bool ami_context_menu_history(const struct history *history, int x0, int
 	gwin->temp++;
 	if(gwin->temp > 10) return false;
 
-	IDoMethod(gwin->objects[OID_MENU], PM_INSERT,
+	IDoMethod(ctxmenuobj, PM_INSERT,
 		NewObject(POPUPMENU_GetItemClass(), NULL,
 			PMIA_Title, (ULONG)history_entry_get_title(entry),
 			PMIA_ID, CMID_HISTORY,
@@ -646,7 +646,7 @@ static bool ami_context_menu_history(const struct history *history, int x0, int
 	return true;
 }
 
-uint32 ami_popup_hook(struct Hook *hook,Object *item,APTR reserved)
+static uint32 ami_popup_hook(struct Hook *hook,Object *item,APTR reserved)
 {
 	int32 itemid = 0;
 	struct gui_window *gwin = hook->h_Data;
@@ -671,19 +671,20 @@ void gui_create_form_select_menu(struct browser_window *bw,
 	struct form_option *opt = control->data.select.items;
 	ULONG i = 0;
 
-	if(gwin->shared->objects[OID_MENU]) DisposeObject(gwin->shared->objects[OID_MENU]);
+	if(ctxmenuobj) DisposeObject(ctxmenuobj);
 
-	gwin->shared->popuphook.h_Entry = ami_popup_hook;
-	gwin->shared->popuphook.h_Data = gwin;
+	ctxmenuhook.h_Entry = ami_popup_hook;
+	ctxmenuhook.h_SubEntry = NULL;
+	ctxmenuhook.h_Data = gwin;
 
 	gwin->shared->control = control;
 
-    gwin->shared->objects[OID_MENU] = PMMENU(ami_utf8_easy(control->name)),
-                        PMA_MenuHandler, &gwin->shared->popuphook, End;
+    ctxmenuobj = PMMENU(ami_utf8_easy(control->name)),
+                        PMA_MenuHandler, &ctxmenuhook, End;
 
 	while(opt)
 	{
-		IDoMethod(gwin->shared->objects[OID_MENU], PM_INSERT,
+		IDoMethod(ctxmenuobj, PM_INSERT,
 			NewObject( POPUPMENU_GetItemClass(), NULL,
 				PMIA_Title, (ULONG)ami_utf8_easy(opt->text),
 				PMIA_ID, i,
@@ -698,7 +699,7 @@ void gui_create_form_select_menu(struct browser_window *bw,
 
 	gui_window_set_pointer(gwin, GUI_POINTER_DEFAULT); // Clear the menu-style pointer
 
-	IDoMethod(gwin->shared->objects[OID_MENU], PM_OPEN, gwin->shared->win);
+	IDoMethod(ctxmenuobj, PM_OPEN, gwin->shared->win);
 }
 
 #else
