@@ -27,7 +27,7 @@ void *drag_save_data;
 struct gui_window *drag_save_gui;
 
 void ami_drag_save(struct Window *win);
-void ami_drag_icon_show(struct Window *win, char *type);
+void ami_drag_icon_show(struct Window *win, const char *type);
 void ami_drag_icon_close(struct Window *win);
 void ami_drag_icon_move(void);
 BOOL ami_drag_in_progress(void);
