@@ -97,6 +97,7 @@ enum
 	GID_OPTS_SCALEQ,
 	GID_OPTS_ANIMSPEED,
 	GID_OPTS_ANIMDISABLE,
+	GID_OPTS_DPI_Y,
 	GID_OPTS_FONT_SANS,
 	GID_OPTS_FONT_SERIF,
 	GID_OPTS_FONT_MONO,
@@ -113,10 +114,10 @@ enum
 	GID_OPTS_DLDIR,
 	GID_OPTS_TAB_ACTIVE,
 	GID_OPTS_TAB_2,
+	GID_OPTS_TAB_LAST,
 	GID_OPTS_SEARCH_PROV,
 	GID_OPTS_CLIPBOARD,
 	GID_OPTS_CMENU_ENABLE,
-	GID_OPTS_CMENU_STICKY,
 	GID_OPTS_STARTUP_NO_WIN,
 	GID_OPTS_CLOSE_NO_QUIT,
 	GID_OPTS_DOCKY,
@@ -150,6 +151,7 @@ enum
 	GRP_OPTS_FETCHING,
 	GRP_OPTS_IMAGES,
 	GRP_OPTS_ANIMS,
+	GRP_OPTS_DPI,
 	GRP_OPTS_FONTFACES,
 	GRP_OPTS_FONTSIZE,
 	GRP_OPTS_MEMCACHE,
@@ -176,6 +178,7 @@ enum
 	LAB_OPTS_PT,
 	LAB_OPTS_MB,
 	LAB_OPTS_MM,
+	LAB_OPTS_DPI,
 	LAB_OPTS_LAST
 };
 
@@ -259,6 +262,7 @@ void ami_gui_opts_setup(void)
 	gadlab[GID_OPTS_NATIVEBM] = (char *)ami_utf8_easy((char *)messages_get("CacheNative"));
 	gadlab[GID_OPTS_SCALEQ] = (char *)ami_utf8_easy((char *)messages_get("ScaleQuality"));
 	gadlab[GID_OPTS_ANIMSPEED] = (char *)ami_utf8_easy((char *)messages_get("AnimSpeedLimit"));
+	gadlab[GID_OPTS_DPI_Y] = (char *)ami_utf8_easy((char *)messages_get("ResolutionY"));
 	gadlab[GID_OPTS_ANIMDISABLE] = (char *)ami_utf8_easy((char *)messages_get("AnimDisable"));
 	gadlab[GID_OPTS_FONT_SANS] = (char *)ami_utf8_easy((char *)messages_get("FontSans"));
 	gadlab[GID_OPTS_FONT_SERIF] = (char *)ami_utf8_easy((char *)messages_get("FontSerif"));
@@ -276,10 +280,10 @@ void ami_gui_opts_setup(void)
 	gadlab[GID_OPTS_DLDIR] = (char *)ami_utf8_easy((char *)messages_get("DownloadDir"));
 	gadlab[GID_OPTS_TAB_ACTIVE] = (char *)ami_utf8_easy((char *)messages_get("TabActive"));
 	gadlab[GID_OPTS_TAB_2] = (char *)ami_utf8_easy((char *)messages_get("TabMiddle"));
+	gadlab[GID_OPTS_TAB_LAST] = (char *)ami_utf8_easy((char *)messages_get("TabLast"));
 	gadlab[GID_OPTS_SEARCH_PROV] = (char *)ami_utf8_easy((char *)messages_get("SearchProvider"));
 	gadlab[GID_OPTS_CLIPBOARD] = (char *)ami_utf8_easy((char *)messages_get("ClipboardUTF8"));
 	gadlab[GID_OPTS_CMENU_ENABLE] = (char *)ami_utf8_easy((char *)messages_get("Enable"));
-	gadlab[GID_OPTS_CMENU_STICKY] = (char *)ami_utf8_easy((char *)messages_get("Sticky"));
 	gadlab[GID_OPTS_STARTUP_NO_WIN] = (char *)ami_utf8_easy((char *)messages_get("OptionNoWindow"));
 	gadlab[GID_OPTS_CLOSE_NO_QUIT] = (char *)ami_utf8_easy((char *)messages_get("OptionNoQuit"));
 	gadlab[GID_OPTS_DOCKY] = (char *)ami_utf8_easy((char *)messages_get("OptionDocky"));
@@ -304,6 +308,7 @@ void ami_gui_opts_setup(void)
 	gadlab[LAB_OPTS_PT] = (char *)ami_utf8_easy((char *)messages_get("Pt"));
 	gadlab[LAB_OPTS_MM] = (char *)ami_utf8_easy((char *)messages_get("MM"));
 	gadlab[LAB_OPTS_MB] = (char *)ami_utf8_easy((char *)messages_get("MBytes"));
+	gadlab[LAB_OPTS_DPI] = (char *)ami_utf8_easy((char *)messages_get("DPI"));
 
 	gadlab[GRP_OPTS_HOMEPAGE] = (char *)ami_utf8_easy((char *)messages_get("Home"));
 	gadlab[GRP_OPTS_CONTENTBLOCKING] = (char *)ami_utf8_easy((char *)messages_get("ContentBlocking"));
@@ -317,6 +322,7 @@ void ami_gui_opts_setup(void)
 	gadlab[GRP_OPTS_FETCHING] = (char *)ami_utf8_easy((char *)messages_get("Fetching"));
 	gadlab[GRP_OPTS_IMAGES] = (char *)ami_utf8_easy((char *)messages_get("Images"));
 	gadlab[GRP_OPTS_ANIMS] = (char *)ami_utf8_easy((char *)messages_get("Animations"));
+	gadlab[GRP_OPTS_DPI] = (char *)ami_utf8_easy((char *)messages_get("Resolution"));
 	gadlab[GRP_OPTS_FONTFACES] = (char *)ami_utf8_easy((char *)messages_get("FontFamilies"));
 	gadlab[GRP_OPTS_FONTSIZE] = (char *)ami_utf8_easy((char *)messages_get("FontSize"));
 	gadlab[GRP_OPTS_MEMCACHE] = (char *)ami_utf8_easy((char *)messages_get("CacheMemory"));
@@ -369,12 +375,19 @@ void ami_gui_opts_open(void)
 	ULONG proxytype = 0;
 	BOOL screenmodedisabled = FALSE, screennamedisabled = FALSE;
 	BOOL proxyhostdisabled = TRUE, proxyauthdisabled = TRUE;
-	BOOL disableanims, animspeeddisabled = FALSE;
+	BOOL disableanims, animspeeddisabled = FALSE, acceptlangdisabled = FALSE;
 	BOOL scaleselected = option_scale_quality, scaledisabled = FALSE;
 	BOOL download_notify_disabled = FALSE;
 	char animspeed[10];
 	struct TextAttr fontsans, fontserif, fontmono, fontcursive, fontfantasy, fontunicode;
 
+	if(gow && gow->win)
+	{
+		WindowToFront(gow->win);
+		ActivateWindow(gow->win);
+		return;
+	}
+
 	if(option_use_pubscreen && option_use_pubscreen[0] != '\0')
 	{
 		if(strcmp(option_use_pubscreen,"Workbench") == 0)
@@ -427,6 +440,11 @@ void ami_gui_opts_open(void)
 		animspeeddisabled = TRUE;
 	}
 
+	if(option_accept_lang_locale)
+		acceptlangdisabled = TRUE;
+	else
+		acceptlangdisabled = FALSE;
+
 	if(GfxBase->LibNode.lib_Version < 53)
 	{
 		scaledisabled = TRUE;
@@ -548,13 +566,15 @@ void ami_gui_opts_open(void)
 										LAYOUT_AddChild, gow->objects[GID_OPTS_CONTENTLANG] = StringObject,
 											GA_ID, GID_OPTS_CONTENTLANG,
 											GA_RelVerify, TRUE,
+											GA_Disabled, acceptlangdisabled,
 											STRINGA_TextVal, option_accept_language,
 											STRINGA_BufferPos,0,
 										StringEnd,
-										LAYOUT_AddChild, gow->objects[GID_OPTS_FROMLOCALE] = ButtonObject,
-											GA_ID,GID_OPTS_FROMLOCALE,
-											GA_Text,gadlab[GID_OPTS_FROMLOCALE],
-											GA_RelVerify,TRUE,
+										LAYOUT_AddChild, gow->objects[GID_OPTS_FROMLOCALE] = CheckBoxObject,
+											GA_ID, GID_OPTS_FROMLOCALE,
+											GA_Text, gadlab[GID_OPTS_FROMLOCALE],
+											GA_RelVerify, TRUE,
+											GA_Selected, option_accept_lang_locale,
 										ButtonEnd,
 									//	CHILD_WeightedWidth, 0,
 									LayoutEnd, // content language
@@ -851,6 +871,30 @@ void ami_gui_opts_open(void)
             	    				CheckBoxEnd,
 								LayoutEnd, //animations
 								CHILD_WeightedHeight, 0,
+								LAYOUT_AddChild,VGroupObject,
+									LAYOUT_SpaceOuter, TRUE,
+									LAYOUT_BevelStyle, BVS_GROUP, 
+									LAYOUT_Label, gadlab[GRP_OPTS_DPI],
+									LAYOUT_AddChild, HGroupObject,
+										LAYOUT_LabelColumn, PLACETEXT_RIGHT,
+										LAYOUT_AddChild, gow->objects[GID_OPTS_DPI_Y] = IntegerObject,
+											GA_ID, GID_OPTS_DPI_Y,
+											GA_RelVerify, TRUE,
+											INTEGER_Number, option_amiga_ydpi,
+											INTEGER_Minimum, 60,
+											INTEGER_Maximum, 150,
+											INTEGER_Arrows, TRUE,
+										IntegerEnd,
+										CHILD_WeightedWidth, 0,
+										CHILD_Label, LabelObject,
+											LABEL_Text, gadlab[LAB_OPTS_DPI],
+										LabelEnd,
+									LayoutEnd,
+									CHILD_Label, LabelObject,
+										LABEL_Text, gadlab[GID_OPTS_DPI_Y],
+									LabelEnd,
+								LayoutEnd, //animations
+								CHILD_WeightedHeight, 0,
 							LayoutEnd, // page vgroup
 							CHILD_WeightedHeight, 0,
 						PageEnd, // page object
@@ -1080,6 +1124,12 @@ void ami_gui_opts_open(void)
          	        	   					GA_RelVerify, TRUE,
          	     	      					GA_Text, gadlab[GID_OPTS_TAB_ACTIVE],
   				      		            	GA_Selected, !option_new_tab_active,
+            	    					CheckBoxEnd,
+										LAYOUT_AddChild, gow->objects[GID_OPTS_TAB_LAST] = CheckBoxObject,
+      	              						GA_ID, GID_OPTS_TAB_LAST,
+         	           						GA_RelVerify, TRUE,
+         	           						GA_Text, gadlab[GID_OPTS_TAB_LAST],
+  				      			            GA_Selected, option_new_tab_last,
             	    					CheckBoxEnd,
 										LAYOUT_AddChild, gow->objects[GID_OPTS_TAB_2] = CheckBoxObject,
       	              						GA_ID, GID_OPTS_TAB_2,
@@ -1088,24 +1138,32 @@ void ami_gui_opts_open(void)
   				      			            GA_Selected, option_button_2_tab,
             	    					CheckBoxEnd,
 									LayoutEnd, // tabbed browsing
-									LAYOUT_AddChild,VGroupObject,
+
+									LAYOUT_AddChild, VGroupObject,
 										LAYOUT_SpaceOuter, TRUE,
 										LAYOUT_BevelStyle, BVS_GROUP, 
-										LAYOUT_Label, gadlab[GRP_OPTS_CONTEXTMENU],
-		        	        			LAYOUT_AddChild, gow->objects[GID_OPTS_CMENU_ENABLE] = CheckBoxObject,
-      	    	          					GA_ID, GID_OPTS_CMENU_ENABLE,
+										LAYOUT_Label, gadlab[GRP_OPTS_BEHAVIOUR],
+			                			LAYOUT_AddChild, gow->objects[GID_OPTS_STARTUP_NO_WIN] = CheckBoxObject,
+    	  	              					GA_ID, GID_OPTS_STARTUP_NO_WIN,
         	 	           					GA_RelVerify, TRUE,
-    	     	           					GA_Text, gadlab[GID_OPTS_CMENU_ENABLE],
-	  				      		            GA_Selected, option_context_menu,
-            	    					CheckBoxEnd,
-		                				LAYOUT_AddChild, gow->objects[GID_OPTS_CMENU_STICKY] = CheckBoxObject,
-      	            	  					GA_ID, GID_OPTS_CMENU_STICKY,
-         	    	       					GA_RelVerify, TRUE,
-											GA_Disabled, !option_context_menu,
-        	 	           					GA_Text, gadlab[GID_OPTS_CMENU_STICKY],
-  					      		            GA_Selected, option_sticky_context_menu,
-	            	    				CheckBoxEnd,
-									LayoutEnd, // context menus
+											GA_Text, gadlab[GID_OPTS_STARTUP_NO_WIN],
+  						      	            GA_Selected, option_startup_no_window,
+            		    				CheckBoxEnd,
+		        	        			LAYOUT_AddChild, gow->objects[GID_OPTS_CLOSE_NO_QUIT] = CheckBoxObject,
+      		              					GA_ID, GID_OPTS_CLOSE_NO_QUIT,
+											GA_RelVerify, TRUE,
+											GA_Text, gadlab[GID_OPTS_CLOSE_NO_QUIT],
+											GA_Selected, option_close_no_quit,
+	        	        				CheckBoxEnd,
+		                				LAYOUT_AddChild, gow->objects[GID_OPTS_DOCKY] = CheckBoxObject,
+											GA_ID, GID_OPTS_DOCKY,
+        	 	           					GA_RelVerify, TRUE,
+         		           					GA_Text, gadlab[GID_OPTS_DOCKY],
+  					      		            GA_Selected, !option_hide_docky_icon,
+	            		    			CheckBoxEnd,
+									LayoutEnd, // behaviour
+									CHILD_WeightedHeight, 0,
+
 								LayoutEnd, // hgroup
 								CHILD_WeightedHeight, 0,
 								LAYOUT_AddChild, HGroupObject,
@@ -1140,32 +1198,20 @@ void ami_gui_opts_open(void)
 									CHILD_WeightedHeight, 0,
 								LayoutEnd, // hgroup
 								CHILD_WeightedHeight, 0,
+
 								LAYOUT_AddChild,VGroupObject,
+									LAYOUT_BevelStyle, BVS_GROUP,
+									LAYOUT_Label, gadlab[GRP_OPTS_CONTEXTMENU],
 									LAYOUT_SpaceOuter, TRUE,
-									LAYOUT_BevelStyle, BVS_GROUP, 
-									LAYOUT_Label, gadlab[GRP_OPTS_BEHAVIOUR],
-									LAYOUT_AddChild, HGroupObject,
-			                			LAYOUT_AddChild, gow->objects[GID_OPTS_STARTUP_NO_WIN] = CheckBoxObject,
-    	  	              					GA_ID, GID_OPTS_STARTUP_NO_WIN,
-        	 	           					GA_RelVerify, TRUE,
-											GA_Text, gadlab[GID_OPTS_STARTUP_NO_WIN],
-  						      	            GA_Selected, option_startup_no_window,
-            		    				CheckBoxEnd,
-		        	        			LAYOUT_AddChild, gow->objects[GID_OPTS_CLOSE_NO_QUIT] = CheckBoxObject,
-      		              					GA_ID, GID_OPTS_CLOSE_NO_QUIT,
-											GA_RelVerify, TRUE,
-											GA_Text, gadlab[GID_OPTS_CLOSE_NO_QUIT],
-											GA_Selected, option_close_no_quit,
-	        	        				CheckBoxEnd,
-									LayoutEnd,
-	                				LAYOUT_AddChild, gow->objects[GID_OPTS_DOCKY] = CheckBoxObject,
-										GA_ID, GID_OPTS_DOCKY,
-         	           					GA_RelVerify, TRUE,
-         	           					GA_Text, gadlab[GID_OPTS_DOCKY],
-  				      		            GA_Selected, !option_hide_docky_icon,
-	            	    			CheckBoxEnd,
-								LayoutEnd, // behaviour
+	        	        			LAYOUT_AddChild, gow->objects[GID_OPTS_CMENU_ENABLE] = CheckBoxObject,
+   	    	          					GA_ID, GID_OPTS_CMENU_ENABLE,
+       	 	           					GA_RelVerify, TRUE,
+   	     	           					GA_Text, gadlab[GID_OPTS_CMENU_ENABLE],
+  				      		            GA_Selected, option_context_menu,
+           	    					CheckBoxEnd,
+								LayoutEnd, // context menus
 								CHILD_WeightedHeight, 0,
+
 							LayoutEnd, // page vgroup
 							CHILD_WeightedHeight, 0,
 						PageEnd, // page object
@@ -1354,7 +1400,7 @@ void ami_gui_opts_open(void)
 
 void ami_gui_opts_use(void)
 {
-	ULONG data;
+	ULONG data, id = 0;
 	float animspeed;
 	struct TextAttr *tattr;
 	char *dot;
@@ -1367,6 +1413,10 @@ void ami_gui_opts_use(void)
 	if(option_accept_language) free(option_accept_language);
 	option_accept_language = (char *)strdup((char *)data);
 
+	GetAttr(GA_Selected, gow->objects[GID_OPTS_FROMLOCALE],(ULONG *)&data);
+	if(data) option_accept_lang_locale = true;
+		else option_accept_lang_locale = false;
+
 	GetAttr(GA_Selected,gow->objects[GID_OPTS_HIDEADS],(ULONG *)&data);
 	if(data) option_block_ads = true;
 		else option_block_ads = false;
@@ -1401,12 +1451,12 @@ void ami_gui_opts_use(void)
 		break;
 	}
 
-	GetAttr(GETSCREENMODE_DisplayID,gow->objects[GID_OPTS_SCREENMODE],(ULONG *)&data);
-	if(data)
+	GetAttr(GETSCREENMODE_DisplayID, gow->objects[GID_OPTS_SCREENMODE], (ULONG *)&id);
+	if(id)
 	{
 		if(option_modeid) free(option_modeid);
 		option_modeid = malloc(20);
-		sprintf(option_modeid,"0x%lx",data);
+		sprintf(option_modeid,"0x%lx", id);
 	}
 
 	GetAttr(GETFILE_Drawer,gow->objects[GID_OPTS_THEME],(ULONG *)&data);
@@ -1464,6 +1514,9 @@ void ami_gui_opts_use(void)
 	if(data) option_animate_images = false;
 		else option_animate_images = true;
 
+	GetAttr(INTEGER_Number,gow->objects[GID_OPTS_DPI_Y],(ULONG *)&option_amiga_ydpi);
+	ami_font_setdevicedpi(id); // id set above
+
 	GetAttr(GETFONT_TextAttr,gow->objects[GID_OPTS_FONT_SANS],(ULONG *)&data);
 	tattr = (struct TextAttr *)data;
 	if(option_font_sans) free(option_font_sans);
@@ -1530,6 +1583,10 @@ void ami_gui_opts_use(void)
 	if(data) option_new_tab_active = false;
 		else option_new_tab_active = true;
 
+	GetAttr(GA_Selected,gow->objects[GID_OPTS_TAB_LAST],(ULONG *)&data);
+	if(data) option_new_tab_last = true;
+		else option_new_tab_last = false;
+
 	GetAttr(GA_Selected,gow->objects[GID_OPTS_TAB_2],(ULONG *)&data);
 	if(data) option_button_2_tab = true;
 		else option_button_2_tab = false;
@@ -1546,10 +1603,6 @@ void ami_gui_opts_use(void)
 	if(data) option_context_menu = true;
 		else option_context_menu = false;
 
-	GetAttr(GA_Selected,gow->objects[GID_OPTS_CMENU_STICKY],(ULONG *)&data);
-	if(data) option_sticky_context_menu = true;
-		else option_sticky_context_menu = false;
-
 	GetAttr(GA_Selected,gow->objects[GID_OPTS_STARTUP_NO_WIN],(ULONG *)&data);
 	if(data) option_startup_no_window = true;
 		else option_startup_no_window = false;
@@ -1591,9 +1644,6 @@ void ami_gui_opts_use(void)
 	GetAttr(GA_Selected,gow->objects[GID_OPTS_EXPORT_PASSWORD],(ULONG *)&data);
 	if(data) option_enable_PDF_password = true;
 		else option_enable_PDF_password = false;
-
-	ami_close_fonts();
-	ami_init_fonts();
 }
 
 void ami_gui_opts_close(void)
@@ -1658,10 +1708,13 @@ BOOL ami_gui_opts_event(void)
 					break;
 
 					case GID_OPTS_FROMLOCALE:
-						if(text = ami_locale_langs())
+						RefreshSetGadgetAttrs((struct Gadget *)gow->objects[GID_OPTS_CONTENTLANG],
+							gow->win, NULL, GA_Disabled, code, TAG_DONE);
+
+						if(code && (text = ami_locale_langs()))
 						{
 							RefreshSetGadgetAttrs((struct Gadget *)gow->objects[GID_OPTS_CONTENTLANG],
-								gow->win,NULL,STRINGA_TextVal, text, TAG_DONE);
+								gow->win, NULL, STRINGA_TextVal, text, TAG_DONE);
 							FreeVec(text);
 						}
 					break;
@@ -1784,11 +1837,6 @@ BOOL ami_gui_opts_event(void)
 						IDoMethod(gow->objects[GID_OPTS_DLDIR],
 						GFILE_REQUEST,gow->win);
 					break;
-
-					case GID_OPTS_CMENU_ENABLE:
-						RefreshSetGadgetAttrs((struct Gadget *)gow->objects[GID_OPTS_CMENU_STICKY],
-							gow->win, NULL, GA_Disabled, !code, TAG_DONE);
-					break;
 				}
 			break;
 		}
