@@ -24,27 +24,23 @@
 #define AMIGA_ICON_H
 
 #include "utils/config.h"
+#include "utils/errors.h"
+
 #ifdef WITH_AMIGA_ICON
 
-#include <stdbool.h>
-#include "content/hlcache.h"
+nserror amiga_icon_init(void);
+void amiga_icon_fini(void);
 
-struct rect;
+#else
 
-struct content_amiga_icon_data {
-/* empty */
-};
-
-bool amiga_icon_convert(struct content *c);
-void amiga_icon_destroy(struct content *c);
-bool amiga_icon_redraw(struct content *c, int x, int y,
-		int width, int height, const struct rect *clip,
-		float scale, colour background_colour);
-bool amiga_icon_clone(const struct content *old, struct content *new_content);
+#define amiga_icon_init() NSERROR_OK
+#define amiga_icon_fini() ((void) 0)
 
 #endif /* WITH_AMIGA_ICON */
 
 struct hlcache_handle;
 
 void ami_superimpose_favicon(char *path, struct hlcache_handle *icon, char *type);
+struct DiskObject *amiga_icon_from_bitmap(struct bitmap *bm);
+void amiga_icon_free(struct DiskObject *dobj);
 #endif
