@@ -43,6 +43,7 @@
 #include "amiga/clipboard.h"
 #include "amiga/cookies.h"
 #include "amiga/download.h"
+#include "amiga/filetype.h"
 #include "amiga/gui.h"
 #include "amiga/gui_options.h"
 #include "amiga/history.h"
@@ -68,7 +69,6 @@
 #include "desktop/save_complete.h"
 #include "desktop/selection.h"
 #include "desktop/textinput.h"
-#include "desktop/plugin.h"
 #include "utils/messages.h"
 #include "utils/url.h"
 
@@ -124,7 +124,7 @@ void ami_free_menulabs(struct gui_window_2 *gwin)
 	gwin->menu = NULL;
 }
 
-void ami_init_menulabs(struct gui_window_2 *gwin, struct DrawInfo *dri)
+void ami_init_menulabs(struct gui_window_2 *gwin)
 {
 	int i;
 
@@ -272,7 +272,7 @@ void ami_menu_refresh(struct gui_window_2 *gwin)
 			TAG_DONE);
 
 	ami_free_menulabs(gwin);
-	ami_create_menu(BROWSER_WINDOW_NORMAL, gwin);
+	ami_create_menu(gwin);
 
 	SetAttrs(gwin->objects[OID_MAIN],
 			WINDOW_NewMenu, gwin->menu,
@@ -282,19 +282,13 @@ void ami_menu_refresh(struct gui_window_2 *gwin)
 }
 */
 
-struct NewMenu *ami_create_menu(ULONG type, struct gui_window_2 *gwin, struct DrawInfo *dri)
+struct NewMenu *ami_create_menu(struct gui_window_2 *gwin)
 {
 	int i;
-	ULONG menuflags = 0;
 
-	ami_init_menulabs(gwin, dri);
+	ami_init_menulabs(gwin);
 	gwin->menu = AllocVec(sizeof(struct NewMenu) * (AMI_MENU_AREXX_MAX + 1), MEMF_CLEAR);
 
-	if(type != BROWSER_WINDOW_NORMAL)
-	{
-		menuflags = NM_ITEMDISABLED;
-	}
-
 	for(i=0;i<=AMI_MENU_AREXX_MAX;i++)
 	{
 		gwin->menu[i].nm_Type = gwin->menutype[i];
@@ -303,12 +297,12 @@ struct NewMenu *ami_create_menu(ULONG type, struct gui_window_2 *gwin, struct Dr
 		gwin->menu[i].nm_Flags = 0;
 	}
 
-	gwin->menu[1].nm_Flags = menuflags;
-	gwin->menu[2].nm_Flags = menuflags;
-	gwin->menu[12].nm_Flags = menuflags;
-	gwin->menu[13].nm_Flags = menuflags;
+	gwin->menu[1].nm_Flags = 0;
+	gwin->menu[2].nm_Flags = 0;
+	gwin->menu[12].nm_Flags = 0;
+	gwin->menu[13].nm_Flags = 0;
 
-	gwin->menu[15].nm_Flags = NM_ITEMDISABLED; /* Not implemented in 2.7 */
+	gwin->menu[15].nm_Flags = NM_ITEMDISABLED;
 
 #ifndef WITH_PDF_EXPORT
 	gwin->menu[9].nm_Flags = NM_ITEMDISABLED;
@@ -360,7 +354,7 @@ void ami_menu_arexx_scan(struct gui_window_2 *gwin)
 
 				do
 				{
-					cont = ExAll(lock,buffer,1024,ED_COMMENT,ctrl);
+					cont = ExAll(lock,(struct ExAllData *)buffer,1024,ED_COMMENT,ctrl);
 					if((!cont) && (IoErr() != ERROR_NO_MORE_ENTRIES)) break;
 					if(!ctrl->eac_Entries) continue;
 
@@ -478,7 +472,7 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 	subnum = SUBNUM(code);
 	char *temp, *temp2;
 	BPTR lock = 0;
-	char *source_data;
+	const char *source_data;
 	ULONG source_size;
 	struct bitmap *bm;
 	int sel = 0;
@@ -533,11 +527,11 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
-								strlcpy(&fname,savereq->fr_Drawer,1024);
+								strlcpy(fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 
-								if(ami_download_check_overwrite(fname, gwin->win))
+								if(ami_download_check_overwrite(fname, gwin->win, 0))
 								{
 									if(fh = FOpen(fname,MODE_NEWFILE,0))
 									{
@@ -559,11 +553,11 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
-								strlcpy(&fname,savereq->fr_Drawer,1024);
+								strlcpy(fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 
-								if(ami_download_check_overwrite(fname, gwin->win))
+								if(ami_download_check_overwrite(fname, gwin->win, 0))
 								{
 									save_as_text(gwin->bw->current_content,fname);
 									SetComment(fname,content_get_url(gwin->bw->current_content));
@@ -579,10 +573,10 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
-								strlcpy(&fname,savereq->fr_Drawer,1024);
+								strlcpy(fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-								if(ami_download_check_overwrite(fname, gwin->win))
+								if(ami_download_check_overwrite(fname, gwin->win, 0))
 								{
 									if(lock = CreateDir(fname))
 									{
@@ -626,18 +620,18 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
-								strlcpy(&fname,savereq->fr_Drawer,1024);
+								strlcpy(fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 								if((bm = content_get_bitmap(gwin->bw->current_content)))
 								{
-									bm->url = content_get_url(gwin->bw->current_content);
-									bm->title = content_get_title(gwin->bw->current_content);
+									bm->url = (char *)content_get_url(gwin->bw->current_content);
+									bm->title = (char *)content_get_title(gwin->bw->current_content);
 									if(bitmap_save(bm, fname, 0))
 										SetComment(fname, content_get_url(gwin->bw->current_content));
 								}
 #ifdef WITH_NS_SVG
-								else if(content_get_type(gwin->bw->current_content) == CONTENT_SVG)
+								else if(ami_mime_compare(gwin->bw->current_content, "svg") == true)
 								{
 									if(ami_save_svg(gwin->bw->current_content,fname))
 										SetComment(fname, content_get_url(gwin->bw->current_content));
@@ -724,12 +718,12 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 					}
 					else if(bm = content_get_bitmap(gwin->bw->current_content))
 					{
-						bm->url = content_get_url(gwin->bw->current_content);
-						bm->title = content_get_title(gwin->bw->current_content);
+						bm->url = (char *)content_get_url(gwin->bw->current_content);
+						bm->title = (char *)content_get_title(gwin->bw->current_content);
 						ami_easy_clipboard_bitmap(bm);
 					}
 #ifdef WITH_NS_SVG
-					else if(content_get_type(gwin->bw->current_content) == CONTENT_SVG)
+					else if(ami_mime_compare(gwin->bw->current_content, "svg") == true)
 					{
 						ami_easy_clipboard_svg(gwin->bw->current_content);
 					}
@@ -889,6 +883,8 @@ static const ULONG ami_asl_mime_hook(struct Hook *mh,struct FileRequester *fr,st
 	char fname[1024];
 	BOOL ret = FALSE;
 	char *mt = NULL;
+	lwc_string *lwc_mt = NULL;
+	lwc_error lerror;
 	content_type ct;
 
 	if(ap->ap_Info.fib_DirEntryType > 0) return(TRUE);
@@ -897,13 +893,14 @@ static const ULONG ami_asl_mime_hook(struct Hook *mh,struct FileRequester *fr,st
 	AddPart(fname,ap->ap_Info.fib_FileName,1024);
 
   	mt = fetch_mimetype(fname);
-	ct = content_lookup(mt);
+	lerror = lwc_intern_string(mt, strlen(mt), &lwc_mt);
+	if (lerror != lwc_error_ok)
+		return FALSE;
 
-	if(ct != CONTENT_OTHER) ret = TRUE;
+	ct = content_factory_type_from_mime_type(lwc_mt);
+	lwc_string_unref(lwc_mt);
 
-#ifdef WITH_PLUGIN
-	if(ct == CONTENT_PLUGIN) ret = plugin_handleable(mt);
-#endif
+	if(ct != CONTENT_NONE) ret = TRUE;
 
 	free(mt);
 	return ret;
@@ -913,7 +910,6 @@ void ami_menu_update_disabled(struct gui_window *g, hlcache_handle *c)
 {
 	struct Window *win = g->shared->win;
 
-	if(g->shared->bw->browser_window_type != BROWSER_WINDOW_NORMAL) return;
 	if(option_kiosk_mode == true) return;
 
 	OffMenu(win,AMI_MENU_CUT);
@@ -928,12 +924,12 @@ void ami_menu_update_disabled(struct gui_window *g, hlcache_handle *c)
 #ifdef WITH_PDF_EXPORT
 		OnMenu(win,AMI_MENU_SAVEAS_PDF);
 #endif
-		if(selection_defined(g->shared->bw->sel))
+		if(browser_window_has_selection(g->shared->bw))
 		{
 			OnMenu(win,AMI_MENU_COPY);
 			OnMenu(win,AMI_MENU_CLEAR);
 
-			if(selection_read_only(g->shared->bw->sel) == false)
+			if(selection_read_only(browser_window_get_selection(g->shared->bw)) == false)
 				OnMenu(win,AMI_MENU_CUT);
 		}
 		if(g->c_h) OnMenu(win,AMI_MENU_PASTE);
@@ -954,7 +950,7 @@ void ami_menu_update_disabled(struct gui_window *g, hlcache_handle *c)
 		OffMenu(win,AMI_MENU_FIND);
 
 #ifdef WITH_NS_SVG
-		if(content_get_bitmap(c) || content_get_type(c) == CONTENT_SVG)
+		if(content_get_bitmap(c) || (ami_mime_compare(c, "svg") == true))
 #else
 		if(content_get_bitmap(c))
 #endif
