@@ -24,7 +24,6 @@
 #include <stdbool.h>
 #include <assert.h>
 #include <windom.h>
-#include <cflib.h>
 
 #include "desktop/gui.h"
 #include "desktop/netsurf.h"
@@ -44,12 +43,16 @@
 #include "atari/plot/plotter.h"
 #include "atari/plot.h"
 #include "atari/font.h"
+#include "cflib.h"
 
 extern browser_mouse_state bmstate;
 extern int mouse_click_time[3];
 extern int mouse_hold_start[3];
 extern GEM_PLOTTER plotter;
 extern struct gui_window *input_window;
+extern short last_drag_x;
+extern short last_drag_y;
+
 
 
 static void __CDECL browser_evnt_wdestroy( WINDOW * c, short buff[8], void * data);
@@ -59,11 +62,13 @@ static bool frameinit = true;
 
 
 /* create an browser component */
-struct s_browser * browser_create(  struct gui_window * gw, 
-									struct browser_window *bw, 
-									struct browser_window * clone, 
-									enum browser_type type, 
-									int lt, int w, int flex )
+struct s_browser * browser_create
+(
+	struct gui_window * gw,
+	struct browser_window *bw,
+	struct browser_window * clone,
+	int lt, int w, int flex 
+)
 {
 	LGRECT cwork;
 	COMPONENT * scrollv, * scrollh, * drawable;
@@ -77,14 +82,13 @@ struct s_browser * browser_create(  struct gui_window * gw,
 	if( bnew )
 	{
 		memset(bnew, 0, sizeof(struct s_browser) );
-		bnew->type = type;
 		bnew->bw = bw;
 		bnew->attached = false;
 		if(clone)
 			bw->scale = clone->scale;
 		else
 			bw->scale = 1;
-
+		bnew->redraw.areas_used = 0;
 		bnew->compwin = mt_WindCreate( &app, VSLIDE|HSLIDE, 1, 1, app.w, app.h);
 		bnew->compwin->w_u = 1;
 		bnew->compwin->h_u = 1;
@@ -123,27 +127,13 @@ struct s_browser * browser_create(  struct gui_window * gw,
 
 bool browser_destroy( struct s_browser * b )
 {
-	short type = BT_ROOT;
-	LGRECT restore;
-	/* only free the components if it is an root browser,
-		this should delete all attached COMPONENTS... 
-	*/
 
-	LOG(("%s (%s)\n", b->bw->name,  (b->type == BT_ROOT) ? "ROOT" : "FRAME"));
+	LOG(("%s\n", b->bw->name ));
 	assert( b != NULL );
 	assert( b->comp != NULL );
 	assert( b->bw != NULL );
-	struct gui_window * rootgw = browser_find_root( b->bw->window );
 
-	if( b->type == BT_ROOT ) {
-		
-	} else if( BT_FRAME ){
-		type = BT_FRAME;
-	} else {
-		assert( 1 == 0 ); 
-	}
-
-	if( b->compwin != NULL ){
+	if( b->compwin != NULL ) {
 		COMPONENT * old = b->comp;
 		WINDOW * oldwin = b->compwin;
 		b->comp = NULL;
@@ -156,58 +146,9 @@ bool browser_destroy( struct s_browser * b )
 		WindDelete( oldwin );
 		mt_CompDelete(&app,  old );	
 	}
-
-	if( type == BT_FRAME ) {
-		/* TODO: restore the remaining frameset (rebuild???) */
-	}
 	return( true );
 }
 
-bool browser_attach_frame( struct gui_window * container, struct gui_window * frame )
-{
-	struct browser_window * cbw = container->browser->bw;
-	int lt = CLT_STACK;
-	if (cbw->rows >= cbw->cols)
-		lt = CLT_VERTICAL;
-	else 
-		lt = CLT_HORIZONTAL;
-
-	printf("attaching frame as: %s\n", (lt == CLT_VERTICAL) ? "CLT_VERTICAL" : "CLT_HORIZONTAL" );
-	/* todo: if first frame, remove compwin, or something like that, because it is still occupiying space */
-	container->browser->compwin->h_max = 0;
-	container->browser->compwin->w_max = 0;
-	container->browser->comp->flex = 0;
-	container->browser->comp->size = 300;
-	//container->browser->comp->bounds.max_height = 0;
-	//container->browser->comp->bounds.min_height = 0;
-	//WindClose( container->browser->compwin );
-	
-	container->browser->comp->type = lt;
-	mt_CompAttach( &app,  container->browser->comp, frame->browser->comp );
-	//win_comp_attach(&app, container->browser->compwin, frame->browser->comp);
-	browser_update_rects( container );
-	frame->browser->attached = true;
-	//container->browser->attached = true;
-	/*browser_update_rects( frame );
-	  LGRECT brect;
-	  browser_get_rect( frame, BR_CONTENT, &brect ); 
-	*/
-	
-}
-
-/* find the root of an frame ( or just return gw if is already the root) */
-struct gui_window * browser_find_root( struct gui_window * gw )
-{
-	if( gw->parent == NULL )
-		return( gw ); 
-	struct gui_window * g;
-	for( g=window_list; g; g=g->next){
-		if( g->root == gw->root && g->parent == NULL )
-			return( g );
-	}
-	return( NULL );
-}
-
 void browser_get_rect( struct gui_window * gw, enum browser_rect type, LGRECT * out)
 {
 	GRECT work;
@@ -282,13 +223,12 @@ static void __CDECL browser_evnt_destroy( COMPONENT * c, long buff[8], void * da
 {
 	struct s_browser * b = (struct s_browser*)data;
 	struct gui_window * gw = b->bw->window;
-	LOG(("%s: %s (%s)\n", __FUNCTION__ ,gw->browser->bw->name, ( gw->browser->type ==BT_FRAME) ? "FRAME" : "NOFRAME"));
-	
+	LOG(("%s\n",gw->browser->bw->name));
+
 	assert( b != NULL );
 	assert( gw != NULL );
-	/* TODO: inspect why this assert fails with frames / iframes
-	/* this should have been happened alrdy */
-	/* assert( b->comp == NULL ); */
+
+	assert( b->comp == NULL );
 
 	free( b );
 	gw->browser = NULL;
@@ -302,6 +242,10 @@ static void __CDECL browser_evnt_arrowed( WINDOW *win, short buff[8], void * dat
 	int value = BROWSER_SCROLL_SVAL;
 	struct gui_window * gw = data;
 	LGRECT cwork;
+
+	if( input_window == NULL || input_window != gw ) {
+		return;
+	}
 	browser_get_rect( gw, BR_CONTENT, &cwork );
 
 	switch( buff[4] ) {
@@ -314,7 +258,7 @@ static void __CDECL browser_evnt_arrowed( WINDOW *win, short buff[8], void * dat
 		case WA_RTPAGE:
 				value = cwork.g_w;
 			break;
-		
+
 		default: 
 			break;
 	}
@@ -325,11 +269,15 @@ void __CDECL browser_evnt_slider( WINDOW *win, short buff[8], void * data)
 {
 	int dx = buff[4];
 	int dy = buff[5];
-	struct gui_window * gw = data;
 	GRECT work, screen;
+	struct gui_window * gw = data;
 
 	if (!dx && !dy) return;
 
+	if( input_window == NULL || input_window != gw ) {
+		return;
+	}
+
 	/* update the sliders _before_ we call redraw (which might depend on the slider possitions) */
 	mt_WindSlider( &app, win, (dx?HSLIDER:0) | (dy?VSLIDER:0) );
 
@@ -347,40 +295,44 @@ static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data
 {
 	long lbuff[8];
 	short i;
-	short mbut, mkstat, mx, my;
-	graf_mkstate(&mx, &my, &mbut, &mkstat);
+	short mx, my, dummy, mbut;
 	uint32_t tnow = clock()*1000 / CLOCKS_PER_SEC;
 	LGRECT cwork;
 	struct gui_window * gw = data;
-	input_window = gw;
+	if( input_window != gw ) {
+		return;
+	}
 	window_set_focus( gw, BROWSER, (void*)gw->browser );
 	browser_get_rect( gw, BR_CONTENT, &cwork );
-	mx = evnt.mx - cwork.g_x; /*+ gw->browser->scroll.current.x*/;
-	my = evnt.my - cwork.g_y; /*+ gw->browser->scroll.current.y*/;
-	LOG(("mevent within %s at %d / %d", gw->browser->bw->name, mx, my ));
+	mx = evnt.mx - cwork.g_x; 
+	my = evnt.my - cwork.g_y;
+	LOG(("mevent (%d) within %s at %d / %d\n", evnt.nb_click, gw->browser->bw->name, mx, my ));
 
-	if( (mkstat & K_RSHIFT) || (mkstat & K_LSHIFT) ){
+	if( evnt.mkstate & (K_RSHIFT | K_LSHIFT) ){
 		bmstate |= BROWSER_MOUSE_MOD_1;
 	} else {
 		bmstate &= ~(BROWSER_MOUSE_MOD_1);
 	}
-	if( (mkstat & K_CTRL) ){
+	if( (evnt.mkstate & K_CTRL) ){
 		bmstate |= BROWSER_MOUSE_MOD_2;
 	} else {
-		bmstate &= ~(BROWSER_MOUSE_MOD_1);
+		bmstate &= ~(BROWSER_MOUSE_MOD_2);
 	}
-	if( (mkstat & K_ALT) ){
+	if( (evnt.mkstate & K_ALT) ){
 		bmstate |= BROWSER_MOUSE_MOD_3;
 	} else {
-		bmstate &= ~(BROWSER_MOUSE_MOD_2);
-	}	
+		bmstate &= ~(BROWSER_MOUSE_MOD_3);
+	}
 	int sx = (mx + gw->browser->scroll.current.x);
 	int sy = (my + gw->browser->scroll.current.y);
+
+	graf_mkstate(&dummy, &dummy, &mbut, &dummy);
+	/* todo: if we need right button click, increase loop count */
 	for( i = 1; i<2; i++) {
-		if( (mbut & i) ) {
+		if( (mbut & i)  ) {
 			if( mouse_hold_start[i-1] == 0 ) {
-				mouse_hold_start[i-1] = clock()*1000 / CLOCKS_PER_SEC;
-				LOG(("Drag %d starts", i));
+				mouse_hold_start[i-1] = tnow;
+				LOG(("Drag %d starts at %d,%d\n", i, sx, sy));
 				if( i == 1 ) {
 					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_1,sx,sy);
 					bmstate |= BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_ON;
@@ -388,7 +340,7 @@ static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data
 				if( i == 2 ) {
 					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_2,sx,sy);
 					bmstate |= BROWSER_MOUSE_HOLDING_2 | BROWSER_MOUSE_DRAG_ON;
-				}				
+				}
 			} else {
 				if( i == 1 ) {
 					bmstate |= BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_DRAG_ON;
@@ -397,22 +349,29 @@ static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data
 					bmstate |= BROWSER_MOUSE_DRAG_2 | BROWSER_MOUSE_DRAG_ON;
 				}
 			}
-		} else {
-			/* remember click time, so we can track double clicks: */
-			/* this does not really work, because a normal double click is swallowed by evnt_multi */
-			if( tnow - mouse_click_time[i-1] < 500 ) {
-				/* double click */
+
+			if( i != 0 ){
+				if( (abs(mx-last_drag_x)>5) || (abs(mx-last_drag_y)>5) ){	
+					browser_window_mouse_track( 
+						gw->browser->bw,
+						bmstate, 
+						sx, sy
+					);
+					last_drag_x = mx;
+					last_drag_y = my;
+				}
 			}
+
+		} else {
 			mouse_click_time[i-1] = tnow; /* clock in ms */
 			/* check if this event was during an drag op: */
-			if( mouse_hold_start[i-1] < 10 ) {
+			if( mouse_hold_start[i-1] == 0 ) {
 				if( i == 1) {
-					LOG(("Click within %s at %d / %d", gw->browser->bw->name, sx, sy ));
+					LOG(("Click within %s at %d / %d\n", gw->browser->bw->name, sx, sy ));
 					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_1,sx,sy);
 					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_CLICK_1,sx,sy);
 					bmstate &= ~( BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_CLICK_1);
 				}
-					
 				if( i == 2 ) {
 					LOG(("Click within %s at %d / %d", gw->browser->bw->name, mx, my ));
 					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_1,sx,sy);
@@ -422,7 +381,7 @@ static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data
 			}
 			mouse_hold_start[i-1] = 0;
 		}
-	}	
+	}
 }
 
 void browser_scroll( struct gui_window * gw, short mode, int value, bool abs )
@@ -523,7 +482,7 @@ static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 	GRECT src;
 	GRECT dst;
 	short h,w;
-	
+
 	if( gw->browser->bw->current_content == NULL )
 		return;
 
@@ -547,7 +506,7 @@ static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 		src.g_w = bwrect.g_w;
 		src.g_h = bwrect.g_h - h;
 		dst.g_x = 0;
-		dst.g_y = h;	
+		dst.g_y = h;
 		dst.g_w = src.g_w;
 		dst.g_h = src.g_h;
 		plotter->copy_rect( plotter, src, dst );
@@ -621,7 +580,7 @@ bool browser_input( struct gui_window * gw, unsigned short nkc )
 			case 'A':
 				r = browser_window_key_press(gw->browser->bw, KEY_SELECT_ALL);
 			break;
-	
+
 			case 'C':
 				r = browser_window_key_press(gw->browser->bw, KEY_COPY_SELECTION);
 			break;
@@ -629,14 +588,14 @@ bool browser_input( struct gui_window * gw, unsigned short nkc )
 			case 'X':
 				r = browser_window_key_press(gw->browser->bw, KEY_CUT_SELECTION);
 			break;
-	
+
 			case 'V':
 				r = browser_window_key_press(gw->browser->bw, KEY_PASTE);
 			break;
 
 			default:
-			break;		
-		}		
+			break;
+		}
 	}
 	if( (nkc & NKF_SHIFT) != 0 ) {
 		switch( ascii ) {
@@ -761,52 +720,36 @@ bool browser_input( struct gui_window * gw, unsigned short nkc )
             r = browser_window_key_press(gw->browser->bw, ucs4 );
 		}
 	}
-	return( r );	
+	return( r );
 }
 
 static void __CDECL browser_evnt_redraw_x( WINDOW * c, short buf[8], void * data)
-{	
-	/* just an stub to prevent wndclear */ 
-	/* Probably the browser redraw is better placed here? dunno! */ 
+{
+	/* just an stub to prevent wndclear */
+	/* Probably the browser redraw is better placed here? dunno! */
 	return;
 }
 
 /* determines if a browser window needs redraw */
-bool browser_redraw_required( struct gui_window * gw) 
+bool browser_redraw_required( struct gui_window * gw)
 {
 	bool ret = true;
-	int frames = 0;
 	CMP_BROWSER b = gw->browser;
 
 	if( b->bw->current_content == NULL )
 		return ( false );
 
-	{	
-		/* don't do redraws if we have subframes */
-		/* iframes will be an special case and must be handled special... */
-		struct gui_window * g;
-		for( g=window_list; g; g=g->next ) {
-			if ( g != gw && g->parent == gw ) {
-				if( g->browser->type == BT_FRAME ) {
-					frames++;
-				}
-			}
-		}
-	}
-	ret = ( (b->redraw.required && frames == 0) || 
-			b->scroll.required || 
-			b->caret.redraw );
+	ret = ( ((b->redraw.areas_used > 0) )
+			|| b->scroll.required
+			|| b->caret.redraw );
 	return( ret );
 }
 
 
 /* schedule a redraw of content */
 /* coords are relative to the framebuffer */
-/* the box is modified to fit the current viewport if it doesn't fit in */
 void browser_schedule_redraw_rect(struct gui_window * gw, short x, short y, short w, short h)
 {
-	int diff;
-
 	if( x < 0  ){
 		w += x;
 		x = 0;
@@ -814,22 +757,38 @@ void browser_schedule_redraw_rect(struct gui_window * gw, short x, short y, shor
 
 	if( y < 0 ) {
 		h += y;
-		y = 0;	
+		y = 0;
 	}
 	browser_schedule_redraw( gw, x, y, x+w, y+h );
 }
 
-/* schedule a redraw of content */
-/* coords are relative to the framebuffer */
+static inline bool rect_intersect( struct rect * box1, struct rect * box2 )
+{
+	if (box2->x1 < box1->x0)
+ 	       return false;
+
+	if (box2->y1 < box1->y0)
+        	return false;
+
+	if (box2->x0 > box1->x1)
+        	return false;
+
+	if (box2->y0 > box1->y1)
+        	return false;
+
+	return true;
+}
+
+/*
+	schedule a redraw of content, coords are relative to the framebuffer
+*/
 void browser_schedule_redraw(struct gui_window * gw, short x0, short y0, short x1, short y1)
 {
 	assert( gw != NULL );
 	CMP_BROWSER b = gw->browser;
+	int i;
+	struct rect area;
 	LGRECT work;
-	/* TODO: add rectangle to list, instead of summarizing the rect.? */
-	/* otherwise it can result in large areas, altough it isnt needed. ( like 1 px in the upper left, 
-	   and 1px in bottom right corner ) 
-	*/
 
 	if( y1 < 0 || x1 < 0 )
 		return;
@@ -840,53 +799,67 @@ void browser_schedule_redraw(struct gui_window * gw, short x0, short y0, short x
 	if( y0 > work.g_h )
 		return;
 
-	/* special handling of initial call: */
-	if( b->redraw.required == false ) {
-		b->redraw.required = true;		
-		b->redraw.area.x0 = x0;
-		b->redraw.area.y0 = y0;
-		b->redraw.area.x1 = x1;
-		b->redraw.area.y1 = y1;
-	} else {
-		b->redraw.area.x0 = MIN(b->redraw.area.x0, x0);
-		b->redraw.area.y0 = MIN(b->redraw.area.y0, y0);
-		b->redraw.area.x1 = MAX(b->redraw.area.x1, x1);
-		b->redraw.area.y1 = MAX(b->redraw.area.y1, y1);
+	area.x0 = x0;
+	area.y0 = y0;
+	area.x1 = x1;
+	area.y1 = y1;
+
+	for( i=0; i<b->redraw.areas_used; i++) {
+		if( b->redraw.areas[i].x0 <= x0
+			&& b->redraw.areas[i].x1 >= x1
+			&& b->redraw.areas[i].y0 <= y0
+			&& b->redraw.areas[i].y1 >= y1 ){
+			/* the area is already queued for redraw */
+			return;
+		} else {
+			if( rect_intersect(&b->redraw.areas[i], &area ) ){
+				b->redraw.areas[i].x0 = MIN(b->redraw.areas[i].x0, x0);
+				b->redraw.areas[i].y0 = MIN(b->redraw.areas[i].y0, y0);
+				b->redraw.areas[i].x1 = MAX(b->redraw.areas[i].x1, x1);
+				b->redraw.areas[i].y1 = MAX(b->redraw.areas[i].y1, y1);
+				return;
+			}
+		}
 	} 
+
+	if( b->redraw.areas_used < MAX_REDRW_SLOTS ) {
+		b->redraw.areas[b->redraw.areas_used].x0 = x0;
+		b->redraw.areas[b->redraw.areas_used].x1 = x1;
+		b->redraw.areas[b->redraw.areas_used].y0 = y0;
+		b->redraw.areas[b->redraw.areas_used].y1 = y1;
+		b->redraw.areas_used++;
+	} else {
+		/*
+			we are out of available slots, merge box with last slot
+			this is dumb... but also a very rare case.
+		*/
+		b->redraw.areas[MAX_REDRW_SLOTS-1].x0 = MIN(b->redraw.areas[i].x0, x0);
+		b->redraw.areas[MAX_REDRW_SLOTS-1].y0 = MIN(b->redraw.areas[i].y0, y0);
+		b->redraw.areas[MAX_REDRW_SLOTS-1].x1 = MAX(b->redraw.areas[i].x1, x1);
+		b->redraw.areas[MAX_REDRW_SLOTS-1].y1 = MAX(b->redraw.areas[i].y1, y1);
+	}
+done:
+	return;
 }
 
 static void browser_redraw_content( struct gui_window * gw, int xoff, int yoff )
 {
 	LGRECT work;
 	CMP_BROWSER b = gw->browser;
-	GRECT area;
-	struct rect clip;
 
-	LOG(("%s : %d,%d - %d,%d\n", b->bw->name, b->redraw.area.x0, 
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &atari_plotters
+	};
+
+	LOG(("%s : %d,%d - %d,%d\n", b->bw->name, b->redraw.area.x0,
 		b->redraw.area.y0, b->redraw.area.x1, b->redraw.area.y1
 	));
-	area.g_x = b->redraw.area.x0;
-	area.g_y = b->redraw.area.y0;
-	area.g_w = b->redraw.area.x1 - b->redraw.area.x0;
-	area.g_h = b->redraw.area.y1 - b->redraw.area.y0;
 
-	current_redraw_browser = b->bw;
 
-	clip.x0 = b->redraw.area.x0;
-	clip.y0 = b->redraw.area.y0;
-	clip.x1 = b->redraw.area.x1;
-	clip.y1 = b->redraw.area.y1;
-	
 	browser_window_redraw( b->bw, -b->scroll.current.x,
-			-b->scroll.current.y, &clip );
+			-b->scroll.current.y, &b->redraw.area, &ctx );
 
-	current_redraw_browser = NULL;
-
-	/* reset redraw area */
-	b->redraw.area.x0 = INT_MAX;
-	b->redraw.area.y0 = INT_MAX;
-	b->redraw.area.x1 = INT_MIN;
-	b->redraw.area.y1 = INT_MIN;
 }
 
 
@@ -922,7 +895,7 @@ void browser_redraw_caret( struct gui_window * gw, GRECT * area )
 }
 
 void browser_redraw( struct gui_window * gw ) 
-{	
+{
 	LGRECT bwrect;
 	struct s_browser * b = gw->browser;
 	short todo[4];
@@ -948,43 +921,55 @@ void browser_redraw( struct gui_window * gw )
 		b->scroll.required = false;
 	}
 
-	if (b->redraw.required == true && b->bw->current_content != NULL ) {
+	if ((b->redraw.areas_used > 0) && b->bw->current_content != NULL ) {
 		if( (plotter->flags & PLOT_FLAG_OFFSCREEN) == 0 ) {
+			int i;
 			GRECT area;
-			BBOX cliporg;
+			GRECT fbwork;
 			todo[0] = bwrect.g_x;
 			todo[1] = bwrect.g_y;
 			todo[2] = todo[0] + bwrect.g_w-1;
 			todo[3] = todo[1] + bwrect.g_h-1;
 			vs_clip(plotter->vdi_handle, 1, (short*)&todo[0]);
-
-			area.g_x = b->redraw.area.x0; 
-			area.g_y = b->redraw.area.y0; 
-			area.g_w = b->redraw.area.x1 - b->redraw.area.x0;
-			area.g_h = b->redraw.area.y1 - b->redraw.area.y0; 
-			if( wind_get(gw->root->handle->handle, WF_FIRSTXYWH, 
+			if( wind_get(gw->root->handle->handle, WF_FIRSTXYWH,
 							&todo[0], &todo[1], &todo[2], &todo[3] )!=0 ) {
 				while (todo[2] && todo[3]) {
 					/* convert screen to framebuffer coords: */
-					todo[0] -= bwrect.g_x;
-					todo[1] -= bwrect.g_y;
-					if( todo[0] < 0 ){
-						todo[2] = todo[2] + todo[0];
-						todo[0] = 0;
+					fbwork.g_x = todo[0] - bwrect.g_x;
+					fbwork.g_y = todo[1] - bwrect.g_y;
+					if( fbwork.g_x < 0 ){
+						fbwork.g_w = todo[2] + todo[0];
+						fbwork.g_x = 0;
+					} else {
+						fbwork.g_w = todo[2];
 					}
-					if( todo[1] < 0 ){
-						todo[3] = todo[3] + todo[1];
-						todo[1] = 0;
+					if( fbwork.g_y < 0 ){
+						fbwork.g_h = todo[3] + todo[1];
+						fbwork.g_y = 0;
+					} else {
+						fbwork.g_h = todo[3];
 					}
-					
-					if (rc_intersect((GRECT *)&area,(GRECT *)&todo)) {
-						b->redraw.area.x0 = todo[0];
-						b->redraw.area.y0 = todo[1];
-						b->redraw.area.x1 = b->redraw.area.x0 + todo[2];
-						b->redraw.area.y1 = b->redraw.area.y0 + todo[3];
-						browser_redraw_content( gw, 0, 0 );
+					/* walk the redraw requests: */
+					for( i=0; i<b->redraw.areas_used; i++ ){
+						area.g_x = b->redraw.areas[i].x0;
+						area.g_y = b->redraw.areas[i].y0;
+						area.g_w = b->redraw.areas[i].x1 - b->redraw.areas[i].x0;
+						area.g_h = b->redraw.areas[i].y1 - b->redraw.areas[i].y0; 
+						if (rc_intersect((GRECT *)&fbwork,(GRECT *)&area)) {
+							b->redraw.area.x0 = area.g_x;
+							b->redraw.area.y0 = area.g_y;
+							b->redraw.area.x1 = area.g_x + area.g_w;
+							b->redraw.area.y1 = area.g_y + area.g_h;
+							browser_redraw_content( gw, 0, 0 );
+						} else {
+							/*
+								the area should be kept scheduled for later redraw, but because this
+								is onscreen plotter, it doesn't make much sense anyway...
+							*/
+						}
+
 					}
-					if (wind_get(gw->root->handle->handle, WF_NEXTXYWH, 
+					if (wind_get(gw->root->handle->handle, WF_NEXTXYWH,
 							&todo[0], &todo[1], &todo[2], &todo[3])==0) {
 						break;
 					}
@@ -994,7 +979,7 @@ void browser_redraw( struct gui_window * gw )
 		} else {
 			/* its save to do a complete redraw :) */
 		}
-		b->redraw.required = false;
+		b->redraw.areas_used = 0;
 	}
 	if( b->caret.redraw == true && 	b->bw->current_content != NULL ) {
 		GRECT area;
@@ -1016,39 +1001,47 @@ void browser_redraw( struct gui_window * gw )
 }
 
 static void __CDECL browser_evnt_redraw( COMPONENT * c, long buff[8], void * data)
-{	
+{
 	short pxy[8];
 	WINDOW * w;
 	struct gui_window * gw = (struct gui_window *) data;
 	CMP_BROWSER b = gw->browser;
-	struct gui_window * rgw = browser_find_root( gw );
 	LGRECT work, lclip, rwork;
+
+	// TODO: maybe implement something like validate_gw()
+	// to fetch spurious redraw events? the function should
+	// traverse all gui_windows and see if gw  exists in the list
+
 	int xoff,yoff,width,heigth;
 	short cw, ch, cellw, cellh;
+	/* use that instead of browser_find_root() ? */
 	w = (WINDOW*)mt_CompGetPtr( &app, c, CF_WINDOW );
 	browser_get_rect( gw, BR_CONTENT, &work );
-	browser_get_rect( rgw, BR_CONTENT, &rwork );
 	lclip = work;
 	if ( !rc_lintersect( (LGRECT*)&buff[4], &lclip ) ) return;
-	 
+
 	if( b->bw->current_content == NULL )
 		return;
+
 	/* convert redraw coords to framebuffer coords: */
 	lclip.g_x -= work.g_x;
 	lclip.g_y -= work.g_y;
+
 	if( lclip.g_x < 0 ) {
 		lclip.g_w = work.g_w + lclip.g_x;
 		lclip.g_x = 0;
 	}
+
 	if( lclip.g_y < 0 ) {
 		lclip.g_h = work.g_h + lclip.g_y;
 		lclip.g_y = 0;
 	}
-	if( lclip.g_h > 0 && lclip.g_w > 0 ) { 
-		browser_schedule_redraw( gw, lclip.g_x, lclip.g_y, 
-			lclip.g_x + lclip.g_w, lclip.g_y + lclip.g_h 
-		);	
+
+	if( lclip.g_h > 0 && lclip.g_w > 0 ) {
+		browser_schedule_redraw( gw, lclip.g_x, lclip.g_y,
+			lclip.g_x + lclip.g_w, lclip.g_y + lclip.g_h
+		);
 	}
 
 	return;
-}
\ No newline at end of file
+}
