@@ -57,12 +57,14 @@
 #include "atari/search.h"
 #include "atari/osspec.h"
 
-
-bool cfg_rt_resize = false;
-bool cfg_rt_move = false;
 extern void * h_gem_rsrc;
 extern struct gui_window *input_window;
 extern GEM_PLOTTER plotter;
+extern int mouse_click_time[3];
+extern int mouse_hold_start[3];
+extern browser_mouse_state bmstate;
+extern short last_drag_x;
+extern short last_drag_y;
 
 void __CDECL std_szd( WINDOW * win, short buff[8], void * );
 void __CDECL std_mvd( WINDOW * win, short buff[8], void * );
@@ -77,11 +79,11 @@ static void __CDECL evnt_window_arrowed( WINDOW *win, short buff[8], void *data
 	bool abs = false;
 	LGRECT cwork;
 	int value = BROWSER_SCROLL_SVAL;
-	
+
 	if( input_window == NULL ) {
 		return;
 	}
-	
+
 	browser_get_rect( input_window, BR_CONTENT, &cwork );
 
 	switch( buff[4] ) {
@@ -95,13 +97,168 @@ static void __CDECL evnt_window_arrowed( WINDOW *win, short buff[8], void *data
 		case WA_RTPAGE:
 				value = cwork.g_w;
 			break;
-					
-		default: 
+
+		default:
 			break;
 	}
 	browser_scroll( input_window, buff[4], value, abs );
 }
 
+/*
+	track the mouse state and
+	finally checks for released buttons.
+ */
+static void window_track_mouse_state( LGRECT * bwrect, bool within, short mx, short my, short mbut, short mkstate ){
+	int i = 0;
+	int nx, ny;
+	struct gui_window * gw = input_window;
+
+	if( !gw ) {
+		bmstate = 0;
+		mouse_hold_start[0] = 0;
+		mouse_hold_start[1] = 0;
+		return;
+	}
+
+	/* todo: creat function find_browser_window( mx, my ) */
+	nx = (mx - bwrect->g_x + gw->browser->scroll.current.x);
+	ny = (my - bwrect->g_y + gw->browser->scroll.current.y);
+
+	if( mkstate & (K_RSHIFT | K_LSHIFT) ){
+		bmstate |= BROWSER_MOUSE_MOD_1;
+	} else {
+		bmstate &= ~(BROWSER_MOUSE_MOD_1);
+	}
+	if( (mkstate & K_CTRL) ){
+		bmstate |= BROWSER_MOUSE_MOD_2;
+	} else {
+		bmstate &= ~(BROWSER_MOUSE_MOD_2);
+	}
+	if( (mkstate & K_ALT) ){
+		bmstate |= BROWSER_MOUSE_MOD_3;
+	} else {
+		bmstate &= ~(BROWSER_MOUSE_MOD_3);
+	}
+
+	if( !(mbut&1) && !(mbut&2) ) {
+		if(bmstate & BROWSER_MOUSE_DRAG_ON )
+			bmstate &= ~( BROWSER_MOUSE_DRAG_ON );
+	}
+
+	/* todo: if we need right button click, increase loop count */
+	for( i = 1; i<2; i++ ) {
+		if( !(mbut & i) ) {
+			if( mouse_hold_start[i-1] > 0 ) {
+				mouse_hold_start[i-1] = 0;
+				if( i==1 ) {
+					LOG(("Drag for %d ended", i));
+					bmstate &= ~( BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_1 ) ;
+					if( within ) {
+						/* drag end */
+						browser_window_mouse_track(
+							gw->browser->bw, 0, nx, ny
+						);
+					}
+				}
+				if( i==2 ) {
+					bmstate &= ~( BROWSER_MOUSE_HOLDING_2 | BROWSER_MOUSE_DRAG_2 ) ;
+					LOG(("Drag for %d ended", i));
+					if( within ) {
+						/* drag end */
+						browser_window_mouse_track(
+							gw->browser->bw, 0, nx, ny
+						);
+					}
+				}
+			}
+		}
+	}
+}
+
+
+static void __CDECL evnt_window_m1( WINDOW * win, short buff[8], void * data)
+{
+	struct gui_window * gw = input_window;
+	static bool prev_url = false;
+	static bool prev_sb = false;
+	short mx, my, mbut, mkstate;
+	bool a = false;	/* flags if mouse is within controls or browser canvas */
+	bool within = false;
+	LGRECT urlbox, bwbox, sbbox;
+	int nx, ny; 	/* relative mouse position */
+
+
+	if( gw == NULL)
+		return;
+
+	if( gw != input_window ){
+		return;
+	}
+
+	graf_mkstate(&mx, &my, &mbut, &mkstate);
+
+	browser_get_rect( gw, BR_CONTENT, &bwbox );
+	if( gw->root->toolbar )
+		mt_CompGetLGrect(&app, gw->root->toolbar->url.comp, WF_WORKXYWH, &urlbox);
+	if( gw->root->statusbar )
+		mt_CompGetLGrect(&app, gw->root->statusbar->comp, WF_WORKXYWH, &sbbox);
+
+	if( mx > bwbox.g_x && mx < bwbox.g_x + bwbox.g_w &&
+		my > bwbox.g_y &&  my < bwbox.g_y + bwbox.g_h ){
+		within = true;
+	}
+
+	if( evnt.m1_flag == MO_LEAVE ) {
+		if( MOUSE_IS_DRAGGING() ){
+			window_track_mouse_state( &bwbox, within, mx, my, mbut, mkstate );
+		}
+		if( gw->root->toolbar && within == false ) {
+			if( (mx > urlbox.g_x && mx < urlbox.g_x + urlbox.g_w ) &&
+			 	(my > urlbox.g_y && my < + urlbox.g_y + urlbox.g_h )) {
+				gem_set_cursor( &gem_cursors.ibeam );
+				prev_url = a = true;
+			}
+		}
+		if( gw->root->statusbar && within == false /* && a == false */ ) {
+			if( mx >= sbbox.g_x + (sbbox.g_w-MOVER_WH) && mx <= sbbox.g_x + sbbox.g_w &&
+				my >= sbbox.g_y + (sbbox.g_h-MOVER_WH) && my <= sbbox.g_y + sbbox.g_h ) {
+				/* mouse within sizer box ( bottom right ) */
+				prev_sb = a =  true;
+				gem_set_cursor( &gem_cursors.sizenwse );
+			}
+		}
+		if( !a ) {
+			if( prev_sb )
+				gw->root->statusbar->resize_init = true;
+			if( prev_url || prev_sb ) {
+				gem_set_cursor( &gem_cursors.arrow );
+				prev_url = false;
+				prev_sb = false;
+			}
+			/* report mouse move in the browser window */
+			if( within ){
+				nx = mx - bwbox.g_x;
+				ny = my - bwbox.g_y;
+				if( ( abs(mx-last_drag_x)>5 || abs(mx-last_drag_y)>5 ) ||
+					!MOUSE_IS_DRAGGING() ){
+					browser_window_mouse_track(
+						input_window->browser->bw,
+						bmstate,
+						nx + gw->browser->scroll.current.x,
+						ny + gw->browser->scroll.current.y
+					);
+					if( MOUSE_IS_DRAGGING() ){
+						last_drag_x = mx;
+						last_drag_y = my;
+					}
+				}
+			}
+		}
+	} else {
+		/* set input window? */
+	}
+}
+
 int window_create( struct gui_window * gw, struct browser_window * bw, unsigned long inflags)
 {
 	short buff[8];
@@ -112,13 +269,18 @@ int window_create( struct gui_window * gw, struct browser_window * bw, unsigned
 	sb = (inflags & WIDGET_STATUSBAR); 
 	short w,h, wx, wy, wh, ww;
 	int flags = CLOSER | MOVER | NAME | FULLER | SMALLER ;
-	gw->parent = NULL;
+
 	gw->root = malloc( sizeof(struct s_gui_win_root) );
 	if( gw->root == NULL )
 		return( -1 );
 	memset( gw->root, 0, sizeof(struct s_gui_win_root) );
 	gw->root->title = malloc(atari_sysinfo.aes_max_win_title_len+1);
 	gw->root->handle = WindCreate( flags,40, 40, app.w, app.h );
+	if( gw->root->handle == NULL ) {
+		free( gw->root->title );
+		free( gw->root );
+		return( -1 );
+	}
 	gw->root->cmproot = mt_CompCreate(&app, CLT_VERTICAL, 1, 1);
 	WindSetPtr( gw->root->handle, WF_COMPONENT, gw->root->cmproot, NULL);
 
@@ -126,12 +288,12 @@ int window_create( struct gui_window * gw, struct browser_window * bw, unsigned
 		gw->root->toolbar = tb_create( gw );
 		assert( gw->root->toolbar );
 		mt_CompAttach( &app, gw->root->cmproot, gw->root->toolbar->comp );
-		
+
 	} else {
 		gw->root->toolbar = NULL;
 	}
 
-	gw->browser = browser_create( gw, bw, NULL, BT_ROOT, CLT_HORIZONTAL, 1, 1 );
+	gw->browser = browser_create( gw, bw, NULL, CLT_HORIZONTAL, 1, 1 );
 	mt_CompAttach( &app, gw->root->cmproot,  gw->browser->comp );
 
 	if( sb ) {
@@ -142,27 +304,24 @@ int window_create( struct gui_window * gw, struct browser_window * bw, unsigned
 	}
 
 	WindSetStr(gw->root->handle, WF_ICONTITLE, (char*)"NetSurf");
-	
+
 	/* Event Handlers: */
-	EvntDataAttach( gw->root->handle, WM_CLOSED, evnt_window_close, NULL );
+	EvntDataAttach( gw->root->handle, WM_CLOSED, evnt_window_close, gw );
 	/* capture resize/move events so we can handle that manually */
-	if( !cfg_rt_resize ) {
-		EvntAttach( gw->root->handle, WM_SIZED, evnt_window_resize);
+	EvntDataAttach( gw->root->handle, WM_SIZED, evnt_window_resize, gw );
+	if( !option_atari_realtime_move ) {
+		EvntDataAttach( gw->root->handle, WM_MOVED, evnt_window_move, gw );
 	} else {
-		EvntAdd( gw->root->handle, WM_SIZED, evnt_window_rt_resize, EV_BOT );
+		EvntDataAdd( gw->root->handle, WM_MOVED, evnt_window_rt_resize, gw, EV_BOT );
 	}
-	if( !cfg_rt_move ) {
-		EvntAttach( gw->root->handle, WM_MOVED, evnt_window_move );
-	} else {
-		EvntAdd( gw->root->handle, WM_MOVED, evnt_window_rt_resize, EV_BOT );
-	}
-	EvntAttach( gw->root->handle, WM_FORCE_MOVE, evnt_window_rt_resize );
+	EvntDataAttach( gw->root->handle, WM_FORCE_MOVE, evnt_window_rt_resize, gw );
 	EvntDataAttach( gw->root->handle, AP_DRAGDROP, evnt_window_dd, gw );
-	EvntDataAdd( gw->root->handle, WM_DESTROY,evnt_window_destroy, NULL, EV_TOP );
-	EvntDataAdd( gw->root->handle, WM_ARROWED,evnt_window_arrowed, NULL, EV_TOP );
-	EvntDataAdd( gw->root->handle, WM_NEWTOP, evnt_window_newtop, &evnt_data, EV_BOT);
-	EvntDataAdd( gw->root->handle, WM_TOPPED, evnt_window_newtop, &evnt_data, EV_BOT);
+	EvntDataAdd( gw->root->handle, WM_DESTROY,evnt_window_destroy, gw, EV_TOP );
+	EvntDataAdd( gw->root->handle, WM_ARROWED,evnt_window_arrowed, gw, EV_TOP );
+	EvntDataAdd( gw->root->handle, WM_NEWTOP, evnt_window_newtop, gw, EV_BOT);
+	EvntDataAdd( gw->root->handle, WM_TOPPED, evnt_window_newtop, gw, EV_BOT);
 	EvntDataAttach( gw->root->handle, WM_ICONDRAW, evnt_window_icondraw, gw);
+	EvntDataAttach( gw->root->handle, WM_XM1, evnt_window_m1, gw );
 
 	/*
 	OBJECT * tbut;
@@ -179,15 +338,7 @@ int window_destroy( struct gui_window * gw)
 	short buff[8];
 	int err = 0;
 
-	if( gw->browser->type != BT_ROOT ) {
-		return(0);
-	}
-	
-	/* test this with frames: */
-	/* assert( gw->parent == NULL); */
-
 	search_destroy( gw );
-
 	if( input_window == gw )
 		input_window = NULL;
 
@@ -199,10 +350,13 @@ int window_destroy( struct gui_window * gw)
 			sb_destroy( gw->root->statusbar );
 	}
 
+	search_destroy( gw );
+
 	LOG(("Freeing browser window"));
 	if( gw->browser )
 		browser_destroy( gw->browser );
 
+
 	/* destroy the icon: */
 	/*window_set_icon(gw, NULL, false );*/
 
@@ -223,9 +377,12 @@ int window_destroy( struct gui_window * gw)
 	return( err );
 }
 
+
+
 void window_open( struct gui_window * gw)
 {
 	LGRECT br;
+	GRECT dim;
 	WindOpen(gw->root->handle, 20, 20, app.w/2, app.h/2 );
 	WindSetStr( gw->root->handle, WF_NAME, (char *)"" );
 	/* apply focus to the root frame: */
@@ -233,6 +390,7 @@ void window_open( struct gui_window * gw)
 	mt_CompEvntExec( gl_appvar, gw->browser->comp, lfbuff );
 	/* recompute the nested component sizes and positions: */
 	browser_update_rects( gw );
+	mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
 	browser_get_rect( gw, BR_CONTENT, &br );
 	plotter->move( plotter, br.g_x, br.g_y );
 	plotter->resize( plotter, br.g_w, br.g_h );
@@ -288,7 +446,7 @@ static void window_redraw_controls(struct gui_window *gw, uint32_t flags)
 	browser_get_rect( gw, BR_VSLIDER, &rect);
 	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
 		rect.g_x, rect.g_y, rect.g_w, rect.g_h );
-	
+
 	browser_get_rect( gw, BR_HSLIDER, &rect);
 	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
 		rect.g_x, rect.g_y, rect.g_w, rect.g_h );
@@ -466,8 +624,8 @@ static void __CDECL evnt_window_dd( WINDOW *win, short wbuff[8], void * data )
 			}
 		}
 	}
-error:	
-	ddclose( dd_hdl);	
+error:
+	ddclose( dd_hdl);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -476,14 +634,11 @@ error:
 static void __CDECL evnt_window_destroy( WINDOW *win, short buff[8], void *data )
 {
 	LOG(("%s\n", __FUNCTION__ ));
-
-	if( data )
-		free( data  );
 }
 
 static void __CDECL evnt_window_close( WINDOW *win, short buff[8], void *data )
 {
-	struct gui_window * gw = find_root_gui_window( win );
+	struct gui_window * gw = (struct gui_window *) data ;
 	if( gw != NULL ) {
 		browser_window_destroy( gw->browser->bw );
 	}
@@ -492,7 +647,7 @@ static void __CDECL evnt_window_close( WINDOW *win, short buff[8], void *data )
 
 static void __CDECL evnt_window_newtop( WINDOW *win, short buff[8], void *data )
 {
-	input_window = find_root_gui_window( win );
+	input_window = (struct gui_window *) data;
 	LOG(("newtop: iw: %p, win: %p", input_window, win ));
 	assert( input_window != NULL );
 
@@ -517,7 +672,7 @@ static void __CDECL evnt_window_icondraw( WINDOW *win, short buff[8], void * dat
 
 	WindClear( win);
 	WindGet( win, WF_WORKXYWH, &x, &y, &w, &h);
-	
+
 	if( has_favicon == false ) {
 		OBJECT * tree;
 		RsrcGaddr( h_gem_rsrc, R_TREE, ICONIFY , &tree );
@@ -529,15 +684,15 @@ static void __CDECL evnt_window_icondraw( WINDOW *win, short buff[8], void * dat
 	}
 }
 
-static void __CDECL evnt_window_move( WINDOW *win, short buff[8] )
+static void __CDECL evnt_window_move( WINDOW *win, short buff[8], void * data )
 {
 	short mx,my, mb, ks;
 	short wx, wy, wh, ww, nx, ny;
 	short r;
 	short xoff, yoff;
-	if( cfg_rt_move  ) {
+	if( option_atari_realtime_move  ) {
 		std_mvd( win, buff, &app );
-		evnt_window_rt_resize( win, buff );
+		evnt_window_rt_resize( win, buff, data );
 	} else { 
 		wind_get( win->handle, WF_CURRXYWH, &wx, &wy, &ww, &wh );
 		if( graf_dragbox( ww, wh, wx, wy, app.x-ww, app.y, app.w+ww, app.h+wh, &nx, &ny )){
@@ -546,69 +701,59 @@ static void __CDECL evnt_window_move( WINDOW *win, short buff[8] )
 			buff[6] = ww;
 			buff[7] = wh;
 			std_mvd( win, buff, &app );
-			evnt_window_rt_resize( win, buff );
+			evnt_window_rt_resize( win, buff, data );
 		}
 	}
 }
 
-void __CDECL evnt_window_resize( WINDOW *win, short buff[8] )
+void __CDECL evnt_window_resize( WINDOW *win, short buff[8], void * data )
 {
-	short mx,my, mb, ks;
 	short wx, wy, wh, ww, nw, nh;
 	short r;
-	graf_mkstate( &mx, &my, &mb,  &ks ); 
-	if( cfg_rt_resize  ) {
-		std_szd( win, buff, &app );
-		evnt_window_rt_resize( win, buff );
-	} else { 
-		wind_get( win->handle, WF_CURRXYWH, &wx, &wy, &ww, &wh );
-		r = graf_rubberbox(wx, wy, 20, 20, &nw, &nh);
-		if( nw < 40 && nw < 40 )
-			return;
-		buff[4] = wx;
-		buff[5] = wy;
-		buff[6] = nw;
-		buff[7] = nh;
-		std_szd( win, buff, &app );
-		evnt_window_rt_resize( win, buff );
-	}
+
+	wind_get( win->handle, WF_CURRXYWH, &wx, &wy, &ww, &wh );
+	r = graf_rubberbox(wx, wy, 20, 20, &nw, &nh);
+	if( nw < 40 && nw < 40 )
+		return;
+	buff[4] = wx;
+	buff[5] = wy;
+	buff[6] = nw;
+	buff[7] = nh;
+	std_szd( win, buff, &app );
+	evnt_window_rt_resize( win, buff, data );
 }
 
 /* perform the actual resize */
-static void __CDECL evnt_window_rt_resize( WINDOW *win, short buff[8] )
+static void __CDECL evnt_window_rt_resize( WINDOW *win, short buff[8], void * data )
 {
 	short x,y,w,h;
 	struct gui_window * gw;
 	LGRECT rect;
-	bool resized;
-	bool moved;
-	
+
 	if(buff[0] == WM_FORCE_MOVE ) {
 		std_mvd(win, buff, &app);
 		std_szd(win, buff, &app);
 	}
 
-	wind_get( win->handle, WF_WORKXYWH, &x, &y, &w, &h );
-	gw = find_root_gui_window( win );
+	wind_get( win->handle, WF_CURRXYWH, &x, &y, &w, &h );
+	gw = (struct gui_window *)data;
 
 	assert( gw != NULL );
 
-	if(gw->root->loc.g_x != x || gw->root->loc.g_y != y ){
-		moved = true;
-		gw->root->loc.g_x = x;
-		gw->root->loc.g_y = y;
-		browser_update_rects( gw );
-	}
-
 	if(gw->root->loc.g_w != w || gw->root->loc.g_h != h ){
-		resized = true;
 		/* report resize to component interface: */
 		browser_update_rects( gw );
+		mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
 		browser_get_rect( gw, BR_CONTENT, &rect );
 		if( gw->browser->bw->current_content != NULL )
-			browser_window_reformat(gw->browser->bw, rect.g_w, rect.g_h );
-		gw->root->toolbar->url.scrollx = 0;	
+			browser_window_reformat(gw->browser->bw, false, rect.g_w, rect.g_h );
+		gw->root->toolbar->url.scrollx = 0;
 		window_redraw_controls(gw, 0);
-		/* TODO: recalculate scroll position, istead of zeroing? */
+		/* TODO: recalculate scroll position, instead of zeroing? */
+	} else {
+		if(gw->root->loc.g_x != x || gw->root->loc.g_y != y ){
+       			mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
+			browser_update_rects( gw );
+        	}
 	}
 }
