@@ -29,7 +29,6 @@
 #include <string.h>
 #include <stdbool.h>
 #include <windom.h>
-#include <cflib.h>
 #include <hubbub/hubbub.h>
 
 #include "content/urldb.h"
@@ -75,13 +74,13 @@
 #include "atari/clipboard.h"
 #include "atari/osspec.h"
 #include "atari/search.h"
+#include "cflib.h"
 
 #define TODO() (0)/*printf("%s Unimplemented!\n", __FUNCTION__)*/
 
 char *default_stylesheet_url;
 char *adblock_stylesheet_url;
 char *quirks_stylesheet_url;
-char *options_file_location;
 char *tmp_clipboard;
 struct gui_window *input_window = NULL;
 struct gui_window *window_list = NULL;
@@ -90,11 +89,12 @@ OBJECT * 	h_gem_menu;
 OBJECT **rsc_trindex;
 short vdih;
 short rsc_ntree;
-static clock_t last_multi_task;
 int mouse_click_time[3] = { INT_MAX, INT_MAX, INT_MAX };
 int mouse_hold_start[3];
+short last_drag_x;
+short last_drag_y;
+long next_poll;
 browser_mouse_state bmstate;
-bool lck_multi = false;
 
 /* Comandline / Options: */
 int cfg_width;
@@ -107,95 +107,87 @@ extern GEM_PLOTTER plotter;
 void gui_multitask(void)
 {
 	short winloc[4];
+	short mx, my, dummy, aestop;
 	int flags = MU_MESAG | MU_KEYBD | MU_BUTTON | MU_TIMER;
-	if( /*((clock() * 1000 / CLOCKS_PER_SEC) - last_multi_task ) < 50 || */ lck_multi == true   ) {
-		return;
-	}
-	/* 	todo: instead of time, use dummy window message here, 
-		timer takes at least 10ms, WMs take about 1ms!
-	*/
-	evnt.timer = 1;
-	if(input_window) {
-		wind_get(input_window->root->handle->handle, WF_WORKXYWH, &winloc[0],
-			&winloc[1], &winloc[2], &winloc[3] );
-		graf_mkstate( &prev_inp_state.mx, &prev_inp_state.my,
-				&prev_inp_state.mbut, &prev_inp_state.mkstat );
-		flags |= MU_M1;
-		if( prev_inp_state.mx >= winloc[0] && prev_inp_state.mx <= winloc[0] + winloc[2] &&
-				prev_inp_state.my >= winloc[1] && prev_inp_state.my <= winloc[1] + winloc[3] ){
-			/* if the cursor is within workarea, capture an movement WITHIN: */
-			evnt.m1_flag = MO_LEAVE;
-			evnt.m1_w = 2;
-			evnt.m1_h = 2;
-			evnt.m1_x = prev_inp_state.mx;
-			evnt.m1_y = prev_inp_state.my;
-		} else {
-			/* otherwise capture mouse move INTO the work area: */
-			evnt.m1_flag = MO_ENTER;
-			evnt.m1_w = winloc[2];
-			evnt.m1_h = winloc[3];
-			evnt.m1_x = winloc[0];
-			evnt.m1_y = winloc[1];
-		}
-	}
+	evnt.timer = 0;
 	EvntWindom( flags );
-	if( MOUSE_IS_DRAGGING() )
-		global_track_mouse_state();
-	last_multi_task = clock()*1000 / CLOCKS_PER_SEC;
 }
+
 void gui_poll(bool active)
 {
 	short winloc[4];
-	int timeout = 50; /* timeout in milliseconds */
+	// int timeout; /* timeout in milliseconds */
 	int flags = MU_MESAG | MU_KEYBD | MU_BUTTON ;
-	/* right now, schedule is only used for the spinner, */
-	/* spinner code must be reviewed, so disable schedule for now */
-	timeout = schedule_run(); 
-	if ( active || browser_reformat_pending )
-		timeout = 1;
+	short mx, my, dummy;
+	short aestop;
 
-	if(input_window) {
+	evnt.timer = schedule_run(); 
+
+	wind_get( 0, WF_TOP, &aestop, &winloc[1], &winloc[2], &winloc[3]);
+	if( winloc[1] != _AESapid ){
+		aestop = 0;
+	}
+
+	if( (aestop > 0) && !active ) {
 		flags |= MU_M1;
-		wind_get(input_window->root->handle->handle, WF_WORKXYWH, &winloc[0],
+		wind_get( aestop, WF_WORKXYWH, &winloc[0],
 			&winloc[1], &winloc[2], &winloc[3] );
-		graf_mkstate( &prev_inp_state.mx, &prev_inp_state.my,
-				&prev_inp_state.mbut, &prev_inp_state.mkstat );
-		if( prev_inp_state.mx >= winloc[0] && prev_inp_state.mx <= winloc[0] + winloc[2] &&
-				prev_inp_state.my >= winloc[1] && prev_inp_state.my <= winloc[1] + winloc[3] ){
+		graf_mkstate( &mx, &my, &dummy, &dummy );
+		/* this can be improved a lot under XaAES - there is an event for mouse move */
+		if( mx >= winloc[0] && mx <= winloc[0] + winloc[2] &&
+			my >= winloc[1] && my <= winloc[1] + winloc[3] ){
+			/* Mouse is within the top window area */
 			evnt.m1_flag = MO_LEAVE;
-			evnt.m1_w = 2;
-			evnt.m1_h = 2;
-			evnt.m1_x = prev_inp_state.mx;
-			evnt.m1_y = prev_inp_state.my;
+			evnt.m1_w = evnt.m1_h = 1;
+			evnt.m1_x = mx;
+			evnt.m1_y = my;
 		} else {
+			/* Mouse is outside of top window area. */
+			if( evnt.m1_flag == MO_LEAVE ) {
+				/* Previous move was inside the top window area */
+				struct gui_window * gw = input_window;
+				if(gw != NULL && gw->browser != NULL && gw->browser->bw != NULL ){
+					/* reset mouse state */
+					/* you could also track further, without reset, but */
+					/* when the mouse moves into native scroller area, the mouse is */
+					/* the native scroll bar code gets in between.. */
+                			mouse_hold_start[0] = 0;
+                			mouse_hold_start[1] = 0;
+					bmstate = 0;
+					browser_window_mouse_track( gw->browser->bw, bmstate, mx, my );
+				}
+			}
 			evnt.m1_flag = MO_ENTER;
 			evnt.m1_w = winloc[2];
 			evnt.m1_h = winloc[3];
 			evnt.m1_x = winloc[0];
 			evnt.m1_y = winloc[1];
 		}
-		/* if we have some state that can't be recognized by evnt_multi, don't block
-			so tracking can take place after timeout: */
-		if( MOUSE_IS_DRAGGING() ) 
-			timeout = 1;
 	}
 
-	if( timeout >= 0 ) {
+	/*printf("time: %d, active: %d\n", evnt.timer, active );*/
+	if( active ) {
+		if( clock() >= next_poll ) {
+			evnt.timer = 0;
+			flags |= MU_TIMER;
+			EvntWindom( flags );
+			next_poll = clock() + CLOCKS_PER_SEC;
+		}
+	} else {
 		flags |= MU_TIMER;
-		evnt.timer = timeout;
+		EvntWindom( flags );
 	}
-	lck_multi = true;
-	EvntWindom( flags );
-	lck_multi = false;
-	if( MOUSE_IS_DRAGGING() ) // MOUSE_EVNT_IN_PROGRESS()
-		global_track_mouse_state();
-	last_multi_task = clock()*1000 / CLOCKS_PER_SEC;
+
 	struct gui_window * g;
 	for( g = window_list; g != NULL; g=g->next ) {
 		if( browser_redraw_required( g ) ){
 			browser_redraw( g );
 		}
 	}
+	if( evnt.timer != 0 && !active ){
+		/* this suits for stuff with lower priority */ 
+		hotlist_redraw();
+	}
 }
 
 struct gui_window *
@@ -206,8 +198,8 @@ gui_create_browser_window(struct browser_window *bw,
 	struct gui_window *gw=NULL;
 	struct gui_window * gwroot ;
 	short winloc[4];
-	LOG(( "gw: %p, BW: %p, clone %p, tab: %d, type: %d\n" , gw,  bw, clone, 
-		(int)new_tab, bw->browser_window_type 
+	LOG(( "gw: %p, BW: %p, clone %p, tab: %d\n" , gw,  bw, clone, 
+		(int)new_tab
 	));
 
 	gw = malloc( sizeof(struct gui_window) );
@@ -215,51 +207,15 @@ gui_create_browser_window(struct browser_window *bw,
 		return NULL;
 	memset( gw, 0, sizeof(struct gui_window) );
 
-	switch(bw->browser_window_type) {
-		case BROWSER_WINDOW_NORMAL:
-
-			LOG(("normal browser window: %p, bw: %p\n", gw, bw));
-			window_create(gw, bw, WIDGET_STATUSBAR|WIDGET_TOOLBAR );
-			if( gw->root->handle ) {
-				window_open( gw );
-				/* Recalculate windows browser area now */
-				browser_update_rects( gw );
-				tb_update_buttons( gw );
-				input_window = gw;
-				/* TODO:... this line: placeholder to create a local history widget ... */
-			}
-		break;
-
-		case BROWSER_WINDOW_FRAME:
-			gwroot = bw->parent->window;
-			LOG(("create frame: %p, clone: %p\n", bw, clone));
-			gw->parent = gwroot;
-			gw->root = gwroot->root;
-			gw->browser = browser_create( gw, bw, clone, BT_FRAME, CLT_VERTICAL, 1, 1);
-			/*browser_attach_frame( gwroot, gw );*/
-		break;
-
-		case BROWSER_WINDOW_FRAMESET:
-			LOG(("frameset: %p, clone: %p\n", bw, clone));
-			gwroot = bw->parent->window;
-			gw->parent = gwroot;
-			gw->root = gwroot->root;
-			gw->browser = browser_create( gw, bw, clone, BT_FRAME, CLT_VERTICAL, 1, 1);
-			/*browser_attach_frame( gwroot, gw );*/
-		break;
-
-		case BROWSER_WINDOW_IFRAME:
-			LOG(("iframe: %p, clone: %p\n", bw, clone));
-			/* just dummy code here! */
-			gwroot = bw->parent->window;
-			gw->parent = gwroot;
-			gw->root = bw->parent->window->root;
-			gw->browser = browser_create( gw, bw, NULL, BT_FRAME, CLT_VERTICAL, 1, 1);
-			/*browser_attach_frame( gwroot, gw );*/
-		break;
-
-		default:
-			LOG(("unhandled type!"));
+	LOG(("new window: %p, bw: %p\n", gw, bw));
+	window_create(gw, bw, WIDGET_STATUSBAR|WIDGET_TOOLBAR );
+	if( gw->root->handle ) {
+		window_open( gw );
+		/* Recalculate windows browser area now */
+		browser_update_rects( gw );
+		tb_update_buttons( gw );
+		input_window = gw;
+		/* TODO:... this line: placeholder to create a local history widget ... */
 	}
 
 	/* add the window to the window list: */
@@ -290,31 +246,8 @@ void gui_window_destroy(struct gui_window *w)
 
 	input_window = NULL;
 
-	LGRECT dbg;
-	struct gui_window * root = browser_find_root( w );
-	browser_get_rect( root, BR_CONTENT, &dbg );
-	switch(w->browser->bw->browser_window_type) {
-		case BROWSER_WINDOW_NORMAL:
-			window_destroy( w );
-			break;
-		case BROWSER_WINDOW_FRAME:
-			browser_destroy( w->browser );
-			break;
+	window_destroy( w );
 
-		case BROWSER_WINDOW_FRAMESET:
-			browser_destroy( w->browser );
-			break;
-
-		case BROWSER_WINDOW_IFRAME:
-			/* just dummy code here: */
-			window_destroy( w );
-			break;
-
-		default:
-			LOG(("Unhandled type!"));
-			assert( 1 == 0 );
-			break;
-	}
 	/* unlink the window: */
 	if(w->prev != NULL ) {
 		w->prev->next = w->next;
@@ -344,11 +277,19 @@ void gui_window_get_dimensions(struct gui_window *w, int *width, int *height,
 
 void gui_window_set_title(struct gui_window *gw, const char *title)
 {
+	int l;
+	char * conv;
+
 	if (gw == NULL)
 		return;
-	/* TODO: query AES for max. title length */
-	if( gw->root && gw->parent == NULL ){
-		strncpy(gw->root->title, title, atari_sysinfo.aes_max_win_title_len);
+	if( gw->root ){
+		l = strlen(title);
+		if( utf8_to_local_encoding(title, l, &conv) == UTF8_CONVERT_OK ){
+			strncpy(gw->root->title, conv, atari_sysinfo.aes_max_win_title_len);
+                	free( conv );
+		} else {
+			strncpy(gw->root->title, title, atari_sysinfo.aes_max_win_title_len);
+		}
 		gw->root->title[atari_sysinfo.aes_max_win_title_len] = 0;
 		WindSetStr( gw->root->handle, WF_NAME, gw->root->title );
 	}
@@ -359,6 +300,7 @@ void gui_window_set_title(struct gui_window *gw, const char *title)
  */
 void gui_window_set_status(struct gui_window *w, const char *text)
 {
+
 	if (w == NULL || text == NULL )
 		return;
 	window_set_stauts( w , (char*)text );
@@ -375,8 +317,7 @@ void gui_window_redraw_window(struct gui_window *gw)
 	browser_schedule_redraw( gw, 0, 0, rect.g_w, rect.g_h );
 }
 
-void gui_window_update_box(struct gui_window *gw,
-			   const union content_msg_data *data)
+void gui_window_update_box(struct gui_window *gw, const struct rect *rect)
 {
 	LGRECT work;
 	CMP_BROWSER b;
@@ -384,11 +325,11 @@ void gui_window_update_box(struct gui_window *gw,
 		return;
 	b = gw->browser;
 	/* the box values are actually floats */
-	int x0 = data->redraw.x - b->scroll.current.x;
-	int y0 = data->redraw.y - b->scroll.current.y;
+	int x0 = rect->x0 - b->scroll.current.x;
+	int y0 = rect->y0 - b->scroll.current.y;
 	int w,h;
-	w = data->redraw.width;
-	h = data->redraw.height;
+	w = rect->x1 - rect->x0;
+	h = rect->y1 - rect->y0;
  	browser_schedule_redraw_rect( gw, x0, y0, w,h);
 }
 
@@ -433,19 +374,6 @@ void gui_window_scroll_visible(struct gui_window *w, int x0, int y0, int x1, int
 	browser_schedule_redraw_rect( w, 0, 0, x1-x0,y1-y0);
 }
 
-void gui_window_position_frame(struct gui_window *gw, int x0, int y0, int x1, int y1)
-{
-	TODO();
-	struct browser_window * bw = gw->browser->bw;
-	LGRECT pardim;
-	int width = x1 - x0 + 2, height = y1 - y0 + 2;
-	/* get available width/height: */
-	if( gw->parent ) {
-		browser_get_rect( gw->parent, BR_CONTENT, &pardim );
-		LOG(("posframe %s: x0,y0: %d/%d, x1,y1: %d/%d, w: %d, h: %d \n",gw->browser->bw->name, x0,y0, x1,y1, width, height));
-	}
-}
-
 
 /* It seems this method is called when content size got adjusted,
 	so that we can adjust scroll info. We also have to call it when tab
@@ -457,12 +385,6 @@ void gui_window_update_extent(struct gui_window *gw)
 	oldx = gw->browser->scroll.current.x;
 	oldy = gw->browser->scroll.current.y;
 	if( gw->browser->bw->current_content != NULL ) {
-		/*printf("update_extent %p (\"%s\"), c_w: %d, c_h: %d, scale: %f\n",
-			gw->browser->bw,gw->browser->bw->name,
-			content_get_width(gw->browser->bw->current_content),
-			content_get_height(gw->browser->bw->current_content),
-			gw->browser->bw->scale
-		);*/
 		browser_set_content_size( gw, 
 			content_get_width(gw->browser->bw->current_content),
 			content_get_height(gw->browser->bw->current_content)
@@ -673,7 +595,7 @@ save_complete_gui_save(const char *path,
 		       const char *filename,
 		       size_t len,
 		       const char *sourcedata,
-		       content_type type)
+		       lwc_string *mime_type)
 {
 	TODO();
 	LOG(("%s", filename));
@@ -716,12 +638,6 @@ bool gui_window_box_scroll_start(struct gui_window *w,
 	return true;
 }
 
-bool gui_window_frame_resize_start(struct gui_window *w)
-{
-	TODO();
-	return true;
-}
-
 void gui_window_save_link(struct gui_window *g, const char *url,
 		const char *title)
 {
@@ -729,15 +645,6 @@ void gui_window_save_link(struct gui_window *g, const char *url,
 	TODO();
 }
 
-void gui_window_set_scale(struct gui_window *w, float scale)
-{
-	if (w == NULL)
-		return;
-	w->browser->bw->scale = scale;
-	LOG(("%.2f\n", scale));
-	/* TODO schedule redraw */
-}
-
 void gui_drag_save_object(gui_save_type type, hlcache_handle *c,
 			  struct gui_window *w)
 {
@@ -884,8 +791,7 @@ gui_selection_traverse_handler(const char *text,
 bool gui_copy_to_clipboard(struct selection *s)
 {
 	bool ret = false;
-	if( (s->defined) && (s->bw != NULL) && (s->bw->window != NULL) &&
-		(s->bw->window->root != NULL )) {
+	if( s->defined ) {
 		gui_empty_clipboard();
 		if(selection_traverse(s, gui_selection_traverse_handler, NULL)){
 			ret = gui_commit_clipboard();
@@ -961,20 +867,16 @@ void gui_quit(void)
 
 	while( gw ) {
 		tmp = gw->next;
-		if( gw->parent == NULL ) {
-			browser_window_destroy(gw->browser->bw);
-		}
+		browser_window_destroy(gw->browser->bw);
 		gw = tmp;
 	}
+
+	hotlist_destroy();
+
 	/* send WM_DESTROY to windows purely managed by windom: */
-	while( wglb.first ) {
-		ApplWrite( _AESapid, WM_DESTROY, wglb.first->handle, 0, 0, 0, 0);
-		EvntWindom( MU_MESAG );
-	}
 
 	urldb_save_cookies(option_cookie_file);
 	urldb_save(option_url_file);	
-	hotlist_destroy();
 
 	RsrcXtype( 0, rsc_trindex, rsc_ntree);
 	unbind_global_events();
@@ -1055,9 +957,7 @@ char* gui_get_resource_url(const char *filename)
 	char buf[PATH_MAX];
 	int len;
 	char * ret;
-	printf("gui_find_res: %s\n", filename);
 	atari_find_resource((char*)&buf, filename, filename); 
-	printf("found: %s\n", (char*)&buf);
 	/* TODO: handle failure? */
 	len = strlen( (char*)&buf ) + 1;
 	return( path_to_url((char*)&buf) );
@@ -1124,16 +1024,20 @@ static void gui_init(int argc, char** argv)
 		LOG(("Loading cookies from: %s", option_cookie_file ));		
 	}
 
-
-
 	if (process_cmdline(argc,argv) != true)
 		die("unable to process command line.\n");
 
 	nkc_init();
 	atari_plotter_init( option_atari_screen_driver, option_atari_font_driver );
+	LOG(("Knockout rendering: %s\n",  option_atari_knockout ? "yes" : "no"));
+	plot_set_knockout( option_atari_knockout );
+	/* Interface colours */
+	option_gui_colour_bg_1 = 0xFFFFFF; /** Background          (bbggrr) */
+	option_gui_colour_fg_1 = 0xFF0000; /** Foreground          (bbggrr) */
+	option_gui_colour_fg_2 = 0x000000; /** Foreground selected (bbggrr) */
 }
 
-static char *theapp = "NetSurf";
+static char *theapp = (char*)"NetSurf";
 static void gui_init2(int argc, char** argv)
 {
 	struct browser_window *bw;
@@ -1144,9 +1048,11 @@ static void gui_init2(int argc, char** argv)
 	if (sys_type() & (SYS_MAGIC|SYS_NAES|SYS_XAAES)) {
 		menu_register( _AESapid, (char*)"  NetSurf ");
 	}
+ 	tree_set_icon_dir( option_tree_icons_path );
+	hotlist_init();
 }
 
-
+/* #define WITH_DBG_LOGFILE 1 */
 /** Entry point from OS.
  *
  * /param argc The number of arguments in the string vector.
@@ -1161,19 +1067,14 @@ int main(int argc, char** argv)
 	setbuf(stderr, NULL);
 	setbuf(stdout, NULL);
 #ifdef WITH_DBG_LOGFILE
-	verbose_log = true;
 	freopen("stdout.log", "a+", stdout);
 	freopen("stderr.log", "a+", stderr);
 #endif
-
 	ApplInit();
-
 	graf_mouse(BUSY_BEE, NULL);
 	init_os_info();
-	atari_find_resource(messages, "messages", "res/messages");
-	atari_find_resource(options, "Choices", "Choices");
-	options_file_location = strdup(options);
-
+	atari_find_resource((char*)&messages, "messages", "res/messages");
+	atari_find_resource((char*)&options, "Choices", "Choices");
 	netsurf_init(&argc, &argv, options, messages);
 	gui_init(argc, argv);
 	gui_init2(argc, argv);
@@ -1181,13 +1082,12 @@ int main(int argc, char** argv)
 	graf_mouse( ARROW , NULL);
 	netsurf_main_loop();
 	netsurf_exit();
-	if( options_file_location != NULL ){
-		free( options_file_location );
-	}
+
 	LOG(("ApplExit"));
 	ApplExit();
 #ifdef WITH_DBG_LOGFILE	
 	fclose(stdout);
+	fclose(stderr);
 #endif
 
 	return 0;
