@@ -30,7 +30,7 @@
 	lbuf[7] = (long)sbuf[7];
 
 
-struct gui_window * find_root_gui_window( WINDOW * win );
+struct gui_window * find_gui_window( WINDOW * win );
 struct gui_window * find_cmp_window( COMPONENT * c );
 OBJECT *get_tree( int idx );
 char *get_rsc_string( int idx );
@@ -39,4 +39,4 @@ void dbg_grect( char * str, GRECT * r );
 void * ldg_open( char * name, short * global );
 void * ldg_find( char * name, short * ldg );
 int ldg_close( void * ldg, short * global );
-#endif
\ No newline at end of file
+#endif
