@@ -25,7 +25,6 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <windom.h>
-#include <cflib.h>
 #include <assert.h>
 #include <math.h>
 
@@ -40,9 +39,11 @@
 #include "atari/gui.h"
 #include "atari/toolbar.h"
 #include "atari/browser_win.h"
+#include "atari/browser.h"
 #include "atari/clipboard.h"
 #include "atari/misc.h"
 #include "atari/global_evnt.h"
+#include "cflib.h"
 #include "atari/res/netsurf.rsh"
 #include "atari/plot/plotter.h"
 
@@ -73,8 +74,10 @@ static void __CDECL button_redraw( COMPONENT *c, long buff[8])
 
 	mt_CompGetLGrect(&app, c, WF_WORKXYWH, &work);
 	clip = work;
-	if ( !rc_lintersect( (LGRECT*)&buff[4], &clip ) ) return;
-
+	if ( !rc_lintersect( (LGRECT*)&buff[4], &clip ) ) {
+		LOG(("useless button_redraw"));
+		return;
+	}
 	pxy[0] = clip.g_x;
 	pxy[1] = clip.g_y;
 	pxy[2] = clip.g_w + clip.g_x;
@@ -94,7 +97,10 @@ static void __CDECL button_redraw( COMPONENT *c, long buff[8])
 	pxy[3] = MIN( (short)buff[5] + buff[7], work.g_y + work.g_h - 2);
 	vswr_mode( vdih, MD_REPLACE);
 	v_bar( vdih, (short*)&pxy );
-	mt_objc_draw( tree, 0, 8, clip.g_x, clip.g_y, clip.g_w, clip.g_h, app.aes_global );
+	/* mt_objc_draw( tree, 0, 8, clip.g_x, clip.g_y, clip.g_w, clip.g_h, app.aes_global ); */
+	/* The above seems to have problems with some AES, so we try this: */
+	int err = ObjcDrawParent(OC_OBJC, tree, 0, 0, 1|OC_MSG );
+	LOG(("ObjcDrawParent: %d", err ));
 	if( gw->root->toolbar->buttons[0].comp ==  c && work.g_x == buff[4] ){
 		vsl_color( vdih, LWHITE );
 		pxy[0] = (short)buff[4];
@@ -269,7 +275,7 @@ void __CDECL evnt_url_redraw( COMPONENT *c, long buff[8] )
 		short vqw[4];
 		char t[2];
 		short cw = 0;
-
+		t[0]=tb->url.text[0];
 		t[1]=0;
 		if( atari_sysinfo.sfont_monospaced ) {
 			vqt_width( vdih, t[0], &vqw[0], &vqw[1], &vqw[2] );
@@ -618,8 +624,6 @@ bool tb_url_input( struct gui_window * gw, short nkc )
 	char code = (nkc & 0xFF);
 
 	assert( gw != NULL );
-	/* make sure we navigate within the root window on enter: */
-	assert( gw->parent == NULL );
 
 	if( (code == NK_LEFT) && !shift ){
 		/* TODO: recognize shift + click */
