@@ -289,7 +289,7 @@ char *realpath(const char *f, char *buf)
 		strncpy(buf, f, MAXPATHLEN);
 		return NULL;
 	}
-	printf("RP: '%s'\n", path.Path());
+	//printf("RP: '%s'\n", path.Path());
 	strncpy(buf, path.Path(), MAXPATHLEN);
 	return buf;
 }
@@ -702,7 +702,7 @@ void nsbeos_pipe_message_top(BMessage *message, BWindow *_this, struct beos_scaf
 
 void gui_poll(bool active)
 {
-	CALLED();
+	//CALLED();
 	CURLMcode code;
 
 	fd_set read_fd_set, write_fd_set, exc_fd_set;
@@ -922,10 +922,11 @@ void nsbeos_gui_view_source(struct hlcache_handle *content, struct selection *se
 			warn_user("IOError", strerror(err));
 			return;
 		}
-		const char *mime = content_get_mime_type(content);
+		lwc_string *mime = content_get_mime_type(content);
+		const char *mime_string = lwc_string_data(mime);
 		if (mime)
 			file.WriteAttr("BEOS:TYPE", B_MIME_STRING_TYPE, 0LL, 
-				mime, strlen(mime) + 1);
+				mime_string, lwc_string_length(mime) + 1);
 		
 	}
 
@@ -1005,14 +1006,6 @@ void gui_launch_url(const char *url)
 }
 
 
-bool gui_search_term_highlighted(struct gui_window *g,
-		unsigned start_offset, unsigned end_offset,
-		unsigned *start_idx, unsigned *end_idx)
-{
-	return false;
-}
-
-
 /**
  * Display a warning for a serious problem (eg memory exhaustion).
  *
