@@ -777,7 +777,7 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 			break;
 		}
 		case B_COPY:
-			gui_copy_to_clipboard(bw->sel);
+			gui_copy_to_clipboard(browser_window_get_selection(bw));
 			break;
 		case B_CUT:
 			browser_window_key_press(bw, 24);
@@ -787,7 +787,7 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 			break;
 		case B_SELECT_ALL:
 			LOG(("Selecting all text"));
-			selection_select_all(bw->sel);
+			selection_select_all(browser_window_get_selection(bw));
 			break;
 		case B_NETPOSITIVE_BACK:
 		case BROWSER_NAVIGATE_BACK:
@@ -916,7 +916,8 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 		{
 			if (!bw || !bw->current_content)
 				break;
-			nsbeos_gui_view_source(bw->current_content, bw->sel);
+			nsbeos_gui_view_source(bw->current_content,
+					browser_window_get_selection(bw));
 			break;
 		}
 		case BROWSER_OBJECT:
@@ -1384,6 +1385,11 @@ gboolean nsbeos_history_expose_event(beosWidget *widget,
 	struct beos_history_window *hw = (struct beos_history_window *)g;
 	struct browser_window *bw = nsbeos_get_browser_for_gui(hw->g->top_level);
 
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &nsbeos_plotters
+	};
+
 	current_widget = widget;
 	current_drawable = widget->window;
 	current_gc = gdk_gc_new(current_drawable);
@@ -1392,7 +1398,7 @@ gboolean nsbeos_history_expose_event(beosWidget *widget,
 #endif
 	plot = nsbeos_plotters;
 
-	history_redraw(bw->history);
+	history_redraw(bw->history, &ctx);
 
 	g_object_unref(current_gc);
 #ifdef CAIRO_VERSION
@@ -1644,7 +1650,7 @@ static BMenuItem *make_menu_item(const char *name, BMessage *message)
 		} else if (accel.Length() > 0) {
 			key = accel[0];
 		}
-		printf("MENU: detected 	accel '%s' mods 0x%08lx, key %d\n", accel.String(), mods, key);
+		//printf("MENU: detected 	accel '%s' mods 0x%08lx, key %d\n", accel.String(), mods, key);
 	}
 
 	// turn ... into ellipsis
