@@ -30,6 +30,7 @@ extern "C" {
 #include "desktop/textinput.h"
 #include "render/font.h"
 #include "utils/log.h"
+#include "utils/types.h"
 #include "utils/utf8.h"
 #include "utils/utils.h"
 }
@@ -113,8 +114,6 @@ static struct gui_window *window_list = 0;	/**< first entry in win list*/
 static BString current_selection;
 static BList current_selection_textruns;
 
-static void nsbeos_gui_window_attach_child(struct gui_window *parent,
-					  struct gui_window *child);
 /* Methods which apply only to a gui_window */
 static void nsbeos_window_expose_event(BView *view, gui_window *g, BMessage *message);
 static void nsbeos_window_keypress_event(BView *view, gui_window *g, BMessage *event);
@@ -397,15 +396,10 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	g->prev = NULL;
 	window_list = g;
 
-	if (bw->parent != NULL) {
-		/* Find our parent's scaffolding */
-		g->scaffold = bw->parent->window->scaffold;
-	} else {
-		/* Now construct and attach a scaffold */
-		g->scaffold = nsbeos_new_scaffolding(g);
-		if (!g->scaffold)
-			return NULL;
-	}
+	/* Now construct and attach a scaffold */
+	g->scaffold = nsbeos_new_scaffolding(g);
+	if (!g->scaffold)
+		return NULL;
 
 	/* Construct our primary elements */
 	BRect frame(0,0,-1,-1); // will be resized later
@@ -430,19 +424,11 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 */
 #endif
 
-	if (bw->parent != NULL ) {
-		g->toplevel = false;
-		// XXX handle scrollview later
-		//g->scrollview = new BScrollView(g->view);
 
-		/* Attach ourselves into our parent at the right point */
-		nsbeos_gui_window_attach_child(bw->parent->window, g);
-	} else {
-		g->toplevel = true;
+	g->toplevel = true;
 
-		/* Attach our viewport into the scaffold */
-		nsbeos_attach_toplevel_view(g->scaffold, g->view);
-	}
+	/* Attach our viewport into the scaffold */
+	nsbeos_attach_toplevel_view(g->scaffold, g->view);
 
 #warning WRITEME
 #if 0 /* GTK */
@@ -454,23 +440,12 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	gtk_fixed_put(g->fixed, GTK_WIDGET(g->drawing_area), 0, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(g->fixed), 0);
 
-	if (bw->parent != NULL ) {
-		g->scrolledwindow = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
-		gtk_scrolled_window_add_with_viewport(g->scrolledwindow,
-						      GTK_WIDGET(g->fixed));
-		gtk_scrolled_window_set_shadow_type(g->scrolledwindow,
-						    GTK_SHADOW_NONE);
-		g->viewport = GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(g->scrolledwindow)));
-		/* Attach ourselves into our parent at the right point */
-		nsbeos_gui_window_attach_child(bw->parent->window, g);
-	} else {
-		g->scrolledwindow = 0;
-		g->viewport = GTK_VIEWPORT(gtk_viewport_new(NULL, NULL)); /* Need to attach adjustments */
-		gtk_container_add(GTK_CONTAINER(g->viewport), GTK_WIDGET(g->fixed));
+	g->scrolledwindow = 0;
+	g->viewport = GTK_VIEWPORT(gtk_viewport_new(NULL, NULL)); /* Need to attach adjustments */
+	gtk_container_add(GTK_CONTAINER(g->viewport), GTK_WIDGET(g->fixed));
 
-		/* Attach our viewport into the scaffold */
-		nsbeos_attach_toplevel_viewport(g->scaffold, g->viewport);
-	}
+	/* Attach our viewport into the scaffold */
+	nsbeos_attach_toplevel_viewport(g->scaffold, g->viewport);
 
 	gtk_container_set_border_width(GTK_CONTAINER(g->viewport), 0);
 	gtk_viewport_set_shadow_type(g->viewport, GTK_SHADOW_NONE);
@@ -494,32 +469,10 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 		break;
 	};
 
-	switch(bw->browser_window_type) {
-	case BROWSER_WINDOW_FRAMESET:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       GTK_POLICY_NEVER,
-						       GTK_POLICY_NEVER);
-		break;
-	case BROWSER_WINDOW_FRAME:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       scrollpolicy,
-						       scrollpolicy);
-		break;
-	case BROWSER_WINDOW_NORMAL:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       scrollpolicy,
-						       scrollpolicy);
-		break;
-	case BROWSER_WINDOW_IFRAME:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       scrollpolicy,
-						       scrollpolicy);
-		break;
-	}
+
+	if (g->scrolledwindow)
+		gtk_scrolled_window_set_policy(g->scrolledwindow,
+				scrollpolicy, scrollpolicy);
 
 	/* set the events we're interested in receiving from the browser's
 	 * drawing area.
@@ -553,37 +506,6 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	return g;
 }
 
-static void nsbeos_gui_window_attach_child(struct gui_window *parent,
-					  struct gui_window *child)
-{
-	/* Attach the child gui_window (frame) into the parent.
-	 * It will be resized later on.
-	 */
-	if (parent->view == NULL || child->view == NULL)
-		return;
-	if (!parent->view->LockLooper())
-		return;
-
-	//if (child->scrollview)
-	//	parent->view->AddChild(child->scrollview);
-	//else
-	parent->view->AddChild(child->view);
-
-	// non-top-level views shouldn't resize automatically (?)
-	child->view->SetResizingMode(B_FOLLOW_NONE);
-
-	parent->view->UnlockLooper();
-
-#warning WRITEME
-#if 0 /* GTK */
-	/* Attach the child gui_window (frame) into the parent.
-	 * It will be resized later on.
-	 */
-	GtkFixed *parent_fixed = parent->fixed;
-	GtkWidget *child_widget = GTK_WIDGET(child->scrolledwindow);
-	gtk_fixed_put(parent_fixed, child_widget, 0, 0);
-#endif
-}
 
 void gui_window_scroll_visible(struct gui_window *g, int x0, int y0,
 		int x1, int y1)
@@ -591,65 +513,6 @@ void gui_window_scroll_visible(struct gui_window *g, int x0, int y0,
 	gui_window_set_scroll(g, x0, y0);
 }
 
-void gui_window_position_frame(struct gui_window *g, int x0, int y0, int x1, int y1)
-{
-	CALLED();
-
-	/* g is a child frame, we need to place it relative to its parent */
-#warning XXX: BScrollView ?
-	BView *view = g->view;
-	BView *parent = g->bw->parent->window->view;
-	assert(view);
-	assert(parent);
-	LOG(("%s: %d,%d  %dx%d", g->bw->name, x0, y0, x1-x0+2, y1-y0+2));
-
-	/* if the window has not changed position or size, do not bother
-	 * moving/resising it.
-	 */
-
-	if (!parent->LockLooper())
-		return;
-
-	LOG(("  current: %d,%d  %dx%d",
-		view->Frame().left, view->Frame().top,
-		view->Frame().Width() + 1, view->Frame().Height() + 1));
-
-	if (view->Frame().left != x0 || view->Frame().top != y0 || 
-		view->Frame().Width() + 1 != x1 - x0 + 2 || 
-		view->Frame().Height() + 1 != y1 - y0 + 2) {
-	  	LOG(("  frame has moved/resized."));
-	  	view->MoveTo(x0, y0);
-		view->ResizeTo(x1 - x0 + 2 - 1, y1 - y0 + 2 - 1);
-	}
-
-	parent->UnlockLooper();
-#warning WRITEME
-#if 0 /* GTK */
-	/* g is a child frame, we need to place it relative to its parent */
-	GtkWidget *w = GTK_WIDGET(g->scrolledwindow);
-	GtkFixed *f = g->bw->parent->window->fixed;
-	assert(w);
-	assert(f);
-	LOG(("%s: %d,%d  %dx%d", g->bw->name, x0, y0, x1-x0+2, y1-y0+2));
-
-	/* if the window has not changed position or size, do not bother
-	 * moving/resising it.
-	 */
-
-	LOG(("  current: %d,%d  %dx%d",
-	      	w->allocation.x, w->allocation.y,
-		w->allocation.width, w->allocation.height));
-
-	if (w->allocation.x != x0 || w->allocation.y != y0 ||
-		w->allocation.width != x1 - x0 + 2 ||
-		w->allocation.height != y1 - y0 + 2) {
-	  	LOG(("  frame has moved/resized."));
-		gtk_fixed_move(f, w, x0, y0);
-		gtk_widget_set_size_request(w, x1 - x0 + 2, y1 - y0 + 2);
-	}
-#endif
-}
-
 void nsbeos_dispatch_event(BMessage *message)
 {
 	struct gui_window *gui = NULL;
@@ -867,7 +730,7 @@ void nsbeos_dispatch_event(BMessage *message)
 					where.x / gui->bw->scale,
 					where.y / gui->bw->scale);
 			else 
-				browser_window_mouse_drag_end(gui->bw, (browser_mouse_state)0, 
+				browser_window_mouse_track(gui->bw, (browser_mouse_state)0, 
 					where.x, where.y);
 
 			gui->mouse.state = 0;
@@ -920,6 +783,8 @@ void nsbeos_window_expose_event(BView *view, gui_window *g, BMessage *message)
 	float scale = g->bw->scale;
 	struct rect clip;
 
+	struct redraw_context ctx = { true, &nsbeos_plotters };
+
 	assert(g);
 	assert(g->bw);
 
@@ -947,17 +812,12 @@ void nsbeos_window_expose_event(BView *view, gui_window *g, BMessage *message)
 	if (view->Window())
 		view->Window()->BeginViewTransaction();
 
-	plot = nsbeos_plotters;
-	current_redraw_browser = g->bw;
-
 	clip.x0 = (int)updateRect.left;
 	clip.y0 = (int)updateRect.top;
 	clip.x1 = (int)updateRect.right + 1;
 	clip.y1 = (int)updateRect.bottom + 1;
 
-	browser_window_redraw(g->bw, 0, 0, &clip);
-
-	current_redraw_browser = NULL;
+	browser_window_redraw(g->bw, 0, 0, &clip, &ctx);
 
 	if (g->careth != 0)
 		nsbeos_plot_caret(g->caretx, g->carety, g->careth);
@@ -1273,6 +1133,7 @@ void nsbeos_window_process_reformats(void)
 		view->UnlockLooper();
 #warning XXX why - 1 & - 2 !???
 		browser_window_reformat(g->bw,
+				false,
 				bounds.Width() + 1 /* - 2*/,
 				bounds.Height() + 1);
 	}
@@ -1285,6 +1146,7 @@ void nsbeos_window_process_reformats(void)
 			continue;
 		g->bw->reformat_pending = false;
 		browser_window_reformat(g->bw,
+				false,
 				widget->allocation.width - 2,
 				widget->allocation.height);
 	}
@@ -1386,8 +1248,7 @@ void gui_window_redraw_window(struct gui_window *g)
 	g->view->UnlockLooper();
 }
 
-void gui_window_update_box(struct gui_window *g,
-			   const union content_msg_data *data)
+void gui_window_update_box(struct gui_window *g, const struct rect *rect)
 {
 	hlcache_handle *c = g->bw->current_content;
 
@@ -1402,9 +1263,8 @@ void gui_window_update_box(struct gui_window *g,
 	nsbeos_current_gc_set(g->view);
 
 //XXX +1 ??
-	g->view->Invalidate(BRect(data->redraw.x, data->redraw.y,
-				   data->redraw.x + data->redraw.width - 1, 
-				   data->redraw.y + data->redraw.height - 1));
+	g->view->Invalidate(BRect(rect->x0, rect->y0,
+				   rect->x1 - 1, rect->y1 - 1));
 
 	nsbeos_current_gc_set(NULL);
 	g->view->UnlockLooper();
@@ -1482,18 +1342,6 @@ void gui_window_set_scroll(struct gui_window *g, int sx, int sy)
 }
 
 
-/**
- * Set the scale setting of a window
- *
- * \param  g	  gui window
- * \param  scale  scale value (1.0 == normal scale)
- */
-
-void gui_window_set_scale(struct gui_window *g, float scale)
-{
-}
-
-
 void gui_window_update_extent(struct gui_window *g)
 {
 	CALLED();
@@ -1923,7 +1771,3 @@ void gui_window_get_dimensions(struct gui_window *g, int *width, int *height,
 	}
 }
 
-bool gui_window_frame_resize_start(struct gui_window *g)
-{
-	return true;
-}
