@@ -146,7 +146,10 @@ static inline NSRect cocoa_get_caret_rect( BrowserView *view )
 {
 	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
-	current_redraw_browser = browser;
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &cocoa_plotters
+	};
 	
 	const NSRect *rects = NULL;
 	NSInteger count = 0;
@@ -160,9 +163,8 @@ static inline NSRect cocoa_get_caret_rect( BrowserView *view )
 			.y1 = cocoa_pt_to_px( NSMaxY( rects[i] ) )
 		};
 
-		browser_window_redraw(browser, 0, 0, &clip);
+		browser_window_redraw(browser, 0, 0, &clip, &ctx);
 	}
-	current_redraw_browser = NULL;
 
 	NSRect caretRect = cocoa_get_caret_rect( self );
 	if (hasCaret && caretVisible && [self needsToDrawRect: caretRect]) {
@@ -241,7 +243,7 @@ static browser_mouse_state cocoa_mouse_flags_for_event( NSEvent *evt )
 	
 	if (isDragging) {
 		isDragging = NO;
-		browser_window_mouse_drag_end( browser, modifierFlags, location.x, location.y );
+		browser_window_mouse_track( browser, (browser_mouse_state)0, location.x, location.y );
 	} else {
 		modifierFlags |= BROWSER_MOUSE_CLICK_1;
 		if ([theEvent clickCount] == 2) modifierFlags |= BROWSER_MOUSE_DOUBLE_CLICK;
@@ -486,7 +488,7 @@ static browser_mouse_state cocoa_mouse_flags_for_event( NSEvent *evt )
 - (void) reformat;
 {
 	NSRect size = [[self superview] frame];
-	browser_window_reformat( browser, cocoa_pt_to_px( NSWidth( size ) ), cocoa_pt_to_px( NSHeight( size ) ) );
+	browser_window_reformat( browser, false, cocoa_pt_to_px( NSWidth( size ) ), cocoa_pt_to_px( NSHeight( size ) ) );
 }
 
 + (void)reformatTimerFired: (NSTimer *) timer;
@@ -513,10 +515,13 @@ static browser_mouse_state cocoa_mouse_flags_for_event( NSEvent *evt )
 
 	NSMenu *popupMenu = [[NSMenu alloc] initWithTitle: @""];
 	NSPoint point = [self convertMousePoint: event];
-	
-	struct box *box = NULL;
-	if ((box = box = box_object_at_point( browser->current_content, point.x, point.y )) != NULL) {
-		NSString *imageURL = [NSString stringWithUTF8String: content_get_url( box->object )];
+
+	struct contextual_content cont;
+
+	browser_window_get_contextual_content( browser, point.x, point.y, &cont);
+
+	if (cont.object != NULL) {
+		NSString *imageURL = [NSString stringWithUTF8String: content_get_url( cont.object )];
 		
 		[[popupMenu addItemWithTitle: NSLocalizedString( @"Open image in new tab", @"Context menu" )
 							  action: @selector(cmOpenURLInTab:) 
@@ -529,13 +534,13 @@ static browser_mouse_state cocoa_mouse_flags_for_event( NSEvent *evt )
 					   keyEquivalent: @""] setRepresentedObject: imageURL];
 		[[popupMenu addItemWithTitle: NSLocalizedString( @"Copy image", @"Context menu" )
 							  action: @selector(cmImageCopy:) 
-					   keyEquivalent: @""] setRepresentedObject: (id)content_get_bitmap( box->object )];
+					   keyEquivalent: @""] setRepresentedObject: (id)content_get_bitmap( cont.object )];
 		
 		[popupMenu addItem: [NSMenuItem separatorItem]];
 	}
 	
-	if ((box = box_href_at_point( browser->current_content, point.x, point.y )) != NULL) {
-		NSString *target = [NSString stringWithUTF8String: box->href];
+	if (cont.link_url != NULL) {
+		NSString *target = [NSString stringWithUTF8String: cont.link_url];
 		
 		[[popupMenu addItemWithTitle: NSLocalizedString( @"Open link in new tab", @"Context menu" )
 							  action: @selector(cmOpenURLInTab:) 
@@ -559,6 +564,8 @@ static browser_mouse_state cocoa_mouse_flags_for_event( NSEvent *evt )
 						 action: @selector(reloadPage:) keyEquivalent: @""];
 	[popupMenu addItemWithTitle: NSLocalizedString( @"Forward", @"Context menu" )
 						 action: @selector(goForward:) keyEquivalent: @""];
+	[popupMenu addItemWithTitle: NSLocalizedString( @"View Source", @"Context menu" )
+						 action: @selector(viewSource:) keyEquivalent: @""];
 	
 	[NSMenu popUpContextMenu: popupMenu withEvent: event forView: self];
 	
