@@ -200,10 +200,14 @@ void fetch_curl_register(void)
 	SETOPT(CURLOPT_NOSIGNAL, 1L);
 	SETOPT(CURLOPT_CONNECTTIMEOUT, 30L);
 
-	if (option_ca_bundle && strcmp(option_ca_bundle, ""))
+	if (option_ca_bundle && strcmp(option_ca_bundle, "")) {
+		LOG(("option_ca_bundle: '%s'", option_ca_bundle));
 		SETOPT(CURLOPT_CAINFO, option_ca_bundle);
-	if (option_ca_path && strcmp(option_ca_path, ""))
+	}
+	if (option_ca_path && strcmp(option_ca_path, "")) {
+		LOG(("option_ca_path: '%s'", option_ca_path));
 		SETOPT(CURLOPT_CAPATH, option_ca_path);
+	}
 
 	/* Detect whether the SSL CTX function API works */
 	curl_with_openssl = true;
@@ -497,12 +501,18 @@ void fetch_curl_cache_handle(CURL *handle, char *host)
 		 * memory (except the hostname) and without removing the entry from the
 		 * ring and then re-inserting it, in order to be as efficient as we can.
 		 */
-		h = curl_handle_ring;
-		curl_handle_ring = h->r_next;
-		curl_easy_cleanup(h->handle);
-		h->handle = handle;
-		free(h->host);
-		h->host = strdup(host);
+		if (curl_handle_ring != NULL) {
+			h = curl_handle_ring;
+			curl_handle_ring = h->r_next;
+			curl_easy_cleanup(h->handle);
+			h->handle = handle;
+			free(h->host);
+			h->host = strdup(host);
+		} else {
+			/* Actually, we don't want to cache any handles */
+			curl_easy_cleanup(handle);
+		}
+
 		return;
 	}
 	/* The table isn't full yet, so make a shiny new handle to add to the ring */
@@ -1073,18 +1083,18 @@ size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
 		i++;
 
 		if (i < (int) size) {
-			free(f->realm);
-			f->realm = malloc(size - i + 1);
-			if (f->realm != NULL) {
-				strncpy(f->realm, data + i, size - i);
-				f->realm[size - i] = '\0';
-				for (i = size - i - 1; i >= 0 &&
-						(f->realm[i] == ' ' ||
-						f->realm[i] == '"' ||
-						f->realm[i] == '\t' ||
-						f->realm[i] == '\r' ||
-						f->realm[i] == '\n'); --i)
-					f->realm[i] = '\0';
+			size_t end = i;
+
+			while (end < size && data[end] != '"')
+				++end;
+
+			if (end < size) {
+				free(f->realm);
+				f->realm = malloc(end - i + 1);
+				if (f->realm != NULL) {
+					strncpy(f->realm, data + i, end - i);
+					f->realm[end - i] = '\0';
+				}
 			}
 		}
 	} else if (11 < size && strncasecmp(data, "Set-Cookie:", 11) == 0) {
