@@ -26,6 +26,7 @@
 
 #include "content/content.h"
 #include "content/hlcache.h"
+#include "content/mimesniff.h"
 #include "utils/http.h"
 #include "utils/log.h"
 #include "utils/messages.h"
@@ -41,16 +42,16 @@ typedef struct hlcache_retrieval_ctx hlcache_retrieval_ctx;
 struct hlcache_retrieval_ctx {
 	struct hlcache_retrieval_ctx *r_prev; /**< Previous retrieval context in the ring */
 	struct hlcache_retrieval_ctx *r_next; /**< Next retrieval context in the ring */
-	
+
 	llcache_handle *llcache;	/**< Low-level cache handle */
 
 	hlcache_handle *handle;		/**< High-level handle for object */
 
 	uint32_t flags;			/**< Retrieval flags */
 
-	const content_type *accepted_types;	/**< Accepted types, or NULL */
+	content_type accepted_types;	/**< Accepted types */
 
-	hlcache_child_context child;	/**< Child context */	
+	hlcache_child_context child;	/**< Child context */
 };
 
 /** High-level cache handle */
@@ -69,20 +70,38 @@ struct hlcache_entry {
 	hlcache_entry *prev;		/**< Previous sibling */
 };
 
-/** List of cached content objects */
-static hlcache_entry *hlcache_content_list;
+/** Current state of the cache.
+ *
+ * Global state of the cache.
+ */
+struct hlcache_s {
+	struct hlcache_parameters params;
+
+	/** List of cached content objects */
+	hlcache_entry *content_list;
+
+	/** Ring of retrieval contexts */
+	hlcache_retrieval_ctx *retrieval_ctx_ring;
+
+	/* statsistics */
+	unsigned int hit_count;
+	unsigned int miss_count;
+};
+
+/** high level cache state */
+static struct hlcache_s *hlcache = NULL;
 
-/** Ring of retrieval contexts */
-static hlcache_retrieval_ctx *hlcache_retrieval_ctx_ring;
 
 static void hlcache_clean(void *ignored);
 
 static nserror hlcache_llcache_callback(llcache_handle *handle,
 		const llcache_event *event, void *pw);
-static bool hlcache_type_is_acceptable(llcache_handle *llcache, 
-		const content_type *accepted_types, 
-		content_type *computed_type);
-static nserror hlcache_find_content(hlcache_retrieval_ctx *ctx);
+static nserror hlcache_migrate_ctx(hlcache_retrieval_ctx *ctx,
+		lwc_string *effective_type);
+static bool hlcache_type_is_acceptable(lwc_string *mime_type,
+		content_type accepted_types, content_type *computed_type);
+static nserror hlcache_find_content(hlcache_retrieval_ctx *ctx,
+		lwc_string *effective_type);
 static void hlcache_content_callback(struct content *c,
 		content_msg msg, union content_msg_data data, void *pw);
 
@@ -91,16 +110,29 @@ static void hlcache_content_callback(struct content *c,
  ******************************************************************************/
 
 nserror
-hlcache_initialise(llcache_query_callback cb, void *pw)
+hlcache_initialise(const struct hlcache_parameters *hlcache_parameters)
 {
-	nserror ret = llcache_initialise(cb, pw);
-	
-	if (ret != NSERROR_OK)
+	nserror ret;
+
+	hlcache = calloc(1, sizeof(struct hlcache_s));
+	if (hlcache == NULL) {
+		return NSERROR_NOMEM;
+	}
+
+	ret = llcache_initialise(hlcache_parameters->cb,
+				 hlcache_parameters->cb_ctx,
+				 hlcache_parameters->limit);
+	if (ret != NSERROR_OK) {
+		free(hlcache);
+		hlcache = NULL;
 		return ret;
-	
-	/* Schedule the cache cleanup for 5 seconds time */
-	schedule(500, hlcache_clean, NULL);
-	
+	}
+
+	hlcache->params = *hlcache_parameters;
+
+	/* Schedule the cache cleanup */
+	schedule(hlcache->params.bg_clean_time / 10, hlcache_clean, NULL);
+
 	return NSERROR_OK;
 }
 
@@ -117,33 +149,33 @@ void hlcache_finalise(void)
 	uint32_t num_contents, prev_contents;
 	hlcache_entry *entry;
 	hlcache_retrieval_ctx *ctx, *next;
-	
+
 	/* Obtain initial count of contents remaining */
-	for (num_contents = 0, entry = hlcache_content_list; 
+	for (num_contents = 0, entry = hlcache->content_list;
 			entry != NULL; entry = entry->next) {
 		num_contents++;
 	}
 
-	LOG(("%d contents to before cache drain", num_contents));
-	
+	LOG(("%d contents remain before cache drain", num_contents));
+
 	/* Drain cache */
 	do {
 		prev_contents = num_contents;
 
 		hlcache_clean(NULL);
 
-		for (num_contents = 0, entry = hlcache_content_list; 
+		for (num_contents = 0, entry = hlcache->content_list;
 				entry != NULL; entry = entry->next) {
 			num_contents++;
 		}
 	} while (num_contents > 0 && num_contents != prev_contents);
 
 	LOG(("%d contents remaining:", num_contents));
-	for (entry = hlcache_content_list; entry != NULL; entry = entry->next) {
+	for (entry = hlcache->content_list; entry != NULL; entry = entry->next) {
 		hlcache_handle entry_handle = { entry, NULL, NULL };
 
 		if (entry->content != NULL) {
-			LOG(("	%p : %s (%d users)", entry, 
+			LOG(("	%p : %s (%d users)", entry,
 			     content_get_url(&entry_handle), content_count_users(entry->content)));
 		} else {
 			LOG(("	%p", entry));
@@ -151,8 +183,8 @@ void hlcache_finalise(void)
 	}
 
 	/* Clean up retrieval contexts */
-	if (hlcache_retrieval_ctx_ring != NULL) {
-		ctx = hlcache_retrieval_ctx_ring;
+	if (hlcache->retrieval_ctx_ring != NULL) {
+		ctx = hlcache->retrieval_ctx_ring;
 
 		do {
 			next = ctx->r_next;
@@ -169,10 +201,18 @@ void hlcache_finalise(void)
 			free(ctx);
 
 			ctx = next;
-		} while (ctx != hlcache_retrieval_ctx_ring);
+		} while (ctx != hlcache->retrieval_ctx_ring);
 
-		hlcache_retrieval_ctx_ring = NULL;
+		hlcache->retrieval_ctx_ring = NULL;
 	}
+
+	LOG(("hit/miss %d/%d", hlcache->hit_count, hlcache->miss_count));
+
+	free(hlcache);
+	hlcache = NULL;
+
+	LOG(("Finalising low-level cache"));
+	llcache_finalise();
 }
 
 /* See hlcache.h for documentation */
@@ -188,8 +228,8 @@ nserror hlcache_poll(void)
 nserror hlcache_handle_retrieve(const char *url, uint32_t flags,
 		const char *referer, llcache_post_data *post,
 		hlcache_handle_callback cb, void *pw,
-		hlcache_child_context *child, 
-		const content_type *accepted_types, hlcache_handle **result)
+		hlcache_child_context *child,
+		content_type accepted_types, hlcache_handle **result)
 {
 	hlcache_retrieval_ctx *ctx;
 	nserror error;
@@ -224,8 +264,8 @@ nserror hlcache_handle_retrieve(const char *url, uint32_t flags,
 	ctx->handle->cb = cb;
 	ctx->handle->pw = pw;
 
-	error = llcache_handle_retrieve(url, flags, referer, post, 
-			hlcache_llcache_callback, ctx, 
+	error = llcache_handle_retrieve(url, flags, referer, post,
+			hlcache_llcache_callback, ctx,
 			&ctx->llcache);
 	if (error != NSERROR_OK) {
 		free((char *) ctx->child.charset);
@@ -233,9 +273,9 @@ nserror hlcache_handle_retrieve(const char *url, uint32_t flags,
 		free(ctx);
 		return error;
 	}
-	
-	RING_INSERT(hlcache_retrieval_ctx_ring, ctx);
-	
+
+	RING_INSERT(hlcache->retrieval_ctx_ring, ctx);
+
 	*result = ctx->handle;
 
 	return NSERROR_OK;
@@ -245,27 +285,27 @@ nserror hlcache_handle_retrieve(const char *url, uint32_t flags,
 nserror hlcache_handle_release(hlcache_handle *handle)
 {
 	if (handle->entry != NULL) {
-		content_remove_user(handle->entry->content, 
+		content_remove_user(handle->entry->content,
 				hlcache_content_callback, handle);
 	} else {
 		RING_ITERATE_START(struct hlcache_retrieval_ctx,
-				   hlcache_retrieval_ctx_ring,
+				   hlcache->retrieval_ctx_ring,
 				   ictx) {
 			if (ictx->handle == handle) {
-				/* This is the nascent context for us, 
+				/* This is the nascent context for us,
 				 * so abort the fetch */
 				llcache_handle_abort(ictx->llcache);
 				llcache_handle_release(ictx->llcache);
 				/* Remove us from the ring */
-				RING_REMOVE(hlcache_retrieval_ctx_ring, ictx);
+				RING_REMOVE(hlcache->retrieval_ctx_ring, ictx);
 				/* Throw us away */
 				free((char *) ictx->child.charset);
 				free(ictx);
 				/* And stop */
-				RING_ITERATE_STOP(hlcache_retrieval_ctx_ring, 
+				RING_ITERATE_STOP(hlcache->retrieval_ctx_ring,
 						ictx);
 			}
-		} RING_ITERATE_END(hlcache_retrieval_ctx_ring, ictx);
+		} RING_ITERATE_END(hlcache->retrieval_ctx_ring, ictx);
 	}
 
 	handle->cb = NULL;
@@ -292,71 +332,71 @@ nserror hlcache_handle_abort(hlcache_handle *handle)
 {
 	struct hlcache_entry *entry = handle->entry;
 	struct content *c;
-	
+
 	if (entry == NULL) {
 		/* This handle is not yet associated with a cache entry.
 		 * The implication is that the fetch for the handle has
 		 * not progressed to the point where the entry can be
 		 * created. */
-		
+
 		RING_ITERATE_START(struct hlcache_retrieval_ctx,
-				   hlcache_retrieval_ctx_ring,
+				   hlcache->retrieval_ctx_ring,
 				   ictx) {
 			if (ictx->handle == handle) {
-				/* This is the nascent context for us, 
+				/* This is the nascent context for us,
 				 * so abort the fetch */
 				llcache_handle_abort(ictx->llcache);
 				llcache_handle_release(ictx->llcache);
 				/* Remove us from the ring */
-				RING_REMOVE(hlcache_retrieval_ctx_ring, ictx);
+				RING_REMOVE(hlcache->retrieval_ctx_ring, ictx);
 				/* Throw us away */
 				free((char *) ictx->child.charset);
 				free(ictx);
 				/* And stop */
-				RING_ITERATE_STOP(hlcache_retrieval_ctx_ring, 
+				RING_ITERATE_STOP(hlcache->retrieval_ctx_ring,
 						ictx);
 			}
-		} RING_ITERATE_END(hlcache_retrieval_ctx_ring, ictx);
-		
+		} RING_ITERATE_END(hlcache->retrieval_ctx_ring, ictx);
+
 		return NSERROR_OK;
 	}
 
 	c = entry->content;
-	
+
 	if (content_count_users(c) > 1) {
 		/* We are not the only user of 'c' so clone it. */
 		struct content *clone = content_clone(c);
-		
+
 		if (clone == NULL)
 			return NSERROR_NOMEM;
-		
+
 		entry = calloc(sizeof(struct hlcache_entry), 1);
-		
+
 		if (entry == NULL) {
 			content_destroy(clone);
 			return NSERROR_NOMEM;
 		}
-		
-		if (content_add_user(clone, 
+
+		if (content_add_user(clone,
 				hlcache_content_callback, handle) == false) {
 			content_destroy(clone);
 			free(entry);
 			return NSERROR_NOMEM;
 		}
-		
+
 		content_remove_user(c, hlcache_content_callback, handle);
-		
+
 		entry->content = clone;
 		handle->entry = entry;
 		entry->prev = NULL;
-		entry->next = hlcache_content_list;
-		if (hlcache_content_list != NULL)
-			hlcache_content_list->prev = entry;
-		hlcache_content_list = entry;
-		
+		entry->next = hlcache->content_list;
+		if (hlcache->content_list != NULL)
+			hlcache->content_list->prev = entry;
+		hlcache->content_list = entry;
+
 		c = clone;
 	}
-	
+
 	return content_abort(c);
 }
 
@@ -387,29 +427,29 @@ void hlcache_clean(void *ignored)
 {
 	hlcache_entry *entry, *next;
 
-	for (entry = hlcache_content_list; entry != NULL; entry = next) {
+	for (entry = hlcache->content_list; entry != NULL; entry = next) {
 		next = entry->next;
 
 		if (entry->content == NULL)
 			continue;
 
-		if (content__get_status(entry->content) == 
+		if (content__get_status(entry->content) ==
 				CONTENT_STATUS_LOADING)
 			continue;
 
 		if (content_count_users(entry->content) != 0)
 			continue;
 
-		/** \todo This is over-zealous: all unused contents 
-		 * will be immediately destroyed. Ideally, we want to 
-		 * purge all unused contents that are using stale 
-		 * source data, and enough fresh contents such that 
+		/** \todo This is over-zealous: all unused contents
+		 * will be immediately destroyed. Ideally, we want to
+		 * purge all unused contents that are using stale
+		 * source data, and enough fresh contents such that
 		 * the cache fits in the configured cache size limit.
 		 */
 
 		/* Remove entry from cache */
 		if (entry->prev == NULL)
-			hlcache_content_list = entry->next;
+			hlcache->content_list = entry->next;
 		else
 			entry->prev->next = entry->next;
 
@@ -422,12 +462,12 @@ void hlcache_clean(void *ignored)
 		/* Destroy entry */
 		free(entry);
 	}
-	
+
 	/* Attempt to clean the llcache */
 	llcache_clean();
-	
-	/* Re-schedule ourselves for 5 seconds time */
-	schedule(500, hlcache_clean, NULL);
+
+	/* Re-schedule ourselves */
+	schedule(hlcache->params.bg_clean_time / 10, hlcache_clean, NULL);
 }
 
 /**
@@ -442,76 +482,71 @@ nserror hlcache_llcache_callback(llcache_handle *handle,
 		const llcache_event *event, void *pw)
 {
 	hlcache_retrieval_ctx *ctx = pw;
+	lwc_string *effective_type = NULL;
 	nserror error;
 
 	assert(ctx->llcache == handle);
 
 	switch (event->type) {
 	case LLCACHE_EVENT_HAD_HEADERS:
-	{
-		content_type type = CONTENT_UNKNOWN;
-		
-		/* Unlink the context to prevent recursion */
-		RING_REMOVE(hlcache_retrieval_ctx_ring, ctx);
-		
-		if (hlcache_type_is_acceptable(handle, 
-				ctx->accepted_types, &type)) {
-			error = hlcache_find_content(ctx);
-			if (error != NSERROR_OK) {
-				hlcache_event hlevent;
+		error = mimesniff_compute_effective_type(handle, NULL, 0,
+				ctx->flags & HLCACHE_RETRIEVE_SNIFF_TYPE,
+				&effective_type);
+		if (error == NSERROR_OK || error == NSERROR_NOT_FOUND) {
+			/* If the sniffer was successful or failed to find
+			 * a Content-Type header when sniffing was
+			 * prohibited, we must migrate the retrieval context. */
+			error = hlcache_migrate_ctx(ctx, effective_type);
 
-				hlevent.type = CONTENT_MSG_ERROR;
-				hlevent.data.error = messages_get("MiscError");
-
-				ctx->handle->cb(ctx->handle, &hlevent, 
-						ctx->handle->pw);
-				
-				llcache_handle_abort(handle);
-				llcache_handle_release(handle);
-				free((char *) ctx->child.charset);
-				free(ctx);
-				return error;
-			}
-		} else if (type == CONTENT_OTHER && 
-				ctx->flags & HLCACHE_RETRIEVE_MAY_DOWNLOAD) {
-			/* Unknown type, and we can download, so convert */
-			llcache_handle_force_stream(handle);
-
-			if (ctx->handle->cb != NULL) {
-				hlcache_event hlevent;
-
-				hlevent.type = CONTENT_MSG_DOWNLOAD;
-				hlevent.data.download = handle;
-
-				ctx->handle->cb(ctx->handle, &hlevent,
-						ctx->handle->pw);
-			}
-		} else {
-			/* Unacceptable type: abort fetch and report error */
-			llcache_handle_abort(handle);
-			llcache_handle_release(handle);
-
-			if (ctx->handle->cb != NULL) {
-				hlcache_event hlevent;
-
-				hlevent.type = CONTENT_MSG_ERROR;
-				hlevent.data.error = messages_get("BadType");
-
-				ctx->handle->cb(ctx->handle, &hlevent, 
-						ctx->handle->pw);
-			}
+			if (effective_type != NULL)
+				lwc_string_unref(effective_type);
 		}
 
-		/* No longer require retrieval context */
-		free((char *) ctx->child.charset);
-		free(ctx);
-	}
+		/* No need to report that we need data:
+		 * we'll get some anyway if there is any */
+		if (error == NSERROR_NEED_DATA)
+			error = NSERROR_OK;
+
+		return error;
+
 		break;
 	case LLCACHE_EVENT_HAD_DATA:
-		/* fall through */
+		error = mimesniff_compute_effective_type(handle,
+				event->data.data.buf, event->data.data.len,
+				ctx->flags & HLCACHE_RETRIEVE_SNIFF_TYPE,
+				&effective_type);
+		if (error != NSERROR_OK) {
+			assert(0 && "MIME sniff failed with data");
+		}
+
+		error = hlcache_migrate_ctx(ctx, effective_type);
+
+		lwc_string_unref(effective_type);
+
+		return error;
+
+		break;
 	case LLCACHE_EVENT_DONE:
-		/* should never happen: the handler must be changed */
-		assert(0 && "Unexpected llcache event");
+		/* DONE event before we could determine the effective MIME type.
+		 */
+		error = mimesniff_compute_effective_type(handle,
+				NULL, 0, false, &effective_type);
+		if (error == NSERROR_OK) {
+			error = hlcache_migrate_ctx(ctx, effective_type);
+
+			lwc_string_unref(effective_type);
+
+			return error;
+		}
+
+		if (ctx->handle->cb != NULL) {
+			hlcache_event hlevent;
+
+			hlevent.type = CONTENT_MSG_ERROR;
+			hlevent.data.error = messages_get("BadType");
+
+			ctx->handle->cb(ctx->handle, &hlevent, ctx->handle->pw);
+		}
 		break;
 	case LLCACHE_EVENT_ERROR:
 		if (ctx->handle->cb != NULL) {
@@ -521,7 +556,7 @@ nserror hlcache_llcache_callback(llcache_handle *handle,
 			hlevent.data.error = event->data.msg;
 
 			ctx->handle->cb(ctx->handle, &hlevent, ctx->handle->pw);
-		}	
+		}
 		break;
 	case LLCACHE_EVENT_PROGRESS:
 		break;
@@ -531,74 +566,122 @@ nserror hlcache_llcache_callback(llcache_handle *handle,
 }
 
 /**
- * Determine if the type of a low-level cache object is acceptable
+ * Migrate a retrieval context into its final destination content
  *
- * \param llcache	  Low-level cache object to consider
+ * \param ctx             Context to migrate
+ * \param effective_type  The effective MIME type of the content, or NULL
+ * \return NSERROR_OK on success,
+ *         NSERROR_NEED_DATA on success where data is needed,
+ *         appropriate error otherwise
+ */
+nserror hlcache_migrate_ctx(hlcache_retrieval_ctx *ctx,
+		lwc_string *effective_type)
+{
+	content_type type = CONTENT_NONE;
+	nserror error = NSERROR_OK;
+
+	/* Unlink the context to prevent recursion */
+	RING_REMOVE(hlcache->retrieval_ctx_ring, ctx);
+
+	if (effective_type != NULL &&
+			hlcache_type_is_acceptable(effective_type,
+			ctx->accepted_types, &type)) {
+		error = hlcache_find_content(ctx, effective_type);
+		if (error != NSERROR_OK && error != NSERROR_NEED_DATA) {
+			hlcache_event hlevent;
+
+			hlevent.type = CONTENT_MSG_ERROR;
+			hlevent.data.error = messages_get("MiscError");
+
+			ctx->handle->cb(ctx->handle, &hlevent,
+					ctx->handle->pw);
+
+			llcache_handle_abort(ctx->llcache);
+			llcache_handle_release(ctx->llcache);
+		}
+	} else if (type == CONTENT_NONE &&
+			(ctx->flags & HLCACHE_RETRIEVE_MAY_DOWNLOAD)) {
+		/* Unknown type, and we can download, so convert */
+		llcache_handle_force_stream(ctx->llcache);
+
+		if (ctx->handle->cb != NULL) {
+			hlcache_event hlevent;
+
+			hlevent.type = CONTENT_MSG_DOWNLOAD;
+			hlevent.data.download = ctx->llcache;
+
+			ctx->handle->cb(ctx->handle, &hlevent,
+					ctx->handle->pw);
+		}
+
+		/* Ensure caller knows we need data */
+		error = NSERROR_NEED_DATA;
+	} else {
+		/* Unacceptable type: abort fetch and report error */
+		llcache_handle_abort(ctx->llcache);
+		llcache_handle_release(ctx->llcache);
+
+		if (ctx->handle->cb != NULL) {
+			hlcache_event hlevent;
+
+			hlevent.type = CONTENT_MSG_ERROR;
+			hlevent.data.error = messages_get("BadType");
+
+			ctx->handle->cb(ctx->handle, &hlevent,
+					ctx->handle->pw);
+		}
+	}
+
+	/* No longer require retrieval context */
+	free((char *) ctx->child.charset);
+	free(ctx);
+
+	return error;
+}
+
+/**
+ * Determine if the specified MIME type is acceptable
+ *
+ * \param mime_type       MIME type to consider
  * \param accepted_types  Array of acceptable types, or NULL for any
  * \param computed_type	  Pointer to location to receive computed type of object
  * \return True if the type is acceptable, false otherwise
  */
-bool hlcache_type_is_acceptable(llcache_handle *llcache, 
-		const content_type *accepted_types, content_type *computed_type)
+bool hlcache_type_is_acceptable(lwc_string *mime_type,
+		content_type accepted_types, content_type *computed_type)
 {
-	const char *content_type_header;
-	char *mime_type;
-	http_parameter *params;
 	content_type type;
-	nserror error;
-	bool acceptable;
 
-	content_type_header = 
-			llcache_handle_get_header(llcache, "Content-Type");
-	if (content_type_header == NULL)
-		content_type_header = "text/plain";
-
-	error = http_parse_content_type(content_type_header, &mime_type, 
-			&params);
-	if (error != NSERROR_OK)
-		return false;
-
-	type = content_lookup(mime_type);
-
-	free(mime_type);
-	http_parameter_list_destroy(params);
-
-	if (accepted_types == NULL) {
-		acceptable = type != CONTENT_OTHER;
-	} else {
-		while (*accepted_types != CONTENT_UNKNOWN) {
-			if (*accepted_types == type)
-				break;
-
-			accepted_types++;
-		}
-
-		acceptable = *accepted_types == type;
-	}
+	type = content_factory_type_from_mime_type(mime_type);
 
 	*computed_type = type;
 
-	return acceptable;
+	return ((accepted_types & type) != 0);
 }
 
 /**
  * Find a content for the high-level cache handle
  *
- * \param ctx	High-level cache retrieval context
- * \return NSERROR_OK on success, appropriate error otherwise
+ * \param ctx             High-level cache retrieval context
+ * \param effective_type  Effective MIME type of content
+ * \return NSERROR_OK on success,
+ *         NSERROR_NEED_DATA on success where data is needed,
+ *         appropriate error otherwise
  *
  * \pre handle::state == HLCACHE_HANDLE_NEW
  * \pre Headers must have been received for associated low-level handle
  * \post Low-level handle is either released, or associated with new content
  * \post High-level handle is registered with content
  */
-nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
+nserror hlcache_find_content(hlcache_retrieval_ctx *ctx,
+		lwc_string *effective_type)
 {
 	hlcache_entry *entry;
 	hlcache_event event;
+	nserror error = NSERROR_OK;
 
 	/* Search list of cached contents for a suitable one */
-	for (entry = hlcache_content_list; entry != NULL; entry = entry->next) {
+	for (entry = hlcache->content_list; entry != NULL; entry = entry->next) {
 		hlcache_handle entry_handle = { entry, NULL, NULL };
 		const llcache_handle *entry_llcache;
 
@@ -614,15 +697,15 @@ nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
 			continue;
 
 		/* Ensure that quirks mode is acceptable */
-		if (content_matches_quirks(entry->content, 
+		if (content_matches_quirks(entry->content,
 				ctx->child.quirks) == false)
 			continue;
 
-		/* Ensure that content uses same low-level object as 
+		/* Ensure that content uses same low-level object as
 		 * low-level handle */
 		entry_llcache = content_get_llcache_handle(entry->content);
 
-		if (llcache_handle_references_same_object(entry_llcache, 
+		if (llcache_handle_references_same_object(entry_llcache,
 				ctx->llcache))
 			break;
 	}
@@ -634,8 +717,9 @@ nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
 			return NSERROR_NOMEM;
 
 		/* Create content using llhandle */
-		entry->content = content_create(ctx->llcache, 
-				ctx->child.charset, ctx->child.quirks);
+		entry->content = content_factory_create_content(ctx->llcache,
+				ctx->child.charset, ctx->child.quirks,
+				effective_type);
 		if (entry->content == NULL) {
 			free(entry);
 			return NSERROR_NOMEM;
@@ -643,17 +727,23 @@ nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
 
 		/* Insert into cache */
 		entry->prev = NULL;
-		entry->next = hlcache_content_list;
-		if (hlcache_content_list != NULL)
-			hlcache_content_list->prev = entry;
-		hlcache_content_list = entry;
+		entry->next = hlcache->content_list;
+		if (hlcache->content_list != NULL)
+			hlcache->content_list->prev = entry;
+		hlcache->content_list = entry;
+
+		/* Signal to caller that we created a content */
+		error = NSERROR_NEED_DATA;
+
+		hlcache->miss_count++;
 	} else {
 		/* Found a suitable content: no longer need low-level handle */
-		llcache_handle_release(ctx->llcache);	
+		llcache_handle_release(ctx->llcache);
+		hlcache->hit_count++;
 	}
 
 	/* Associate handle with content */
-	if (content_add_user(entry->content, 
+	if (content_add_user(entry->content,
 			hlcache_content_callback, ctx->handle) == false)
 		return NSERROR_NOMEM;
 
@@ -673,7 +763,7 @@ nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
 
 			if (ctx->handle->cb != NULL) {
 				event.type = CONTENT_MSG_READY;
-				ctx->handle->cb(ctx->handle, &event, 
+				ctx->handle->cb(ctx->handle, &event,
 						ctx->handle->pw);
 			}
 		} else if (status == CONTENT_STATUS_DONE) {
@@ -682,19 +772,19 @@ nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
 
 			if (ctx->handle->cb != NULL) {
 				event.type = CONTENT_MSG_READY;
-				ctx->handle->cb(ctx->handle, &event, 
+				ctx->handle->cb(ctx->handle, &event,
 						ctx->handle->pw);
 			}
 
 			if (ctx->handle->cb != NULL) {
 				event.type = CONTENT_MSG_DONE;
-				ctx->handle->cb(ctx->handle, &event, 
+				ctx->handle->cb(ctx->handle, &event,
 						ctx->handle->pw);
 			}
 		}
 	}
 
-	return NSERROR_OK;
+	return error;
 }
 
 /**
@@ -705,7 +795,7 @@ nserror hlcache_find_content(hlcache_retrieval_ctx *ctx)
  * \param data	Data for message
  * \param pw	Pointer to private data (hlcache_handle)
  */
-void hlcache_content_callback(struct content *c, content_msg msg, 
+void hlcache_content_callback(struct content *c, content_msg msg,
 		union content_msg_data data, void *pw)
 {
 	hlcache_handle *handle = pw;
@@ -721,4 +811,3 @@ void hlcache_content_callback(struct content *c, content_msg msg,
 	if (error != NSERROR_OK)
 		LOG(("Error in callback: %d", error));
 }
-
