@@ -50,6 +50,7 @@
 #include "desktop/gui.h"
 #include "desktop/knockout.h"
 #include "desktop/options.h"
+#include "desktop/scrollbar.h"
 #include "desktop/selection.h"
 #include "desktop/textinput.h"
 #include "desktop/plotters.h"
@@ -64,9 +65,6 @@
 #include "utils/utils.h"
 #include "utils/utf8.h"
 
-/** browser window which is being redrawn. Valid only during redraw. */
-struct browser_window *current_redraw_browser;
-
 /** one or more windows require a reformat */
 bool browser_reformat_pending;
 
@@ -89,33 +87,140 @@ static void browser_window_set_scale_internal(struct browser_window *bw,
 static void browser_window_find_target_internal(struct browser_window *bw,
 		const char *target, int depth, struct browser_window *page,
 		int *rdepth, struct browser_window **bw_target);
+static void browser_window_mouse_drag_end(struct browser_window *bw,
+		browser_mouse_state mouse, int x, int y);
+
+
+/**
+ * Get position of scrollbar widget within browser window.
+ *
+ * \param  bw		The browser window
+ * \param  horizontal	Whether to get position of horizontal scrollbar
+ * \param  x		Updated to x-coord of top left of scrollbar widget
+ * \param  y		Updated to y-coord of top left of scrollbar widget
+ */
+
+static inline void browser_window_get_scrollbar_pos(struct browser_window *bw,
+		bool horizontal, int *x, int *y)
+{
+	if (horizontal) {
+		*x = 0;
+		*y = bw->height - SCROLLBAR_WIDTH;
+	} else {
+		*x = bw->width - SCROLLBAR_WIDTH;
+		*y = 0;
+	}
+}
+
+
+/**
+ * Get browser window scrollbar widget length
+ *
+ * \param  bw		The browser window
+ * \param  horizontal	Whether to get length of horizontal scrollbar
+ * \return the scrollbar's length
+ */
+
+static inline int browser_window_get_scrollbar_len(struct browser_window *bw,
+		bool horizontal)
+{
+	if (horizontal)
+		return bw->width - (bw->scroll_y != NULL ? SCROLLBAR_WIDTH : 0);
+	else
+		return bw->height;
+}
 
 /* exported interface, documented in browser.h */
 bool browser_window_redraw(struct browser_window *bw, int x, int y,
-		const struct rect *clip)
+		const struct rect *clip, const struct redraw_context *ctx)
 {
+	struct redraw_context new_ctx = *ctx;
 	int width = 0;
 	int height = 0;
 	bool plot_ok = true;
 	content_type content_type;
+	struct content_redraw_data data;
+	struct rect content_clip;
 
 	if (bw == NULL) {
 		LOG(("NULL browser window"));
 		return false;
 	}
 
-	if (bw->current_content == NULL) {
+	if (bw->current_content == NULL && bw->children == NULL) {
 		/* Browser window has no content, render blank fill */
-		plot.clip(clip);
-		return plot.rectangle(clip->x0, clip->y0, clip->x1, clip->y1,
+		ctx->plot->clip(clip);
+		return ctx->plot->rectangle(clip->x0, clip->y0,
+				clip->x1, clip->y1,
 				plot_style_fill_white);
 	}
 
-	/* Browser window has content */
-	if (plot.option_knockout)
-		knockout_plot_start(&plot);
+	/* Browser window has content OR children (frames) */
 
-	plot.clip(clip);
+	if (bw->window != NULL && ctx->plot->option_knockout) {
+		/* Root browser window: start knockout */
+		knockout_plot_start(ctx, &new_ctx);
+	}
+
+	new_ctx.plot->clip(clip);
+
+	/* Handle redraw of any browser window children */
+	if (bw->children) {
+		struct browser_window *child;
+		int cur_child;
+		int children = bw->rows * bw->cols;
+
+		if (bw->window != NULL)
+			/* Root browser window; start with blank fill */
+			plot_ok &= new_ctx.plot->rectangle(clip->x0, clip->y0,
+					clip->x1, clip->y1,
+					plot_style_fill_white);
+
+		/* Loop through all children of bw */
+		for (cur_child = 0; cur_child < children; cur_child++) {
+			/* Set current child */
+			child = &bw->children[cur_child];
+
+			/* Get frame edge box in global coordinates */
+			content_clip.x0 = (x + child->x) * child->scale;
+			content_clip.y0 = (y + child->y) * child->scale;
+			content_clip.x1 = content_clip.x0 +
+					child->width * child->scale;
+			content_clip.y1 = content_clip.y0 +
+					child->height * child->scale;
+
+			/* Intersect it with clip rectangle */
+			if (content_clip.x0 < clip->x0)
+				content_clip.x0 = clip->x0;
+			if (content_clip.y0 < clip->y0)
+				content_clip.y0 = clip->y0;
+			if (clip->x1 < content_clip.x1)
+				content_clip.x1 = clip->x1;
+			if (clip->y1 < content_clip.y1)
+				content_clip.y1 = clip->y1;
+
+			/* Skip this frame if it lies outside clip rectangle */
+			if (content_clip.x0 >= content_clip.x1 ||
+					content_clip.y0 >= content_clip.y1)
+				continue;
+
+			/* Redraw frame */
+			plot_ok &= browser_window_redraw(child,
+					x + child->x, y + child->y,
+					&content_clip, &new_ctx);
+		}
+
+		/* Nothing else to redraw for browser windows with children;
+		 * cleanup and return */
+		if (bw->window != NULL && ctx->plot->option_knockout) {
+			/* Root browser window: knockout end */
+			knockout_plot_end();
+		}
+
+		return plot_ok;
+	}
+
+	/* Handle browser windows with content to redraw */
 
 	content_type = content_get_type(bw->current_content);
 	if (content_type != CONTENT_HTML && content_type != CONTENT_TEXTPLAIN) {
@@ -125,16 +230,67 @@ bool browser_window_redraw(struct browser_window *bw, int x, int y,
 
 		/* Non-HTML may not fill viewport to extents, so plot white
 		 * background fill */
-		plot_ok &= plot.rectangle(clip->x0, clip->y0,
+		plot_ok &= new_ctx.plot->rectangle(clip->x0, clip->y0,
 				clip->x1, clip->y1, plot_style_fill_white);
 	}
+
+	/* Set up content redraw data */
+	data.x = x - scrollbar_get_offset(bw->scroll_x);
+	data.y = y - scrollbar_get_offset(bw->scroll_y);
+	data.width = width;
+	data.height = height;
+
+	data.background_colour = 0xFFFFFF;
+	data.scale = bw->scale;
+	data.repeat_x = false;
+	data.repeat_y = false;
+
+	content_clip = *clip;
+
+	if (!bw->window) {
+		int x0 = x * bw->scale;
+		int y0 = y * bw->scale;
+		int x1 = (x + bw->width - ((bw->scroll_y != NULL) ?
+				SCROLLBAR_WIDTH : 0)) * bw->scale;
+		int y1 = (y + bw->height - ((bw->scroll_x != NULL) ?
+				SCROLLBAR_WIDTH : 0)) * bw->scale;
+
+		if (content_clip.x0 < x0) content_clip.x0 = x0;
+		if (content_clip.y0 < y0) content_clip.y0 = y0;
+		if (x1 < content_clip.x1) content_clip.x1 = x1;
+		if (y1 < content_clip.y1) content_clip.y1 = y1;
+	}
  
 	/* Render the content */
-	plot_ok &= content_redraw(bw->current_content, x, y, width, height,
-			clip, bw->scale, 0xFFFFFF);
+	plot_ok &= content_redraw(bw->current_content, &data,
+			&content_clip, &new_ctx);
+
+	/* Back to full clip rect */
+	new_ctx.plot->clip(clip);
+
+	if (!bw->window) {
+		/* Render scrollbars */
+		int off_x, off_y;
+		if (bw->scroll_x != NULL) {
+			browser_window_get_scrollbar_pos(bw, true,
+					&off_x, &off_y);
+			plot_ok &= scrollbar_redraw(bw->scroll_x,
+					x + off_x, y + off_y, clip,
+					bw->scale, &new_ctx);
+		}
+		if (bw->scroll_y != NULL) {
+			browser_window_get_scrollbar_pos(bw, false,
+					&off_x, &off_y);
+			plot_ok &= scrollbar_redraw(bw->scroll_y,
+					x + off_x, y + off_y, clip,
+					bw->scale, &new_ctx);
+		}
+	}
 	
-	if (plot.option_knockout)
+	if (bw->window != NULL && ctx->plot->option_knockout) {
+		/* Root browser window: end knockout */
 		knockout_plot_end();
+	}
 
 	return plot_ok;
 }
@@ -153,8 +309,226 @@ bool browser_window_redraw_ready(struct browser_window *bw)
 	return true;
 }
 
+/* exported interface, documented in browser.h */
+void browser_window_update_extent(struct browser_window *bw)
+{
+	if (bw->window != NULL)
+		/* Front end window */
+		gui_window_update_extent(bw->window);
+	else
+		/* Core-managed browser window */
+		browser_window_handle_scrollbars(bw);
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_get_position(struct browser_window *bw, bool root,
+		int *pos_x, int *pos_y)
+{
+	*pos_x = 0;
+	*pos_y = 0;
+
+	assert(bw != NULL);
+
+	while (bw) {
+		switch (bw->browser_window_type) {
+
+		case BROWSER_WINDOW_FRAME:
+		case BROWSER_WINDOW_FRAMESET:
+			*pos_x += bw->x * bw->scale;
+			*pos_y += bw->y * bw->scale;
+			break;
+
+		case BROWSER_WINDOW_NORMAL:
+			/* There is no offset to the root browser window */
+			break;
+
+		case BROWSER_WINDOW_IFRAME:
+			*pos_x += (bw->x - scrollbar_get_offset(bw->scroll_x)) *
+					bw->scale;
+			*pos_y += (bw->y - scrollbar_get_offset(bw->scroll_y)) *
+					bw->scale;
+			break;
+		}
+
+		bw = bw->parent;
+
+		if (!root) {
+			/* return if we just wanted the position in the parent
+			 * browser window. */
+			return;
+		}
+	}
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_set_position(struct browser_window *bw, int x, int y)
+{
+	assert(bw != NULL);
+
+	if (bw->window == NULL) {
+		/* Core managed browser window */
+		bw->x = x;
+		bw->y = y;
+	} else {
+		LOG(("Asked to set position of front end window."));
+		assert(0);
+	}
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_set_drag_type(struct browser_window *bw,
+		browser_drag_type type)
+{
+	struct browser_window *top_bw = browser_window_get_root(bw);
+
+	if (type == DRAGGING_NONE)
+		top_bw->drag_window = NULL;
+	else
+		top_bw->drag_window = bw;
+
+	bw->drag_type = type;
+}
+
+/* exported interface, documented in browser.h */
+struct browser_window * browser_window_get_root(struct browser_window *bw)
+{
+	while (bw && !bw->window && bw->parent) {
+		bw = bw->parent;
+	}
+	return bw;
+}
+
+/* exported interface, documented in browser.h */
+bool browser_window_has_selection(struct browser_window *bw)
+{
+	assert(bw->window);
+
+	if (bw->cur_sel != NULL && selection_defined(bw->cur_sel)) {
+		return true;
+	} else {
+		return false;
+	}
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_set_selection(struct browser_window *bw,
+		struct selection *s)
+{
+	assert(bw->window);
+
+	if (bw->cur_sel != s && bw->cur_sel != NULL) {
+		/* Clear any existing selection */
+		selection_clear(bw->cur_sel, true);
+	}
+
+	/* Replace current selection pointer */
+	if (s == NULL && bw->current_content != NULL) {
+		bw->cur_sel = content_get_selection(bw->current_content);
+	} else {
+		bw->cur_sel = s;
+	}
+}
+
+/* exported interface, documented in browser.h */
+struct selection *browser_window_get_selection(struct browser_window *bw)
+{
+	assert(bw->window);
+
+	return bw->cur_sel;
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_scroll_visible(struct browser_window *bw,
+		const struct rect *rect)
+{
+	assert(bw != NULL);
+
+	if (bw->window != NULL) {
+		/* Front end window */
+		gui_window_scroll_visible(bw->window,
+				rect->x0, rect->y0, rect->x1, rect->y1);
+	} else {
+		/* Core managed browser window */
+		if (bw->scroll_x != NULL)
+			scrollbar_set(bw->scroll_x, rect->x0, false);
+		if (bw->scroll_y != NULL)
+			scrollbar_set(bw->scroll_y, rect->y0, false);
+	}
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_set_scroll(struct browser_window *bw, int x, int y)
+{
+	if (bw->window != NULL) {
+		gui_window_set_scroll(bw->window, x, y);
+	} else {
+		if (bw->scroll_x != NULL)
+			scrollbar_set(bw->scroll_x, x, false);
+		if (bw->scroll_y != NULL)
+			scrollbar_set(bw->scroll_y, y, false);
+	}
+}
+
 /**
- * Create and open a new browser window with the given page.
+ * Internal helper for browser_window_get_contextual_content
+ */
+static void browser_window__get_contextual_content(struct browser_window *bw,
+		int x, int y, struct contextual_content *data)
+{
+	/* Handle (i)frame scroll offset (core-managed browser windows only) */
+	x += scrollbar_get_offset(bw->scroll_x);
+	y += scrollbar_get_offset(bw->scroll_y);
+
+	if (bw->children) {
+		/* Browser window has children, so pass request on to
+		 * appropriate child */
+		struct browser_window *bwc;
+		int cur_child;
+		int children = bw->rows * bw->cols;
+
+		/* Loop through all children of bw */
+		for (cur_child = 0; cur_child < children; cur_child++) {
+			/* Set current child */
+			bwc = &bw->children[cur_child];
+
+			/* Skip this frame if (x, y) coord lies outside */
+			if (x < bwc->x || bwc->x + bwc->width < x ||
+					y < bwc->y || bwc->y + bwc->height < y)
+				continue;
+
+			/* Pass request into this child */
+			browser_window__get_contextual_content(bwc,
+					(x - bwc->x), (y - bwc->y), data);
+			return;
+		}
+
+		/* Coordinate not contained by any frame */
+		return;
+	}
+
+	if (bw->current_content == NULL)
+		/* No content; nothing to set */
+		return;
+
+	/* Pass request to content */
+	content_get_contextual_content(bw->current_content, x, y, data);
+	data->main = bw->current_content;
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_get_contextual_content(struct browser_window *bw,
+		int x, int y, struct contextual_content *data)
+{
+	data->link_url = NULL;
+	data->object = NULL;
+	data->main = NULL;
+
+	browser_window__get_contextual_content(bw, x, y, data);
+}
+
+
+/**
+ * Create and open a new root browser window with the given page.
  *
  * \param  url	    URL to start fetching in the new window (copied)
  * \param  clone    The browser window to clone
@@ -166,6 +540,7 @@ struct browser_window *browser_window_create(const char *url,
 		const char *referer, bool history_add, bool new_tab)
 {
 	struct browser_window *bw;
+	struct browser_window *top;
 
 	assert(clone || history_add);
 
@@ -183,10 +558,15 @@ struct browser_window *browser_window_create(const char *url,
 	bw->border = true;
 	bw->no_resize = true;
 	bw->last_action = wallclock();
-
-	bw->window = gui_create_browser_window(bw, clone, new_tab);
+	bw->focus = bw;
 
 	/* gui window */
+	/* from the front end's pov, it clones the top level browser window,
+	 * so find that. */
+	top = browser_window_get_root(clone);
+
+	bw->window = gui_create_browser_window(bw, top, new_tab);
+
 	if (bw->window == NULL) {
 		browser_window_destroy(bw);
 		return NULL;
@@ -217,13 +597,19 @@ void browser_window_initialise_common(struct browser_window *bw,
 		bw->history = history_clone(clone->history);
 
 	/* window characteristics */
-	bw->sel = selection_create(bw);
+	bw->cur_sel = NULL;
+	bw->cur_search = NULL;
 	bw->refresh_interval = -1;
 
 	bw->reformat_pending = false;
 	bw->drag_type = DRAGGING_NONE;
 	bw->scale = (float) option_scale / 100.0;
 
+	bw->scroll_x = NULL;
+	bw->scroll_y = NULL;
+
+	bw->focus = NULL;
+
 	/* initialise status text cache */
 	bw->status_text = NULL;
 	bw->status_text_len = 0;
@@ -449,12 +835,13 @@ void browser_window_go_post(struct browser_window *bw, const char *url,
 	bw->history_add = add_to_history;
 
 	error = hlcache_handle_retrieve(url2,
-			fetch_flags | HLCACHE_RETRIEVE_MAY_DOWNLOAD, 
+			fetch_flags | HLCACHE_RETRIEVE_MAY_DOWNLOAD | 
+					HLCACHE_RETRIEVE_SNIFF_TYPE, 
 			referer,
 			fetch_is_post ? &post : NULL,
 			browser_window_callback, bw,
 			parent != NULL ? &child : NULL,
-			NULL, &c);
+			CONTENT_ANY, &c);
 	if (error == NSERROR_NO_FETCH_HANDLER) {
 		gui_launch_url(url2);
 		free(url2);
@@ -534,14 +921,13 @@ nserror browser_window_callback(hlcache_handle *c,
 		bw->loading_content = NULL;
 
 		/* Format the new content to the correct dimensions */
-		gui_window_get_dimensions(bw->window, &width, &height, true);
-		content_reformat(c, width, height);
+		browser_window_get_dimensions(bw, &width, &height, true);
+		content_reformat(c, false, width, height);
 
 		browser_window_remove_caret(bw);
 
-		bw->scroll = NULL;
-
-		gui_window_new_content(bw->window);
+		if (bw->window)
+			gui_window_new_content(bw->window);
 
 		browser_window_refresh_url_bar(bw,
 				content_get_url(bw->current_content),
@@ -566,18 +952,11 @@ nserror browser_window_callback(hlcache_handle *c,
 				global_history_add(urldb_get_url(url));
 			}
 		}
-		
-		/* favicon preload */
-		if (content_get_type(c) == CONTENT_HTML)
-			gui_window_set_icon(bw->window, 
-					    html_get_favicon(bw->current_content));
-		
-		/* text selection */
-		if (content_get_type(c) == CONTENT_HTML)
-			selection_init(bw->sel,
-					html_get_box_tree(bw->current_content));
-		if (content_get_type(c) == CONTENT_TEXTPLAIN)
-			selection_init(bw->sel, NULL);
+
+		if (bw->window != NULL) {
+			browser_window_set_selection(bw,
+					content_get_selection(c));
+		}
 
 		/* frames */
 		if (content_get_type(c) == CONTENT_HTML && 
@@ -593,6 +972,14 @@ nserror browser_window_callback(hlcache_handle *c,
 	case CONTENT_MSG_DONE:
 		assert(bw->current_content == c);
 
+		if (bw->window == NULL) {
+			/* Updated browser window's scrollbars.
+			 * TODO: do this before CONTENT_MSG_DONE */
+			browser_window_reformat(bw, true,
+					bw->width, bw->height);
+			browser_window_handle_scrollbars(bw);
+		}
+
 		browser_window_update(bw, false);
 		browser_window_set_status(bw, content_get_status_message(c));
 		browser_window_stop_throbber(bw);
@@ -618,8 +1005,6 @@ nserror browser_window_callback(hlcache_handle *c,
 		else if (c == bw->current_content) {
 			bw->current_content = NULL;
 			browser_window_remove_caret(bw);
-			bw->scroll = NULL;
-			selection_init(bw->sel, NULL);
 		}
 
 		hlcache_handle_release(c);
@@ -640,18 +1025,35 @@ nserror browser_window_callback(hlcache_handle *c,
 			/* reflow iframe positions */
 			if (html_get_iframe(c) != NULL)
 				browser_window_recalculate_iframes(bw);
-			/* box tree may have changed, need to relabel */
-			selection_reinit(bw->sel, html_get_box_tree(c));
 		}
 
 		if (bw->move_callback)
-			bw->move_callback(bw, bw->caret_p);
+			bw->move_callback(bw, bw->caret_p1, bw->caret_p2);
 
-		browser_window_update(bw, false);
+		if (!(event->data.background)) {
+			/* Reformatted content should be redrawn */
+			browser_window_update(bw, false);
+		}
 		break;
 
 	case CONTENT_MSG_REDRAW:
-		gui_window_update_box(bw->window, &event->data);
+	{
+		struct rect rect = {
+			.x0 = event->data.redraw.x,
+			.y0 = event->data.redraw.y,
+			.x1 = event->data.redraw.x + event->data.redraw.width,
+			.y1 = event->data.redraw.y + event->data.redraw.height
+		};
+
+		if (bw->browser_window_type == BROWSER_WINDOW_FRAME) {
+			rect.x0 -= scrollbar_get_offset(bw->scroll_x);
+			rect.y0 -= scrollbar_get_offset(bw->scroll_y);
+			rect.x1 -= scrollbar_get_offset(bw->scroll_x);
+			rect.y1 -= scrollbar_get_offset(bw->scroll_y);
+		}
+
+		browser_window_update_box(bw, &rect);
+	}
 		break;
 
 	case CONTENT_MSG_REFRESH:
@@ -660,7 +1062,10 @@ nserror browser_window_callback(hlcache_handle *c,
 		
 	case CONTENT_MSG_FAVICON_REFRESH:
 		/* Cause the GUI to update */
-		gui_window_set_icon(bw->window, html_get_favicon(bw->current_content));
+		if (bw->browser_window_type == BROWSER_WINDOW_NORMAL) {
+			gui_window_set_icon(bw->window,
+					html_get_favicon(bw->current_content));
+		}
 		break;
 
 	default:
@@ -671,6 +1076,55 @@ nserror browser_window_callback(hlcache_handle *c,
 }
 
 
+/*
+ * Get the dimensions of the area a browser window occupies
+ *
+ * \param  bw      The browser window to get dimensions of
+ * \param  width   Updated to the browser window viewport width
+ * \param  height  Updated to the browser window viewport height
+ * \param  scaled  Whether we want the height with scale applied
+ */
+
+void browser_window_get_dimensions(struct browser_window *bw,
+		int *width, int *height, bool scaled)
+{
+	assert(bw);
+
+	if (bw->window == NULL) {
+		/* Core managed browser window */
+		*width = bw->width;
+		*height = bw->height;
+	} else {
+		/* Front end window */
+		gui_window_get_dimensions(bw->window, width, height, scaled);
+	}
+}
+
+
+/*
+ * Set the dimensions of the area a browser window occupies
+ *
+ * \param  bw      The browser window to set dimensions of
+ * \param  width   Width in pixels
+ * \param  height  Height in pixels
+ */
+
+void browser_window_set_dimensions(struct browser_window *bw,
+		int width, int height)
+{
+	assert(bw);
+
+	if (bw->window == NULL) {
+		/* Core managed browser window */
+		bw->width = width;
+		bw->height = height;
+	} else {
+		LOG(("Asked to set dimensions of front end window."));
+		assert(0);
+	}
+}
+
+
 /**
  * Transfer the loading_content to a new download window.
  */
@@ -833,22 +1287,103 @@ void browser_window_update(struct browser_window *bw, bool scroll_to_top)
 	if (bw->current_content == NULL)
 		return;
 
-	gui_window_set_title(bw->window, 
-			content_get_title(bw->current_content));
+	switch (bw->browser_window_type) {
 
-	gui_window_update_extent(bw->window);
+	case BROWSER_WINDOW_NORMAL:
+		/* Root browser window, constituting a front end window/tab */
+		gui_window_set_title(bw->window, 
+				content_get_title(bw->current_content));
 
-	if (scroll_to_top)
-		gui_window_set_scroll(bw->window, 0, 0);
+		browser_window_update_extent(bw);
 
-	/** \todo don't do this if the user has scrolled */
-	/* if frag_id exists, then try to scroll to it */
-	if (bw->frag_id && html_get_id_offset(bw->current_content, bw->frag_id,
-			&x, &y)) {
-		gui_window_set_scroll(bw->window, x, y);
+		if (scroll_to_top)
+			browser_window_set_scroll(bw, 0, 0);
+
+		/* if frag_id exists, then try to scroll to it */
+		/** \TODO don't do this if the user has scrolled */
+		if (bw->frag_id && html_get_id_offset(bw->current_content,
+				bw->frag_id, &x, &y)) {
+			browser_window_set_scroll(bw, x, y);
+		}
+
+		gui_window_redraw_window(bw->window);
+
+		break;
+
+	case BROWSER_WINDOW_IFRAME:
+		/* Internal iframe browser window */
+
+		browser_window_update_extent(bw);
+
+		if (scroll_to_top)
+			browser_window_set_scroll(bw, 0, 0);
+
+		/* if frag_id exists, then try to scroll to it */
+		/** \TODO don't do this if the user has scrolled */
+		if (bw->frag_id && html_get_id_offset(bw->current_content,
+				bw->frag_id, &x, &y)) {
+			browser_window_set_scroll(bw, x, y);
+		}
+
+		html_redraw_a_box(bw->parent->current_content, bw->box);
+		break;
+
+	case BROWSER_WINDOW_FRAME:
+	{
+		struct rect rect;
+		browser_window_update_extent(bw);
+
+		if (scroll_to_top)
+			browser_window_set_scroll(bw, 0, 0);
+
+		/* if frag_id exists, then try to scroll to it */
+		/** \TODO don't do this if the user has scrolled */
+		if (bw->frag_id && html_get_id_offset(bw->current_content,
+				bw->frag_id, &x, &y)) {
+			browser_window_set_scroll(bw, x, y);
+		}
+
+		rect.x0 = 0;
+		rect.y0 = 0;
+		rect.x1 = bw->width;
+		rect.y1 = bw->height;
+
+		browser_window_update_box(bw, &rect);
 	}
+		break;
 
-	gui_window_redraw_window(bw->window);
+	default:
+	case BROWSER_WINDOW_FRAMESET:
+		/* Nothing to do */
+		break;
+	}
+}
+
+
+void browser_window_update_box(struct browser_window *bw, struct rect *rect)
+{
+	int pos_x;
+	int pos_y;
+	struct browser_window *top;
+
+	assert(bw);
+
+	if (bw->window != NULL) {
+		/* Front end window */
+		gui_window_update_box(bw->window, rect);
+	} else {
+		/* Core managed browser window */
+		browser_window_get_position(bw, true, &pos_x, &pos_y);
+
+		top = browser_window_get_root(bw);
+
+		rect->x0 += pos_x / bw->scale;
+		rect->y0 += pos_y / bw->scale;
+		rect->x1 += pos_x / bw->scale;
+		rect->y1 += pos_y / bw->scale;
+
+		gui_window_update_box(top->window, rect);
+	}
 }
 
 
@@ -993,9 +1528,15 @@ void browser_window_set_status(struct browser_window *bw, const char *text)
  * \param  shape    shape to use
  */
 
-void browser_window_set_pointer(struct gui_window *g, gui_pointer_shape shape)
+void browser_window_set_pointer(struct browser_window *bw,
+		gui_pointer_shape shape)
 {
-	gui_window_set_pointer(g, shape);
+	struct browser_window *root = browser_window_get_root(bw);
+
+	assert(root);
+	assert(root->window);
+
+	gui_window_set_pointer(root->window, shape);
 }
 
 
@@ -1061,14 +1602,33 @@ void browser_window_destroy_internal(struct browser_window *bw)
 
 	schedule_remove(browser_window_refresh, bw);
 
+	/* If this brower window is not the root window, and has focus, unset
+	 * the root browser window's focus pointer. */
+	if (!bw->window) {
+		struct browser_window *top = browser_window_get_root(bw);
+
+		if (top->focus == bw)
+			top->focus = top;
+
+		if (bw->current_content != NULL &&
+				top->cur_sel == content_get_selection(
+						bw->current_content)) {
+			browser_window_set_selection(top, NULL);
+		}
+	}
+
 	/* Destruction order is important: we must ensure that the frontend 
 	 * destroys any window(s) associated with this browser window before 
 	 * we attempt any destructive cleanup. 
 	 */
 
-	gui_window_destroy(bw->window);
+	if (bw->window) {
+		/* Only the root window has a GUI window */
+		gui_window_destroy(bw->window);
+	}
 
 	if (bw->loading_content != NULL) {
+		hlcache_handle_abort(bw->loading_content);
 		hlcache_handle_release(bw->loading_content);
 		bw->loading_content = NULL;
 	}
@@ -1083,8 +1643,12 @@ void browser_window_destroy_internal(struct browser_window *bw)
 		bw->current_content = NULL;
 	}
 
+	if (bw->box != NULL) {
+		bw->box->iframe = NULL;
+		bw->box = NULL;
+	}
+
 	/* These simply free memory, so are safe here */
-	selection_destroy(bw->sel);
 	history_destroy(bw->history);
 
 	free(bw->name);
@@ -1096,35 +1660,6 @@ void browser_window_destroy_internal(struct browser_window *bw)
 }
 
 
-/**
- * Returns the browser window that is responsible for the child.
- *
- * \param  bw	The browser window to find the owner of
- * \return the browser window's owner
- */
-
-struct browser_window *browser_window_owner(struct browser_window *bw)
-{
-  	/* an iframe's parent is just the parent window */
-  	if (bw->browser_window_type == BROWSER_WINDOW_IFRAME)
-  		return bw->parent;
-
-  	/* the parent of a frameset is either a NORMAL window or an IFRAME */
-	while (bw->parent != NULL) {
-		switch (bw->browser_window_type) {
- 		case BROWSER_WINDOW_NORMAL:
-  		case BROWSER_WINDOW_IFRAME:
-  			return bw;
-		case BROWSER_WINDOW_FRAME:
- 		case BROWSER_WINDOW_FRAMESET:
-  			bw = bw->parent;
-			break;
-		}
-	}
-	return bw;
-}
-
-
 /**
  * Reformat a browser window contents to a new width or height.
  *
@@ -1133,14 +1668,30 @@ struct browser_window *browser_window_owner(struct browser_window *bw)
  * \param  height  new height
  */
 
-void browser_window_reformat(struct browser_window *bw, int width, int height)
+void browser_window_reformat(struct browser_window *bw, bool background,
+		int width, int height)
 {
 	hlcache_handle *c = bw->current_content;
 
 	if (c == NULL)
 		return;
 
-	content_reformat(c, width / bw->scale, height / bw->scale);
+	if (bw->browser_window_type != BROWSER_WINDOW_IFRAME) {
+		/* Iframe dimensions are already scaled in parent's layout */
+		width  /= bw->scale;
+		height /= bw->scale;
+	}
+
+	if (bw->window == NULL) {
+		/* Core managed browser window; subtract scrollbar width */
+		width  -= bw->scroll_y ? SCROLLBAR_WIDTH : 0;
+		height -= bw->scroll_x ? SCROLLBAR_WIDTH : 0;
+
+		width  = width  > 0 ? width  : 0;
+		height = height > 0 ? height : 0;
+	}
+
+	content_reformat(c, background, width, height);
 }
 
 
@@ -1185,8 +1736,6 @@ void browser_window_set_scale_internal(struct browser_window *bw, float scale)
 	 	}
 	}
 
-	gui_window_set_scale(bw->window, scale);
-
 	for (i = 0; i < (bw->cols * bw->rows); i++)
 		browser_window_set_scale_internal(&bw->children[i], scale);
 	for (i = 0; i < bw->iframe_count; i++)
@@ -1209,8 +1758,11 @@ void browser_window_refresh_url_bar(struct browser_window *bw, const char *url,
 
 	assert(bw);
 	assert(url);
-	
-	bw->visible_select_menu = NULL;
+
+	if (bw->parent != NULL) {
+		/* Not root window; don't set a URL in GUI URL bar */
+		return;
+	}
 
 	if (frag == NULL) {
 		/* With no fragment, we may as well pass url straight through
@@ -1402,7 +1954,7 @@ void browser_window_find_target_internal(struct browser_window *bw,
 
 
 /**
- * Handle mouse movements in a browser window.
+ * Handle non-click mouse action in a browser window. (drag ends, movements)
  *
  * \param  bw	  browser window
  * \param  mouse  state of mouse buttons and modifier keys
@@ -1414,19 +1966,128 @@ void browser_window_mouse_track(struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
 	hlcache_handle *c = bw->current_content;
+	const char *status = NULL;
+	gui_pointer_shape pointer = GUI_POINTER_DEFAULT;
+
+	if (bw->window != NULL && bw->drag_window && bw != bw->drag_window) {
+		/* This is the root browser window and there's an active drag
+		 * in a sub window.
+		 * Pass the mouse action straight on to that bw. */
+		struct browser_window *drag_bw = bw->drag_window;
+		int off_x = 0;
+		int off_y = 0;
+
+		browser_window_get_position(drag_bw, true, &off_x, &off_y);
+
+		if (drag_bw->browser_window_type == BROWSER_WINDOW_FRAME) {
+			off_x -= scrollbar_get_offset(drag_bw->scroll_x);
+			off_y -= scrollbar_get_offset(drag_bw->scroll_y);
+
+			browser_window_mouse_track(drag_bw, mouse,
+					x - off_x, y - off_y);
+
+		} else if (drag_bw->browser_window_type ==
+				BROWSER_WINDOW_IFRAME) {
+			browser_window_mouse_track(drag_bw, mouse,
+					x - off_x / bw->scale,
+					y - off_y / bw->scale);
+		}
+		return;
+	}
+
+	if (bw->children) {
+		/* Browser window has children (frames) */
+		struct browser_window *child;
+		int cur_child;
+		int children = bw->rows * bw->cols;
+
+		for (cur_child = 0; cur_child < children; cur_child++) {
+
+			child = &bw->children[cur_child];
+
+			if (x < child->x || y < child->y ||
+					child->x + child->width < x ||
+					child->y + child->height < y) {
+				/* Click not in this child */
+				continue;
+			}
+
+			/* It's this child that contains the mouse; pass
+			 * mouse action on to child */
+			browser_window_mouse_track(child, mouse,
+					x - child->x + scrollbar_get_offset(
+							child->scroll_x),
+					y - child->y + scrollbar_get_offset(
+							child->scroll_y));
+
+			/* Mouse action was for this child, we're done */
+			return;
+		}
+
+		/* Odd if we reached here, but nothing else can use the click
+		 * when there are children. */
+		return;
+	}
 
 	if (c == NULL && bw->drag_type != DRAGGING_FRAME)
 		return;
 
-	/* detect end of drag operation in case the platform-specific code
-	   doesn't call browser_mouse_drag_end() (RISC OS code does) */
-
 	if (bw->drag_type != DRAGGING_NONE && !mouse) {
 		browser_window_mouse_drag_end(bw, mouse, x, y);
 	}
 
+	/* Browser window's horizontal scrollbar */
+	if (bw->scroll_x != NULL && bw->drag_type != DRAGGING_SCR_Y) {
+		int scr_x, scr_y;
+		browser_window_get_scrollbar_pos(bw, true, &scr_x, &scr_y);
+		scr_x = x - scr_x - scrollbar_get_offset(bw->scroll_x);
+		scr_y = y - scr_y - scrollbar_get_offset(bw->scroll_y);
+
+		if ((scr_x > 0 && scr_x < browser_window_get_scrollbar_len(bw,
+						true) &&
+				scr_y > 0 && scr_y < SCROLLBAR_WIDTH &&
+				bw->drag_type == DRAGGING_NONE) ||
+				bw->drag_type == DRAGGING_SCR_X) {
+			/* Start a scrollbar drag, or continue existing drag */
+			status = scrollbar_mouse_action(bw->scroll_x, mouse,
+					scr_x, scr_y);
+			pointer = GUI_POINTER_DEFAULT;
+
+			if (status != NULL)
+				browser_window_set_status(bw, status);
+
+			browser_window_set_pointer(bw, pointer);
+			return;
+		}
+	}
+
+	/* Browser window's vertical scrollbar */
+	if (bw->scroll_y != NULL) {
+		int scr_x, scr_y;
+		browser_window_get_scrollbar_pos(bw, false, &scr_x, &scr_y);
+		scr_x = x - scr_x - scrollbar_get_offset(bw->scroll_x);
+		scr_y = y - scr_y - scrollbar_get_offset(bw->scroll_y);
+
+		if ((scr_y > 0 && scr_y < browser_window_get_scrollbar_len(bw,
+						false) &&
+				scr_x > 0 && scr_x < SCROLLBAR_WIDTH &&
+				bw->drag_type == DRAGGING_NONE) ||
+				bw->drag_type == DRAGGING_SCR_Y) {
+			/* Start a scrollbar drag, or continue existing drag */
+			status = scrollbar_mouse_action(bw->scroll_y, mouse,
+					scr_x, scr_y);
+			pointer = GUI_POINTER_DEFAULT;
+
+			if (status != NULL)
+				browser_window_set_status(bw, status);
+
+			browser_window_set_pointer(bw, pointer);
+			return;
+		}
+	}
+
 	if (bw->drag_type == DRAGGING_FRAME) {
-		browser_window_resize_frame(bw, bw->x0 + x, bw->y0 + y);
+		browser_window_resize_frame(bw, bw->x + x, bw->y + y);
 	} else if (bw->drag_type == DRAGGING_PAGE_SCROLL) {
 		/* mouse movement since drag started */
 		int scrollx = bw->drag_start_x - x;
@@ -1439,7 +2100,7 @@ void browser_window_mouse_track(struct browser_window *bw,
 		bw->drag_start_scroll_x = scrollx;
 		bw->drag_start_scroll_y = scrolly;
 
-		gui_window_set_scroll(bw->window, scrollx, scrolly);
+		browser_window_set_scroll(bw, scrollx, scrolly);
 	} else {
 		assert(c != NULL);
 		content_mouse_track(c, bw, mouse, x, y);
@@ -1460,10 +2121,86 @@ void browser_window_mouse_click(struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
 	hlcache_handle *c = bw->current_content;
+	const char *status = NULL;
+	gui_pointer_shape pointer = GUI_POINTER_DEFAULT;
+
+	if (bw->children) {
+		/* Browser window has children (frames) */
+		struct browser_window *child;
+		int cur_child;
+		int children = bw->rows * bw->cols;
+
+		for (cur_child = 0; cur_child < children; cur_child++) {
+
+			child = &bw->children[cur_child];
+
+			if (x < child->x || y < child->y ||
+					child->x + child->width < x ||
+					child->y + child->height < y) {
+				/* Click not in this child */
+				continue;
+			}
+
+			/* It's this child that contains the click; pass it
+			 * on to child. */
+			browser_window_mouse_click(child, mouse,
+					x - child->x + scrollbar_get_offset(
+							child->scroll_x),
+					y - child->y + scrollbar_get_offset(
+							child->scroll_y));
+
+			/* Mouse action was for this child, we're done */
+			return;
+		}
+
+		return;
+	}
 
 	if (!c)
 		return;
 
+	if (bw->scroll_x != NULL) {
+		int scr_x, scr_y;
+		browser_window_get_scrollbar_pos(bw, true, &scr_x, &scr_y);
+		scr_x = x - scr_x - scrollbar_get_offset(bw->scroll_x);
+		scr_y = y - scr_y - scrollbar_get_offset(bw->scroll_y);
+
+		if (scr_x > 0 && scr_x < browser_window_get_scrollbar_len(bw,
+						true) &&
+				scr_y > 0 && scr_y < SCROLLBAR_WIDTH) {
+			status = scrollbar_mouse_action(bw->scroll_x, mouse,
+					scr_x, scr_y);
+			pointer = GUI_POINTER_DEFAULT;
+
+			if (status != NULL)
+				browser_window_set_status(bw, status);
+
+			browser_window_set_pointer(bw, pointer);
+			return;
+		}
+	}
+
+	if (bw->scroll_y != NULL) {
+		int scr_x, scr_y;
+		browser_window_get_scrollbar_pos(bw, false, &scr_x, &scr_y);
+		scr_x = x - scr_x - scrollbar_get_offset(bw->scroll_x);
+		scr_y = y - scr_y - scrollbar_get_offset(bw->scroll_y);
+
+		if (scr_y > 0 && scr_y < browser_window_get_scrollbar_len(bw,
+						false) &&
+				scr_x > 0 && scr_x < SCROLLBAR_WIDTH) {
+			status = scrollbar_mouse_action(bw->scroll_y, mouse,
+					scr_x, scr_y);
+			pointer = GUI_POINTER_DEFAULT;
+
+			if (status != NULL)
+				browser_window_set_status(bw, status);
+
+			browser_window_set_pointer(bw, pointer);
+			return;
+		}
+	}
+
 	switch (content_get_type(c)) {
 	case CONTENT_HTML:
 	case CONTENT_TEXTPLAIN:
@@ -1481,7 +2218,7 @@ void browser_window_mouse_click(struct browser_window *bw,
 		else if (mouse & (BROWSER_MOUSE_DRAG_1 |
 				BROWSER_MOUSE_DRAG_2)) {
 			browser_window_page_drag_start(bw, x, y);
-			browser_window_set_pointer(bw->window, GUI_POINTER_MOVE);
+			browser_window_set_pointer(bw, GUI_POINTER_MOVE);
 		}
 		break;
 	}
@@ -1495,57 +2232,53 @@ void browser_window_mouse_click(struct browser_window *bw,
  * \param  mouse  state of mouse buttons and modifier keys
  * \param  x	  coordinate of mouse
  * \param  y	  coordinate of mouse
+ *
+ * TODO: Remove this function, once these things are associated with content,
+ *       rather than bw.
  */
 
-// TODO: MOVE content specific stuff out
-
 void browser_window_mouse_drag_end(struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
-	
-	if (bw->visible_select_menu != NULL) {
-		form_select_mouse_drag_end(bw->visible_select_menu, mouse,
-				x, y);
-		return;
-	}
-	
-	if (bw->scroll != NULL) {
-		html_overflow_scroll_drag_end(bw->scroll, mouse, x, y);
-		return;
-	}
-	
+	int scr_x, scr_y;
+
 	switch (bw->drag_type) {
-	case DRAGGING_SELECTION: {
-		hlcache_handle *h = bw->current_content;
-		if (h) {
-			int dir = -1;
-			size_t idx;
+	case DRAGGING_SELECTION:
+		/* Drag handled by content handler */
+		break;
 
-			if (selection_dragging_start(bw->sel))
-				dir = 1;
+	case DRAGGING_OTHER:
+		/* Drag handled by content handler */
+		break;
 
-			if (content_get_type(h) == CONTENT_HTML) {
-				idx = html_selection_drag_end(h, mouse, x, y,
-						dir);
-				if (idx != 0)
-					selection_track(bw->sel, mouse, idx);
-			} else {
-				assert(content_get_type(h) ==
-						CONTENT_TEXTPLAIN);
-				idx = textplain_offset_from_coords(h, x, y,
-						dir);
-				selection_track(bw->sel, mouse, idx);
-			}
-		}
-		selection_drag_end(bw->sel);
-	}
-	break;
+	case DRAGGING_SCR_X:
+
+		browser_window_get_scrollbar_pos(bw, true, &scr_x, &scr_y);
+
+		scr_x = x - scr_x - scrollbar_get_offset(bw->scroll_x);
+		scr_y = y - scr_y - scrollbar_get_offset(bw->scroll_y);
+
+		scrollbar_mouse_drag_end(bw->scroll_x, mouse, scr_x, scr_y);
+
+		bw->drag_type = DRAGGING_NONE;
+		break;
+
+	case DRAGGING_SCR_Y:
+
+		browser_window_get_scrollbar_pos(bw, false, &scr_x, &scr_y);
+
+		scr_x = x - scr_x - scrollbar_get_offset(bw->scroll_x);
+		scr_y = y - scr_y - scrollbar_get_offset(bw->scroll_y);
+
+		scrollbar_mouse_drag_end(bw->scroll_y, mouse, scr_x, scr_y);
+
+		bw->drag_type = DRAGGING_NONE;
+		break;
 
 	default:
+		browser_window_set_drag_type(bw, DRAGGING_NONE);
 		break;
 	}
-
-	bw->drag_type = DRAGGING_NONE;
 }
 
 
@@ -1581,10 +2314,17 @@ void browser_window_page_drag_start(struct browser_window *bw, int x, int y)
 	bw->drag_start_x = x;
 	bw->drag_start_y = y;
 
-	gui_window_get_scroll(bw->window, &bw->drag_start_scroll_x,
-			&bw->drag_start_scroll_y);
+	if (bw->window != NULL) {
+		/* Front end window */
+		gui_window_get_scroll(bw->window, &bw->drag_start_scroll_x,
+				&bw->drag_start_scroll_y);
 
-	gui_window_scroll_start(bw->window);
+		gui_window_scroll_start(bw->window);
+	} else {
+		/* Core managed browser window */
+		bw->drag_start_scroll_x = scrollbar_get_offset(bw->scroll_x);
+		bw->drag_start_scroll_y = scrollbar_get_offset(bw->scroll_y);
+	}
 }
 
 
