@@ -34,6 +34,7 @@
 #include "desktop/frames.h"
 #include "desktop/history_core.h"
 #include "desktop/gui.h"
+#include "desktop/scrollbar.h"
 #include "desktop/selection.h"
 #include "utils/log.h"
 #include "utils/messages.h"
@@ -47,6 +48,132 @@ static bool browser_window_resolve_frame_dimension(struct browser_window *bw,
 		bool height);
 
 
+/**
+ * Callback for (i)frame scrollbars.
+ */
+void browser_window_scroll_callback(void *client_data,
+		struct scrollbar_msg_data *scrollbar_data)
+{
+	struct browser_window *bw = client_data;
+
+	switch(scrollbar_data->msg) {
+	case SCROLLBAR_MSG_REDRAW:
+		/* TODO: Is this needed? */
+		break;
+	case SCROLLBAR_MSG_MOVED:
+		if (bw->browser_window_type == BROWSER_WINDOW_IFRAME) {
+			html_redraw_a_box(bw->parent->current_content, bw->box);
+		} else {
+			struct rect rect;
+
+			rect.x0 = 0;
+			rect.y0 = 0;
+			rect.x1 = bw->width;
+			rect.y1 = bw->height;
+
+			browser_window_update_box(bw, &rect);
+		}
+		break;
+	case SCROLLBAR_MSG_SCROLL_START:
+		if (scrollbar_is_horizontal(scrollbar_data->scrollbar))
+			browser_window_set_drag_type(bw, DRAGGING_SCR_X);
+		else
+			browser_window_set_drag_type(bw, DRAGGING_SCR_Y);
+
+		break;
+	case SCROLLBAR_MSG_SCROLL_FINISHED:
+		browser_window_set_drag_type(bw, DRAGGING_NONE);
+
+		browser_window_set_pointer(bw, GUI_POINTER_DEFAULT);
+		break;
+	}
+}
+
+/* exported interface, documented in browser.h */
+void browser_window_handle_scrollbars(struct browser_window *bw)
+{
+	hlcache_handle *h = bw->current_content;
+	bool scroll_x;
+	bool scroll_y;
+	int c_width = 0;
+	int c_height = 0;
+
+	assert(!bw->window); /* Core-handled windows only */
+
+	if (h != NULL) {
+		c_width  = content_get_width(h);
+		c_height = content_get_height(h);
+	}
+
+	if (bw->scrolling == SCROLLING_YES) {
+		scroll_x = true;
+		scroll_y = true;
+	} else if (bw->scrolling == SCROLLING_AUTO &&
+			bw->current_content) {
+		int bw_width = bw->width;
+		int bw_height = bw->height;
+
+		/* subtract existing scrollbar width */
+		bw_width -= bw->scroll_y ? SCROLLBAR_WIDTH : 0;
+		bw_height -= bw->scroll_x ? SCROLLBAR_WIDTH : 0;
+
+		scroll_y = (c_height > bw_height) ? true : false;
+		scroll_x = (c_width > bw_width) ? true : false;
+	} else {
+		/* No scrollbars */
+		scroll_x = false;
+		scroll_y = false;
+	}
+
+	if (!scroll_x && bw->scroll_x != NULL) {
+		scrollbar_destroy(bw->scroll_x);
+		bw->scroll_x = NULL;
+	}
+
+	if (!scroll_y && bw->scroll_y != NULL) {
+		scrollbar_destroy(bw->scroll_y);
+		bw->scroll_y = NULL;
+	}
+
+	if (scroll_y) {
+		int length = bw->height;
+		int visible = bw->height - (scroll_x ? SCROLLBAR_WIDTH : 0);
+
+		if (bw->scroll_y == NULL) {
+			/* create vertical scrollbar */
+			if (!scrollbar_create(false, length, c_height, visible,
+					bw, browser_window_scroll_callback,
+					&(bw->scroll_y)))
+				return;
+		} else {
+			/* update vertical scrollbar */
+			scrollbar_set_extents(bw->scroll_y, length,
+					visible, c_height);
+		}
+	}
+
+	if (scroll_x) {
+		int length = bw->width - (scroll_y ? SCROLLBAR_WIDTH : 0);
+		int visible = length;
+
+		if (bw->scroll_x == NULL) {
+			/* create horizontal scrollbar */
+			if (!scrollbar_create(true, length, c_width, visible,
+					bw, browser_window_scroll_callback,
+					&(bw->scroll_x)))
+				return;
+		} else {
+			/* update horizontal scrollbar */
+			scrollbar_set_extents(bw->scroll_x, length,
+					visible, c_width);
+		}
+	}
+
+	if (scroll_x && scroll_y)
+		scrollbar_make_pair(bw->scroll_x, bw->scroll_y);
+}
+
+
 /**
  * Create and open a iframes for a browser window.
  *
@@ -55,9 +182,11 @@ static bool browser_window_resolve_frame_dimension(struct browser_window *bw,
  */
 
 void browser_window_create_iframes(struct browser_window *bw,
-		struct content_html_iframe *iframe) {
+		struct content_html_iframe *iframe)
+{
 	struct browser_window *window;
 	struct content_html_iframe *cur;
+	struct rect rect;
 	int iframes = 0;
 	int index;
 
@@ -83,6 +212,8 @@ void browser_window_create_iframes(struct browser_window *bw,
 		window->no_resize = true;
 		window->margin_width = cur->margin_width;
 		window->margin_height = cur->margin_height;
+		window->cur_sel = bw->cur_sel;
+		window->scale = bw->scale;
 		if (cur->name) {
 			window->name = strdup(cur->name);
 			if (!window->name)
@@ -92,22 +223,29 @@ void browser_window_create_iframes(struct browser_window *bw,
 		/* linking */
 		window->box = cur->box;
 		window->parent = bw;
+		window->box->iframe = window;
 
-		/* gui window */
-		window->window = gui_create_browser_window(window, bw, false);
+		/* iframe dimensions */
+		box_bounds(window->box, &rect);
+
+		browser_window_set_position(window, rect.x0, rect.y0);
+		browser_window_set_dimensions(window, rect.x1 - rect.x0,
+				rect.y1 - rect.y0);
 	}
 
 	/* calculate dimensions */
-	gui_window_update_extent(bw->window);
+	browser_window_update_extent(bw);
 	browser_window_recalculate_iframes(bw);
 
 	index = 0;
 	for (cur = iframe; cur; cur = cur->next) {
 		window = &(bw->iframes[index++]);
-		if (cur->url)
+		if (cur->url) {
+			/* fetch iframe's content */
 			browser_window_go_unverifiable(window, cur->url,
 					content_get_url(bw->current_content),
 					false, bw->current_content);
+		}
 	}
 }
 
@@ -118,42 +256,16 @@ void browser_window_create_iframes(struct browser_window *bw,
  * \param  bw	    The browser window to reposition iframes for
  */
 
-void browser_window_recalculate_iframes(struct browser_window *bw) {
+void browser_window_recalculate_iframes(struct browser_window *bw)
+{
 	struct browser_window *window;
-	struct rect rect;
-	int bw_width, bw_height;
 	int index;
 
-#ifdef nsamiga
-	/* In the Amiga frontend we can switch off IFrames because they
-	 * turn into pop-up hell due to broken frames implementation.
-	 * This stops NetSurf asserting in this specific instance.
-	 */
-	if(bw && bw->window == NULL) return;
-#endif
-
-	assert(bw != NULL);
-	assert(bw->window != NULL);
-
-	/* update window dimensions */
-	gui_window_get_dimensions(bw->window, &bw_width, &bw_height, false);
-	if (!bw->parent) {
-		bw->x0 = 0;
-		bw->y0 = 0;
-		bw->x1 = bw_width;
-		bw->y1 = bw_height;
-	}
-
 	for (index = 0; index < bw->iframe_count; index++) {
 		window = &(bw->iframes[index]);
-		
-		if ((window != NULL) && (window->box != NULL)) {
-			box_bounds(window->box, &rect);
-			gui_window_position_frame(window->window, 
-						  rect.x0, rect.y0,
-						  rect.x1, rect.y1);
-		} else {
-			LOG(("Bad IFrame window=%p, box=%p",window,window->box));
+
+		if (window != NULL) {
+			browser_window_handle_scrollbars(window);
 		}
 	}
 }
@@ -214,12 +326,12 @@ void browser_window_create_frameset(struct browser_window *bw,
 					warn_user("NoMemory", 0);
 			}
 
+			window->cur_sel = bw->cur_sel;
+			window->scale = bw->scale;
+
 			/* linking */
 			window->parent = bw;
 
-			/* gui window */
-			window->window = gui_create_browser_window(window, bw, false);
-
 			if (window->name)
 				LOG(("Created frame '%s'", window->name));
 			else
@@ -228,7 +340,7 @@ void browser_window_create_frameset(struct browser_window *bw,
 	}
 
 	/* 2. Calculate dimensions */
-	gui_window_update_extent(bw->window);
+	browser_window_update_extent(bw);
 	browser_window_recalculate_frameset(bw);
 
 	/* 3. Recurse for grandchildren */
@@ -289,19 +401,20 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 	float relative;
 	int size, extent;
 	int x, y;
+	int new_width, new_height;
 
 	assert(bw);
 
 	/* window dimensions */
 	if (!bw->parent) {
-		gui_window_get_dimensions(bw->window, &bw_width, &bw_height, false);
-		bw->x0 = 0;
-		bw->y0 = 0;
-		bw->x1 = bw_width;
-		bw->y1 = bw_height;
+		browser_window_get_dimensions(bw, &bw_width, &bw_height, true);
+		bw->x = 0;
+		bw->y = 0;
+		bw->width = bw_width;
+		bw->height = bw_height;
 	} else {
-		bw_width = bw->x1 - bw->x0;
-		bw_height = bw->y1 - bw->y0;
+		bw_width = bw->width;
+		bw_height = bw->height;
 	}
 	bw_width++;
 	bw_height++;
@@ -367,8 +480,6 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 					widths[col][row] = bw_width;
 				} else {
 					size = bw_width * widths[col][row] / extent;
-					bw_width -= size;
-					extent -= widths[col][row];
 					widths[col][row] = size;
 				}
 			}
@@ -439,8 +550,6 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 					heights[col][row] = bw_height;
 				} else {
 					size = bw_height * heights[col][row] / extent;
-					bw_height -= size;
-					extent -= heights[col][row];
 					heights[col][row] = size;
 				}
 			}
@@ -457,10 +566,27 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 			y = 0;
 			for (row2 = 0; row2 < row; row2++)
 				y+= heights[col][row2];
-			gui_window_position_frame(window->window, x, y,
-					x + widths[col][row] - 1,
-					y + heights[col][row] - 1);
+
+			window->x = x;
+			window->y = y;
+
+			new_width = widths[col][row] - 1;
+			new_height = heights[col][row] - 1;
+
+			if (window->width != new_width ||
+					window->height != new_height) {
+				/* Change in frame size */
+				browser_window_reformat(window, false,
+						new_width * bw->scale,
+						new_height * bw->scale);
+				window->width = new_width;
+				window->height = new_height;
+
+				browser_window_handle_scrollbars(window);
+			}
+
 			x += widths[col][row];
+
 			if (window->children)
 				browser_window_recalculate_frameset(window);
 		}
@@ -526,24 +652,24 @@ bool browser_window_resolve_frame_dimension(struct browser_window *bw, struct br
 	/* extend/shrink the box to the pointer */
 	if (width) {
 		if (bw->drag_resize_left)
-			bw_dimension = bw->x1 - x;
+			bw_dimension = bw->x + bw->width - x;
 		else
-			bw_dimension = x - bw->x0;
-		bw_pixels = (bw->x1 - bw->x0);
-		sibling_pixels = (sibling->x1 - sibling->x0);
+			bw_dimension = x - bw->x;
+		bw_pixels = bw->width;
+		sibling_pixels = sibling->width;
 		bw_d = &bw->frame_width;
 		sibling_d = &sibling->frame_width;
-		frame_size = bw->parent->x1 - bw->parent->x0;
+		frame_size = bw->parent->width;
 	} else {
 		if (bw->drag_resize_up)
-			bw_dimension = bw->y1 - y;
+			bw_dimension = bw->y + bw->height - y;
 		else
-			bw_dimension = y - bw->y0;
-		bw_pixels = (bw->y1 - bw->y0);
-		sibling_pixels = (sibling->y1 - sibling->y0);
+			bw_dimension = y - bw->y;
+		bw_pixels = bw->height;
+		sibling_pixels = sibling->height;
 		bw_d = &bw->frame_height;
 		sibling_d = &sibling->frame_height;
-		frame_size = bw->parent->y1 - bw->parent->y0;
+		frame_size = bw->parent->height;
 	}
 	sibling_dimension = bw_pixels + sibling_pixels - bw_dimension;
 
@@ -635,21 +761,22 @@ bool browser_window_resize_frames(struct browser_window *bw, browser_mouse_state
 	bool left, right, up, down;
 	int i, resize_margin;
 
-	if ((x < bw->x0) || (x > bw->x1) || (y < bw->y0) || (y > bw->y1))
+	if ((x < bw->x) || (x > bw->x + bw->width) ||
+			(y < bw->y) || (y > bw->y + bw->height))
 		return false;
 
 	parent = bw->parent;
 	if ((!bw->no_resize) && parent) {
 		resize_margin = FRAME_RESIZE;
-		if (resize_margin * 2 > (bw->x1 - bw->x0))
-			resize_margin = (bw->x1 - bw->x0) / 2;
-		left = (x < bw->x0 + resize_margin);
-		right = (x > bw->x1 - resize_margin);
+		if (resize_margin * 2 > bw->width)
+			resize_margin = bw->width / 2;
+		left = (x < bw->x + resize_margin);
+		right = (x > bw->x + bw->width - resize_margin);
 		resize_margin = FRAME_RESIZE;
-		if (resize_margin * 2 > (bw->y1 - bw->y0))
-			resize_margin = (bw->y1 - bw->y0) / 2;
-		up = (y < bw->y0 + resize_margin);
-		down = (y > bw->y1 - resize_margin);
+		if (resize_margin * 2 > bw->height)
+			resize_margin = bw->height / 2;
+		up = (y < bw->y + resize_margin);
+		down = (y > bw->y + bw-> height - resize_margin);
 
 		/* check if the edges can actually be moved */
 		if (left || right || up || down) {
@@ -716,14 +843,22 @@ bool browser_window_resize_frames(struct browser_window *bw, browser_mouse_state
 				*pointer = GUI_POINTER_DOWN;
 			}
 			if (mouse & (BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_DRAG_2)) {
-				bw->drag_type = DRAGGING_FRAME;
+				browser_window_set_drag_type(bw, DRAGGING_FRAME);
 				bw->drag_start_x = x;
 				bw->drag_start_y = y;
 				bw->drag_resize_left = left;
 				bw->drag_resize_right = right;
 				bw->drag_resize_up = up;
 				bw->drag_resize_down = down;
-				gui_window_frame_resize_start(bw->window);
+
+				/* TODO: Tell the front end the valid pointer
+				 *       movement range for the drag, so that
+				 *       they can clamp pointer.
+				 *
+				 *       Probably need a general function for
+				 *       this, to be used by all core-managed
+				 *       drag ops.
+				 */
 
 				*status = messages_get("FrameDrag");
 				*action = true;
