@@ -42,6 +42,8 @@
 #include "utils/utils.h"
 #include "utils/url.h"
 
+#undef TREE_NOISY_DEBUG
+
 #define MAXIMUM_URL_LENGTH 1024
 
 #define TREE_TEXT_SIZE_PT 11
@@ -271,6 +273,8 @@ static void tree_recalculate_node_element(struct tree *tree,
 {
 	struct bitmap *bitmap = NULL;
 	int width, height;
+	static char *cache_text = NULL;
+	static int cache_size = 0;
 
 	assert(element != NULL);
 
@@ -284,10 +288,21 @@ static void tree_recalculate_node_element(struct tree *tree,
 			textarea_get_dimensions(tree->textarea,
 						&element->box.width, NULL);
 		} else {
-			nsfont.font_width(&plot_fstyle,
-					  element->text,
-					  strlen(element->text),
-					  &element->box.width);
+			if ((cache_text != NULL) &&
+				(strcmp(cache_text, element->text) == 0)) {
+				element->box.width = cache_size;
+				#ifdef TREE_NOISY_DEBUG
+					LOG(("Tree font width cache hit"));
+				#endif
+			} else {
+				if(cache_text != NULL) free(cache_text);
+				nsfont.font_width(&plot_fstyle,
+						  element->text,
+						  strlen(element->text),
+						  &cache_size);
+				element->box.width = cache_size;
+				cache_text = strdup(element->text);
+			}
 		}
 
 		element->box.width += 8;
@@ -462,19 +477,22 @@ static void tree_recalculate_node_sizes(struct tree *tree, struct node *node,
 		bool recalculate_sizes)
 {
 	struct node_element *element;
-	int width, height;
+	int height;
 
 	assert(node != NULL);
 
-	width = node->box.width;
 	height = node->box.height;
 	node->box.width = 0;
 	node->box.height = 0;
 	if (node->expanded) {
 		for (element = &node->data; element != NULL;
 		     element = element->next) {
-			if (recalculate_sizes)
+			if (recalculate_sizes) {
+				#ifdef TREE_NOISY_DEBUG
+					if(element->text) LOG(("%s", element->text));
+				#endif
 				tree_recalculate_node_element(tree, element);
+			}
 			node->box.width = (node->box.width > element->box.x +
 					   element->box.width - node->box.x) ?
 				node->box.width :
@@ -485,10 +503,13 @@ static void tree_recalculate_node_sizes(struct tree *tree, struct node *node,
 	} else {
 		if (recalculate_sizes)
 			for (element = &node->data; element != NULL;
-			     element = element->next)
+			     element = element->next) {
+				#ifdef TREE_NOISY_DEBUG
+					if(element->text) LOG(("%s", element->text));
+				#endif
 				tree_recalculate_node_element(tree, element);
-		else
-			tree_recalculate_node_element(tree, &node->data);
+			}
+
 		node->box.width = node->data.box.width;
 		node->box.height = node->data.box.height;
 	}
@@ -815,7 +836,7 @@ void tree_link_node(struct tree *tree, struct node *link, struct node *node,
  * \param element  the node element to update
  */
 static void tree_handle_node_element_changed(struct tree *tree,
-		struct node_element *element)
+		struct node_element *element, bool text_changed)
 {
 	int width, height;
 
@@ -823,7 +844,13 @@ static void tree_handle_node_element_changed(struct tree *tree,
 
 	width = element->box.width;
 	height = element->box.height;
-	tree_recalculate_node_element(tree, element);
+
+	if(text_changed == true) {
+		#ifdef TREE_NOISY_DEBUG
+			if(element->text) LOG(("%s", element->text));
+		#endif
+		tree_recalculate_node_element(tree, element);
+	}
 
 	if (element->box.height != height) {
 		tree_recalculate_node_sizes(tree, element->parent, false);
@@ -924,7 +951,7 @@ static void tree_stop_edit(struct tree *tree, bool keep_changes)
 	if (text != NULL)
 		tree_update_node_element(tree, element, text, NULL);
 	else
-		tree_handle_node_element_changed(tree, element);
+		tree_handle_node_element_changed(tree, element, true);
 
 
 	tree_recalculate_size(tree);
@@ -1242,11 +1269,12 @@ void tree_set_node_selected(struct tree *tree, struct node *node, bool all,
 		if (node->selected != selected) {
 			node->selected = selected;
 			if (tree != NULL && tree->redraw)
-				tree->callbacks->redraw_request(node->box.x,
-								node->box.y,
-								node->box.width,
-								node->data.box.height,
-								tree->client_data);
+				tree->callbacks->redraw_request(
+						node->data.box.x,
+						node->data.box.y,
+						node->data.box.width,
+						node->data.box.height,
+						tree->client_data);
 		}
 		if (all && (node->child != NULL) && (node->expanded))
 			tree_set_node_selected(tree, node->child, all,
@@ -1411,6 +1439,7 @@ void tree_update_node_element(struct tree *tree, struct node_element *element,
 {
 	node_callback_resp response;
 	struct node_msg_data msg_data;
+	bool text_changed = false;
 
 	assert(element != NULL);
 
@@ -1420,6 +1449,8 @@ void tree_update_node_element(struct tree *tree, struct node_element *element,
 	if (text != NULL && (element->type == NODE_ELEMENT_TEXT ||
 			     element->type == NODE_ELEMENT_TEXT_PLUS_ICON)) {
 		if (element->text != NULL) {
+			if(strcmp(element->text, text) == 0) text_changed = true;
+
 			response = NODE_CALLBACK_NOT_HANDLED;
 			if (!element->editable &&
 			    element->parent->user_callback !=
@@ -1454,10 +1485,15 @@ void tree_update_node_element(struct tree *tree, struct node_element *element,
 			if (response != NODE_CALLBACK_HANDLED)
 				free(element->bitmap);
 		}
+		else {
+			/* Increase the box width to accomodate the new icon */
+			element->box.width += NODE_INSTEP;
+		}
+
 		element->bitmap = bitmap;
 	}
 
-	tree_handle_node_element_changed(tree, element);
+	tree_handle_node_element_changed(tree, element, text_changed);
 }
 
 
@@ -1535,14 +1571,17 @@ struct node *tree_node_get_next(struct node *node)
 /**
  * Draws an element's expansion icon
  *
- * \param tree	   the tree to draw the expansion for
- * \param element  the element to draw the expansion for
+ * \param tree		the tree to draw the expansion for
+ * \param element	the element to draw the expansion for
  * \param tree_x	X coordinate of the tree
  * \param tree_y	Y coordinate of the tree
+ * \param ctx		current redraw context
  */
 static void tree_draw_node_expansion_toggle(struct tree *tree,
-		struct node *node, int tree_x, int tree_y)
+		struct node *node, int tree_x, int tree_y,
+		const struct redraw_context *ctx)
 {
+	const struct plotter_table *plot = ctx->plot;
 	int x, y;
 
 	assert(tree != NULL);
@@ -1551,15 +1590,15 @@ static void tree_draw_node_expansion_toggle(struct tree *tree,
 	if ((node->child != NULL) || (node->data.next != NULL)) {
 		x = tree_x + node->box.x - (NODE_INSTEP / 2) - 4;
 		y = tree_y + node->box.y + (TREE_LINE_HEIGHT - 9) / 2;
-		plot.rectangle(x, y, x + 9, y + 9,
+		plot->rectangle(x, y, x + 9, y + 9,
 				&plot_style_fill_tree_furniture);
-		plot.rectangle(x , y, x + 8, y + 8,
-			       &plot_style_stroke_tree_furniture);
-		plot.line(x + 2, y + 4, x + 7, y + 4,
-			  &plot_style_stroke_tree_furniture);
+		plot->rectangle(x , y, x + 8, y + 8,
+				&plot_style_stroke_tree_furniture);
+		plot->line(x + 2, y + 4, x + 7, y + 4,
+				&plot_style_stroke_tree_furniture);
 		if (!node->expanded)
-			plot.line(x + 4, y + 2, x + 4, y + 7,
-				  &plot_style_stroke_tree_furniture);
+			plot->line(x + 4, y + 2, x + 4, y + 7,
+					&plot_style_stroke_tree_furniture);
 
 	}
 
@@ -1574,12 +1613,13 @@ static void tree_draw_node_expansion_toggle(struct tree *tree,
  * \param tree_x	X coordinate to draw the tree at (wrt plot origin)
  * \param tree_y	Y coordinate to draw the tree at (wrt plot origin)
  * \param clip		clipping rectangle (wrt plot origin)
+ * \param ctx		current redraw context
  */
 static void tree_draw_node_element(struct tree *tree,
 		struct node_element *element, int tree_x, int tree_y,
-		const struct rect *clip)
+		const struct rect *clip, const struct redraw_context *ctx)
 {
-
+	const struct plotter_table *plot = ctx->plot;
 	struct bitmap *bitmap = NULL;
 	int x, y, width;
 	bool selected = false;
@@ -1620,13 +1660,24 @@ static void tree_draw_node_element(struct tree *tree,
 
 			if (c.x1 > c.x0 && c.y1 > c.y0) {
 				/* Valid clip rectangles only */
-				plot.clip(&c);
-				content_redraw(icon , x, y + icon_inset,
-						TREE_ICON_SIZE, TREE_ICON_SIZE,
-						&c, 1, 0);
+				struct content_redraw_data data;
+
+				plot->clip(&c);
+
+				data.x = x;
+				data.y = y + icon_inset;
+				data.width = TREE_ICON_SIZE;
+				data.height = TREE_ICON_SIZE;
+
+				data.background_colour = 0xFFFFFF;
+				data.scale = 1;
+				data.repeat_x = false;
+				data.repeat_y = false;
+
+				content_redraw(icon, &data, &c, ctx);
 
 				/* Restore previous clipping area */
-				plot.clip(clip);
+				plot->clip(clip);
 			}
 		}
 
@@ -1643,14 +1694,14 @@ static void tree_draw_node_element(struct tree *tree,
 
 		if (selected) {
 			fstyle = &plot_fstyle_selected;
-			plot.rectangle(x, y, x + width,
+			plot->rectangle(x, y, x + width,
 				       y + element->box.height,
 				       &plot_style_fill_tree_selected);
 		} else {
 			fstyle = &plot_fstyle;
 		}
 
-		plot.text(x + 4, y + (TREE_LINE_HEIGHT * 3 + 2) / 4,
+		plot->text(x + 4, y + (TREE_LINE_HEIGHT * 3 + 2) / 4,
 				element->text, strlen(element->text),
 				fstyle);
 		break;
@@ -1658,11 +1709,11 @@ static void tree_draw_node_element(struct tree *tree,
 		bitmap = element->bitmap;
 		if (bitmap == NULL)
 			break;
-		plot.bitmap(x, y, element->box.width - 1,
+		plot->bitmap(x, y, element->box.width - 1,
 			    element->box.height - 2,
 			    bitmap, 0xFFFFFF, BITMAPF_NONE);
 		if (!(tree->flags & TREE_NO_FURNITURE))
-			plot.rectangle(x, y, x + element->box.width - 1,
+			plot->rectangle(x, y, x + element->box.width - 1,
 				       y + element->box.height - 3,
 				       &plot_style_stroke_tree_furniture);
 
@@ -1680,10 +1731,13 @@ static void tree_draw_node_element(struct tree *tree,
  * \param tree_x	X coordinate to draw the tree at (wrt plot origin)
  * \param tree_y	Y coordinate to draw the tree at (wrt plot origin)
  * \param clip		clipping rectangle (wrt plot origin)
+ * \param ctx		current redraw context
  */
 static void tree_draw_node(struct tree *tree, struct node *node,
-		int tree_x, int tree_y, struct rect clip)
+		int tree_x, int tree_y, struct rect clip,
+		const struct redraw_context *ctx)
 {
+	const struct plotter_table *plot = ctx->plot;
 	struct node_element *element;
 	struct node *parent;
 	int x0, y0, x1, y1;
@@ -1720,7 +1774,7 @@ static void tree_draw_node(struct tree *tree, struct node *node,
 	}
 
 	/* Set up the clipping area */
-	plot.clip(&clip);
+	plot->clip(&clip);
 
 	/* Draw node's furniture */
 	if (!(tree->flags & TREE_NO_FURNITURE)) {
@@ -1731,7 +1785,7 @@ static void tree_draw_node(struct tree *tree, struct node *node,
 			x0 = x1 = tree_x + node->box.x - (NODE_INSTEP / 2);
 			y0 = tree_y + node->previous->box.y;
 			y1 = tree_y + node->box.y + (TREE_LINE_HEIGHT / 2);
-			plot.line(x0, y0, x1, y1,
+			plot->line(x0, y0, x1, y1,
 					&plot_style_stroke_tree_furniture);
 		}
 		if (node->next != NULL) {
@@ -1740,7 +1794,7 @@ static void tree_draw_node(struct tree *tree, struct node *node,
 			x0 = x1 = tree_x + node->box.x - (NODE_INSTEP / 2);
 			y0 = tree_y + node->box.y + (TREE_LINE_HEIGHT / 2);
 			y1 = tree_y + node->next->box.y;
-			plot.line(x0, y0, x1, y1,
+			plot->line(x0, y0, x1, y1,
 					&plot_style_stroke_tree_furniture);
 		}
 
@@ -1752,7 +1806,7 @@ static void tree_draw_node(struct tree *tree, struct node *node,
 			y0 = tree_y + parent->data.box.y +
 					parent->data.box.height;
 			y1 = y0 + (TREE_LINE_HEIGHT / 2);
-			plot.line(x0, y0, x1, y1,
+			plot->line(x0, y0, x1, y1,
 					&plot_style_stroke_tree_furniture);
 		}
 		/* Line from expansion toggle to icon */
@@ -1760,9 +1814,10 @@ static void tree_draw_node(struct tree *tree, struct node *node,
 		x1 = x0 + (NODE_INSTEP / 2) - 2;
 		y0 = y1 = tree_y + node->data.box.y + node->data.box.height -
 				(TREE_LINE_HEIGHT / 2);
-		plot.line(x0, y0, x1, y1, &plot_style_stroke_tree_furniture);
+		plot->line(x0, y0, x1, y1, &plot_style_stroke_tree_furniture);
 
-		tree_draw_node_expansion_toggle(tree, node, tree_x, tree_y);
+		tree_draw_node_expansion_toggle(tree, node,
+				tree_x, tree_y, ctx);
 	}
 
 	/* Draw node's element(s)
@@ -1772,12 +1827,12 @@ static void tree_draw_node(struct tree *tree, struct node *node,
 				element = element->next) {
 			/* Draw each element of expanded node */
 			tree_draw_node_element(tree, element, tree_x, tree_y,
-					&clip);
+					&clip, ctx);
 		}
 	} else {
 		/* Draw main title element of node */
 		tree_draw_node_element(tree, &node->data, tree_x, tree_y,
-				&clip);
+				&clip, ctx);
 	}
 }
 
@@ -1790,9 +1845,11 @@ static void tree_draw_node(struct tree *tree, struct node *node,
  * \param tree_x	X coordinate to draw the tree at (wrt plot origin)
  * \param tree_y	Y coordinate to draw the tree at (wrt plot origin)
  * \param clip		clipping rectangle (wrt plot origin)
+ * \param ctx		current redraw context
  */
 static void tree_draw_tree(struct tree *tree, struct node *node,
-		int tree_x, int tree_y, struct rect clip)
+		int tree_x, int tree_y, struct rect clip,
+		const struct redraw_context *ctx)
 {
 	struct node *child;
 
@@ -1812,11 +1869,11 @@ static void tree_draw_tree(struct tree *tree, struct node *node,
 			return;
 
 		/* Draw current child */
-		tree_draw_node(tree, child, tree_x, tree_y, clip);
+		tree_draw_node(tree, child, tree_x, tree_y, clip, ctx);
 		/* And its children */
 		if ((child->child != NULL) && (child->expanded)) {
 			/* Child has children and they are visible */
-			tree_draw_tree(tree, child, tree_x, tree_y, clip);
+			tree_draw_tree(tree, child, tree_x, tree_y, clip, ctx);
 		}
 	}
 }
@@ -1832,35 +1889,38 @@ static void tree_draw_tree(struct tree *tree, struct node *node,
  * \param clip_y	minimum y of the clipping rectangle (wrt tree origin)
  * \param clip_width	width of the clipping rectangle
  * \param clip_height	height of the clipping rectangle
+ * \param ctx		current redraw context
  */
 void tree_draw(struct tree *tree, int x, int y,
-	       int clip_x, int clip_y, int clip_width, int clip_height)
+		int clip_x, int clip_y, int clip_width, int clip_height,
+		const struct redraw_context *ctx)
 {
+	struct redraw_context new_ctx = *ctx;
 	struct rect clip;
 
 	assert(tree != NULL);
 	assert(tree->root != NULL);
 
 	/* Start knockout rendering if it's available for this plotter */
-	if (plot.option_knockout)
-		knockout_plot_start(&plot);
+	if (ctx->plot->option_knockout)
+		knockout_plot_start(ctx, &new_ctx);
 
 	/* Set up clip rectangle */
 	clip.x0 = x + clip_x;
 	clip.y0 = y + clip_y;
 	clip.x1 = clip.x0 + clip_width;
 	clip.y1 = clip.y0 + clip_height;
-	plot.clip(&clip);
+	new_ctx.plot->clip(&clip);
 
 	/* Flat fill extents of clipping area */
-	plot.rectangle(clip.x0, clip.y0, clip.x1, clip.y1,
+	new_ctx.plot->rectangle(clip.x0, clip.y0, clip.x1, clip.y1,
 		       &plot_style_fill_tree_background);
 
 	/* don't draw empty trees or trees with redraw flag set to false */
 	if (tree->root->child != NULL && tree->redraw) {
 
 		/* Draw the tree */
-		tree_draw_tree(tree, tree->root, x, y, clip);
+		tree_draw_tree(tree, tree->root, x, y, clip, &new_ctx);
 
 		/* Draw textarea, if present */
 		if (tree->editing != NULL) {
@@ -1868,12 +1928,12 @@ void tree_draw(struct tree *tree, int x, int y,
 			y = y + tree->editing->box.y;
 			if (tree->editing->type == NODE_ELEMENT_TEXT_PLUS_ICON)
 				x += NODE_INSTEP;
-			textarea_redraw(tree->textarea, x, y, &clip);
+			textarea_redraw(tree->textarea, x, y, &clip, &new_ctx);
 		}
 	}
 
 	/* Rendering complete */
-	if (plot.option_knockout)
+	if (ctx->plot->option_knockout)
 		knockout_plot_end();
 }
 
@@ -2093,20 +2153,29 @@ struct node *tree_get_link_details(struct tree *tree, int x, int y,
  *
  * \param tree	the tree for which all nodes will be launched
  * \param node	the node which will be checked together with its children
+ * \param tabs	launch node in a new tab instead of a new window
  */
-static void tree_launch_selected_internal(struct tree *tree, struct node *node)
+static void tree_launch_selected_internal(struct tree *tree, struct node *node,
+		bool tabs)
 {
 	struct node_msg_data msg_data;
 
+	msg_data.data.bw = NULL;
+
 	for (; node != NULL; node = node->next) {
 		if (node->selected && node->user_callback != NULL) {
 			msg_data.msg = NODE_LAUNCH;
-			msg_data.flag = TREE_ELEMENT_TITLE;
+			if (tabs == true) {
+				msg_data.flag = TREE_ELEMENT_LAUNCH_IN_TABS;
+			} else {
+				msg_data.flag = TREE_ELEMENT_TITLE;
+			}
+
 			msg_data.node = node;
 			node->user_callback(node->callback_data, &msg_data);
 		}
 		if (node->child != NULL)
-			tree_launch_selected_internal(tree, node->child);
+			tree_launch_selected_internal(tree, node->child, tabs);
 	}
 }
 
@@ -2115,11 +2184,12 @@ static void tree_launch_selected_internal(struct tree *tree, struct node *node)
  * Launches all the selected nodes of the tree
  *
  * \param tree	the tree for which all nodes will be launched
+ * \param tabs	launch nodes in new tabs instead of new windows
  */
-void tree_launch_selected(struct tree *tree)
+void tree_launch_selected(struct tree *tree, bool tabs)
 {
 	if (tree->root->child != NULL)
-		tree_launch_selected_internal(tree, tree->root->child);
+		tree_launch_selected_internal(tree, tree->root->child, tabs);
 }
 
 
@@ -2307,7 +2377,7 @@ bool tree_mouse_action(struct tree *tree, browser_mouse_state mouse, int x,
 			tree_set_node_selected(tree, tree->root->child, true,
 					       false);
 			node->selected = true;
-			tree_handle_node_element_changed(tree, &node->data);
+			tree_handle_node_element_changed(tree, &node->data, false);
 		}
 		return true;
 	}
@@ -2317,7 +2387,7 @@ bool tree_mouse_action(struct tree *tree, browser_mouse_state mouse, int x,
 		if (tree->flags & TREE_NO_SELECT)
 			return true;
 		node->selected = !node->selected;
-		tree_handle_node_element_changed(tree, &node->data);
+		tree_handle_node_element_changed(tree, &node->data, false);
 		return true;
 	}
 
@@ -2331,7 +2401,7 @@ bool tree_mouse_action(struct tree *tree, browser_mouse_state mouse, int x,
 			tree_set_node_selected(tree, tree->root->child, true,
 					       false);
 			node->selected = true;
-			tree_handle_node_element_changed(tree, &node->data);
+			tree_handle_node_element_changed(tree, &node->data, false);
 		}
 
 		if (tree->flags & TREE_MOVABLE)
@@ -2395,11 +2465,11 @@ static void tree_handle_selection_area_node(struct tree *tree,
 				if (invert) {
 					node->selected = !node->selected;
 					tree_handle_node_element_changed(tree,
-							&node->data);
+							&node->data, false);
 				} else if (!node->selected) {
 					node->selected = true;
 					tree_handle_node_element_changed(tree,
-							&node->data);
+							&node->data, false);
 				}
 			}
 		}
@@ -2704,7 +2774,7 @@ void tree_start_edit(struct tree *tree, struct node_element *element)
 	}
 	textarea_set_text(tree->textarea, element->text);
 
-	tree_handle_node_element_changed(tree, element);
+	tree_handle_node_element_changed(tree, element, true);
 	tree_recalculate_size(tree);
 	tree->callbacks->scroll_visible(element->box.y, element->box.height,
 					tree->client_data);
@@ -2771,7 +2841,8 @@ hlcache_handle *tree_load_icon(const char *name)
 
 	/* Fetch the icon */
 	err = hlcache_handle_retrieve(icon_url, 0, 0, 0,
-				      tree_icon_callback, 0, 0, 0, &c);
+				      tree_icon_callback, 0, 0, 
+				      CONTENT_IMAGE, &c);
 
 
 	/* If we built the URL here, free it */
