@@ -36,11 +36,11 @@
 #include "desktop/plotters.h"
 #include "desktop/netsurf.h"
 #include "desktop/options.h"
-#include "desktop/shape.h"
 #include "utils/filepath.h"
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/schedule.h"
+#include "utils/types.h"
 #include "utils/url.h"
 #include "utils/utils.h"
 #include "desktop/textinput.h"
@@ -307,6 +307,10 @@ fb_redraw(fbtk_widget_t *widget,
 	int x;
 	int y;
 	struct rect clip;
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &fb_plotters
+	};
 
 	LOG(("%d,%d to %d,%d",
 	     bwidget->redraw_box.x0,
@@ -326,8 +330,6 @@ fb_redraw(fbtk_widget_t *widget,
 	nsfb_claim(fbtk_get_nsfb(widget), &bwidget->redraw_box);
 
 	/* redraw bounding box is relative to window */
-	current_redraw_browser = bw;
-
 	clip.x0 = bwidget->redraw_box.x0;
 	clip.y0 = bwidget->redraw_box.y0;
 	clip.x1 = bwidget->redraw_box.x1;
@@ -336,9 +338,7 @@ fb_redraw(fbtk_widget_t *widget,
 	browser_window_redraw(bw,
 			(x - bwidget->scrollx) / bw->scale,
 			(y - bwidget->scrolly) / bw->scale,
-			&clip);
-
-	current_redraw_browser = NULL;
+			&clip, &ctx);
 
 	nsfb_update(fbtk_get_nsfb(widget), &bwidget->redraw_box);
 
@@ -457,12 +457,12 @@ gui_init(int argc, char** argv)
 	LOG(("Using '%s' as AdBlock CSS URL", adblock_stylesheet_url));
 
 	if (option_cookie_file == NULL) {
-		option_cookie_file = filepath_find(respaths, "Cookies");
+		option_cookie_file = strdup("~/.netsurf/Cookies");
 		LOG(("Using '%s' as Cookies file", option_cookie_file));
 	}
 
 	if (option_cookie_jar == NULL) {
-		option_cookie_jar = filepath_find(respaths, "Cookies");
+		option_cookie_jar = strdup("~/.netsurf/Cookies");
 		LOG(("Using '%s' as Cookie Jar file", option_cookie_jar));
 	}
 
@@ -1266,29 +1266,11 @@ gui_create_browser_window(struct browser_window *bw,
 	 */
 	gw->bw = bw;
 
+	create_normal_browser_window(gw, option_fb_furniture_size);
+	gw->localhistory = fb_create_localhistory(bw, fbtk, option_fb_furniture_size);
 
-	switch(bw->browser_window_type) {
-	case BROWSER_WINDOW_NORMAL:
-		create_normal_browser_window(gw, option_fb_furniture_size);
-		gw->localhistory = fb_create_localhistory(bw, fbtk, option_fb_furniture_size);
-
-		/* map and request redraw of gui window */
-		fbtk_set_mapping(gw->window, true);
-
-		break;
-
-	case BROWSER_WINDOW_FRAME:
-		gw->window = fbtk_create_window(bw->parent->window->window, 0, 0, 0, 0, 0);
-		create_browser_widget(gw, 0, 0);
-		LOG(("create frame"));
-		break;
-
-	default:
-		gw->window = fbtk_create_window(bw->parent->window->window, 0, 0, 0, 0, 0);
-		create_browser_widget(gw, 0, 0);
-		LOG(("unhandled type"));
-
-	}
+	/* map and request redraw of gui window */
+	fbtk_set_mapping(gw->window, true);
 
 	return gw;
 }
@@ -1316,16 +1298,14 @@ gui_window_redraw_window(struct gui_window *g)
 }
 
 void
-gui_window_update_box(struct gui_window *g, const union content_msg_data *data)
+gui_window_update_box(struct gui_window *g, const struct rect *rect)
 {
 	struct browser_widget_s *bwidget = fbtk_get_userpw(g->browser);
 	fb_queue_redraw(g->browser,
-			data->redraw.x - bwidget->scrollx,
-			data->redraw.y - bwidget->scrolly,
-			data->redraw.x - bwidget->scrollx +
-			data->redraw.width,
-			data->redraw.y - bwidget->scrolly +
-			data->redraw.height);
+			rect->x0 - bwidget->scrollx,
+			rect->y0 - bwidget->scrolly,
+			rect->x1 - bwidget->scrollx,
+			rect->y1 - bwidget->scrolly);
 }
 
 bool
@@ -1357,34 +1337,6 @@ gui_window_scroll_visible(struct gui_window *g, int x0, int y0,
 	LOG(("%s:(%p, %d, %d, %d, %d)", __func__, g, x0, y0, x1, y1));
 }
 
-void
-gui_window_position_frame(struct gui_window *g, int x0, int y0, int x1, int y1)
-{
-	struct gui_window *parent;
-	int px, py;
-	int w, h;
-	LOG(("%s: %d, %d, %d, %d", g->bw->name, x0, y0, x1, y1));
-	parent = g->bw->parent->window;
-
-	if (parent->window == NULL)
-		return; /* doesnt have an fbtk widget */
-
-	px = fbtk_get_absx(parent->browser) + x0;
-	py = fbtk_get_absy(parent->browser) + y0;
-	w = x1 - x0;
-	h = y1 - y0;
-	if (w > (fbtk_get_width(parent->browser) - px))
-		w = fbtk_get_width(parent->browser) - px;
-
-	if (h > (fbtk_get_height(parent->browser) - py))
-		h = fbtk_get_height(parent->browser) - py;
-
-	fbtk_set_pos_and_size(g->window, px, py , w , h);
-
-	fbtk_request_redraw(parent->browser);
-
-}
-
 void
 gui_window_get_dimensions(struct gui_window *g,
 			  int *width,
@@ -1559,24 +1511,11 @@ gui_window_box_scroll_start(struct gui_window *g,
 	return true;
 }
 
-bool
-gui_window_frame_resize_start(struct gui_window *g)
-{
-	LOG(("resize frame\n"));
-	return true;
-}
-
 void
 gui_window_save_link(struct gui_window *g, const char *url, const char *title)
 {
 }
 
-void
-gui_window_set_scale(struct gui_window *g, float scale)
-{
-	LOG(("set scale\n"));
-}
-
 /**
  * set favicon
  */
