@@ -30,7 +30,7 @@
 */
 
 bool save_complete_gui_save(const char *path, const char *filename, size_t len,
-		const char *sourcedata, content_type type)
+		const char *sourcedata, lwc_string *mime_type)
 {
 	return false;
 }
