@@ -169,8 +169,10 @@ static struct gtk_scaffolding *current_model;
 /** global list for interface changes */
 nsgtk_scaffolding *scaf_list = NULL;
 
-/** The box containing a link under the mouse, or 0 if none */
-static struct box *current_menu_link_box;
+/** holds the context data for what's under the pointer, when the contextual
+ *  menu is opened. */
+static struct contextual_content current_menu_ctx = { NULL, NULL, NULL };
+
 
 /**
  * Helper to hide popup menu entries by grouping
@@ -322,7 +324,7 @@ void nsgtk_scaffolding_destroy(nsgtk_scaffolding *g)
 static void nsgtk_window_update_back_forward(struct gtk_scaffolding *g)
 {
 	int width, height;
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	g->buttons[BACK_BUTTON]->sensitivity =
 			history_back_available(bw->history);
@@ -380,8 +382,8 @@ static guint nsgtk_scaffolding_update_edit_actions_sensitivity(
 		g->buttons[PASTE_BUTTON]->sensitivity = true;
 	} else {
 		struct browser_window *bw =
-				gui_window_get_browser_window(g->top_level);
-		has_selection = bw->sel->defined;
+				nsgtk_get_browser_window(g->top_level);
+		has_selection = browser_window_has_selection(bw);
 
 		g->buttons[COPY_BUTTON]->sensitivity = has_selection;
 		g->buttons[CUT_BUTTON]->sensitivity = (has_selection &&
@@ -448,7 +450,7 @@ static gboolean nsgtk_window_popup_menu_hidden(GtkWidget *widget,
 gboolean nsgtk_window_url_activate_event(GtkWidget *widget, gpointer data)
 {
 	struct gtk_scaffolding *g = data;
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	char *url;
 	if (search_is_url(gtk_entry_get_text(GTK_ENTRY(g->url_bar)))
 			== false)
@@ -512,7 +514,7 @@ static void nsgtk_window_tabs_num_changed(GtkNotebook *notebook,
  */
 static void nsgtk_openfile_open(const char *filename)
 {
-	struct browser_window *bw = gui_window_get_browser_window(
+	struct browser_window *bw = nsgtk_get_browser_window(
 			current_model->top_level);
 	char url[strlen(filename) + FILE_SCHEME_PREFIX_LEN + 1];
 
@@ -526,8 +528,14 @@ static void nsgtk_openfile_open(const char *filename)
 
 MULTIHANDLER(newwindow)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
-	const char *url = gtk_entry_get_text(GTK_ENTRY(g->url_bar));
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
+	const char *url = option_homepage_url;
+
+	if ((url != NULL) && (url[0] == '\0'))
+		url = NULL;
+
+	if (url == NULL)
+		url = NETSURF_HOMEPAGE;
 
 	browser_window_create(url, bw, NULL, false, false);
 
@@ -536,7 +544,7 @@ MULTIHANDLER(newwindow)
 
 MULTIHANDLER(newtab)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	if (option_new_blank) {
 		browser_window_create(NULL, bw, NULL, false, true);
@@ -591,7 +599,7 @@ static gboolean nsgtk_filter_directory(const GtkFileFilterInfo *info,
 
 MULTIHANDLER(savepage)
 {
-	if (gui_window_get_browser_window(g->top_level)->current_content
+	if (nsgtk_get_browser_window(g->top_level)->current_content
 			== NULL)
 		return FALSE;
 
@@ -611,7 +619,7 @@ MULTIHANDLER(savepage)
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fc), filter);
 	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(fc), filter);
 
-	res = url_nice(content_get_url(gui_window_get_browser_window(
+	res = url_nice(content_get_url(nsgtk_get_browser_window(
 			g->top_level)->current_content), &path, false);
 	if (res != URL_FUNC_OK) {
 		path = strdup(messages_get("SaveText"));
@@ -648,7 +656,7 @@ MULTIHANDLER(savepage)
 	}
 	closedir(d);
 	save_complete_init();
-	save_complete(gui_window_get_browser_window(
+	save_complete(nsgtk_get_browser_window(
 			g->top_level)->current_content, path);
 	g_free(path);
 
@@ -663,7 +671,7 @@ MULTIHANDLER(pdf)
 #ifdef WITH_PDF_EXPORT
 
 	GtkWidget *save_dialog;
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	struct print_settings *settings;
 	char filename[PATH_MAX];
 	char dirname[PATH_MAX];
@@ -733,7 +741,7 @@ MULTIHANDLER(pdf)
 
 MULTIHANDLER(plaintext)
 {
-	if (gui_window_get_browser_window(g->top_level)->current_content
+	if (nsgtk_get_browser_window(g->top_level)->current_content
 			== NULL)
 		return FALSE;
 
@@ -746,7 +754,7 @@ MULTIHANDLER(plaintext)
 	char *filename;
 	url_func_result res;
 
-	res = url_nice(content_get_url(gui_window_get_browser_window(
+	res = url_nice(content_get_url(nsgtk_get_browser_window(
 			g->top_level)->current_content), &filename, false);
 	if (res != URL_FUNC_OK) {
 		filename = strdup(messages_get("SaveText"));
@@ -764,7 +772,7 @@ MULTIHANDLER(plaintext)
 
 	if (gtk_dialog_run(GTK_DIALOG(fc)) == GTK_RESPONSE_ACCEPT) {
 		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fc));
-		save_as_text(gui_window_get_browser_window(
+		save_as_text(nsgtk_get_browser_window(
 				g->top_level)->current_content, filename);
 		g_free(filename);
 	}
@@ -791,7 +799,7 @@ MULTIHANDLER(printpreview)
 
 MULTIHANDLER(print)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	GtkPrintOperation *print_op;
 	GtkPageSetup *page_setup;
@@ -880,13 +888,13 @@ MENUHANDLER(savelink)
 {
 	struct gtk_scaffolding *g = (struct gtk_scaffolding *) data;
 	struct gui_window *gui = g->top_level;
-	struct browser_window *bw = gui_window_get_browser_window(gui);
+	struct browser_window *bw = nsgtk_get_browser_window(gui);
 
-	if (!current_menu_link_box)
+	if (current_menu_ctx.link_url == NULL)
 		return FALSE;
 
-	browser_window_download(bw, current_menu_link_box->href,
-		content_get_url(bw->current_content));
+	browser_window_download(bw, current_menu_ctx.link_url,
+			current_menu_ctx.link_url);
 
 	return TRUE;
 }
@@ -898,12 +906,12 @@ MENUHANDLER(link_openwin)
 {
 	struct gtk_scaffolding *g = (struct gtk_scaffolding *) data;
 	struct gui_window *gui = g->top_level;
-	struct browser_window *bw = gui_window_get_browser_window(gui);
+	struct browser_window *bw = nsgtk_get_browser_window(gui);
 
-	if (current_menu_link_box == NULL)
+	if (current_menu_ctx.link_url == NULL)
 		return FALSE;
 
-	browser_window_create(current_menu_link_box->href, bw, NULL, true, false);
+	browser_window_create(current_menu_ctx.link_url, bw, NULL, true, false);
 
 	return TRUE;
 }
@@ -915,15 +923,13 @@ MENUHANDLER(link_opentab)
 {
 	struct gtk_scaffolding *g = (struct gtk_scaffolding *) data;
 	struct gui_window *gui = g->top_level;
-	struct browser_window *bw = gui_window_get_browser_window(gui);
+	struct browser_window *bw = nsgtk_get_browser_window(gui);
 
-	temp_open_background = 1;
-
-	if (current_menu_link_box == NULL)
+	if (current_menu_ctx.link_url == NULL)
 		return FALSE;
 
-	browser_window_create(current_menu_link_box->href, bw, NULL, true,
-			true);
+	temp_open_background = 1;
+	browser_window_create(current_menu_ctx.link_url, bw, NULL, true, true);
 	temp_open_background = -1;
 
 	return TRUE;
@@ -932,7 +938,7 @@ MENUHANDLER(link_opentab)
 
 MULTIHANDLER(cut)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	GtkWidget *focused = gtk_window_get_focus(g->window);
 
 	/* If the url bar has focus, let gtk handle it */
@@ -946,14 +952,14 @@ MULTIHANDLER(cut)
 
 MULTIHANDLER(copy)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	GtkWidget *focused = gtk_window_get_focus(g->window);
 
 	/* If the url bar has focus, let gtk handle it */
 	if (GTK_IS_EDITABLE (focused))
 		gtk_editable_copy_clipboard(GTK_EDITABLE(g->url_bar));
 	else
-		gui_copy_to_clipboard(bw->sel);
+		gui_copy_to_clipboard(browser_window_get_selection(bw));
 
 	return TRUE;
 }
@@ -986,14 +992,14 @@ MENUHANDLER(customize)
 
 MULTIHANDLER(selectall)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	if (nsgtk_widget_has_focus(GTK_WIDGET(g->url_bar))) {
 		LOG(("Selecting all URL bar text"));
 		gtk_editable_select_region(GTK_EDITABLE(g->url_bar), 0, -1);
 	} else {
 		LOG(("Selecting all document text"));
-		selection_select_all(bw->sel);
+		selection_select_all(browser_window_get_selection(bw));
 	}
 
 	return TRUE;
@@ -1007,7 +1013,7 @@ MULTIHANDLER(find)
 
 MULTIHANDLER(preferences)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	if (g->preferences_dialog == NULL)
 		g->preferences_dialog = nsgtk_options_init(bw, g->window);
 	else
@@ -1018,7 +1024,7 @@ MULTIHANDLER(preferences)
 
 MULTIHANDLER(zoomplus)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	float old_scale = nsgtk_get_scale_for_gui(g->top_level);
 
 	browser_window_set_scale(bw, old_scale + 0.05, true);
@@ -1028,7 +1034,7 @@ MULTIHANDLER(zoomplus)
 
 MULTIHANDLER(zoomnormal)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	browser_window_set_scale(bw, 1.0, true);
 
@@ -1037,7 +1043,7 @@ MULTIHANDLER(zoomnormal)
 
 MULTIHANDLER(zoomminus)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 	float old_scale = nsgtk_get_scale_for_gui(g->top_level);
 
 	browser_window_set_scale(bw, old_scale - 0.05, true);
@@ -1061,7 +1067,7 @@ MULTIHANDLER(fullscreen)
 MULTIHANDLER(viewsource)
 {
 	nsgtk_source_dialog_init(g->window,
-			gui_window_get_browser_window(g->top_level));
+			nsgtk_get_browser_window(g->top_level));
 	return TRUE;
 }
 
@@ -1203,7 +1209,7 @@ MULTIHANDLER(saveboxtree)
 				"Unable to open file for writing.");
 		} else {
 			struct browser_window *bw;
-			bw = gui_window_get_browser_window(g->top_level);
+			bw = nsgtk_get_browser_window(g->top_level);
 
 			if (bw->current_content &&
 					content_get_type(bw->current_content) ==
@@ -1252,7 +1258,7 @@ MULTIHANDLER(savedomtree)
 				"Unable to open file for writing.");
 		} else {
 			struct browser_window *bw;
-			bw = gui_window_get_browser_window(g->top_level);
+			bw = nsgtk_get_browser_window(g->top_level);
 
 			if (bw->current_content &&
 					content_get_type(bw->current_content) ==
@@ -1276,7 +1282,7 @@ MULTIHANDLER(savedomtree)
 MULTIHANDLER(stop)
 {
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 
 	browser_window_stop(bw);
 
@@ -1286,13 +1292,13 @@ MULTIHANDLER(stop)
 MULTIHANDLER(reload)
 {
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 	if (bw == NULL)
 		return TRUE;
 
 	/* clear potential search effects */
-	if (bw->search_context != NULL)
-		search_destroy_context(bw->search_context);
+	browser_window_search_destroy_context(bw);
+
 	nsgtk_search_set_forward_state(true, bw);
 	nsgtk_search_set_back_state(true, bw);
 
@@ -1304,14 +1310,14 @@ MULTIHANDLER(reload)
 MULTIHANDLER(back)
 {
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 
 	if ((bw == NULL) || (!history_back_available(bw->history)))
 		return TRUE;
 
 	/* clear potential search effects */
-	if (bw->search_context != NULL)
-		search_destroy_context(bw->search_context);
+	browser_window_search_destroy_context(bw);
+
 	nsgtk_search_set_forward_state(true, bw);
 	nsgtk_search_set_back_state(true, bw);
 
@@ -1324,14 +1330,14 @@ MULTIHANDLER(back)
 MULTIHANDLER(forward)
 {
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 
 	if ((bw == NULL) || (!history_forward_available(bw->history)))
 		return TRUE;
 
 	/* clear potential search effects */
-	if (bw->search_context != NULL)
-		search_destroy_context(bw->search_context);
+	browser_window_search_destroy_context(bw);
+
 	nsgtk_search_set_forward_state(true, bw);
 	nsgtk_search_set_back_state(true, bw);
 
@@ -1345,7 +1351,7 @@ MULTIHANDLER(home)
 {
 	static const char *addr = NETSURF_HOMEPAGE;
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 
 	if (option_homepage_url != NULL && option_homepage_url[0] != '\0')
 		addr = option_homepage_url;
@@ -1358,7 +1364,7 @@ MULTIHANDLER(home)
 MULTIHANDLER(localhistory)
 {
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 
 	int x,y, width, height, mainwidth, mainheight, margin = 20;
 	/* if entries of the same url but different frag_ids have been added
@@ -1396,7 +1402,7 @@ MULTIHANDLER(globalhistory)
 
 MULTIHANDLER(addbookmarks)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	if (bw == NULL || bw->current_content == NULL ||
 			content_get_url(bw->current_content) == NULL)
@@ -1451,7 +1457,7 @@ MULTIHANDLER(closetab)
 
 MULTIHANDLER(contents)
 {
-	browser_window_go(gui_window_get_browser_window(g->top_level), 
+	browser_window_go(nsgtk_get_browser_window(g->top_level), 
 			  "http://www.netsurf-browser.org/documentation/", 0, true);
 
 	return TRUE;
@@ -1459,7 +1465,7 @@ MULTIHANDLER(contents)
 
 MULTIHANDLER(guide)
 {
-	browser_window_go(gui_window_get_browser_window(g->top_level), 
+	browser_window_go(nsgtk_get_browser_window(g->top_level), 
 			  "http://www.netsurf-browser.org/documentation/guide", 0, true);
 
 	return TRUE;
@@ -1467,7 +1473,7 @@ MULTIHANDLER(guide)
 
 MULTIHANDLER(info)
 {
-	browser_window_go(gui_window_get_browser_window(g->top_level), 
+	browser_window_go(nsgtk_get_browser_window(g->top_level), 
 			  "http://www.netsurf-browser.org/documentation/info", 0, true);
 
 	return TRUE;
@@ -1476,7 +1482,7 @@ MULTIHANDLER(info)
 MULTIHANDLER(about)
 {
 	nsgtk_about_dialog_init(g->window,
-			gui_window_get_browser_window(g->top_level),
+			nsgtk_get_browser_window(g->top_level),
 			netsurf_version);
 	return TRUE;
 }
@@ -1499,29 +1505,30 @@ static gboolean nsgtk_history_expose_event(GtkWidget *widget,
 	struct rect clip;
 	struct gtk_history_window *hw = (struct gtk_history_window *)g;
 	struct browser_window *bw =
-			gui_window_get_browser_window(hw->g->top_level);
+			nsgtk_get_browser_window(hw->g->top_level);
+
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &nsgtk_plotters
+	};
 
 	current_widget = widget;
 	current_drawable = widget->window;
-	current_gc = gdk_gc_new(current_drawable);
-#ifdef CAIRO_VERSION
+
 	current_cr = gdk_cairo_create(current_drawable);
-#endif
-	plot = nsgtk_plotters;
 
 	clip.x0 = event->area.x;
 	clip.y0 = event->area.y;
 	clip.x1 = event->area.x + event->area.width;
 	clip.y1 = event->area.y + event->area.height;
-	plot.clip(&clip);
+	ctx.plot->clip(&clip);
 
-	history_redraw(bw->history);
+	history_redraw(bw->history, &ctx);
 
 	current_widget = NULL;
-	g_object_unref(current_gc);
-#ifdef CAIRO_VERSION
+
 	cairo_destroy(current_cr);
-#endif
+
 	return FALSE;
 }
 
@@ -1531,7 +1538,7 @@ static gboolean nsgtk_history_button_press_event(GtkWidget *widget,
 {
 	struct gtk_history_window *hw = (struct gtk_history_window *)g;
 	struct browser_window *bw =
-			gui_window_get_browser_window(hw->g->top_level);
+			nsgtk_get_browser_window(hw->g->top_level);
 
 	LOG(("X=%g, Y=%g", event->x, event->y));
 
@@ -1994,11 +2001,6 @@ void gui_window_set_icon(struct gui_window *_g, hlcache_handle *icon)
 	if (g->top_level != _g)
 		return;
 
-#ifdef WITH_BMP
-	if (icon != NULL && content_get_type(icon) == CONTENT_ICO)
-		nsico_set_bitmap_from_size(icon, 16, 16);
-#endif
-
 	icon_bitmap = (icon != NULL) ? content_get_bitmap(icon) : NULL;
 
 	if (icon_bitmap != NULL) {
@@ -2039,11 +2041,6 @@ void gui_window_set_search_ico(hlcache_handle *ico)
 	if (ico == NULL && (ico = search_web_ico()) == NULL)
 		return;
 
-#ifdef WITH_BMP
-	if (content_get_type(ico) == CONTENT_ICO)
-		nsico_set_bitmap_from_size(ico, 20, 20);
-#endif
-
 	ico_bitmap = content_get_bitmap(ico);
 	if (ico_bitmap == NULL)
 		return;
@@ -2220,11 +2217,11 @@ void nsgtk_scaffolding_toggle_search_bar_visibility(nsgtk_scaffolding *g)
 {
 	gboolean vis;
 	struct browser_window *bw =
-			gui_window_get_browser_window(g->top_level);
+			nsgtk_get_browser_window(g->top_level);
 	g_object_get(G_OBJECT(g->search->bar), "visible", &vis, NULL);
 	if (vis) {
-		if ((bw != NULL) && (bw->search_context != NULL))
-			search_destroy_context(bw->search_context);
+		if (bw != NULL)
+			browser_window_search_destroy_context(bw);
 		nsgtk_search_set_forward_state(true, bw);
 		nsgtk_search_set_back_state(true, bw);
 		gtk_widget_hide(GTK_WIDGET(g->search->bar));
@@ -2243,7 +2240,7 @@ struct gui_window *nsgtk_scaffolding_top_level(nsgtk_scaffolding *g)
 void nsgtk_scaffolding_set_top_level (struct gui_window *gw)
 {
 	nsgtk_get_scaffold(gw)->top_level = gw;
-	struct browser_window *bw = gui_window_get_browser_window(gw);
+	struct browser_window *bw = nsgtk_get_browser_window(gw);
 
 	assert(bw != NULL);
 
@@ -2251,8 +2248,7 @@ void nsgtk_scaffolding_set_top_level (struct gui_window *gw)
 	nsgtk_window_update_back_forward(nsgtk_get_scaffold(gw));
 
 	/* clear effects of potential searches */
-	if (bw->search_context != NULL)
-		search_destroy_context(bw->search_context);
+	browser_window_search_destroy_context(bw);
 
 	nsgtk_search_set_forward_state(true, bw);
 	nsgtk_search_set_back_state(true, bw);
@@ -2331,19 +2327,19 @@ void nsgtk_scaffolding_initial_sensitivity(struct gtk_scaffolding *g)
 /**
  * Checks if a location is over a link.
  *
- * Side effect of this function is to set the global current_menu_link_box
+ * Side effect of this function is to set the global current_menu_ctx
  */
 static bool is_menu_over_link(struct gtk_scaffolding *g, gdouble x, gdouble y)
 {
-	struct browser_window *bw = gui_window_get_browser_window(g->top_level);
-	current_menu_link_box = NULL;
+	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	if ((bw->current_content != NULL) &&
 	    (content_get_type(bw->current_content) == CONTENT_HTML)) {
-		current_menu_link_box = box_href_at_point(bw->current_content, x, y);
+		browser_window_get_contextual_content(bw, x, y,
+				&current_menu_ctx);
 	}
 
-	if (current_menu_link_box == NULL)
+	if (current_menu_ctx.link_url == NULL)
 		return false;
 
 	return true;
