@@ -101,7 +101,7 @@ nsgtk_scaffolding *nsgtk_get_scaffold(struct gui_window *g)
 	return g->scaffold;
 }
 
-struct browser_window *gui_window_get_browser_window(struct gui_window *g)
+struct browser_window *nsgtk_get_browser_window(struct gui_window *g)
 {
 	return g->bw;
 }
@@ -137,27 +137,16 @@ float nsgtk_get_scale_for_gui(struct gui_window *g)
 	return g->bw->scale;
 }
 
-/** Attaches the child gui_window (frame) into the parent.
- *
- * The window will be resized later.
- *
- * \param parent The parent gui window.
- * \param child The gui window to attach to the parent.
- */
-static void nsgtk_gui_window_attach_child(struct gui_window *parent,
-					  struct gui_window *child)
-{
-	GtkLayout *parent_layout = parent->layout;
-	GtkWidget *child_widget = GTK_WIDGET(child->scrolledwindow);
-	gtk_layout_put(parent_layout, child_widget, 0, 0);
-}
-
 static gboolean nsgtk_window_expose_event(GtkWidget *widget,
 				   GdkEventExpose *event, gpointer data)
 {
 	struct gui_window *g = data;
 	struct gui_window *z;
 	struct rect clip;
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &nsgtk_plotters
+	};
 
 	assert(g);
 	assert(g->bw);
@@ -170,31 +159,20 @@ static gboolean nsgtk_window_expose_event(GtkWidget *widget,
 
 	current_widget = (GtkWidget *)g->layout;
 	current_drawable = g->layout->bin_window;
-	current_gc = gdk_gc_new(current_drawable);
-#ifdef CAIRO_VERSION
 	current_cr = gdk_cairo_create(current_drawable);
-#endif
-
-	plot = nsgtk_plotters;
-	current_redraw_browser = g->bw;
 
 	clip.x0 = event->area.x;
 	clip.y0 = event->area.y;
 	clip.x1 = event->area.x + event->area.width;
 	clip.y1 = event->area.y + event->area.height;
 
-	browser_window_redraw(g->bw, 0, 0, &clip);
-
-	current_redraw_browser = NULL;
+	browser_window_redraw(g->bw, 0, 0, &clip, &ctx);
 
 	if (g->careth != 0)
 		nsgtk_plot_caret(g->caretx, g->carety, g->careth);
 
 	current_widget = NULL;
-	g_object_unref(current_gc);
-#ifdef CAIRO_VERSION
 	cairo_destroy(current_cr);
-#endif
 
 	return FALSE;
 }
@@ -323,7 +301,7 @@ static gboolean nsgtk_window_button_release_event(GtkWidget *widget,
 				event->x / g->bw->scale,
 				event->y / g->bw->scale);
 	} else {
-		browser_window_mouse_drag_end(g->bw, 0, event->x / g->bw->scale,
+		browser_window_mouse_track(g->bw, 0, event->x / g->bw->scale,
 				event->y / g->bw->scale);
 	}
 
@@ -527,14 +505,14 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 
 	g->careth = 0;
 
-	if (bw->parent != NULL)
-		/* Find our parent's scaffolding */
-		g->scaffold = bw->parent->window->scaffold;
-	else if (new_tab)
+	if (new_tab) {
+		assert(clone != NULL);
 		g->scaffold = clone->window->scaffold;
-	else
+	} else {
 		/* Now construct and attach a scaffold */
 		g->scaffold = nsgtk_new_scaffolding(g);
+	}
+
 	if (g->scaffold == NULL) {
 		warn_user("NoMemory", 0);
 		free(g);
@@ -542,64 +520,46 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	}
 
 	/* Construct our primary elements */
-	if (bw->parent == NULL) {
-		/* top-level document (not a frame) => create a new tab */
-		GladeXML *xml = glade_xml_new(glade_netsurf_file_location, "tabContents", NULL);
-		if (!xml) {
-			warn_user("MiscError", "Failed to create tab contents");
-			free(g);
-			return 0;
-		}
 
-		GtkWidget *tab_contents = glade_xml_get_widget(xml, "tabContents");
-		g->layout = GTK_LAYOUT(glade_xml_get_widget(xml, "layout"));
-		g->status_bar = GTK_LABEL(glade_xml_get_widget(xml, "status_bar"));
-		g->paned = GTK_PANED(glade_xml_get_widget(xml, "hpaned1"));
-
-		/* connect the scrollbars to the layout widget */
-		gtk_layout_set_hadjustment(g->layout,
-				gtk_range_get_adjustment(GTK_RANGE(
-				glade_xml_get_widget(xml, "hscrollbar"))));
-		gtk_layout_set_vadjustment(g->layout,
-				gtk_range_get_adjustment(GTK_RANGE(
-				glade_xml_get_widget(xml, "vscrollbar"))));
-
-		/* add the tab to the scaffold */
-		bool tempback = true;
-		switch (temp_open_background) {
-		case -1:
-			tempback = !(option_focus_new);
-			break;
-		case 0:
-			tempback = false;
-			break;
-		case 1:
-			tempback = true;
-			break;
-		}
-		g_object_set_data(G_OBJECT(tab_contents), "gui_window", g);
-		nsgtk_tab_add(g, tab_contents, tempback);
-
-		g_object_unref(xml);
-
-	} else {
-		/* frame or iframe => create a child layout */
-		g->layout = GTK_LAYOUT(gtk_layout_new(NULL, NULL));
-		gtk_container_set_border_width(GTK_CONTAINER(g->layout), 0);
-
-		g->scrolledwindow = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
-		g_object_set_data(G_OBJECT(g->scrolledwindow), "gui_window", g);
-		gtk_container_add(GTK_CONTAINER(g->scrolledwindow), GTK_WIDGET(g->layout));
-		gtk_scrolled_window_set_shadow_type(g->scrolledwindow,
-						    GTK_SHADOW_NONE);
-		g->tab = NULL;
-
-		/* Attach ourselves into our parent at the right point */
-		nsgtk_gui_window_attach_child(bw->parent->window, g);
-
-		gtk_widget_show(GTK_WIDGET(g->scrolledwindow));
+	/* top-level document (not a frame) => create a new tab */
+	GladeXML *xml = glade_xml_new(glade_netsurf_file_location, "tabContents", NULL);
+	if (!xml) {
+		warn_user("MiscError", "Failed to create tab contents");
+		free(g);
+		return 0;
 	}
 
+	GtkWidget *tab_contents = glade_xml_get_widget(xml, "tabContents");
+	g->layout = GTK_LAYOUT(glade_xml_get_widget(xml, "layout"));
+	g->status_bar = GTK_LABEL(glade_xml_get_widget(xml, "status_bar"));
+	g->paned = GTK_PANED(glade_xml_get_widget(xml, "hpaned1"));
+
+	/* connect the scrollbars to the layout widget */
+	gtk_layout_set_hadjustment(g->layout,
+			gtk_range_get_adjustment(GTK_RANGE(
+			glade_xml_get_widget(xml, "hscrollbar"))));
+	gtk_layout_set_vadjustment(g->layout,
+			gtk_range_get_adjustment(GTK_RANGE(
+			glade_xml_get_widget(xml, "vscrollbar"))));
+
+	/* add the tab to the scaffold */
+	bool tempback = true;
+	switch (temp_open_background) {
+	case -1:
+		tempback = !(option_focus_new);
+		break;
+	case 0:
+		tempback = false;
+		break;
+	case 1:
+		tempback = true;
+		break;
+	}
+	g_object_set_data(G_OBJECT(tab_contents), "gui_window", g);
+	nsgtk_tab_add(g, tab_contents, tempback);
+
+	g_object_unref(xml);
+
 	switch(bw->scrolling) {
 	case SCROLLING_NO:
 		scrollpolicy = GTK_POLICY_NEVER;
@@ -613,32 +573,9 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 		break;
 	}
 
-	switch (bw->browser_window_type) {
-	case BROWSER_WINDOW_FRAMESET:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       GTK_POLICY_NEVER,
-						       GTK_POLICY_NEVER);
-		break;
-	case BROWSER_WINDOW_FRAME:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       scrollpolicy,
-						       scrollpolicy);
-		break;
-	case BROWSER_WINDOW_NORMAL:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       scrollpolicy,
-						       scrollpolicy);
-		break;
-	case BROWSER_WINDOW_IFRAME:
-		if (g->scrolledwindow)
-			gtk_scrolled_window_set_policy(g->scrolledwindow,
-						       scrollpolicy,
-						       scrollpolicy);
-		break;
-	}
+	if (g->scrolledwindow)
+		gtk_scrolled_window_set_policy(g->scrolledwindow,
+				scrollpolicy, scrollpolicy);
 
 	/* Attach ourselves to the list (push_top) */
 	if (window_list)
@@ -688,43 +625,6 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 }
 
 
-void gui_window_position_frame(struct gui_window *g, int x0, int y0, int x1, int y1)
-{
-	/* g is a child frame, we need to place it relative to its parent */
-	GtkWidget *w = GTK_WIDGET(g->scrolledwindow);
-	GtkLayout *f = g->bw->parent->window->layout;
-	int width = x1 - x0 + 2, height = y1 - y0 + 2;
-	assert(w);
-	assert(f);
-
-	if (width < 1)
-		width = 1;
-	if (height < 1)
-		height = 1;
-
-	LOG(("%s: %d,%d	 %dx%d", g->bw->name, x0, y0, width, height));
-
-	/* if the window has not changed position or size, do not bother
-	 * moving/resising it.
-	 */
-
-	LOG(("	current: %d,%d	%dx%d",
-		w->allocation.x, w->allocation.y,
-		w->allocation.width, w->allocation.height));
-
-	if (w->allocation.x != x0 || w->allocation.y != y0 ||
-		w->allocation.width != width ||
-		w->allocation.height != height) {
-		LOG(("	frame has moved/resized."));
-		gtk_layout_move(f, w, x0, y0);
-		gtk_widget_set_size_request(w, width, height);
-	}
-}
-
-
-
-
-
 
 void nsgtk_reflow_all_windows(void)
 {
@@ -752,7 +652,7 @@ void nsgtk_window_process_reformats(void)
 		if (!g->bw->reformat_pending)
 			continue;
 		g->bw->reformat_pending = false;
-		browser_window_reformat(g->bw,
+		browser_window_reformat(g->bw, false,
 				widget->allocation.width,
 				widget->allocation.height);
 	}
@@ -834,8 +734,7 @@ void gui_window_redraw_window(struct gui_window *g)
 	gtk_widget_queue_draw(GTK_WIDGET(g->layout));
 }
 
-void gui_window_update_box(struct gui_window *g,
-			   const union content_msg_data *data)
+void gui_window_update_box(struct gui_window *g, const struct rect *rect)
 {
 	int sx, sy;
 	hlcache_handle *c = g->bw->current_content;
@@ -846,10 +745,10 @@ void gui_window_update_box(struct gui_window *g,
 	gui_window_get_scroll(g, &sx, &sy);
 
 	gtk_widget_queue_draw_area(GTK_WIDGET(g->layout),
-				   data->redraw.x * g->bw->scale - sx,
-				   data->redraw.y * g->bw->scale - sy,
-				   data->redraw.width * g->bw->scale,
-				   data->redraw.height * g->bw->scale);
+				   rect->x0 * g->bw->scale - sx,
+				   rect->y0 * g->bw->scale - sy,
+				   (rect->x1 - rect->x0) * g->bw->scale,
+				   (rect->y1 - rect->y0) * g->bw->scale);
 }
 
 void gui_window_set_status(struct gui_window *g, const char *text)
@@ -905,18 +804,6 @@ void gui_window_scroll_visible(struct gui_window *g, int x0, int y0,
 }
 
 
-/**
- * Set the scale setting of a window
- *
- * \param  g	  gui window
- * \param  scale  scale value (1.0 == normal scale)
- */
-
-void gui_window_set_scale(struct gui_window *g, float scale)
-{
-}
-
-
 void gui_window_update_extent(struct gui_window *g)
 {
 	if (!g->bw->current_content)
@@ -1102,7 +989,3 @@ void gui_window_get_dimensions(struct gui_window *g, int *width, int *height,
 	LOG(("\tWINDOW HEIGHT: %i\n", *height));
 }
 
-bool gui_window_frame_resize_start(struct gui_window *g)
-{
-	return true;
-}
