@@ -31,11 +31,11 @@
 #include "content/hlcache.h"
 #include "css/css.h"
 #include "css/dump.h"
-#include "desktop/scroll.h"
+#include "desktop/scrollbar.h"
 #include "desktop/options.h"
 #include "render/box.h"
 #include "render/form.h"
-#include "render/html.h"
+#include "render/html_internal.h"
 #include "utils/log.h"
 #include "utils/talloc.h"
 #include "utils/utils.h"
@@ -165,6 +165,7 @@ struct box * box_create(css_select_results *styles, css_computed_style *style,
 	box->background = NULL;
 	box->object = NULL;
 	box->object_params = NULL;
+	box->iframe = NULL;
 
 	return box;
 }
@@ -277,9 +278,9 @@ void box_free_box(struct box *box)
 		if (box->gadget)
 			form_free_control(box->gadget);
 		if (box->scroll_x != NULL)
-			scroll_destroy(box->scroll_x);
+			scrollbar_destroy(box->scroll_x);
 		if (box->scroll_y != NULL)
-			scroll_destroy(box->scroll_y);
+			scrollbar_destroy(box->scroll_y);
 		if (box->styles != NULL)
 			css_select_results_destroy(box->styles);
 	}
@@ -307,8 +308,8 @@ void box_coords(struct box *box, int *x, int *y)
 			} while (!box->float_children);
 		} else
 			box = box->parent;
-		*x += box->x - scroll_get_offset(box->scroll_x);
-		*y += box->y - scroll_get_offset(box->scroll_y);
+		*x += box->x - scrollbar_get_offset(box->scroll_x);
+		*y += box->y - scrollbar_get_offset(box->scroll_y);
 	}
 }
 
@@ -388,9 +389,9 @@ struct box *box_at_point(struct box *box, const int x, const int y,
 	for (child = box->float_children; child; child = child->next_float) {
 		if (box_contains_point(child, x - bx, y - by, &physically)) {
 			*box_x = bx + child->x -
-					scroll_get_offset(child->scroll_x);
+					scrollbar_get_offset(child->scroll_x);
 			*box_y = by + child->y -
-					scroll_get_offset(child->scroll_y);
+					scrollbar_get_offset(child->scroll_y);
 
 			if (physically)
 				return child;
@@ -407,9 +408,9 @@ non_float_children:
 			continue;
 		if (box_contains_point(child, x - bx, y - by, &physically)) {
 			*box_x = bx + child->x -
-					scroll_get_offset(child->scroll_x);
+					scrollbar_get_offset(child->scroll_x);
 			*box_y = by + child->y -
-					scroll_get_offset(child->scroll_y);
+					scrollbar_get_offset(child->scroll_y);
 
 			if (physically)
 				return child;
@@ -433,17 +434,17 @@ siblings:
 	/* siblings and siblings of ancestors */
 	while (box) {
 		if (box_is_float(box)) {
-			bx -= box->x - scroll_get_offset(box->scroll_x);
-			by -= box->y - scroll_get_offset(box->scroll_y);
+			bx -= box->x - scrollbar_get_offset(box->scroll_x);
+			by -= box->y - scrollbar_get_offset(box->scroll_y);
 			for (sibling = box->next_float; sibling;
 					sibling = sibling->next_float) {
 				if (box_contains_point(sibling,
 						x - bx, y - by, &physically)) {
 					*box_x = bx + sibling->x -
-							scroll_get_offset(
+							scrollbar_get_offset(
 							sibling->scroll_x);
 					*box_y = by + sibling->y -
-							scroll_get_offset(
+							scrollbar_get_offset(
 							sibling->scroll_y);
 
 					if (physically)
@@ -463,8 +464,8 @@ siblings:
 			goto non_float_children;
 
 		} else {
-			bx -= box->x - scroll_get_offset(box->scroll_x);
-			by -= box->y - scroll_get_offset(box->scroll_y);
+			bx -= box->x - scrollbar_get_offset(box->scroll_x);
+			by -= box->y - scrollbar_get_offset(box->scroll_y);
 			for (sibling = box->next; sibling;
 					sibling = sibling->next) {
 				if (box_is_float(sibling))
@@ -472,10 +473,10 @@ siblings:
 				if (box_contains_point(sibling, x - bx, y - by,
 						&physically)) {
 					*box_x = bx + sibling->x -
-							scroll_get_offset(
+							scrollbar_get_offset(
 							sibling->scroll_x);
 					*box_y = by + sibling->y -
-							scroll_get_offset(
+							scrollbar_get_offset(
 							sibling->scroll_y);
 
 					if (physically)
@@ -563,16 +564,12 @@ bool box_contains_point(struct box *box, int x, int y, bool *physically)
 
 struct box *box_object_at_point(hlcache_handle *h, int x, int y)
 {
-	struct content *c = hlcache_handle_get_content(h);
 	struct box *box;
 	int box_x = 0, box_y = 0;
 	hlcache_handle *content = h;
 	struct box *object_box = 0;
 
-	assert(c != NULL);
-	assert(c->type == CONTENT_HTML);
-
-	box = c->data.html.layout;
+	box = html_get_box_tree(h);
 
 	while ((box = box_at_point(box, x, y, &box_x, &box_y, &content))) {
 		if (box->style && css_computed_visibility(box->style) ==
@@ -597,16 +594,12 @@ struct box *box_object_at_point(hlcache_handle *h, int x, int y)
 
 struct box *box_href_at_point(hlcache_handle *h, int x, int y)
 {
-	struct content *c = hlcache_handle_get_content(h);
 	struct box *box;
 	int box_x = 0, box_y = 0;
 	hlcache_handle *content = h;
 	struct box *href_box = 0;
 
-	assert(c != NULL);
-	assert(c->type == CONTENT_HTML);
-
-	box = c->data.html.layout;
+	box = html_get_box_tree(h);
 
 	while ((box = box_at_point(box, x, y, &box_x, &box_y, &content))) {
 		if (box->style && css_computed_visibility(box->style) ==
@@ -743,14 +736,14 @@ bool box_nearest_text_box(struct box *box, int bx, int by,
 		if (child->type == BOX_FLOAT_LEFT ||
 				child->type == BOX_FLOAT_RIGHT) {
 			c_bx = fx + child->x -
-					scroll_get_offset(child->scroll_x);
+					scrollbar_get_offset(child->scroll_x);
 			c_by = fy + child->y -
-					scroll_get_offset(child->scroll_y);
+					scrollbar_get_offset(child->scroll_y);
 		} else {
 			c_bx = bx + child->x -
-					scroll_get_offset(child->scroll_x);
+					scrollbar_get_offset(child->scroll_x);
 			c_by = by + child->y -
-					scroll_get_offset(child->scroll_y);
+					scrollbar_get_offset(child->scroll_y);
 		}
 		if (child->float_children) {
 			c_fx = c_bx;
@@ -791,7 +784,7 @@ bool box_nearest_text_box(struct box *box, int bx, int by,
  * the mouse pointer, or nearest in the given direction if the pointer is
  * not over a text box.
  *
- * \param h	html content's high level cache handle
+ * \param html	an HTML content
  * \param x	coordinate of mouse
  * \param y	coordinate of mouse
  * \param dir	direction to search (-1 = above-left, +1 = below-right)
@@ -799,47 +792,51 @@ bool box_nearest_text_box(struct box *box, int bx, int by,
  * \param dy	receives y ordinate of mouse relative to text box
  */
 
-struct box *box_pick_text_box(hlcache_handle *h,
+struct box *box_pick_text_box(struct html_content *html,
 		int x, int y, int dir, int *dx, int *dy)
 {
 	struct box *text_box = NULL;
+	struct box *box;
+	int nr_xd, nr_yd;
+	int bx, by;
+	int fx, fy;
+	int tx, ty;
 
-	if (h && content_get_type(h) == CONTENT_HTML) {
-		struct box *box = html_get_box_tree(h);
-		int nr_xd, nr_yd;
-		int bx = box->margin[LEFT];
-		int by = box->margin[TOP];
-		int fx = bx;
-		int fy = by;
-		int tx, ty;
+	if (html == NULL)
+		return NULL;
 
-		if (!box_nearest_text_box(box, bx, by, fx, fy, x, y,
-				dir, &text_box, &tx, &ty, &nr_xd, &nr_yd)) {
-			if (text_box && text_box->text && !text_box->object) {
-				int w = (text_box->padding[LEFT] +
-						text_box->width +
-						text_box->padding[RIGHT]);
-				int h = (text_box->padding[TOP] +
-						text_box->height +
-						text_box->padding[BOTTOM]);
-				int x1, y1;
+	box = html->layout;
+	bx = box->margin[LEFT];
+	by = box->margin[TOP];
+	fx = bx;
+	fy = by;
 
-				y1 = ty + h;
-				x1 = tx + w;
+	if (!box_nearest_text_box(box, bx, by, fx, fy, x, y,
+			dir, &text_box, &tx, &ty, &nr_xd, &nr_yd)) {
+		if (text_box && text_box->text && !text_box->object) {
+			int w = (text_box->padding[LEFT] +
+					text_box->width +
+					text_box->padding[RIGHT]);
+			int h = (text_box->padding[TOP] +
+					text_box->height +
+					text_box->padding[BOTTOM]);
+			int x1, y1;
 
-				/* ensure point lies within the text box */
-				if (x < tx) x = tx;
-				if (y < ty) y = ty;
-				if (y > y1) y = y1;
-				if (x > x1) x = x1;
-			}
+			y1 = ty + h;
+			x1 = tx + w;
+
+			/* ensure point lies within the text box */
+			if (x < tx) x = tx;
+			if (y < ty) y = ty;
+			if (y > y1) y = y1;
+			if (x > x1) x = x1;
 		}
-
-		/* return coordinates relative to box */
-		*dx = x - tx;
-		*dy = y - ty;
 	}
 
+	/* return coordinates relative to box */
+	*dx = x - tx;
+	*dy = y - ty;
+
 	return text_box;
 }
 
@@ -940,6 +937,9 @@ void box_dump(FILE *stream, struct box *box, unsigned int depth)
 		fprintf(stream, "(object '%s') ", 
 				content_get_url(box->object));
 	}
+	if (box->iframe) {
+		fprintf(stream, "(iframe) ");
+	}
 	if (box->gadget)
 		fprintf(stream, "(gadget) ");
 	if (box->style)
@@ -1003,92 +1003,95 @@ void box_dump(FILE *stream, struct box *box, unsigned int depth)
  * Applies the given scroll setup to a box. This includes scroll
  * creation/deletion as well as scroll dimension updates.
  *
- * \param bw		browser window in which the box is located
+ * \param c		content in which the box is located
  * \param box		the box to handle the scrolls for
  * \param bottom	whether the horizontal scrollbar should be present
  * \param right		whether the vertical scrollbar should be present
  * \return		true on success false otherwise
  */
-bool box_handle_scrollbars(struct browser_window *bw, struct box *box,
+bool box_handle_scrollbars(struct content *c, struct box *box,
 		bool bottom, bool right)
 {
-	struct browser_scroll_data *data;
-	int padding_width, padding_height;
-	
-	padding_width = box->width + box->padding[RIGHT] + box->padding[LEFT];
-	padding_height = box->height + box->padding[TOP] + box->padding[BOTTOM];
-	
+	struct html_scrollbar_data *data;
+	int visible_width, visible_height;
+	int full_width, full_height;
+
 	if (!bottom && box->scroll_x != NULL) {
-		data = scroll_get_data(box->scroll_x);
-		scroll_destroy(box->scroll_x);
+		data = scrollbar_get_data(box->scroll_x);
+		scrollbar_destroy(box->scroll_x);
 		free(data);
 		box->scroll_x = NULL;
 	}
-	
+
 	if (!right && box->scroll_y != NULL) {
-		data = scroll_get_data(box->scroll_y);
-		scroll_destroy(box->scroll_y);
+		data = scrollbar_get_data(box->scroll_y);
+		scrollbar_destroy(box->scroll_y);
 		free(data);
 		box->scroll_y = NULL;
 	}
-	
+
 	if (!bottom && !right)
 		return true;
-	
+
+	visible_width = box->width + box->padding[RIGHT] + box->padding[LEFT];
+	visible_height = box->height + box->padding[TOP] + box->padding[BOTTOM];
+
+	full_width = ((box->descendant_x1 - box->border[RIGHT].width) >
+			visible_width) ?
+			box->descendant_x1 + box->padding[RIGHT] :
+			visible_width;
+	full_height = ((box->descendant_y1 - box->border[BOTTOM].width) >
+			visible_height) ?
+			box->descendant_y1 + box->padding[BOTTOM] :
+			visible_height;
+
 	if (right) {
 		if (box->scroll_y == NULL) {
-			data = malloc(sizeof(struct browser_scroll_data));
+			data = malloc(sizeof(struct html_scrollbar_data));
 			if (data == NULL) {
 				LOG(("malloc failed"));
 				warn_user("NoMemory", 0);
 				return false;
 			}
-			data->bw = bw;
+			data->c = c;
 			data->box = box;
-			if (!scroll_create(false,
-					padding_height,
-					box->descendant_y1 - box->padding[TOP],
-     					box->height,
-					data,
-     					html_overflow_scroll_callback,
+			if (!scrollbar_create(false, visible_height,
+					full_height, visible_height,
+					data, html_overflow_scroll_callback,
 					&(box->scroll_y)))
 				return false;
-		} else 
-			scroll_set_extents(box->scroll_y,
-					padding_height, box->height,
-     					box->descendant_y1 -
-					box->padding[TOP]);
+		} else  {
+			scrollbar_set_extents(box->scroll_y, visible_height,
+					visible_height, full_height);
+		}
 	}
 	if (bottom) {
 		if (box->scroll_x == NULL) {
-			data = malloc(sizeof(struct browser_scroll_data));
+			data = malloc(sizeof(struct html_scrollbar_data));
 			if (data == NULL) {
 				LOG(("malloc failed"));
 				warn_user("NoMemory", 0);
 				return false;
 			}
-			data->bw = bw;
+			data->c = c;
 			data->box = box;
-			if (!scroll_create(true,
-					padding_width -
+			if (!scrollbar_create(true,
+					visible_width -
 					(right ? SCROLLBAR_WIDTH : 0),
-					box->descendant_x1 - box->padding[LEFT],
-     					box->width,
-					data,
-     					html_overflow_scroll_callback,
+					full_width, visible_width,
+					data, html_overflow_scroll_callback,
 					&box->scroll_x))
 				return false;
-		} else
-			scroll_set_extents(box->scroll_x,
-					padding_width -
+		} else {
+			scrollbar_set_extents(box->scroll_x,
+					visible_width -
 					(right ? SCROLLBAR_WIDTH : 0),
-					box->width,
-     					box->descendant_x1 -
-					box->padding[LEFT]);
+					visible_width, full_width);
+		}
 	}
 	
 	if (right && bottom)
-		scroll_make_pair(box->scroll_x, box->scroll_y);
+		scrollbar_make_pair(box->scroll_x, box->scroll_y);
 	
 	return true;
 }
