@@ -42,7 +42,7 @@
 #include "desktop/options.h"
 #include "render/box.h"
 #include "render/form.h"
-#include "render/html.h"
+#include "render/html_internal.h"
 #include "desktop/gui.h"
 #include "utils/locale.h"
 #include "utils/log.h"
@@ -52,45 +52,7 @@
 #include "utils/utils.h"
 
 
-static const content_type image_types[] = {
-#ifdef WITH_JPEG
-	CONTENT_JPEG,
-#endif
-#ifdef WITH_GIF
-	CONTENT_GIF,
-#endif
-#ifdef WITH_BMP
-	CONTENT_BMP,
-#endif
-#if defined(WITH_MNG) || defined(WITH_PNG)
-	CONTENT_PNG,
-#endif
-#ifdef WITH_MNG
-	CONTENT_JNG,
-	CONTENT_MNG,
-#endif
-#if defined(WITH_NS_SVG) || defined(WITH_RSVG)
-	CONTENT_SVG,
-#endif
-#if defined(WITH_SPRITE) || defined(WITH_NSSPRITE)
-	CONTENT_SPRITE,
-#endif
-#ifdef WITH_DRAW
-	CONTENT_DRAW,
-#endif
-#ifdef WITH_ARTWORKS
-	CONTENT_ARTWORKS,
-#endif
-#ifdef WITH_WEBP
-	CONTENT_WEBP,
-#endif
-#ifdef WITH_AMIGA_ICON
-	CONTENT_AMIGA_ICON,
-#endif
-#ifdef WITH_APPLE_IMAGE
-	CONTENT_APPLE_IMAGE,
-#endif
-	CONTENT_UNKNOWN };
+static const content_type image_types = CONTENT_IMAGE;
 
 /* the strings are not important, since we just compare the pointers */
 const char *TARGET_SELF = "_self";
@@ -98,25 +60,25 @@ const char *TARGET_PARENT = "_parent";
 const char *TARGET_TOP = "_top";
 const char *TARGET_BLANK = "_blank";
 
-static bool convert_xml_to_box(xmlNode *n, struct content *content,
+static bool convert_xml_to_box(xmlNode *n, html_content *content,
 		const css_computed_style *parent_style,
 		struct box *parent, struct box **inline_container,
 		char *href, const char *target, char *title);
-bool box_construct_element(xmlNode *n, struct content *content,
+bool box_construct_element(xmlNode *n, html_content *content,
 		const css_computed_style *parent_style,
 		struct box *parent, struct box **inline_container,
 		char *href, const char *target, char *title);
-void box_construct_after(xmlNode *n, struct content *content,
-		struct box *box, const css_computed_style *after_style);
-bool box_construct_text(xmlNode *n, struct content *content,
+void box_construct_generate(xmlNode *n, html_content *content,
+		struct box *box, const css_computed_style *style);
+bool box_construct_text(xmlNode *n, html_content *content,
 		const css_computed_style *parent_style,
 		struct box *parent, struct box **inline_container,
 		char *href, const char *target, char *title);
-static css_select_results * box_get_style(struct content *c,
+static css_select_results * box_get_style(html_content *c,
 		const css_computed_style *parent_style, xmlNode *n);
 static void box_text_transform(char *s, unsigned int len,
 		enum css_text_transform_e tt);
-#define BOX_SPECIAL_PARAMS xmlNode *n, struct content *content, \
+#define BOX_SPECIAL_PARAMS xmlNode *n, html_content *content, \
 		struct box *box, bool *convert_children
 static bool box_a(BOX_SPECIAL_PARAMS);
 static bool box_body(BOX_SPECIAL_PARAMS);
@@ -129,12 +91,11 @@ static bool box_input_text(BOX_SPECIAL_PARAMS, bool password);
 static bool box_button(BOX_SPECIAL_PARAMS);
 static bool box_frameset(BOX_SPECIAL_PARAMS);
 static bool box_create_frameset(struct content_html_frames *f, xmlNode *n,
-		struct content *content);
+		html_content *content);
 static bool box_select_add_option(struct form_control *control, xmlNode *n);
 static bool box_object(BOX_SPECIAL_PARAMS);
 static bool box_embed(BOX_SPECIAL_PARAMS);
 static bool box_pre(BOX_SPECIAL_PARAMS);
-/*static bool box_applet(BOX_SPECIAL_PARAMS);*/
 static bool box_iframe(BOX_SPECIAL_PARAMS);
 static bool box_get_attribute(xmlNode *n, const char *attribute,
 		void *context, char **value);
@@ -148,7 +109,6 @@ struct element_entry {
 };
 static const struct element_entry element_table[] = {
 	{"a", box_a},
-/*	{"applet", box_applet},*/
 	{"body", box_body},
 	{"br", box_br},
 	{"button", box_button},
@@ -173,13 +133,11 @@ static const struct element_entry element_table[] = {
  * \return  true on success, false on memory exhaustion
  */
 
-bool xml_to_box(xmlNode *n, struct content *c)
+bool xml_to_box(xmlNode *n, html_content *c)
 {
 	struct box root;
 	struct box *inline_container = NULL;
 
-	assert(c->type == CONTENT_HTML);
-
 	root.type = BOX_BLOCK;
 	root.style = NULL;
 	root.next = NULL;
@@ -198,8 +156,8 @@ bool xml_to_box(xmlNode *n, struct content *c)
 	if (!box_normalise_block(&root, c))
 		return false;
 
-	c->data.html.layout = root.children;
-	c->data.html.layout->parent = NULL;
+	c->layout = root.children;
+	c->layout->parent = NULL;
 
 	return true;
 }
@@ -243,7 +201,7 @@ static const box_type box_map[] = {
  * \return  true on success, false on memory exhaustion
  */
 
-bool convert_xml_to_box(xmlNode *n, struct content *content,
+bool convert_xml_to_box(xmlNode *n, html_content *content,
 		const css_computed_style *parent_style,
 		struct box *parent, struct box **inline_container,
 		char *href, const char *target, char *title)
@@ -277,7 +235,7 @@ bool convert_xml_to_box(xmlNode *n, struct content *content,
  * \return  true on success, false on memory exhaustion
  */
 
-bool box_construct_element(xmlNode *n, struct content *content,
+bool box_construct_element(xmlNode *n, html_content *content,
 		const css_computed_style *parent_style,
 		struct box *parent, struct box **inline_container,
 		char *href, const char *target, char *title)
@@ -358,6 +316,18 @@ bool box_construct_element(xmlNode *n, struct content *content,
 				n->parent == NULL)];
 	}
 
+	/* handle the :before pseudo element */
+
+	/* TODO: Replace with true implementation.
+	 * Currently we only implement enough of this to support the
+	 * 'clearfix' hack, which is in widespread use and the layout
+	 * of many sites depend on. As such, only bother if box is a
+	 * block for now. */
+	if (box->type == BOX_BLOCK) {
+		box_construct_generate(n, content, box,
+				box->styles->styles[CSS_PSEUDO_ELEMENT_BEFORE]);
+	}
+
 	/* special elements */
 	element = bsearch((const char *) n->name, element_table,
 			ELEMENT_TABLE_COUNT, sizeof(element_table[0]),
@@ -531,7 +501,7 @@ bool box_construct_element(xmlNode *n, struct content *content,
 				if (!html_fetch_object(content,
 						lwc_string_data(image_uri),
 						marker, image_types,
-						content->available_width,
+						content->base.available_width,
 						1000, false))
 					return false;
 			}
@@ -595,7 +565,7 @@ bool box_construct_element(xmlNode *n, struct content *content,
 				bgimage_uri != NULL) {
 		if (!html_fetch_object(content,
 				lwc_string_data(bgimage_uri),
-				box, image_types, content->available_width,
+				box, image_types, content->base.available_width,
 				1000, true))
 			return false;
 	}
@@ -608,21 +578,20 @@ bool box_construct_element(xmlNode *n, struct content *content,
 	 * of many sites depend on. As such, only bother if box is a
 	 * block for now. */
 	if (box->type == BOX_BLOCK) {
-		box_construct_after(n, content, box,
+		box_construct_generate(n, content, box,
 				box->styles->styles[CSS_PSEUDO_ELEMENT_AFTER]);
 	}
 
 	return true;
 }
 
-
 /**
- * Construct the box required for an :after pseudo element.
+ * Construct the box required for a generated element.
  *
  * \param  n		XML node of type XML_ELEMENT_NODE
  * \param  content	content of type CONTENT_HTML that is being processed
- * \param  box		box which may have an :after s
- * \param  after_style	complete computed style for after pseudo element
+ * \param  box		box which may have generated content
+ * \param  style	complete computed style for pseudo element
  *
  * TODO:
  * This is currently incomplete. It just does enough to support the clearfix
@@ -634,36 +603,36 @@ bool box_construct_element(xmlNode *n, struct content *content,
  * We don't actually support generated content yet.
  */
 
-void box_construct_after(xmlNode *n, struct content *content,
-		struct box *box, const css_computed_style *after_style)
+void box_construct_generate(xmlNode *n, html_content *content,
+		struct box *box, const css_computed_style *style)
 {
-	struct box *after = 0;
+	struct box *gen = NULL;
 	const css_computed_content_item *c_item;
 
-	if (after_style == NULL ||
-			css_computed_content(after_style, &c_item) ==
+	if (style == NULL ||
+			css_computed_content(style, &c_item) ==
 			CSS_CONTENT_NORMAL) {
 		/* No pseudo element */
 		return;
 	}
 
 	/* create box for this element */
-	if (css_computed_display(after_style, n->parent == NULL) ==
+	if (css_computed_display(style, n->parent == NULL) ==
 			CSS_DISPLAY_BLOCK) {
 		/* currently only support block level after elements */
 
 		/** \todo Not wise to drop const from the computed style */ 
-		after = box_create(NULL, (css_computed_style *)after_style,
+		gen = box_create(NULL, (css_computed_style *) style,
 				false, NULL, NULL, NULL, NULL, content);
-		if (after == NULL) {
+		if (gen == NULL) {
 			return;
 		}
 
 		/* set box type from computed display */
-		after->type = box_map[css_computed_display(
-				after_style, n->parent == NULL)];
+		gen->type = box_map[css_computed_display(
+				style, n->parent == NULL)];
 
-		box_add_child(box, after);
+		box_add_child(box, gen);
 	}
 }
 
@@ -683,7 +652,7 @@ void box_construct_after(xmlNode *n, struct content *content,
  * \return  true on success, false on memory exhaustion
  */
 
-bool box_construct_text(xmlNode *n, struct content *content,
+bool box_construct_text(xmlNode *n, html_content *content,
 		const css_computed_style *parent_style,
 		struct box *parent, struct box **inline_container,
 		char *href, const char *target, char *title)
@@ -904,7 +873,7 @@ bool box_construct_text(xmlNode *n, struct content *content,
  * \param  n		   node in xml tree
  * \return  the new style, or NULL on memory exhaustion
  */
-css_select_results *box_get_style(struct content *c,
+css_select_results *box_get_style(html_content *c,
 		const css_computed_style *parent_style, xmlNode *n)
 {
 	char *s;
@@ -912,13 +881,14 @@ css_select_results *box_get_style(struct content *c,
 	css_error error;
 	css_stylesheet *inline_style = NULL;
 	css_select_results *styles;
+	nscss_select_ctx ctx;
 
 	/* Firstly, construct inline stylesheet, if any */
 	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "style"))) {
 		inline_style = nscss_create_inline_style(
 				(uint8_t *) s, strlen(s),
-				c->data.html.encoding, content__get_url(c), 
-				c->data.html.quirks != BINDING_QUIRKS_MODE_NONE,
+				c->encoding, content__get_url(&c->base), 
+				c->quirks != BINDING_QUIRKS_MODE_NONE,
 				box_style_alloc, NULL);
 
 		xmlFree(s);
@@ -927,8 +897,13 @@ css_select_results *box_get_style(struct content *c,
 			return NULL;
 	}
 
+	/* Populate selection context */
+	ctx.ctx = c->select_ctx;
+	ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
+	ctx.base_url = c->base_url;
+
 	/* Select partial style for element */
-	styles = nscss_get_style(c, n, CSS_MEDIA_SCREEN, inline_style,
+	styles = nscss_get_style(&ctx, n, CSS_MEDIA_SCREEN, inline_style,
 			box_style_alloc, NULL);
 
 	/* No longer need inline style */
@@ -1055,9 +1030,9 @@ bool box_body(BOX_SPECIAL_PARAMS)
 
 	css_computed_background_color(box->style, &color);
 	if (nscss_color_is_transparent(color))
-		content->data.html.background_colour = NS_TRANSPARENT;
+		content->background_colour = NS_TRANSPARENT;
 	else
-		content->data.html.background_colour = nscss_color_to_ns(color);
+		content->background_colour = nscss_color_to_ns(color);
 
 	return true;
 }
@@ -1095,7 +1070,7 @@ bool box_a(BOX_SPECIAL_PARAMS)
 
 	if ((s = xmlGetProp(n, (const xmlChar *) "href"))) {
 		ok = box_extract_link((const char *) s,
-				content->data.html.base_url, &url);
+				content->base_url, &url);
 		xmlFree(s);
 		if (!ok)
 			return false;
@@ -1148,6 +1123,11 @@ bool box_image(BOX_SPECIAL_PARAMS)
 	bool ok;
 	char *s, *url;
 	xmlChar *alt, *src;
+	enum css_width_e wtype;
+	enum css_height_e htype;
+	css_fixed value = 0;
+	css_unit wunit = CSS_UNIT_PX;
+	css_unit hunit = CSS_UNIT_PX;
 
 	if (box->style && css_computed_display(box->style, 
 			n->parent == NULL) == CSS_DISPLAY_NONE)
@@ -1175,7 +1155,7 @@ bool box_image(BOX_SPECIAL_PARAMS)
 	/* get image URL */
 	if (!(src = xmlGetProp(n, (const xmlChar *) "src")))
 		return true;
-	if (!box_extract_link((char *) src, content->data.html.base_url, &url))
+	if (!box_extract_link((char *) src, content->base_url, &url))
 		return false;
 	xmlFree(src);
 	if (!url)
@@ -1183,8 +1163,19 @@ bool box_image(BOX_SPECIAL_PARAMS)
 
 	/* start fetch */
 	ok = html_fetch_object(content, url, box, image_types,
-			content->available_width, 1000, false);
+			content->base.available_width, 1000, false);
 	free(url);
+
+	wtype = css_computed_width(box->style, &value, &wunit);
+	htype = css_computed_height(box->style, &value, &hunit);
+
+	if (wtype == CSS_WIDTH_SET && wunit != CSS_UNIT_PCT &&
+			htype == CSS_HEIGHT_SET && hunit != CSS_UNIT_PCT) {
+		/* We know the dimensions the image will be shown at before it's
+		 * fetched. */
+		box->flags |= REPLACE_DIM;
+	}
+
 	return ok;
 }
 
@@ -1223,13 +1214,13 @@ bool box_object(BOX_SPECIAL_PARAMS)
 	 * (codebase is the base for the other two) */
 	if ((codebase = xmlGetProp(n, (const xmlChar *) "codebase"))) {
 		if (!box_extract_link((char *) codebase,
-				content->data.html.base_url,
+				content->base_url,
 				&params->codebase))
 			return false;
 		xmlFree(codebase);
 	}
 	if (!params->codebase)
-		params->codebase = content->data.html.base_url;
+		params->codebase = content->base_url;
 
 	if ((classid = xmlGetProp(n, (const xmlChar *) "classid"))) {
 		if (!box_extract_link((char *) classid, params->codebase,
@@ -1250,14 +1241,10 @@ bool box_object(BOX_SPECIAL_PARAMS)
 		return true;
 
 	/* Don't include ourself */
-	if (params->classid &&
-			strcmp(content->data.html.base_url,
-				params->classid) == 0)
+	if (params->classid && strcmp(content->base_url, params->classid) == 0)
 		return true;
 
-	if (params->data &&
-			strcmp(content->data.html.base_url,
-				params->data) == 0)
+	if (params->data && strcmp(content->base_url, params->data) == 0)
 		return true;
 
 	/* codetype and type are MIME types */
@@ -1270,15 +1257,44 @@ bool box_object(BOX_SPECIAL_PARAMS)
 	 * (classid || !classid) && data => data is used (consult type)
 	 * !classid && !data => invalid; ignored */
 
-	if (params->classid && !params->data && params->codetype &&
-			content_lookup(params->codetype) == CONTENT_OTHER)
-		/* can't handle this MIME type */
-		return true;
+	if (params->classid != NULL && params->data == NULL && 
+			params->codetype != NULL) {
+		lwc_string *icodetype;
+		lwc_error lerror;
 
-	if (params->data && params->type &&
-			content_lookup(params->type) == CONTENT_OTHER)
-		/* can't handle this MIME type */
-		return true;
+		lerror = lwc_intern_string(params->codetype, 
+				strlen(params->codetype), &icodetype);
+		if (lerror != lwc_error_ok)
+			return false;
+
+		if (content_factory_type_from_mime_type(icodetype) ==
+				CONTENT_NONE) {
+			/* can't handle this MIME type */
+			lwc_string_unref(icodetype);
+			return true;
+		}
+
+		lwc_string_unref(icodetype);
+	}
+
+	if (params->data != NULL && params->type != NULL) {
+		lwc_string *itype;
+		lwc_error lerror;
+
+		lerror = lwc_intern_string(params->type, strlen(params->type),
+				&itype);
+		if (lerror != lwc_error_ok)
+			return false;
+
+		if (content_factory_type_from_mime_type(itype) == 
+				CONTENT_NONE) {
+			/* can't handle this MIME type */
+			lwc_string_unref(itype);
+			return true;
+		}
+
+		lwc_string_unref(itype);
+	}
 
 	/* add parameters to linked list */
 	for (c = n->children; c; c = c->next) {
@@ -1322,7 +1338,8 @@ bool box_object(BOX_SPECIAL_PARAMS)
 	/* start fetch (MIME type is ok or not specified) */
 	if (!html_fetch_object(content,
 			params->data ? params->data : params->classid,
-			box, 0, content->available_width, 1000, false))
+			box, CONTENT_ANY, content->base.available_width, 1000, 
+			false))
 		return false;
 
 	*convert_children = false;
@@ -1330,144 +1347,6 @@ bool box_object(BOX_SPECIAL_PARAMS)
 }
 
 
-#if 0 /**
- * "Java applet" [13.4].
- *
- * \todo This needs reworking to be compliant to the spec
- * For now, we simply ignore all applet tags.
- */
-
-struct box_result box_applet(xmlNode *n, struct box_status *status,
-		struct css_style *style)
-{
-	struct box *box;
-	struct object_params *po;
-	struct object_param *pp = NULL;
-	char *s;
-	xmlNode *c;
-
-	po = calloc(1, sizeof(struct object_params));
-	if (!po)
-		return (struct box_result) {0, false, true};
-
-	box = box_create(style, false, status->href, 0, status->id,
-			status->content->data.html.box_pool);
-	if (!box) {
-		free(po);
-		return (struct box_result) {0, false, true};
-	}
-
-	/* archive */
-	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "archive")) != NULL) {
-		/** \todo tokenise this comma separated list */
-		LOG(("archive '%s'", s));
-		po->data = strdup(s);
-		xmlFree(s);
-		if (!po->data)
-			goto no_memory;
-	}
-	/* code */
-	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "code")) != NULL) {
-		LOG(("applet '%s'", s));
-		po->classid = strdup(s);
-		xmlFree(s);
-		if (!po->classid)
-			goto no_memory;
-	}
-
-	/* object codebase */
-	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "codebase")) != NULL) {
-		po->codebase = strdup(s);
-		LOG(("codebase: %s", s));
-		xmlFree(s);
-		if (!po->codebase)
-			goto no_memory;
-	}
-
-	/* parameters
-	 * parameter data is stored in a singly linked list.
-	 * po->params points to the head of the list.
-	 * new parameters are added to the head of the list.
-	 */
-	for (c = n->children; c != 0; c = c->next) {
-		if (c->type != XML_ELEMENT_NODE)
-			continue;
-
-		if (strcmp((const char *) c->name, "param") == 0) {
-			pp = calloc(1, sizeof(struct object_param));
-			if (!pp)
-				goto no_memory;
-
-			if ((s = (char *) xmlGetProp(c,
-					(const xmlChar *) "name")) != NULL) {
-				pp->name = strdup(s);
-				xmlFree(s);
-				if (!pp->name)
-					goto no_memory;
-			}
-			if ((s = (char *) xmlGetProp(c,
-					(const xmlChar *) "value")) != NULL) {
-				pp->value = strdup(s);
-				xmlFree(s);
-				if (!pp->value)
-					goto no_memory;
-			}
-			if ((s = (char *) xmlGetProp(c,
-					(const xmlChar *) "type")) != NULL) {
-				pp->type = strdup(s);
-				xmlFree(s);
-				if (!pp->type)
-					goto no_memory;
-			}
-			if ((s = (char *) xmlGetProp(c, (const xmlChar *)
-							"valuetype")) != NULL) {
-				pp->valuetype = strdup(s);
-				xmlFree(s);
-				if (!pp->valuetype)
-					goto no_memory;
-			} else {
-				pp->valuetype = strdup("data");
-				if (!pp->valuetype)
-					goto no_memory;
-			}
-
-			pp->next = po->params;
-			po->params = pp;
-		} else {
-				 /* The first non-param child is the start
-				  * of the alt html. Therefore, we should
-				  * break out of this loop.
-				  */
-				break;
-		}
-	}
-
-	box->object_params = po;
-
-	/* start fetch */
-	if (plugin_decode(status->content, box))
-		return (struct box_result) {box, false, false};
-
-	return (struct box_result) {box, true, false};
-
-no_memory:
-	if (pp && pp != po->params) {
-		/* ran out of memory creating parameter struct */
-		free(pp->name);
-		free(pp->value);
-		free(pp->type);
-		free(pp->valuetype);
-		free(pp);
-	}
-
-	box_free_object_params(po);
-	box_free_box(box);
-
-	return (struct box_result) {0, false, true};
-}
-#endif
-
-
 /**
  * Window subdivision [16.2.1].
  */
@@ -1476,7 +1355,7 @@ bool box_frameset(BOX_SPECIAL_PARAMS)
 {
 	bool ok;
 
-	if (content->data.html.frameset) {
+	if (content->frameset) {
 		LOG(("Error: multiple framesets in document."));
 		/* Don't convert children */
 		if (convert_children)
@@ -1486,12 +1365,11 @@ bool box_frameset(BOX_SPECIAL_PARAMS)
 		return true;
 	}
 
-	content->data.html.frameset = talloc_zero(content,
-						struct content_html_frames);
-	if (!content->data.html.frameset)
+	content->frameset = talloc_zero(content, struct content_html_frames);
+	if (!content->frameset)
 		return false;
 
-	ok = box_create_frameset(content->data.html.frameset, n, content);
+	ok = box_create_frameset(content->frameset, n, content);
 	if (ok)
 		box->type = BOX_NONE;
 
@@ -1501,7 +1379,7 @@ bool box_frameset(BOX_SPECIAL_PARAMS)
 }
 
 bool box_create_frameset(struct content_html_frames *f, xmlNode *n,
-		struct content *content) {
+		html_content *content) {
 	unsigned int row, col, index, i;
 	unsigned int rows = 1, cols = 1;
 	char *s, *url;
@@ -1618,15 +1496,14 @@ bool box_create_frameset(struct content_html_frames *f, xmlNode *n,
 			url = NULL;
 			if ((s = (char *) xmlGetProp(c,
 					(const xmlChar *) "src"))) {
-				box_extract_link(s, content->data.html.base_url,
-									&url);
+				box_extract_link(s, content->base_url, &url);
 				xmlFree(s);
 			}
 
 			/* copy url */
 			if (url) {
 			  	/* no self-references */
-			  	if (strcmp(content->data.html.base_url, url))
+			  	if (strcmp(content->base_url, url))
 					frame->url = talloc_strdup(content,
 									url);
 				free(url);
@@ -1711,7 +1588,7 @@ bool box_iframe(BOX_SPECIAL_PARAMS)
 	if (!(s = (char *) xmlGetProp(n,
 			(const xmlChar *) "src")))
 		return true;
-	if (!box_extract_link(s, content->data.html.base_url, &url)) {
+	if (!box_extract_link(s, content->base_url, &url)) {
 		xmlFree(s);
 		return false;
 	}
@@ -1720,7 +1597,7 @@ bool box_iframe(BOX_SPECIAL_PARAMS)
 		return true;
 
 	/* don't include ourself */
-	if (strcmp(content->data.html.base_url, url) == 0) {
+	if (strcmp(content->base_url, url) == 0) {
 		free(url);
 		return true;
 	}
@@ -1740,17 +1617,15 @@ bool box_iframe(BOX_SPECIAL_PARAMS)
 	iframe->border = true;
 
 	/* Add this iframe to the linked list of iframes */
-	iframe->next = content->data.html.iframe;
-	content->data.html.iframe = iframe;
+	iframe->next = content->iframe;
+	content->iframe = iframe;
 
 	/* fill in specified values */
-	if ((s = (char *) xmlGetProp(n,
-			(const xmlChar *) "name"))) {
+	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "name"))) {
 		iframe->name = talloc_strdup(content, s);
 		xmlFree(s);
 	}
-	if ((s = (char *) xmlGetProp(n,
-			(const xmlChar *) "frameborder"))) {
+	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "frameborder"))) {
 		i = atoi(s);
 		iframe->border = (i != 0);
 		xmlFree(s);
@@ -1763,21 +1638,18 @@ bool box_iframe(BOX_SPECIAL_PARAMS)
 
 		xmlFree(s);
 	}
-	if ((s = (char *) xmlGetProp(n,
-			(const xmlChar *) "scrolling"))) {
+	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "scrolling"))) {
 		if (!strcasecmp(s, "yes"))
 			iframe->scrolling = SCROLLING_YES;
 		else if (!strcasecmp(s, "no"))
 			iframe->scrolling = SCROLLING_NO;
 		xmlFree(s);
 	}
-	if ((s = (char *) xmlGetProp(n,
-			(const xmlChar *) "marginwidth"))) {
+	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "marginwidth"))) {
 		iframe->margin_width = atoi(s);
 		xmlFree(s);
 	}
-	if ((s = (char *) xmlGetProp(n,
-			(const xmlChar *) "marginheight"))) {
+	if ((s = (char *) xmlGetProp(n, (const xmlChar *) "marginheight"))) {
 		iframe->margin_height = atoi(s);
 		xmlFree(s);
 	}
@@ -1786,8 +1658,8 @@ bool box_iframe(BOX_SPECIAL_PARAMS)
 	free(url);
 
 	/* box */
-	box->type = BOX_INLINE_BLOCK;
 	assert(box->style);
+	box->flags |= IFRAME;
 
 	/* Showing iframe, so don't show alternate content */
 	if (convert_children)
@@ -1808,8 +1680,7 @@ bool box_input(BOX_SPECIAL_PARAMS)
 
 	type = (char *) xmlGetProp(n, (const xmlChar *) "type");
 
-	gadget = binding_get_control_for_node(content->data.html.parser_binding,
-			n);
+	gadget = binding_get_control_for_node(content->parser_binding, n);
 	if (!gadget)
 		goto no_memory;
 	box->gadget = gadget;
@@ -1874,8 +1745,7 @@ bool box_input(BOX_SPECIAL_PARAMS)
 				n->parent == NULL) != CSS_DISPLAY_NONE) {
 			if ((s = (char *) xmlGetProp(n,
 					(const xmlChar*) "src"))) {
-				res = url_join(s,
-					content->data.html.base_url, &url);
+				res = url_join(s, content->base_url, &url);
 				xmlFree(s);
 				/* if url is equivalent to the parent's url,
 				 * we've got infinite inclusion. stop it here
@@ -1883,11 +1753,10 @@ bool box_input(BOX_SPECIAL_PARAMS)
 				 */
 				if (res == URL_FUNC_OK &&
 						strcasecmp(url,
-						content->data.
-						html.base_url) != 0) {
+						content->base_url) != 0) {
 					if (!html_fetch_object(content, url,
 							box, image_types,
-							content->
+							content->base.
 							available_width,
 							1000, false)) {
 						free(url);
@@ -1970,8 +1839,7 @@ bool box_button(BOX_SPECIAL_PARAMS)
 {
 	struct form_control *gadget;
 
-	gadget = binding_get_control_for_node(content->data.html.parser_binding,
-			n);
+	gadget = binding_get_control_for_node(content->parser_binding, n);
 	if (!gadget)
 		return false;
 
@@ -1997,8 +1865,7 @@ bool box_select(BOX_SPECIAL_PARAMS)
 	struct form_control *gadget;
 	xmlNode *c, *c2;
 
-	gadget = binding_get_control_for_node(content->data.html.parser_binding,
-			n);
+	gadget = binding_get_control_for_node(content->parser_binding, n);
 	if (!gadget)
 		return false;
 
@@ -2144,8 +2011,7 @@ bool box_textarea(BOX_SPECIAL_PARAMS)
 	size_t len;
 
 	box->type = BOX_INLINE_BLOCK;
-	box->gadget = binding_get_control_for_node(
-			content->data.html.parser_binding, n);
+	box->gadget = binding_get_control_for_node(content->parser_binding, n);
 	if (!box->gadget)
 		return false;
 	box->gadget->box = box;
@@ -2274,15 +2140,14 @@ bool box_embed(BOX_SPECIAL_PARAMS)
 	/* src is a URL */
 	if (!(src = xmlGetProp(n, (const xmlChar *) "src")))
 		return true;
-	if (!box_extract_link((char *) src, content->data.html.base_url,
-			&params->data))
+	if (!box_extract_link((char *) src, content->base_url, &params->data))
 		return false;
 	xmlFree(src);
 	if (!params->data)
 		return true;
 
 	/* Don't include ourself */
-	if (strcmp(content->data.html.base_url, params->data) == 0)
+	if (strcmp(content->base_url, params->data) == 0)
 		return true;
 
 	/* add attributes as parameters to linked list */
@@ -2312,8 +2177,8 @@ bool box_embed(BOX_SPECIAL_PARAMS)
 	box->object_params = params;
 
 	/* start fetch */
-	return html_fetch_object(content, params->data, box, 0,
-			content->available_width, 1000, false);
+	return html_fetch_object(content, params->data, box, CONTENT_ANY,
+			content->base.available_width, 1000, false);
 }
 
 /**
