@@ -41,11 +41,12 @@
 #include "desktop/knockout.h"
 #include "desktop/plot_style.h"
 #include "desktop/plotters.h"
-#include "desktop/scroll.h"
+#include "desktop/scrollbar.h"
 #include "render/box.h"
 #include "render/font.h"
 #include "render/form.h"
 #include "render/html.h"
+#include "render/html_internal.h"
 #include "render/layout.h"
 #include "utils/log.h"
 #include "utils/messages.h"
@@ -62,12 +63,12 @@
 struct form_select_menu {
 	int line_height;
 	int width, height;
-	struct scroll *scroll;
+	struct scrollbar *scrollbar;
 	int f_size;
 	bool scroll_capture;
 	select_menu_redraw_callback callback;
 	void *client_data;
-	struct browser_window *bw;
+	struct content *c;
 };
 
 static plot_style_t plot_style_fill_selected = {
@@ -90,7 +91,7 @@ static char *form_encode_item(const char *item, const char *charset,
 static void form_select_menu_clicked(struct form_control *control,
 		int x, int y);
 static void form_select_menu_scroll_callback(void *client_data,
-		struct scroll_msg_data *scroll_data);
+		struct scrollbar_msg_data *scrollbar_data);
 
 /**
  * Create a struct form.
@@ -868,86 +869,72 @@ char *form_encode_item(const char *item, const char *charset,
 bool form_open_select_menu(void *client_data,
 		struct form_control *control,
 		select_menu_redraw_callback callback,
-		struct browser_window *bw)
+		struct content *c)
 {
-	int i, line_height_with_spacing, scroll;
-	struct form_option *option;
+	int line_height_with_spacing;
 	struct box *box;
 	plot_font_style_t fstyle;
 	int total_height;
 	struct form_select_menu *menu;
-		
+
 
 	/* if the menu is opened for the first time */
 	if (control->data.select.menu == NULL) {
-		
+
 		menu = calloc(1, sizeof (struct form_select_menu));
 		if (menu == NULL) {
 			warn_user("NoMemory", 0);
 			return false;
 		}
-		
+
 		control->data.select.menu = menu;
-				
+
 		box = control->box;
 
 		menu->width = box->width +
 				box->border[RIGHT].width +
 				box->border[LEFT].width +
 				box->padding[RIGHT] + box->padding[LEFT];
-		
+
 		font_plot_style_from_css(control->box->style,
 				&fstyle);
 		menu->f_size = fstyle.size;
-		
-		menu->line_height = FIXTOINT(FDIV((FMUL(FLTTOFIX(1.2), FMUL(nscss_screen_dpi, INTTOFIX((fstyle.size / FONT_SIZE_SCALE))))), F_72));
-		
+
+		menu->line_height = FIXTOINT(FDIV((FMUL(FLTTOFIX(1.2),
+				FMUL(nscss_screen_dpi,
+				INTTOFIX(fstyle.size / FONT_SIZE_SCALE)))),
+				F_72));
+
 		line_height_with_spacing = menu->line_height +
 				menu->line_height *
 				SELECT_LINE_SPACING;
-		
+
 		total_height = control->data.select.num_items *
 				line_height_with_spacing;
 		menu->height = total_height;
-	
-		scroll = 0;
+
 		if (menu->height > MAX_SELECT_HEIGHT) {
-			
+
 			menu->height = MAX_SELECT_HEIGHT;
-			
-			if (control->data.select.num_selected > 0) {
-				i = 0;
-				option = control->data.select.items;
-				while (!option->selected) {
-					option = option->next;
-					i++;
-				}
-				
-				if ((i + 1) * line_height_with_spacing >
-						MAX_SELECT_HEIGHT)
-					scroll = (i + 1) *
-							line_height_with_spacing
-							- MAX_SELECT_HEIGHT;
-			}
 		}
 		menu->client_data = client_data;
 		menu->callback = callback;
-		if (!scroll_create(false,
+		if (!scrollbar_create(false,
 				menu->height,
     				total_height,
 				menu->height,
 				control,
 				form_select_menu_scroll_callback,
-				&(menu->scroll))) {
+				&(menu->scrollbar))) {
 			free(menu);
 			return false;
 		}
-		menu->bw = bw;
+		menu->c = c;
 	}
 	else menu = control->data.select.menu;
-		
+
 	menu->callback(client_data, 0, 0, menu->width, menu->height);
-	
+
 	return true;
 }
 
@@ -960,8 +947,8 @@ bool form_open_select_menu(void *client_data,
  */
 void form_free_select_menu(struct form_control *control)
 {
-	if (control->data.select.menu->scroll != NULL)
-		scroll_destroy(control->data.select.menu->scroll);
+	if (control->data.select.menu->scrollbar != NULL)
+		scrollbar_destroy(control->data.select.menu->scrollbar);
 	free(control->data.select.menu);
 	control->data.select.menu = NULL;
 }
@@ -973,15 +960,15 @@ void form_free_select_menu(struct form_control *control)
  * \param x		the X coordinate to draw the menu at
  * \param x		the Y coordinate to draw the menu at
  * \param scale		current redraw scale
- * \param clip_x0	minimum x of clipping rectangle
- * \param clip_y0	minimum y of clipping rectangle
- * \param clip_x1	maximum x of clipping rectangle
- * \param clip_y1	maximum y of clipping rectangle
+ * \param clip		clipping rectangle
+ * \param ctx		current redraw context
  * \return		true on success, false otherwise
  */
 bool form_redraw_select_menu(struct form_control *control, int x, int y,
-		float scale, const struct rect *clip)
+		float scale, const struct rect *clip,
+		const struct redraw_context *ctx)
 {
+	const struct plotter_table *plot = ctx->plot;
 	struct box *box;
 	struct form_select_menu *menu = control->data.select.menu;
 	struct form_option *option;
@@ -1006,7 +993,7 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
 	
 	line_height_with_spacing = line_height +
 			line_height * SELECT_LINE_SPACING;
-	scroll = scroll_get_offset(menu->scroll);
+	scroll = scrollbar_get_offset(menu->scrollbar);
 	
 	if (scale != 1.0) {
 		x *= scale;
@@ -1034,9 +1021,9 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
 	r.y0 = y0;
 	r.x1 = x1 + 1;
 	r.y1 = y1 + 1;
-	if (!plot.clip(&r))
+	if (!plot->clip(&r))
 		return false;
-	if (!plot.rectangle(x0, y0, x1, y1 ,plot_style_stroke_darkwbasec))
+	if (!plot->rectangle(x0, y0, x1, y1 ,plot_style_stroke_darkwbasec))
 		return false;
 		
 	
@@ -1050,9 +1037,9 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
 	r.y0 = y0;
 	r.x1 = x1 + 1;
 	r.y1 = y1 + 1;
-	if (!plot.clip(&r))
+	if (!plot->clip(&r))
 		return false;
-	if (!plot.rectangle(x0, y0, x1 + 1, y1 + 1,
+	if (!plot->rectangle(x0, y0, x1 + 1, y1 + 1,
 			plot_style_fill_lightwbasec))
 		return false;
 	option = control->data.select.items;
@@ -1074,7 +1061,7 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
  		if (option->selected) {
  			y2 = y + item_y - scroll;
  			y3 = y + item_y + line_height_with_spacing - scroll;
- 			if (!plot.rectangle(x0, (y0 > y2 ? y0 : y2),
+ 			if (!plot->rectangle(x0, (y0 > y2 ? y0 : y2),
 					scrollbar_x + 1,
      					(y3 < y1 + 1 ? y3 : y1 + 1),
 					&plot_style_fill_selected))
@@ -1082,7 +1069,7 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
  		}
 		
 		y2 = text_pos_offset + item_y;
-		if (!plot.text(text_x, y2, option->text,
+		if (!plot->text(text_x, y2, option->text,
 				strlen(option->text), &plot_fstyle_entry))
 			return false;
 		
@@ -1090,10 +1077,10 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
 		option = option->next;
 	}
 		
-	if (!scroll_redraw(menu->scroll,
+	if (!scrollbar_redraw(menu->scrollbar,
 			x_cp + menu->width - SCROLLBAR_WIDTH,
       			y_cp,
-			clip, scale))
+			clip, scale, ctx))
 		return false;
 	
 	return true;
@@ -1143,12 +1130,13 @@ bool form_clip_inside_select_menu(struct form_control *control, float scale,
 void form_select_menu_clicked(struct form_control *control, int x, int y)
 {	
 	struct form_select_menu *menu = control->data.select.menu;
-	struct form_option *option;	
+	struct form_option *option;
+	html_content *html = (html_content *)menu->c;
 	int line_height, line_height_with_spacing;	
 	int item_bottom_y;
 	int scroll, i;
 	
-	scroll = scroll_get_offset(menu->scroll);
+	scroll = scrollbar_get_offset(menu->scrollbar);
 	
 	line_height = menu->line_height;
 	line_height_with_spacing = line_height +
@@ -1163,9 +1151,11 @@ void form_select_menu_clicked(struct form_control *control, int x, int y)
 		i++;
 	}
 	
-	if (option != NULL)
-		form_select_process_selection(menu->bw->current_content,
+	if (option != NULL) {
+		/* TODO: going via the bw to get a hlcache_handle is nasty */
+		form_select_process_selection(html->bw->current_content,
 				control, i);
+	}
 	
 	menu->callback(menu->client_data, 0, 0, menu->width, menu->height);
 }
@@ -1200,7 +1190,7 @@ const char *form_select_mouse_action(struct form_control *control,
 		 * event is taking place on the scrollbar widget area
 		 */
 		x -= scrollbar_x;
-		return scroll_mouse_action(menu->scroll,
+		return scrollbar_mouse_action(menu->scrollbar,
 				    mouse, x, y);
 	}
 	
@@ -1255,7 +1245,7 @@ void form_select_mouse_drag_end(struct form_control *control,
 
 	if (menu->scroll_capture) {
 		x -= menu->width - SCROLLBAR_WIDTH;
-		scroll_mouse_drag_end(menu->scroll, mouse, x, y);
+		scrollbar_mouse_drag_end(menu->scrollbar, mouse, x, y);
 		return;
 	}
 	
@@ -1274,33 +1264,39 @@ void form_select_mouse_drag_end(struct form_control *control,
  * Callback for the select menus scroll
  */
 void form_select_menu_scroll_callback(void *client_data,
-		struct scroll_msg_data *scroll_data)
+		struct scrollbar_msg_data *scrollbar_data)
 {
 	struct form_control *control = client_data;
 	struct form_select_menu *menu = control->data.select.menu;
+	html_content *html = (html_content *)menu->c;
+	struct browser_window *root_bw;
 	
-	switch (scroll_data->msg) {
-		case SCROLL_MSG_REDRAW:
+	switch (scrollbar_data->msg) {
+		case SCROLLBAR_MSG_REDRAW:
 			menu->callback(menu->client_data,
 				       	menu->width -
-					SCROLLBAR_WIDTH + scroll_data->x0,
-     					scroll_data->y0,
-					scroll_data->x1 - scroll_data->x0,
-					scroll_data->y1 - scroll_data->y0);
+					SCROLLBAR_WIDTH + scrollbar_data->x0,
+     					scrollbar_data->y0,
+					scrollbar_data->x1 - scrollbar_data->x0,
+					scrollbar_data->y1 - scrollbar_data->y0);
 			break;
-		case SCROLL_MSG_MOVED:
+		case SCROLLBAR_MSG_MOVED:
 			menu->callback(menu->client_data,
 				    	0, 0,
 					menu->width - SCROLLBAR_WIDTH,
      					menu->height);
 			break;
-		case SCROLL_MSG_SCROLL_START:
+		case SCROLLBAR_MSG_SCROLL_START:
+			browser_window_set_drag_type(html->bw, DRAGGING_OTHER);
+
 			menu->scroll_capture = true;
-			gui_window_box_scroll_start(menu->bw->window,
-					scroll_data->x0, scroll_data->y0,
-     					scroll_data->x1, scroll_data->y1);
+
+			root_bw = browser_window_get_root(html->bw);
+			gui_window_box_scroll_start(root_bw->window,
+					scrollbar_data->x0, scrollbar_data->y0,
+     					scrollbar_data->x1, scrollbar_data->y1);
 			break;
-		case SCROLL_MSG_SCROLL_FINISHED:
+		case SCROLLBAR_MSG_SCROLL_FINISHED:
 			menu->scroll_capture = false;
 			break;
 		default:
@@ -1397,18 +1393,18 @@ void form_select_process_selection(hlcache_handle *h,
 void form_select_menu_callback(void *client_data,
 		int x, int y, int width, int height)
 {
-	struct browser_window *bw = client_data;
+	html_content *html = client_data;
 	int menu_x, menu_y;
 	struct box *box;
 	
-	box = bw->visible_select_menu->box;
+	box = html->visible_select_menu->box;
 	box_coords(box, &menu_x, &menu_y);
 		
 	menu_x -= box->border[LEFT].width;
 	menu_y += box->height + box->border[BOTTOM].width +
 			box->padding[BOTTOM] +
 			box->padding[TOP];
-	browser_window_redraw_rect(bw, menu_x + x, menu_y + y,
+	content__request_redraw((struct content *)html, menu_x + x, menu_y + y,
 			width, height);
 }
 
@@ -1466,7 +1462,6 @@ void form_submit(hlcache_handle *h, struct browser_window *target,
 	url_func_result res;
 
 	assert(form != NULL);
-	assert(content_get_type(h) == CONTENT_HTML);
 
 	if (form_successful_controls(form, submit_button, &success) == false) {
 		warn_user("NoMemory", 0);
