@@ -31,14 +31,15 @@
 #include "desktop/frames.h"
 #include "desktop/mouse.h"
 #include "desktop/options.h"
-#include "desktop/scroll.h"
+#include "desktop/scrollbar.h"
 #include "desktop/selection.h"
 #include "desktop/textinput.h"
 #include "render/box.h"
 #include "render/font.h"
 #include "render/form.h"
-#include "render/html.h"
+#include "render/html_internal.h"
 #include "render/imagemap.h"
+#include "render/textinput.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
 
@@ -47,6 +48,39 @@ static gui_pointer_shape get_pointer_shape(struct browser_window *bw,
 		struct box *box, bool imagemap);
 static void html_box_drag_start(struct box *box, int x, int y);
 
+
+/**
+ * End overflow scroll scrollbar drags
+ *
+ * \param  h      html content's high level cache entry
+ * \param  mouse  state of mouse buttons and modifier keys
+ * \param  x	  coordinate of mouse
+ * \param  y	  coordinate of mouse
+ */
+static size_t html_selection_drag_end(struct html_content *html,
+		browser_mouse_state mouse, int x, int y, int dir)
+{
+	int pixel_offset;
+	struct box *box;
+	int dx, dy;
+	size_t idx = 0;
+
+	box = box_pick_text_box(html, x, y, dir, &dx, &dy);
+	if (box) {
+		plot_font_style_t fstyle;
+
+		font_plot_style_from_css(box->style, &fstyle);
+
+		nsfont.font_position_in_string(&fstyle, box->text, box->length,
+				dx, &idx, &pixel_offset);
+
+		idx += box->byte_offset;
+	}
+
+	return idx;
+}
+
+
 /**
  * Handle mouse tracking (including drags) in an HTML content window.
  *
@@ -60,7 +94,22 @@ static void html_box_drag_start(struct box *box, int x, int y);
 void html_mouse_track(struct content *c, struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
-	hlcache_handle *h = bw->current_content;
+	html_content *html = (html_content*) c;
+
+	if (bw->drag_type == DRAGGING_SELECTION && !mouse) {
+		int dir = -1;
+		size_t idx;
+
+		if (selection_dragging_start(&html->sel))
+			dir = 1;
+
+		idx = html_selection_drag_end(html, mouse, x, y, dir);
+
+		if (idx != 0)
+			selection_track(&html->sel, mouse, idx);
+
+		browser_window_set_drag_type(bw, DRAGGING_NONE);
+	}
 
 	switch (bw->drag_type) {
 		case DRAGGING_SELECTION: {
@@ -68,10 +117,11 @@ void html_mouse_track(struct content *c, struct browser_window *bw,
 			int dir = -1;
 			int dx, dy;
 
-			if (selection_dragging_start(bw->sel)) dir = 1;
+			if (selection_dragging_start(&html->sel))
+				dir = 1;
+
+			box = box_pick_text_box(html, x, y, dir, &dx, &dy);
 
-			box = box_pick_text_box(h, x, y, dir,
-					&dx, &dy);
 			if (box) {
 				int pixel_offset;
 				size_t idx;
@@ -83,7 +133,7 @@ void html_mouse_track(struct content *c, struct browser_window *bw,
 						box->text, box->length,
 						dx, &idx, &pixel_offset);
 
-				selection_track(bw->sel, mouse,
+				selection_track(&html->sel, mouse,
 						box->byte_offset + idx);
 			}
 		}
@@ -115,6 +165,7 @@ void html_mouse_track(struct content *c, struct browser_window *bw,
 void html_mouse_action(struct content *c, struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
+	html_content *html = (html_content *) c;
 	enum { ACTION_NONE, ACTION_SUBMIT, ACTION_GO } action = ACTION_NONE;
 	char *title = 0;
 	const char *url = 0;
@@ -135,61 +186,76 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	hlcache_handle *gadget_content = h;
 	struct form_control *gadget = 0;
 	hlcache_handle *object = NULL;
+	struct browser_window *iframe = NULL;
 	struct box *next_box;
 	struct box *drag_candidate = NULL;
-	struct scroll *scroll = NULL;
+	struct scrollbar *scrollbar = NULL;
 	plot_font_style_t fstyle;
 	int scroll_mouse_x = 0, scroll_mouse_y = 0;
 	int padding_left, padding_right, padding_top, padding_bottom;
-	
 
-	if (bw->visible_select_menu != NULL) {
-		box = bw->visible_select_menu->box;
+	if (bw->drag_type != DRAGGING_NONE && !mouse &&
+			html->visible_select_menu != NULL) {
+		/* drag end: select menu */
+		form_select_mouse_drag_end(html->visible_select_menu,
+				mouse, x, y);
+	}
+
+	if (html->visible_select_menu != NULL) {
+		box = html->visible_select_menu->box;
 		box_coords(box, &box_x, &box_y);
 
 		box_x -= box->border[LEFT].width;
 		box_y += box->height + box->border[BOTTOM].width +
 				box->padding[BOTTOM] + box->padding[TOP];
-		status = form_select_mouse_action(bw->visible_select_menu,
+		status = form_select_mouse_action(html->visible_select_menu,
 				mouse, x - box_x, y - box_y);
 		if (status != NULL)
 			browser_window_set_status(bw, status);
 		else {
 			int width, height;
-			form_select_get_dimensions(bw->visible_select_menu,
+			form_select_get_dimensions(html->visible_select_menu,
 					&width, &height);
-			bw->visible_select_menu = NULL;
+			html->visible_select_menu = NULL;
 			browser_window_redraw_rect(bw, box_x, box_y,
 					width, height);					
 		}
 		return;
 	}
-	
-	if (bw->scroll != NULL) {
-		struct browser_scroll_data *data = scroll_get_data(bw->scroll);
+
+	if (bw->drag_type != DRAGGING_NONE && !mouse &&
+			html->scrollbar != NULL) {
+		/* drag end: scrollbar */
+		html_overflow_scroll_drag_end(html->scrollbar, mouse, x, y);
+	}
+
+	if (html->scrollbar != NULL) {
+		struct html_scrollbar_data *data =
+				scrollbar_get_data(html->scrollbar);
 		box = data->box;
 		box_coords(box, &box_x, &box_y);
-		if (scroll_is_horizontal(bw->scroll)) {
+		if (scrollbar_is_horizontal(html->scrollbar)) {
 			scroll_mouse_x = x - box_x ;
 			scroll_mouse_y = y - (box_y + box->padding[TOP] +
 					box->height + box->padding[BOTTOM] -
 					SCROLLBAR_WIDTH);
-			status = scroll_mouse_action(bw->scroll, mouse,
+			status = scrollbar_mouse_action(html->scrollbar, mouse,
 					scroll_mouse_x, scroll_mouse_y);
 		} else {
 			scroll_mouse_x = x - (box_x + box->padding[LEFT] +
 					box->width + box->padding[RIGHT] -
 					SCROLLBAR_WIDTH);
 			scroll_mouse_y = y - box_y;
-			status = scroll_mouse_action(bw->scroll, mouse, 
+			status = scrollbar_mouse_action(html->scrollbar, mouse, 
 					scroll_mouse_x, scroll_mouse_y);
 		}
 		
 		browser_window_set_status(bw, status);
 		return;
 	}
-	
-	bw->drag_type = DRAGGING_NONE;
+
+	/* Content related drags handled by now */
+	browser_window_set_drag_type(bw, DRAGGING_NONE);
 
 	/* search the box tree for a link, imagemap, form control, or
 	 * box with scrollbars */
@@ -202,8 +268,6 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 
 	while ((next_box = box_at_point(box, x, y, &box_x, &box_y, &content)) !=
 			NULL) {
-		enum css_overflow_e overflow = CSS_OVERFLOW_VISIBLE;
-
 		box = next_box;
 
 		if (box->style && css_computed_visibility(box->style) == 
@@ -213,6 +277,9 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 		if (box->object)
 			object = box->object;
 
+		if (box->iframe)
+			iframe = box->iframe;
+
 		if (box->href) {
 			url = box->href;
 			target = box->target;
@@ -220,7 +287,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 		}
 
 		if (box->usemap) {
-			url = imagemap_get(content, box->usemap,
+			url = imagemap_get(html, box->usemap,
 					box_x, box_y, x, y, &target);
 			if (url) {
 				imagemap = true;
@@ -243,18 +310,17 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 
 		pointer = get_pointer_shape(bw, box, false);
 
-		if (box->style)
-			overflow = css_computed_overflow(box->style);
-
 		if ((box->scroll_x != NULL || box->scroll_y != NULL) &&
 				   drag_candidate == NULL)
 			drag_candidate = box;
 		
 		if (box->scroll_y != NULL || box->scroll_x != NULL) {
-			padding_left = box_x + scroll_get_offset(box->scroll_x);
+			padding_left = box_x +
+					scrollbar_get_offset(box->scroll_x);
 			padding_right = padding_left + box->padding[LEFT] +
 					box->width + box->padding[RIGHT];
-			padding_top = box_y + scroll_get_offset(box->scroll_y);
+			padding_top = box_y +
+					scrollbar_get_offset(box->scroll_y);
 			padding_bottom = padding_top + box->padding[TOP] +
 					box->height + box->padding[BOTTOM];
 			
@@ -266,7 +332,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 						SCROLLBAR_WIDTH) {
 					/* mouse above vertical box scroll */
 					
-					scroll = box->scroll_y;
+					scrollbar = box->scroll_y;
 					scroll_mouse_x = x - (padding_right -
 							     SCROLLBAR_WIDTH);
 					scroll_mouse_y = y - padding_top;
@@ -277,7 +343,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 						SCROLLBAR_WIDTH) {
 					/* mouse above horizontal box scroll */
 							
-					scroll = box->scroll_x;
+					scrollbar = box->scroll_x;
 					scroll_mouse_x = x - padding_left;
 					scroll_mouse_y = y - (padding_bottom -
 							SCROLLBAR_WIDTH);
@@ -295,8 +361,8 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	/* use of box_x, box_y, or content below this point is probably a
 	 * mistake; they will refer to the last box returned by box_at_point */
 
-	if (scroll) {
-		status = scroll_mouse_action(scroll, mouse,
+	if (scrollbar) {
+		status = scrollbar_mouse_action(scrollbar, mouse,
 				scroll_mouse_x, scroll_mouse_y);
 		pointer = GUI_POINTER_DEFAULT;
 	} else if (gadget) {
@@ -306,10 +372,10 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 			pointer = GUI_POINTER_MENU;
 			if (mouse & BROWSER_MOUSE_CLICK_1 &&
 					option_core_select_menu) {
-				bw->visible_select_menu = gadget;
-				form_open_select_menu(bw, gadget,
+				html->visible_select_menu = gadget;
+				form_open_select_menu(c, gadget,
 						form_select_menu_callback,
-						bw);
+						c);
 				pointer =  GUI_POINTER_DEFAULT;
 			} else if (mouse & BROWSER_MOUSE_CLICK_1)
 				gui_create_form_select_menu(bw, gadget);
@@ -353,12 +419,11 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 
 			if (mouse & (BROWSER_MOUSE_PRESS_1 |
 					BROWSER_MOUSE_PRESS_2)) {
-				if (text_box && selection_root(bw->sel) !=
+				if (text_box && selection_root(&html->sel) !=
 						gadget_box)
-					selection_init(bw->sel, gadget_box);
+					selection_init(&html->sel, gadget_box);
 
-				browser_window_textarea_click(bw,
-						mouse,
+				textinput_textarea_click(c, mouse,
 						gadget_box,
 						gadget_box_x,
 						gadget_box_y,
@@ -380,17 +445,17 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 					&idx,
 					&pixel_offset);
 
-				selection_click(bw->sel, mouse,
+				selection_click(&html->sel, mouse,
 						text_box->byte_offset + idx);
 
-				if (selection_dragging(bw->sel)) {
+				if (selection_dragging(&html->sel)) {
 					bw->drag_type = DRAGGING_SELECTION;
 					status = messages_get("Selecting");
 				} else
 					status = content_get_status_message(h);
 			}
 			else if (mouse & BROWSER_MOUSE_PRESS_1)
-				selection_clear(bw->sel, true);
+				selection_clear(&html->sel, true);
 			break;
 		case GADGET_TEXTBOX:
 		case GADGET_PASSWORD:
@@ -400,7 +465,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 			if ((mouse & BROWSER_MOUSE_PRESS_1) &&
 					!(mouse & (BROWSER_MOUSE_MOD_1 |
 					BROWSER_MOUSE_MOD_2))) {
-				browser_window_input_click(bw,
+				textinput_input_click(c,
 						gadget_box,
 						gadget_box_x,
 						gadget_box_y,
@@ -413,7 +478,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 
 				if (mouse & (BROWSER_MOUSE_DRAG_1 |
 						BROWSER_MOUSE_DRAG_2))
-					selection_init(bw->sel, gadget_box);
+					selection_init(&html->sel, gadget_box);
 
 				font_plot_style_from_css(text_box->style,
 						&fstyle);
@@ -425,14 +490,14 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 					&idx,
 					&pixel_offset);
 
-				selection_click(bw->sel, mouse,
+				selection_click(&html->sel, mouse,
 						text_box->byte_offset + idx);
 
-				if (selection_dragging(bw->sel))
+				if (selection_dragging(&html->sel))
 					bw->drag_type = DRAGGING_SELECTION;
 			}
 			else if (mouse & BROWSER_MOUSE_PRESS_1)
-				selection_clear(bw->sel, true);
+				selection_clear(&html->sel, true);
 			break;
 		case GADGET_HIDDEN:
 			/* not possible: no box generated */
@@ -461,6 +526,20 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 		/* \todo should have a drag-saving object msg */
 		status = content_get_status_message(h);
 
+	} else if (iframe) {
+		int pos_x, pos_y;
+		browser_window_get_position(iframe, false, &pos_x, &pos_y);
+		pos_x /= bw->scale;
+		pos_y /= bw->scale;
+
+		if (mouse & BROWSER_MOUSE_CLICK_1 ||
+				mouse & BROWSER_MOUSE_CLICK_2) {
+			browser_window_mouse_click(iframe, mouse,
+					x - pos_x, y - pos_y);
+		} else {
+			browser_window_mouse_track(iframe, mouse,
+					x - pos_x, y - pos_y);
+		}
 	} else if (url) {
 		if (title) {
 			snprintf(status_buffer, sizeof status_buffer, "%s: %s",
@@ -492,7 +571,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 			for (parent = bw->parent; parent->parent;
 					parent = parent->parent);
 			browser_window_resize_frames(parent, mouse,
-					x + bw->x0, y + bw->y0,
+					x + bw->x, y + bw->y,
 					&pointer, &status, &done);
 		}
 
@@ -501,11 +580,10 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 		if (!done) {
 			struct box *layout = html_get_box_tree(h);
 
-			if (text_box &&
-				(mouse & (BROWSER_MOUSE_CLICK_1 |
-						BROWSER_MOUSE_CLICK_2)) &&
-					selection_root(bw->sel) != layout)
-				selection_init(bw->sel, layout);
+			if (mouse && (mouse < BROWSER_MOUSE_MOD_1) &&
+					selection_root(&html->sel) != layout) {
+				selection_init(&html->sel, layout);
+			}
 
 			if (text_box) {
 				int pixel_offset;
@@ -521,13 +599,13 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 					&idx,
 					&pixel_offset);
 
-				if (selection_click(bw->sel, mouse,
+				if (selection_click(&html->sel, mouse,
 						text_box->byte_offset + idx)) {
 					/* key presses must be directed at the
 					 * main browser window, paste text
 					 * operations ignored */
 
-					if (selection_dragging(bw->sel)) {
+					if (selection_dragging(&html->sel)) {
 						bw->drag_type =
 							DRAGGING_SELECTION;
 						status =
@@ -539,7 +617,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 				}
 			}
 			else if (mouse & BROWSER_MOUSE_PRESS_1)
-				selection_clear(bw->sel, true);
+				selection_clear(&html->sel, true);
 		}
 
 		if (!done) {
@@ -584,8 +662,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 				}
 			}
 		}
-		if ((mouse & BROWSER_MOUSE_CLICK_1) &&
-				!selection_defined(bw->sel)) {
+		if (mouse && mouse < BROWSER_MOUSE_MOD_1) {
 			/* ensure key presses still act on the browser window */
 			browser_window_remove_caret(bw);
 		}
@@ -598,7 +675,8 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	if (status != NULL)
 		browser_window_set_status(bw, status);
 
-	browser_window_set_pointer(bw->window, pointer);
+	if (!iframe)
+		browser_window_set_pointer(bw, pointer);
 
 	/* deferred actions that can cause this browser_window to be destroyed
 	 * and must therefore be done after set_status/pointer
@@ -728,53 +806,59 @@ gui_pointer_shape get_pointer_shape(struct browser_window *bw, struct box *box,
 
 
 /**
- * Callback for in-page scrolls.
+ * Callback for in-page scrollbars.
  */
 void html_overflow_scroll_callback(void *client_data,
-		struct scroll_msg_data *scroll_data)
+		struct scrollbar_msg_data *scrollbar_data)
 {
-	struct browser_scroll_data *data = client_data;
-	struct browser_window *bw = data->bw;
+	struct html_scrollbar_data *data = client_data;
+	html_content *html = (html_content *)data->c;
 	struct box *box = data->box;
 	int x, y, box_x, box_y, diff_x, diff_y;
+	struct browser_window *root_bw;
 	
 	
-	switch(scroll_data->msg) {
-		case SCROLL_MSG_REDRAW:
+	switch(scrollbar_data->msg) {
+		case SCROLLBAR_MSG_REDRAW:
 			diff_x = box->padding[LEFT] + box->width +
 					box->padding[RIGHT] - SCROLLBAR_WIDTH;
 			diff_y = box->padding[TOP] + box->height +
 					box->padding[BOTTOM] - SCROLLBAR_WIDTH;
 	
 			box_coords(box, &box_x, &box_y);
-			if (scroll_is_horizontal(scroll_data->scroll)) {
-				x = box_x + scroll_get_offset(box->scroll_x);
-				y = box_y + scroll_get_offset(box->scroll_y) +
+			if (scrollbar_is_horizontal(
+					scrollbar_data->scrollbar)) {
+				x = box_x + scrollbar_get_offset(box->scroll_x);
+				y = box_y + scrollbar_get_offset(box->scroll_y) +
 						diff_y;
 			} else {
-				x = box_x + scroll_get_offset(box->scroll_x) +
+				x = box_x + scrollbar_get_offset(box->scroll_x) +
 						diff_x;
-				y = box_y + scroll_get_offset(box->scroll_y);
+				y = box_y + scrollbar_get_offset(box->scroll_y);
 			}
-			browser_window_redraw_rect(bw,
-					x + scroll_data->x0,
-					y + scroll_data->y0,
-     					scroll_data->x1 - scroll_data->x0,
-					scroll_data->y1 - scroll_data->y0);
+			content__request_redraw((struct content *)html,
+					x + scrollbar_data->x0,
+					y + scrollbar_data->y0,
+     					scrollbar_data->x1 - scrollbar_data->x0,
+					scrollbar_data->y1 - scrollbar_data->y0);
 			break;
-		case SCROLL_MSG_MOVED:
-			html_redraw_a_box(bw->current_content, box);
+		case SCROLLBAR_MSG_MOVED:
+			html_redraw_a_box(html->bw->current_content, box);
 			break;
-		case SCROLL_MSG_SCROLL_START:
-			bw->scroll = scroll_data->scroll;
-			gui_window_box_scroll_start(bw->window,
-					scroll_data->x0, scroll_data->y0,
-     					scroll_data->x1, scroll_data->y1);
+		case SCROLLBAR_MSG_SCROLL_START:
+			browser_window_set_drag_type(html->bw, DRAGGING_OTHER);
+
+			html->scrollbar = scrollbar_data->scrollbar;
+
+			root_bw = browser_window_get_root(html->bw);
+			gui_window_box_scroll_start(root_bw->window,
+					scrollbar_data->x0, scrollbar_data->y0,
+     					scrollbar_data->x1, scrollbar_data->y1);
 			break;
-		case SCROLL_MSG_SCROLL_FINISHED:
-			bw->scroll = NULL;
+		case SCROLLBAR_MSG_SCROLL_FINISHED:
+			html->scrollbar = NULL;
 			
-			browser_window_set_pointer(bw->window,
+			browser_window_set_pointer(html->bw,
 					GUI_POINTER_DEFAULT);
 			break;
 	}
@@ -789,66 +873,34 @@ void html_overflow_scroll_callback(void *client_data,
  * \param  x	   coordinate of mouse
  * \param  y	   coordinate of mouse
  */
-void html_overflow_scroll_drag_end(struct scroll *scroll,
+void html_overflow_scroll_drag_end(struct scrollbar *scrollbar,
 		browser_mouse_state mouse, int x, int y)
 {
 	int scroll_mouse_x, scroll_mouse_y, box_x, box_y;
-	struct browser_scroll_data *data = scroll_get_data(scroll);
+	struct html_scrollbar_data *data = scrollbar_get_data(scrollbar);
 	struct box *box;
 
 	box = data->box;
 	box_coords(box, &box_x, &box_y);
 
-	if (scroll_is_horizontal(scroll)) {
+	if (scrollbar_is_horizontal(scrollbar)) {
 		scroll_mouse_x = x - box_x;
 		scroll_mouse_y = y - (box_y + box->padding[TOP] +
 				box->height + box->padding[BOTTOM] -
 				SCROLLBAR_WIDTH);
-		scroll_mouse_drag_end(scroll, mouse,
+		scrollbar_mouse_drag_end(scrollbar, mouse,
 				scroll_mouse_x, scroll_mouse_y);
 	} else {
 		scroll_mouse_x = x - (box_x + box->padding[LEFT] +
 				box->width + box->padding[RIGHT] -
 				SCROLLBAR_WIDTH);
 		scroll_mouse_y = y - box_y;
-		scroll_mouse_drag_end(scroll, mouse,
+		scrollbar_mouse_drag_end(scrollbar, mouse,
 				scroll_mouse_x, scroll_mouse_y);
 	}
 }
 
 
-/**
- * End overflow scroll scrollbar drags
- *
- * \param  h      html content's high level cache entry
- * \param  mouse  state of mouse buttons and modifier keys
- * \param  x	  coordinate of mouse
- * \param  y	  coordinate of mouse
- */
-size_t html_selection_drag_end(hlcache_handle *h, browser_mouse_state mouse,
-		int x, int y, int dir)
-{
-	int pixel_offset;
-	struct box *box;
-	int dx, dy;
-	size_t idx = 0;
-
-	box = box_pick_text_box(h, x, y, dir, &dx, &dy);
-	if (box) {
-		plot_font_style_t fstyle;
-
-		font_plot_style_from_css(box->style, &fstyle);
-
-		nsfont.font_position_in_string(&fstyle, box->text, box->length,
-				dx, &idx, &pixel_offset);
-
-		idx += box->byte_offset;
-	}
-
-	return idx;
-}
-
-
 /**
  * Start drag scrolling the contents of a box
  *
@@ -859,7 +911,8 @@ size_t html_selection_drag_end(hlcache_handle *h, browser_mouse_state mouse,
 
 void html_box_drag_start(struct box *box, int x, int y)
 {
-	int box_x, box_y, scroll_mouse_x, scroll_mouse_y;
+	int box_x, box_y;
+	int scroll_mouse_x, scroll_mouse_y;
 	
 	box_coords(box, &box_x, &box_y);
 	
@@ -868,7 +921,7 @@ void html_box_drag_start(struct box *box, int x, int y)
 		scroll_mouse_y = y - (box_y + box->padding[TOP] +
 				box->height + box->padding[BOTTOM] -
 				SCROLLBAR_WIDTH);
-		scroll_start_content_drag(box->scroll_x,
+		scrollbar_start_content_drag(box->scroll_x,
 				scroll_mouse_x, scroll_mouse_y);
 	} else if (box->scroll_y != NULL) {
 		scroll_mouse_x = x - (box_x + box->padding[LEFT] +
@@ -876,7 +929,7 @@ void html_box_drag_start(struct box *box, int x, int y)
 				SCROLLBAR_WIDTH);
 		scroll_mouse_y = y - box_y;
 		
-		scroll_start_content_drag(box->scroll_y,
+		scrollbar_start_content_drag(box->scroll_y,
 				scroll_mouse_x, scroll_mouse_y);
 	}
 }
