@@ -42,6 +42,7 @@
 #include "desktop/selection.h"
 #include "render/box.h"
 #include "render/font.h"
+#include "render/search.h"
 #include "render/textplain.h"
 #include "utils/http.h"
 #include "utils/log.h"
@@ -50,6 +51,30 @@
 #include "utils/utils.h"
 #include "utils/utf8.h"
 
+struct textplain_line {
+	size_t	start;
+	size_t	length;
+};
+
+typedef struct textplain_content {
+	struct content base;
+
+	lwc_string *encoding;
+	void *inputstream;
+	char *utf8_data;
+	size_t utf8_data_size;
+	size_t utf8_data_allocated;
+	unsigned long physical_line_count;
+	struct textplain_line *physical_line;
+	int formatted_width;
+	struct browser_window *bw;
+
+	struct selection sel;	/** Selection state */
+
+	/** Context for free text search, or NULL if none */
+	struct search_context *search;
+} textplain_content;
+
 
 #define CHUNK 32768 /* Must be a power of 2 */
 #define MARGIN 4
@@ -69,35 +94,154 @@ static plot_font_style_t textplain_style = {
 
 static int textplain_tab_width = 256;  /* try for a sensible default */
 
-static bool textplain_create_internal(struct content *c, const char *encoding);
+static void textplain_fini(void);
+static nserror textplain_create(const content_handler *handler,
+		lwc_string *imime_type, const http_parameter *params,
+		llcache_handle *llcache, const char *fallback_charset,
+		bool quirks, struct content **c);
+static nserror textplain_create_internal(textplain_content *c, 
+		lwc_string *charset);
+static bool textplain_process_data(struct content *c, 
+		const char *data, unsigned int size);
+static bool textplain_convert(struct content *c);
+static void textplain_mouse_track(struct content *c, struct browser_window *bw,
+			browser_mouse_state mouse, int x, int y);
+static void textplain_mouse_action(struct content *c, struct browser_window *bw,
+			browser_mouse_state mouse, int x, int y);
+static void textplain_reformat(struct content *c, int width, int height);
+static void textplain_destroy(struct content *c);
+static bool textplain_redraw(struct content *c, struct content_redraw_data *data,
+		const struct rect *clip, const struct redraw_context *ctx);
+static void textplain_open(struct content *c, struct browser_window *bw,
+		struct content *page, struct box *box,
+		struct object_params *params);
+void textplain_close(struct content *c);
+struct selection *textplain_get_selection(struct content *c);
+struct search_context *textplain_get_search(struct content *c);
+static nserror textplain_clone(const struct content *old, 
+		struct content **newc);
+static content_type textplain_content_type(void);
+
 static parserutils_error textplain_charset_hack(const uint8_t *data, size_t len,
 		uint16_t *mibenum, uint32_t *source);
-static bool textplain_drain_input(struct content *c, 
+static bool textplain_drain_input(textplain_content *c, 
 		parserutils_inputstream *stream, parserutils_error terminator);
-static bool textplain_copy_utf8_data(struct content *c,
+static bool textplain_copy_utf8_data(textplain_content *c,
 		const uint8_t *buf, size_t len);
 static int textplain_coord_from_offset(const char *text, size_t offset,
 	size_t length);
 static float textplain_line_height(void);
 
+static const content_handler textplain_content_handler = {
+	.fini = textplain_fini,
+	.create = textplain_create,
+	.process_data = textplain_process_data,
+	.data_complete = textplain_convert,
+	.reformat = textplain_reformat,
+	.destroy = textplain_destroy,
+	.mouse_track = textplain_mouse_track,
+	.mouse_action = textplain_mouse_action,
+	.redraw = textplain_redraw,
+	.open = textplain_open,
+	.close = textplain_close,
+	.get_selection = textplain_get_selection,
+	.clone = textplain_clone,
+	.type = textplain_content_type,
+	.no_share = true,
+};
+
+static lwc_string *textplain_charset;
+static lwc_string *textplain_default_charset;
+
+/**
+ * Initialise the text content handler
+ */
+nserror textplain_init(void)
+{
+	lwc_error lerror;
+	nserror error;
+
+	lerror = lwc_intern_string("charset", SLEN("charset"),
+			&textplain_charset);
+	if (lerror != lwc_error_ok) {
+		return NSERROR_NOMEM;
+	}
+
+	lerror = lwc_intern_string("Windows-1252", SLEN("Windows-1252"),
+			&textplain_default_charset);
+	if (lerror != lwc_error_ok) {
+		lwc_string_unref(textplain_charset);
+		return NSERROR_NOMEM;
+	}
+
+	error = content_factory_register_handler("text/plain", 
+			&textplain_content_handler);
+	if (error != NSERROR_OK) {
+		lwc_string_unref(textplain_default_charset);
+		lwc_string_unref(textplain_charset);
+	}
+
+	return error;
+}
+
+/**
+ * Clean up after the text content handler
+ */
+void textplain_fini(void)
+{
+	if (textplain_default_charset != NULL) {
+		lwc_string_unref(textplain_default_charset);
+		textplain_default_charset = NULL;
+	}
+
+	if (textplain_charset != NULL) {
+		lwc_string_unref(textplain_charset);
+		textplain_charset = NULL;
+	}
+}
 
 /**
  * Create a CONTENT_TEXTPLAIN.
  */
 
-bool textplain_create(struct content *c, const http_parameter *params)
+nserror textplain_create(const content_handler *handler,
+		lwc_string *imime_type, const http_parameter *params,
+		llcache_handle *llcache, const char *fallback_charset,
+		bool quirks, struct content **c)
 {
-	const char *encoding;
+	textplain_content *text;
 	nserror error;
+	lwc_string *encoding;
 
-	textplain_style.size = (option_font_size * FONT_SIZE_SCALE) / 10;
+	text = talloc_zero(0, textplain_content);
+	if (text == NULL)
+		return NSERROR_NOMEM;
 
-	error = http_parameter_list_find_item(params, "charset", &encoding);
+	error = content__init(&text->base, handler, imime_type, params,
+			llcache, fallback_charset, quirks);
 	if (error != NSERROR_OK) {
-		encoding = "Windows-1252";
+		talloc_free(text);
+		return error;
 	}
 
-	return textplain_create_internal(c, encoding);
+	error = http_parameter_list_find_item(params, textplain_charset, 
+			&encoding);
+	if (error != NSERROR_OK) {
+		encoding = lwc_string_ref(textplain_default_charset);
+	}
+
+	error = textplain_create_internal(text, encoding);
+	if (error != NSERROR_OK) {
+		lwc_string_unref(encoding);
+		talloc_free(text);
+		return error;
+	}
+
+	lwc_string_unref(encoding);
+
+	*c = (struct content *) text;
+
+	return NSERROR_OK;
 }
 
 /*
@@ -116,18 +260,20 @@ parserutils_error textplain_charset_hack(const uint8_t *data, size_t len,
 	return PARSERUTILS_OK;
 } 
 
-bool textplain_create_internal(struct content *c, const char *encoding)
+nserror textplain_create_internal(textplain_content *c, lwc_string *encoding)
 {
 	char *utf8_data;
 	parserutils_inputstream *stream;
 	parserutils_error error;
 	union content_msg_data msg_data;
 
+	textplain_style.size = (option_font_size * FONT_SIZE_SCALE) / 10;
+
 	utf8_data = talloc_array(c, char, CHUNK);
 	if (utf8_data == NULL)
 		goto no_memory;
 
-	error = parserutils_inputstream_create(encoding, 0, 
+	error = parserutils_inputstream_create(lwc_string_data(encoding), 0, 
 			textplain_charset_hack, ns_realloc, NULL, &stream);
 	if (error == PARSERUTILS_BADENCODING) {
 		/* Fall back to Windows-1252 */
@@ -140,30 +286,28 @@ bool textplain_create_internal(struct content *c, const char *encoding)
 		goto no_memory;
 	}
 	
-	c->data.textplain.encoding = strdup(encoding);
-	if (c->data.textplain.encoding == NULL) {
-		talloc_free(utf8_data);
-		parserutils_inputstream_destroy(stream);
-		goto no_memory;
-	}
-		
-	c->data.textplain.inputstream = stream;
-	c->data.textplain.utf8_data = utf8_data;
-	c->data.textplain.utf8_data_size = 0;
-	c->data.textplain.utf8_data_allocated = CHUNK;
-	c->data.textplain.physical_line = 0;
-	c->data.textplain.physical_line_count = 0;
-	c->data.textplain.formatted_width = 0;
+	c->encoding = lwc_string_ref(encoding);
+	c->inputstream = stream;
+	c->utf8_data = utf8_data;
+	c->utf8_data_size = 0;
+	c->utf8_data_allocated = CHUNK;
+	c->physical_line = 0;
+	c->physical_line_count = 0;
+	c->formatted_width = 0;
+	c->bw = NULL;
 
-	return true;
+	selection_prepare(&c->sel, (struct content *)c, false);
+
+	return NSERROR_OK;
 
 no_memory:
 	msg_data.error = messages_get("NoMemory");
-	content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
-	return false;
+	content_broadcast(&c->base, CONTENT_MSG_ERROR, msg_data);
+	return NSERROR_NOMEM;
 }
 
-bool textplain_drain_input(struct content *c, parserutils_inputstream *stream,
+bool textplain_drain_input(textplain_content *c, 
+		parserutils_inputstream *stream,
 		parserutils_error terminator)
 {
 	static const uint8_t *u_fffd = (const uint8_t *) "\xef\xbf\xfd";
@@ -228,26 +372,25 @@ bool textplain_drain_input(struct content *c, parserutils_inputstream *stream,
 	return true;
 }
 
-bool textplain_copy_utf8_data(struct content *c, const uint8_t *buf, size_t len)
+bool textplain_copy_utf8_data(textplain_content *c, 
+		const uint8_t *buf, size_t len)
 {
-	if (c->data.textplain.utf8_data_size + len >= 
-			c->data.textplain.utf8_data_allocated) {
+	if (c->utf8_data_size + len >= c->utf8_data_allocated) {
 		/* Compute next multiple of chunk above the required space */
-		size_t allocated = (c->data.textplain.utf8_data_size + len + 
+		size_t allocated = (c->utf8_data_size + len + 
 				CHUNK - 1) & ~(CHUNK - 1);
 		char *utf8_data = talloc_realloc(c,
-				c->data.textplain.utf8_data,
+				c->utf8_data,
 				char, allocated);
 		if (utf8_data == NULL)
 			return false;
 
-		c->data.textplain.utf8_data = utf8_data;
-		c->data.textplain.utf8_data_allocated = allocated;
+		c->utf8_data = utf8_data;
+		c->utf8_data_allocated = allocated;
 	}
 
-	memcpy(c->data.textplain.utf8_data + 
-			c->data.textplain.utf8_data_size, buf, len);
-	c->data.textplain.utf8_data_size += len;
+	memcpy(c->utf8_data + c->utf8_data_size, buf, len);
+	c->utf8_data_size += len;
 
 	return true;
 }
@@ -260,7 +403,8 @@ bool textplain_copy_utf8_data(struct content *c, const uint8_t *buf, size_t len)
 bool textplain_process_data(struct content *c, 
 		const char *data, unsigned int size)
 {
-	parserutils_inputstream *stream = c->data.textplain.inputstream;
+	textplain_content *text = (textplain_content *) c;
+	parserutils_inputstream *stream = text->inputstream;
 	union content_msg_data msg_data;
 	parserutils_error error;
 
@@ -270,7 +414,7 @@ bool textplain_process_data(struct content *c,
 		goto no_memory;
 	}
 
-	if (textplain_drain_input(c, stream, PARSERUTILS_NEEDDATA) == false)
+	if (textplain_drain_input(text, stream, PARSERUTILS_NEEDDATA) == false)
 		goto no_memory;
 
 	return true;
@@ -288,7 +432,8 @@ no_memory:
 
 bool textplain_convert(struct content *c)
 {
-	parserutils_inputstream *stream = c->data.textplain.inputstream;
+	textplain_content *text = (textplain_content *) c;
+	parserutils_inputstream *stream = text->inputstream;
 	parserutils_error error;
 
 	error = parserutils_inputstream_append(stream, NULL, 0);
@@ -296,11 +441,11 @@ bool textplain_convert(struct content *c)
 		return false;
 	}
 
-	if (textplain_drain_input(c, stream, PARSERUTILS_EOF) == false)
+	if (textplain_drain_input(text, stream, PARSERUTILS_EOF) == false)
 		return false;
 
 	parserutils_inputstream_destroy(stream);
-	c->data.textplain.inputstream = NULL;
+	text->inputstream = NULL;
 
 	content_set_ready(c);
 	content_set_done(c);
@@ -316,10 +461,11 @@ bool textplain_convert(struct content *c)
 
 void textplain_reformat(struct content *c, int width, int height)
 {
-	char *utf8_data = c->data.textplain.utf8_data;
-	size_t utf8_data_size = c->data.textplain.utf8_data_size;
+	textplain_content *text = (textplain_content *) c;
+	char *utf8_data = text->utf8_data;
+	size_t utf8_data_size = text->utf8_data_size;
 	unsigned long line_count = 0;
-	struct textplain_line *line = c->data.textplain.physical_line;
+	struct textplain_line *line = text->physical_line;
 	struct textplain_line *line1;
 	size_t i, space, col;
 	size_t columns = 80;
@@ -333,12 +479,12 @@ void textplain_reformat(struct content *c, int width, int height)
 	columns = (width - MARGIN - MARGIN) * 8 / character_width;
 	textplain_tab_width = (TAB_WIDTH * character_width) / 8;
 
-	c->data.textplain.formatted_width = width;
+	text->formatted_width = width;
 
-	c->data.textplain.physical_line_count = 0;
+	text->physical_line_count = 0;
 
 	if (!line) {
-		c->data.textplain.physical_line = line =
+		text->physical_line = line =
 				talloc_array(c, struct textplain_line, 1024 + 3);
 		if (!line)
 			goto no_memory;
@@ -359,8 +505,7 @@ void textplain_reformat(struct content *c, int width, int height)
 						struct textplain_line, line_count + 1024 + 3);
 				if (!line1)
 					goto no_memory;
-				c->data.textplain.physical_line =
-						line = line1;
+				text->physical_line = line = line1;
 			}
 			if (term) {
 				line[line_count-1].length = i - line_start;
@@ -392,7 +537,7 @@ void textplain_reformat(struct content *c, int width, int height)
 	line[line_count-1].length = i - line[line_count-1].start;
 	line[line_count].start = utf8_data_size;
 
-	c->data.textplain.physical_line_count = line_count;
+	text->physical_line_count = line_count;
 	c->width = width;
 	c->height = line_count * textplain_line_height() + MARGIN + MARGIN;
 
@@ -410,39 +555,63 @@ no_memory:
 
 void textplain_destroy(struct content *c)
 {
-	if (c->data.textplain.encoding != NULL)
-		free(c->data.textplain.encoding);
+	textplain_content *text = (textplain_content *) c;
 
-	if (c->data.textplain.inputstream != NULL)
-		parserutils_inputstream_destroy(c->data.textplain.inputstream);
+	lwc_string_unref(text->encoding);
+
+	if (text->inputstream != NULL)
+		parserutils_inputstream_destroy(text->inputstream);
 }
 
 
-bool textplain_clone(const struct content *old, struct content *new_content)
+nserror textplain_clone(const struct content *old, struct content **newc)
 {
+	const textplain_content *old_text = (textplain_content *) old;
+	textplain_content *text;
+	nserror error;
 	const char *data;
 	unsigned long size;
 
-	/* Simply replay create/process/convert */
-	if (textplain_create_internal(new_content, 
-			old->data.textplain.encoding) == false)
-		return false;
+	text = talloc_zero(0, textplain_content);
+	if (text == NULL)
+		return NSERROR_NOMEM;
 
-	data = content__get_source_data(new_content, &size);
+	error = content__clone(old, &text->base);
+	if (error != NSERROR_OK) {
+		content_destroy(&text->base);
+		return error;
+	}
+
+	/* Simply replay create/process/convert */
+	error = textplain_create_internal(text, old_text->encoding);
+	if (error != NSERROR_OK) {
+		content_destroy(&text->base);
+		return error;
+	}
+
+	data = content__get_source_data(&text->base, &size);
 	if (size > 0) {
-		if (textplain_process_data(new_content, data, size) == false)
-			return false;
+		if (textplain_process_data(&text->base, data, size) == false) {
+			content_destroy(&text->base);
+			return NSERROR_NOMEM;
+		}
 	}
 
 	if (old->status == CONTENT_STATUS_READY ||
 			old->status == CONTENT_STATUS_DONE) {
-		if (textplain_convert(new_content) == false)
-			return false;
+		if (textplain_convert(&text->base) == false) {
+			content_destroy(&text->base);
+			return NSERROR_CLONE_FAILED;
+		}
 	}
 
-	return true;
+	return NSERROR_OK;
 }
 
+content_type textplain_content_type(void)
+{
+	return CONTENT_TEXTPLAIN;
+}
 
 /**
  * Handle mouse tracking (including drags) in a TEXTPLAIN content window.
@@ -457,17 +626,31 @@ bool textplain_clone(const struct content *old, struct content *new_content)
 void textplain_mouse_track(struct content *c, struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
+	textplain_content *text = (textplain_content *) c;
+
+	if (bw->drag_type == DRAGGING_SELECTION && !mouse) {
+		int dir = -1;
+		size_t idx;
+
+		if (selection_dragging_start(&text->sel))
+			dir = 1;
+
+		idx = textplain_offset_from_coords(c, x, y, dir);
+		selection_track(&text->sel, mouse, idx);
+
+		browser_window_set_drag_type(bw, DRAGGING_NONE);
+	}
+
 	switch (bw->drag_type) {
 
 		case DRAGGING_SELECTION: {
-			hlcache_handle *h = bw->current_content;
 			int dir = -1;
 			size_t idx;
 
-			if (selection_dragging_start(bw->sel)) dir = 1;
+			if (selection_dragging_start(&text->sel)) dir = 1;
 
-			idx = textplain_offset_from_coords(h, x, y, dir);
-			selection_track(bw->sel, mouse, idx);
+			idx = textplain_offset_from_coords(c, x, y, dir);
+			selection_track(&text->sel, mouse, idx);
 		}
 		break;
 
@@ -491,32 +674,30 @@ void textplain_mouse_track(struct content *c, struct browser_window *bw,
 void textplain_mouse_action(struct content *c, struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y)
 {
-	hlcache_handle *h = bw->current_content;
+	textplain_content *text = (textplain_content *) c;
 	gui_pointer_shape pointer = GUI_POINTER_DEFAULT;
 	const char *status = 0;
 	size_t idx;
 	int dir = 0;
 
-	bw->drag_type = DRAGGING_NONE;
+	browser_window_set_drag_type(bw, DRAGGING_NONE);
 
-	if (!bw->sel) return;
+	idx = textplain_offset_from_coords(c, x, y, dir);
+	if (selection_click(&text->sel, mouse, idx)) {
 
-	idx = textplain_offset_from_coords(h, x, y, dir);
-	if (selection_click(bw->sel, mouse, idx)) {
-
-		if (selection_dragging(bw->sel)) {
+		if (selection_dragging(&text->sel)) {
 			bw->drag_type = DRAGGING_SELECTION;
 			status = messages_get("Selecting");
 		}
 		else
-			status = content_get_status_message(h);
+			status = content__get_status_message(c);
 	}
 	else {
 		if (bw->loading_content)
 			status = content_get_status_message(
 					bw->loading_content);
 		else
-			status = content_get_status_message(h);
+			status = content__get_status_message(c);
 
 		if (mouse & (BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_DRAG_2)) {
 			browser_window_page_drag_start(bw, x, y);
@@ -527,39 +708,38 @@ void textplain_mouse_action(struct content *c, struct browser_window *bw,
 	if (status != NULL)
 		browser_window_set_status(bw, status);
 
-	browser_window_set_pointer(bw->window, pointer);
+	browser_window_set_pointer(bw, pointer);
 }
 
 
 /**
  * Draw a CONTENT_TEXTPLAIN using the current set of plotters (plot).
  *
- * \param  c		     content of type CONTENT_TEXTPLAIN
- * \param  x		     coordinate for top-left of redraw
- * \param  y		     coordinate for top-left of redraw
- * \param  width	     available width
- * \param  height	     available height
- * \param  clip		     clip rectangle
- * \param  scale	     scale for redraw
- * \param  background_colour the background colour
+ * \param  c	 content of type CONTENT_TEXTPLAIN
+ * \param  data	 redraw data for this content redraw
+ * \param  clip	 current clip region
+ * \param  ctx	 current redraw context
  * \return true if successful, false otherwise
  *
  * x, y, clip_[xy][01] are in target coordinates.
  */
 
-bool textplain_redraw(struct content *c, int x, int y,
-		int width, int height, const struct rect *clip,
-		float scale, colour background_colour)
+bool textplain_redraw(struct content *c, struct content_redraw_data *data,
+		const struct rect *clip, const struct redraw_context *ctx)
 {
-	struct browser_window *bw = current_redraw_browser;
-	char *utf8_data = c->data.textplain.utf8_data;
+	textplain_content *text = (textplain_content *) c;
+	struct browser_window *bw = text->bw;
+	const struct plotter_table *plot = ctx->plot;
+	char *utf8_data = text->utf8_data;
 	long lineno;
-	unsigned long line_count = c->data.textplain.physical_line_count;
+	int x = data->x;
+	int y = data->y;
+	unsigned long line_count = text->physical_line_count;
 	float line_height = textplain_line_height();
-	float scaled_line_height = line_height * scale;
-	long line0 = (clip->y0 - y * scale) / scaled_line_height - 1;
-	long line1 = (clip->y1 - y * scale) / scaled_line_height + 1;
-	struct textplain_line *line = c->data.textplain.physical_line;
+	float scaled_line_height = line_height * data->scale;
+	long line0 = (clip->y0 - y * data->scale) / scaled_line_height - 1;
+	long line1 = (clip->y1 - y * data->scale) / scaled_line_height + 1;
+	struct textplain_line *line = text->physical_line;
 	size_t length;
 	plot_style_t *plot_style_highlight;
 
@@ -574,7 +754,7 @@ bool textplain_redraw(struct content *c, int x, int y,
 	if (line1 < line0)
 		line1 = line0;
 
-	if (!plot.rectangle(clip->x0, clip->y0, clip->x1, clip->y1,
+	if (!plot->rectangle(clip->x0, clip->y0, clip->x1, clip->y1,
 			plot_style_fill_white))
 		return false;
 
@@ -582,19 +762,19 @@ bool textplain_redraw(struct content *c, int x, int y,
 		return true;
 
 	/* choose a suitable background colour for any highlighted text */
-	if ((background_colour & 0x808080) == 0x808080)
+	if ((data->background_colour & 0x808080) == 0x808080)
 		plot_style_highlight = plot_style_fill_black;
 	else
 		plot_style_highlight = plot_style_fill_white;
 
 	/* Set up font plot style */
-	textplain_style.background = background_colour;
+	textplain_style.background = data->background_colour;
 
-	x = (x + MARGIN) * scale;
-	y = (y + MARGIN) * scale;
+	x = (x + MARGIN) * data->scale;
+	y = (y + MARGIN) * data->scale;
 	for (lineno = line0; lineno != line1; lineno++) {
-		const char *text = utf8_data + line[lineno].start;
-		int tab_width = textplain_tab_width * scale;
+		const char *text_d = utf8_data + line[lineno].start;
+		int tab_width = textplain_tab_width * data->scale;
 		size_t offset = 0;
 		int tx = x;
 
@@ -609,23 +789,25 @@ bool textplain_redraw(struct content *c, int x, int y,
 			int width;
 			int ntx;
 
-			while (next_offset < length && text[next_offset] != '\t')
-				next_offset = utf8_next(text, length, next_offset);
+			while (next_offset < length && text_d[next_offset] != '\t')
+				next_offset = utf8_next(text_d, length, next_offset);
 
-			if (!text_redraw(text + offset, next_offset - offset,
+			if (!text_redraw(text_d + offset, next_offset - offset,
 					line[lineno].start + offset, 0,
 					&textplain_style,
 					tx, y + (lineno * scaled_line_height),
-					clip, line_height, scale, false))
+					clip, line_height, data->scale, false,
+					(struct content *)text, &text->sel,
+					text->search, ctx))
 				return false;
 
 			if (next_offset >= length)
 				break;
 
 			/* locate end of string and align to next tab position */
-			if (nsfont.font_width(&textplain_style, &text[offset],
+			if (nsfont.font_width(&textplain_style, &text_d[offset],
 					next_offset - offset, &width))
-				tx += (int)(width * scale);
+				tx += (int)(width * data->scale);
 
 			ntx = x + ((1 + (tx - x) / tab_width) * tab_width);
 
@@ -635,7 +817,7 @@ bool textplain_redraw(struct content *c, int x, int y,
 
 			if (bw) {
 				unsigned tab_ofst = line[lineno].start + next_offset;
-				struct selection *sel = bw->sel;
+				struct selection *sel = &text->sel;
 				bool highlighted = false;
 
 				if (selection_defined(sel)) {
@@ -646,20 +828,18 @@ bool textplain_redraw(struct content *c, int x, int y,
 						highlighted = true;
 				}
 
-				if (!highlighted && (bw->search_context 
-						!= NULL)) {
+				if (!highlighted && (text->search != NULL)) {
 					unsigned start_idx, end_idx;
-					if (gui_search_term_highlighted(
-							bw->window,
+					if (search_term_highlighted(c,
 							tab_ofst, tab_ofst + 1,
 							&start_idx, &end_idx,
-							bw->search_context))
+							text->search))
 						highlighted = true;
 				}
 
 				if (highlighted) {
 					int sy = y + (lineno * scaled_line_height);
-					if (!plot.rectangle(tx, sy, 
+					if (!plot->rectangle(tx, sy, 
 							    ntx, sy + scaled_line_height,
 							    plot_style_highlight))
 						return false;
@@ -674,19 +854,93 @@ bool textplain_redraw(struct content *c, int x, int y,
 	return true;
 }
 
+
+/**
+ * Handle a window containing a CONTENT_TEXTPLAIN being opened.
+ */
+
+void textplain_open(struct content *c, struct browser_window *bw,
+		struct content *page, struct box *box,
+		struct object_params *params)
+{
+	textplain_content *text = (textplain_content *) c;
+
+	text->bw = bw;
+
+	/* text selection */
+	selection_init(&text->sel, NULL);
+}
+
+
+/**
+ * Handle a window containing a CONTENT_TEXTPLAIN being closed.
+ */
+
+void textplain_close(struct content *c)
+{
+	textplain_content *text = (textplain_content *) c;
+
+	if (text->search != NULL)
+		search_destroy_context(text->search);
+
+	text->bw = NULL;
+}
+
+
+/**
+ * Return an textplain content's selection context
+ */
+
+struct selection *textplain_get_selection(struct content *c)
+{
+	textplain_content *text = (textplain_content *) c;
+
+	return &text->sel;
+}
+
+
+/**
+ * Set an TEXTPLAIN content's search context
+ *
+ * \param c	content of type text
+ * \param s	search context, or NULL if none
+ */
+
+void textplain_set_search(struct content *c, struct search_context *s)
+{
+	textplain_content *text = (textplain_content *) c;
+
+	text->search = s;
+}
+
+
+/**
+ * Return an TEXTPLAIN content's search context
+ *
+ * \param c	content of type text
+ * \return content's search context, or NULL if none
+ */
+
+struct search_context *textplain_get_search(struct content *c)
+{
+	textplain_content *text = (textplain_content *) c;
+
+	return text->search;
+}
+
 /**
  * Retrieve number of lines in content
  *
  * \param h  Content to retrieve line count from
  * \return Number of lines
  */
-unsigned long textplain_line_count(hlcache_handle *h)
+unsigned long textplain_line_count(struct content *c)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *text = (textplain_content *) c;
 
 	assert(c != NULL);
 
-	return c->data.textplain.physical_line_count;
+	return text->physical_line_count;
 }
 
 /**
@@ -695,13 +949,13 @@ unsigned long textplain_line_count(hlcache_handle *h)
  * \param h  Content to retrieve size of
  * \return Size, in bytes, of data
  */
-size_t textplain_size(hlcache_handle *h)
+size_t textplain_size(struct content *c)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *text = (textplain_content *) c;
 
 	assert(c != NULL);
 
-	return c->data.textplain.utf8_data_size;
+	return text->utf8_data_size;
 }
 
 /**
@@ -717,9 +971,9 @@ size_t textplain_size(hlcache_handle *h)
  * \return byte offset of character containing (or nearest to) point
  */
 
-size_t textplain_offset_from_coords(hlcache_handle *h, int x, int y, int dir)
+size_t textplain_offset_from_coords(struct content *c, int x, int y, int dir)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *textc = (textplain_content *) c;
 	float line_height = textplain_line_height();
 	struct textplain_line *line;
 	const char *text;
@@ -728,12 +982,11 @@ size_t textplain_offset_from_coords(hlcache_handle *h, int x, int y, int dir)
 	int idx;
 
 	assert(c != NULL);
-	assert(c->type == CONTENT_TEXTPLAIN);
 
 	y = (int)((float)(y - MARGIN) / line_height);
 	x -= MARGIN;
 
-	nlines = c->data.textplain.physical_line_count;
+	nlines = textc->physical_line_count;
 	if (!nlines)
 		return 0;
 
@@ -741,8 +994,8 @@ size_t textplain_offset_from_coords(hlcache_handle *h, int x, int y, int dir)
 	else if ((unsigned)y >= nlines)
 		y = nlines - 1;
 
-	line = &c->data.textplain.physical_line[y];
-	text = c->data.textplain.utf8_data + line->start;
+	line = &textc->physical_line[y];
+	text = textc->utf8_data + line->start;
 	length = line->length;
 	idx = 0;
 
@@ -791,25 +1044,24 @@ size_t textplain_offset_from_coords(hlcache_handle *h, int x, int y, int dir)
  * Given a byte offset within the text, return the line number
  * of the line containing that offset (or -1 if offset invalid)
  *
- * \param  h       content of type CONTENT_TEXTPLAIN
+ * \param  c       content of type CONTENT_TEXTPLAIN
  * \param  offset  byte offset within textual representation
  * \return line number, or -1 if offset invalid (larger than size)
  */
 
-int textplain_find_line(hlcache_handle *h, unsigned offset)
+int textplain_find_line(struct content *c, unsigned offset)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *text = (textplain_content *) c;
 	struct textplain_line *line;
 	int nlines;
 	int lineno = 0;
 
 	assert(c != NULL);
-	assert(c->type == CONTENT_TEXTPLAIN);
 
-	line = c->data.textplain.physical_line;
-	nlines = c->data.textplain.physical_line_count;
+	line = text->physical_line;
+	nlines = text->physical_line_count;
 
-	if (offset > c->data.textplain.utf8_data_size)
+	if (offset > text->utf8_data_size)
 		return -1;
 
 /* \todo - implement binary search here */
@@ -872,10 +1124,10 @@ int textplain_coord_from_offset(const char *text, size_t offset, size_t length)
  * \param  r      rectangle to be completed
  */
 
-void textplain_coords_from_range(hlcache_handle *h, unsigned start, 
+void textplain_coords_from_range(struct content *c, unsigned start, 
 		unsigned end, struct rect *r)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *text = (textplain_content *) c;
 	float line_height = textplain_line_height();
 	char *utf8_data;
 	struct textplain_line *line;
@@ -883,16 +1135,15 @@ void textplain_coords_from_range(hlcache_handle *h, unsigned start,
 	unsigned nlines;
 
 	assert(c != NULL);
-	assert(c->type == CONTENT_TEXTPLAIN);
 	assert(start <= end);
-	assert(end <= c->data.textplain.utf8_data_size);
+	assert(end <= text->utf8_data_size);
 
-	utf8_data = c->data.textplain.utf8_data;
-	nlines = c->data.textplain.physical_line_count;
-	line = c->data.textplain.physical_line;
+	utf8_data = text->utf8_data;
+	nlines = text->physical_line_count;
+	line = text->physical_line;
 
 	/* find start */
-	lineno = textplain_find_line(h, start);
+	lineno = textplain_find_line(c, start);
 
 	r->y0 = (int)(MARGIN + lineno * line_height);
 
@@ -901,10 +1152,10 @@ void textplain_coords_from_range(hlcache_handle *h, unsigned start,
 			forwards most of the time */
 
 		/* find end */
-		lineno = textplain_find_line(h, end);
+		lineno = textplain_find_line(c, end);
 
 		r->x0 = 0;
-		r->x1 = c->data.textplain.formatted_width;
+		r->x1 = text->formatted_width;
 	}
 	else {
 		/* single line */
@@ -931,22 +1182,21 @@ void textplain_coords_from_range(hlcache_handle *h, unsigned start,
  * \return pointer to text, or NULL if invalid line number
  */
 
-char *textplain_get_line(hlcache_handle *h, unsigned lineno,
+char *textplain_get_line(struct content *c, unsigned lineno,
 		size_t *poffset, size_t *plen)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *text = (textplain_content *) c;
 	struct textplain_line *line;
 
 	assert(c != NULL);
-	assert(c->type == CONTENT_TEXTPLAIN);
 
-	if (lineno >= c->data.textplain.physical_line_count)
+	if (lineno >= text->physical_line_count)
 		return NULL;
-	line = &c->data.textplain.physical_line[lineno];
+	line = &text->physical_line[lineno];
 
 	*poffset = line->start;
 	*plen = line->length;
-	return c->data.textplain.utf8_data + line->start;
+	return text->utf8_data + line->start;
 }
 
 
@@ -962,16 +1212,15 @@ char *textplain_get_line(hlcache_handle *h, unsigned lineno,
  * \return pointer to text, or NULL if no text
  */
 
-char *textplain_get_raw_data(hlcache_handle *h, unsigned start, unsigned end,
+char *textplain_get_raw_data(struct content *c, unsigned start, unsigned end,
 		size_t *plen)
 {
-	struct content *c = hlcache_handle_get_content(h);
+	textplain_content *text = (textplain_content *) c;
 	size_t utf8_size;
 
 	assert(c != NULL);
-	assert(c->type == CONTENT_TEXTPLAIN);
 
-	utf8_size = c->data.textplain.utf8_data_size;
+	utf8_size = text->utf8_data_size;
 
 	/* any text at all? */
 	if (!utf8_size) return NULL;
@@ -982,7 +1231,7 @@ char *textplain_get_raw_data(hlcache_handle *h, unsigned start, unsigned end,
 
 	*plen = end - start;
 
-	return c->data.textplain.utf8_data + start;
+	return text->utf8_data + start;
 }
 
 /**
@@ -999,3 +1248,19 @@ float textplain_line_height(void)
 		INTTOFIX((textplain_style.size / FONT_SIZE_SCALE))))), F_72));
 }
 
+/**
+ * Get the browser window containing a textplain content
+ *
+ * \param  c	text/plain content
+ * \return the browser window
+ */
+struct browser_window *textplain_get_browser_window(struct content *c)
+{
+	textplain_content *text = (textplain_content *) c;
+
+	assert(c != NULL);
+	assert(c->handler == &textplain_content_handler);
+
+	return text->bw;
+}
+
