@@ -62,15 +62,18 @@
 #include "desktop/netsurf.h"
 #include "desktop/options.h"
 #include "desktop/save_complete.h"
+#include "desktop/sslcert.h"
 #include "desktop/tree.h"
 #include "desktop/tree_url_node.h"
 #include "render/box.h"
 #include "render/font.h"
 #include "render/html.h"
+#include "riscos/content-handlers/artworks.h"
 #include "riscos/bitmap.h"
 #include "riscos/buffer.h"
 #include "riscos/cookies.h"
 #include "riscos/dialog.h"
+#include "riscos/content-handlers/draw.h"
 #include "riscos/global_history.h"
 #include "riscos/gui.h"
 #include "riscos/help.h"
@@ -86,6 +89,7 @@
 #include "riscos/query.h"
 #include "riscos/save.h"
 #include "riscos/sslcert.h"
+#include "riscos/content-handlers/sprite.h"
 #include "riscos/system_colour.h"
 #include "riscos/textselection.h"
 #include "riscos/theme.h"
@@ -774,6 +778,10 @@ int main(int argc, char** argv)
 
 	netsurf_init(&argc, &argv, "NetSurf:Choices", NULL);
 
+	artworks_init();
+	draw_init();
+	sprite_init();
+
 	/* Choose the interface language to use */
 	ro_gui_choose_language();
 
@@ -810,6 +818,7 @@ void gui_quit(void)
 	history_global_cleanup();
 	cookies_cleanup();
 	hotlist_cleanup(option_hotlist_save);
+	sslcert_cleanup();
 	ro_gui_saveas_quit();
 	rufl_quit();
 	free(gui_sprites);
@@ -1489,10 +1498,6 @@ void ro_msg_dataload(wimp_message *message)
 	if (g) {
 		if (ro_gui_window_dataload(g, message))
 			return;
-
-		/* Get top-level window for loading into */
-		while (g->bw->parent)
-			g = g->bw->parent->window;
 	}
 	else {
 		g = ro_gui_toolbar_lookup(message->data.data_xfer.w);
