@@ -581,6 +581,12 @@ void ro_gui_url_bar_redraw(struct url_bar *url_bar, wimp_draw *redraw)
 
 		xwimp_plot_icon(&icon);
 	} else {
+		struct content_redraw_data data;
+		struct redraw_context ctx = {
+			.interactive = true,
+			.plot = &ro_plotters
+		};
+
 		xwimp_set_colour(wimp_COLOUR_WHITE);
 		xos_plot(os_MOVE_TO,
 				(redraw->box.x0 - redraw->xscroll) +
@@ -598,13 +604,18 @@ void ro_gui_url_bar_redraw(struct url_bar *url_bar, wimp_draw *redraw)
 		clip.x1 = (redraw->clip.x1 - ro_plot_origin_x) / 2;
 		clip.y1 = (ro_plot_origin_y - redraw->clip.y1) / 2;
 
-		content_redraw(url_bar->favicon_content,
-				(url_bar->favicon_extent.x0 +
-					url_bar->favicon_offset.x) / 2,
-				(url_bar->favicon_offset.y -
-					url_bar->favicon_extent.y1) / 2,
-				url_bar->favicon_width, url_bar->favicon_height,
-				&clip, 1, 0);
+		data.x = (url_bar->favicon_extent.x0 +
+				url_bar->favicon_offset.x) / 2;
+		data.y = (url_bar->favicon_offset.y -
+					url_bar->favicon_extent.y1) / 2;
+		data.width = url_bar->favicon_width;
+		data.height = url_bar->favicon_height;
+		data.background_colour = 0xFFFFFF;
+		data.scale = 1;
+		data.repeat_x = false;
+		data.repeat_y = false;
+
+		content_redraw(url_bar->favicon_content, &data, &clip, &ctx);
 	}
 }
 
@@ -875,7 +886,7 @@ bool ro_gui_url_bar_test_for_text_field_keypress(struct url_bar *url_bar,
 bool ro_gui_url_bar_set_site_favicon(struct url_bar *url_bar,
 		struct hlcache_handle *h)
 {
-	content_type		type = CONTENT_OTHER;
+	content_type		type = CONTENT_NONE;
 
 	if (url_bar == NULL)
 		return false;
@@ -885,7 +896,7 @@ bool ro_gui_url_bar_set_site_favicon(struct url_bar *url_bar,
 
 	// \TODO -- Maybe test for CONTENT_ICO ???
 
-	if (type != CONTENT_OTHER && type != CONTENT_UNKNOWN) {
+	if (type == CONTENT_IMAGE) {
 		url_bar->favicon_content = h;
 		url_bar->favicon_width = content_get_width(h);
 		url_bar->favicon_height = content_get_height(h);
@@ -938,7 +949,7 @@ bool ro_gui_url_bar_set_content_favicon(struct url_bar *url_bar,
 		return false;
 
 	if (h != NULL)
-		type = ro_content_filetype_from_type(content_get_type(h));
+		type = ro_content_filetype(h);
 
 	if (type != 0) {
 		snprintf(sprite, URLBAR_FAVICON_NAME_LENGTH,
