@@ -33,10 +33,6 @@
 #include "content/content.h"
 #include "content/hlcache.h"
 #include "desktop/plotters.h"
-#include "render/box.h"
-#include "render/font.h"
-#include "render/html.h"
-#include "render/layout.h"
 #include "riscos/dialog.h"
 #include "riscos/menus.h"
 #include "riscos/print.h"
@@ -592,8 +588,7 @@ bool print_document(struct gui_window *g, const char *filename)
 	saved_width = content_get_width(h);
 	saved_height = content_get_height(h);
 	if (content_get_type(h) == CONTENT_HTML)
-		/* TODO: Front end code shouldn't see contents */
-		layout_document(hlcache_handle_get_content(h), width, height);
+		content_reformat(h, false, width, height);
 
 	/* open printer file */
 	error = xosfind_openoutw(osfind_NO_PATH | osfind_ERROR_IF_DIR |
@@ -623,11 +618,7 @@ bool print_document(struct gui_window *g, const char *filename)
 			goto error;
 	}
 
-	plot = ro_plotters;
-	plot.option_knockout = false;
 	ro_gui_current_redraw_gui = g;
-	current_redraw_browser = NULL;  /* we don't want to print the
-	                                  selection */
 
 	/* print is now active */
 	print_active = true;
@@ -693,18 +684,30 @@ bool print_document(struct gui_window *g, const char *filename)
 		}
 
 		while (more) {
+			struct content_redraw_data data;
+			/* TODO: turn knockout off for print */
+			struct redraw_context ctx = {
+				.interactive = false,
+				.plot = &ro_plotters
+			};
+
 			LOG(("redrawing area: [(%d, %d), (%d, %d)]",
 					b.x0, b.y0, b.x1, b.y1));
 			clip.x0 = (b.x0 - ro_plot_origin_x) / 2;
 			clip.y0 = (ro_plot_origin_y - b.y1) / 2;
 			clip.x1 = (b.x1 - ro_plot_origin_x) / 2;
 			clip.y1 = (ro_plot_origin_y - b.y0) / 2;
-			if (!content_redraw(h, 0, 0,
-					content_get_width(h),
-					content_get_height(h),
-					&clip,
-					print_scale,
-					0xFFFFFF)) {
+
+			data.x = 0;
+			data.y = 0;
+			data.width = content_get_width(h);
+			data.height = content_get_height(h);
+			data.background_colour = 0xFFFFFF;
+			data.scale = print_scale;
+			data.repeat_x = false;
+			data.repeat_y = false;
+
+			if (!content_redraw(h, &data, &clip, &ctx)) {
 				error_message = "redraw error";
 				goto error;
 			}
@@ -763,9 +766,7 @@ bool print_document(struct gui_window *g, const char *filename)
 
 	/* restore document layout and redraw browser window */
 	if (content_get_type(h) == CONTENT_HTML)
-		/* TODO: Front end code shouldn't see contents */
-		layout_document(hlcache_handle_get_content(h),
-				saved_width, saved_height);
+		content_reformat(h, false, saved_width, saved_height);
 
 	gui_window_redraw_window(g);
 
@@ -785,9 +786,7 @@ error:
 
 	/* restore document layout */
 	if (content_get_type(h)  == CONTENT_HTML)
-		/* TODO: Front end code shouldn't see contents */
-		layout_document(hlcache_handle_get_content(h),
-				saved_width, saved_height);
+		content_reformat(h, false, saved_width, saved_height);
 
 	return false;
 }
@@ -804,8 +803,13 @@ const char *print_declare_fonts(hlcache_handle *h)
 {
 	unsigned int i;
 	struct rect clip;
+	struct content_redraw_data data;
 	const char *error_message = 0;
 	os_error *error;
+	struct redraw_context ctx = {
+		.interactive = false,
+		.plot = &print_fonts_plotters
+	};
 
 	free(print_fonts_list);
 	print_fonts_list = 0;
@@ -815,10 +819,16 @@ const char *print_declare_fonts(hlcache_handle *h)
 	clip.x0 = clip.y0 = INT_MIN;
 	clip.x1 = clip.y1 = INT_MAX;
 
-	plot = print_fonts_plotters;
-	if (!content_redraw(h, 0, 0, content_get_width(h),
-			content_get_height(h),
-			&clip, 1, 0xffffff)) {
+	data.x = 0;
+	data.y = 0;
+	data.width = content_get_width(h);
+	data.height = content_get_height(h);
+	data.background_colour = 0xFFFFFF;
+	data.scale = 1;
+	data.repeat_x = false;
+	data.repeat_y = false;
+
+	if (!content_redraw(h, &data, &clip, &ctx)) {
 		if (print_fonts_error)
 			return print_fonts_error;
 		return "Declaring fonts failed.";
