@@ -317,8 +317,6 @@ void ro_treeview_redraw_request(int x, int y, int width, int height,
 		wimp_draw		update;
 		osbool			more;
 
-		plot = ro_plotters;
-
 		update.w = tv->w;
 		update.box.x0 = (2 * x) + tv->origin.x;
 		update.box.y0 = (-2 * (y + height)) + tv->origin.y;
@@ -357,8 +355,6 @@ void ro_treeview_redraw(wimp_draw *redraw)
 		 */
 	}
 
-	plot = ro_plotters;
-
 	error = xwimp_redraw_window(redraw, &more);
 	if (error) {
 		LOG(("xwimp_redraw_window: 0x%x: %s",
@@ -380,7 +376,11 @@ void ro_treeview_redraw(wimp_draw *redraw)
 
 void ro_treeview_redraw_loop(wimp_draw *redraw, ro_treeview *tv, osbool more)
 {
-	os_error		*error;
+	os_error *error;
+	struct redraw_context ctx = {
+		.interactive = true,
+		.plot = &ro_plotters
+	};
 
 	while (more) {
 		ro_plot_origin_x = redraw->box.x0 - redraw->xscroll;
@@ -394,14 +394,15 @@ void ro_treeview_redraw_loop(wimp_draw *redraw, ro_treeview *tv, osbool more)
 					((ro_plot_origin_y+tv->origin.y)
 					-redraw->clip.y1)/2,
 					(redraw->clip.x1 - redraw->clip.x0)/2,
-					(redraw->clip.y1 - redraw->clip.y0)/2);
+					(redraw->clip.y1 - redraw->clip.y0)/2,
+					&ctx);
 
 			/* Put the graphcis window back how the Wimp set it. */
 			clip.x0 = (redraw->clip.x0 - ro_plot_origin_x) / 2;
 			clip.y0 = (ro_plot_origin_y - redraw->clip.y1) / 2;
 			clip.x1 = (redraw->clip.x1 - ro_plot_origin_x) / 2;
 			clip.y1 = (ro_plot_origin_y - redraw->clip.y0) / 2;
-			plot.clip(&clip);
+			ro_plotters.clip(&clip);
 	 	}
 
 		error = xwimp_get_rectangle(redraw, &more);
@@ -749,6 +750,7 @@ static bool ro_treeview_mouse_click(wimp_pointer *pointer)
 	wimp_window_state	state;
 	int			xpos, ypos;
 	browser_mouse_state	mouse;
+	bool			handled = false;
 
 	tv = (ro_treeview *) ro_gui_wimp_event_get_user_data(pointer->w);
 	if (tv == NULL) {
@@ -800,7 +802,7 @@ static bool ro_treeview_mouse_click(wimp_pointer *pointer)
 	}
 
 	if (mouse != 0) {
-		tree_mouse_action(tv->tree, mouse, xpos, ypos);
+		handled = tree_mouse_action(tv->tree, mouse, xpos, ypos);
 
 		tv->drag = tree_drag_status(tv->tree);
 		if (tv->drag != TREE_NO_DRAG) {
@@ -822,13 +824,27 @@ static bool ro_treeview_mouse_click(wimp_pointer *pointer)
 			tv->callbacks->toolbar_button_update();
 	}
 
-	/* We assume that the owning module will have attached a window menu
-	 * to our parent window with the auto flag unset (so that we can fudge
-	 * the selection above).  If it hasn't, this call will quietly fail.
+	/* Special actions for some mouse buttons.  Adjust closes the dialog;
+	 * Menu opens a menu.  For the latter, we assume that the owning module
+	 * will have attached a window menu to our parent window with the auto
+	 * flag unset (so that we can fudge the selection above).  If it hasn't,
+	 * the call will quietly fail.
+	 *
+	 * \TODO -- Adjust-click close isn't a perfect copy of what the RO
+	 *          version did: adjust clicks anywhere close the tree, and
+	 *          selections persist.
 	 */
 
-	if (pointer->buttons == wimp_CLICK_MENU)
+	switch(pointer->buttons) {
+	case wimp_CLICK_ADJUST:
+		if (handled)
+			ro_gui_dialog_close(tv->w);
+		break;
+
+	case wimp_CLICK_MENU:
 		ro_gui_wimp_event_process_window_menu_click(pointer);
+		break;
+	}
 
 	return true;
 }
@@ -1226,35 +1242,7 @@ void tree_icon_name_from_content_type(char *buffer, content_type type)
 	case CONTENT_HTML:
 	case CONTENT_TEXTPLAIN:
 	case CONTENT_CSS:
-#if defined(WITH_MNG) || defined(WITH_PNG)
-	case CONTENT_PNG:
-#endif
-#ifdef WITH_MNG
-	case CONTENT_JNG:
-	case CONTENT_MNG:
-#endif
-#ifdef WITH_JPEG
-	case CONTENT_JPEG:
-#endif
-#ifdef WITH_GIF
-	case CONTENT_GIF:
-#endif
-#ifdef WITH_BMP
-	case CONTENT_BMP:
-	case CONTENT_ICO:
-#endif
-#ifdef WITH_SPRITE
-	case CONTENT_SPRITE:
-#endif
-#ifdef WITH_DRAW
-	case CONTENT_DRAW:
-#endif
-#ifdef WITH_ARTWORKS
-	case CONTENT_ARTWORKS:
-#endif
-#ifdef WITH_NS_SVG
-	case CONTENT_SVG:
-#endif
+	case CONTENT_IMAGE:
 	default:
 		sprintf(buffer, tree_content_icon_name);
 		break;
