@@ -358,6 +358,7 @@ static void nsws_window_update_forward_back(struct gui_window *w)
 static void nsws_update_edit(struct gui_window *w)
 {
 	bool paste, copy, del;
+	bool sel = (w->bw != NULL && browser_window_has_selection(w->bw));
 	if (GetFocus() == w->urlbar) {
 		DWORD i, ii;
 		SendMessage(w->urlbar, EM_GETSEL, (WPARAM)&i, (LPARAM)&ii);
@@ -365,11 +366,10 @@ static void nsws_update_edit(struct gui_window *w)
 		copy = (i != ii);
 		del = (i != ii);
 
-	} else if ((w->bw != NULL) && (w->bw->sel != NULL)){
+	} else if (sel){
 		paste = (w->bw->paste_callback != NULL);
-		copy = w->bw->sel->defined;
-		del = ((w->bw->sel->defined) &&
-		       (w->bw->caret_callback != NULL));
+		copy = sel;
+		del = (sel && (w->bw->caret_callback != NULL));
 	} else {
 		paste = false;
 		copy = false;
@@ -777,7 +777,8 @@ nsws_window_command(HWND hwnd,
 		if (GetFocus() == gw->urlbar) {
 			SendMessage(gw->urlbar, WM_COPY, 0, 0);
 		} else if (gw->bw != NULL) {
-			gui_copy_to_clipboard(gw->bw->sel);
+			gui_copy_to_clipboard(
+					browser_window_get_selection(gw->bw));
 		}
 		break;
 
@@ -808,7 +809,8 @@ nsws_window_command(HWND hwnd,
 		if (GetFocus() == gw->urlbar)
 			SendMessage(gw->urlbar, EM_SETSEL, 0, -1);
 		else
-			selection_select_all(gw->bw->sel);
+			selection_select_all(
+					browser_window_get_selection(gw->bw));
 		break;
 
 	case IDM_EDIT_SEARCH:
@@ -858,7 +860,7 @@ nsws_window_command(HWND hwnd,
 		gui_window_get_scroll(gw, &x, &y);
 		if (gw->bw != NULL) {
 			browser_window_set_scale(gw->bw, gw->bw->scale * 1.1, true);
-			browser_window_reformat(gw->bw, gw->width, gw->height);
+			browser_window_reformat(gw->bw, false, gw->width, gw->height);
 		}
 		gui_window_redraw_window(gw);
 		gui_window_set_scroll(gw, x, y);
@@ -871,7 +873,7 @@ nsws_window_command(HWND hwnd,
 		if (gw->bw != NULL) {
 			browser_window_set_scale(gw->bw,
 						 gw->bw->scale * 0.9, true);
-			browser_window_reformat(gw->bw, gw->width, gw->height);
+			browser_window_reformat(gw->bw, false, gw->width, gw->height);
 		}
 		gui_window_redraw_window(gw);
 		gui_window_set_scroll(gw, x, y);
@@ -883,7 +885,7 @@ nsws_window_command(HWND hwnd,
 		gui_window_get_scroll(gw, &x, &y);
 		if (gw->bw != NULL) {
 			browser_window_set_scale(gw->bw, 1.0, true);
-			browser_window_reformat(gw->bw, gw->width, gw->height);
+			browser_window_reformat(gw->bw, false, gw->width, gw->height);
 		}
 		gui_window_redraw_window(gw);
 		gui_window_set_scroll(gw, x, y);
@@ -948,7 +950,9 @@ nsws_window_command(HWND hwnd,
 	case IDM_VIEW_TOGGLE_DEBUG_RENDERING:
 		html_redraw_debug = !html_redraw_debug;
 		if (gw->bw != NULL) {
-			browser_window_reformat(gw->bw, gw->width, gw->height);
+			/* TODO: This should only redraw, not reformat.
+			 * (Layout doesn't change, so reformat is a waste of time) */
+			browser_window_reformat(gw->bw, false, gw->width, gw->height);
 		}
 		break;
 
@@ -1205,46 +1209,18 @@ gui_create_browser_window(struct browser_window *bw,
 	gw->next = window_list;
 	window_list = gw;
 
-	switch(bw->browser_window_type) {
-	case BROWSER_WINDOW_NORMAL:
-		gw->main = nsws_window_create(gw);
-		gw->toolbar = nsws_window_create_toolbar(gw, gw->main);
-		gw->statusbar = nsws_window_create_statusbar(gw);
-		gw->drawingarea = nsws_window_create_drawable(hInstance, gw->main, gw);
+	gw->main = nsws_window_create(gw);
+	gw->toolbar = nsws_window_create_toolbar(gw, gw->main);
+	gw->statusbar = nsws_window_create_statusbar(gw);
+	gw->drawingarea = nsws_window_create_drawable(hInstance, gw->main, gw);
 
-		LOG(("BROWSER_WINDOW_NORMAL: main:%p toolbar:%p statusbar %p drawingarea %p", gw->main, gw->toolbar, gw->statusbar, gw->drawingarea));
+	LOG(("new window: main:%p toolbar:%p statusbar %p drawingarea %p",
+			gw->main, gw->toolbar, gw->statusbar, gw->drawingarea));
 
-		font_hwnd = gw->drawingarea;
-		input_window = gw;
-		open_windows++;
-		ShowWindow(gw->main, SW_SHOWNORMAL);
-		break;
-
-	case BROWSER_WINDOW_FRAME:
-		gw->drawingarea = nsws_window_create_drawable(hInstance, bw->parent->window->drawingarea, gw);
-
-		ShowWindow(gw->drawingarea, SW_SHOWNORMAL);
-
-		LOG(("BROWSER_WINDOW_FRAME: main:%p toolbar:%p statusbar %p drawingarea %p", gw->main, gw->toolbar, gw->statusbar, gw->drawingarea));
-
-		break;
-
-	case BROWSER_WINDOW_FRAMESET:
-		LOG(("create frameset"));
-		break;
-
-	case BROWSER_WINDOW_IFRAME:
-		gw->drawingarea = nsws_window_create_drawable(hInstance, bw->parent->window->drawingarea, gw);
-
-		ShowWindow(gw->drawingarea, SW_SHOWNORMAL);
-
-		LOG(("BROWSER_WINDOW_IFRAME: main:%p toolbar:%p statusbar %p drawingarea %p", gw->main, gw->toolbar, gw->statusbar, gw->drawingarea));
-
-		break;
-
-	default:
-		LOG(("unhandled type"));
-	}
+	font_hwnd = gw->drawingarea;
+	input_window = gw;
+	open_windows++;
+	ShowWindow(gw->main, SW_SHOWNORMAL);
 
 	return gw;
 }
@@ -1422,8 +1398,7 @@ void gui_window_redraw_window(struct gui_window *gw)
 	RedrawWindow(gw->drawingarea, NULL, NULL, RDW_INVALIDATE | RDW_NOERASE);
 }
 
-void gui_window_update_box(struct gui_window *gw,
-			   const union content_msg_data *data)
+void gui_window_update_box(struct gui_window *gw, const struct rect *rect)
 {
 	/* LOG(("gw:%p %f,%f %f,%f", gw, data->redraw.x, data->redraw.y, data->redraw.width, data->redraw.height)); */
 
@@ -1432,10 +1407,10 @@ void gui_window_update_box(struct gui_window *gw,
 
 	RECT redrawrect;
 
-	redrawrect.left = (long)data->redraw.x - (gw->scrollx / gw->bw->scale);
-	redrawrect.top = (long)data->redraw.y - (gw->scrolly / gw->bw->scale);
-	redrawrect.right =(long)(data->redraw.x + data->redraw.width);
-	redrawrect.bottom = (long)(data->redraw.y + data->redraw.height);
+	redrawrect.left = (long)rect->x0 - (gw->scrollx / gw->bw->scale);
+	redrawrect.top = (long)rect->y0 - (gw->scrolly / gw->bw->scale);
+	redrawrect.right =(long)rect->x1;
+	redrawrect.bottom = (long)rect->y1;
 
 	RedrawWindow(gw->drawingarea, &redrawrect, NULL, RDW_INVALIDATE | RDW_NOERASE);
 
@@ -1521,16 +1496,6 @@ void gui_window_scroll_visible(struct gui_window *w, int x0, int y0,
 	LOG(("scroll visible (%p, %d, %d, %d, %d)", w, x0, y0, x1, y1));
 }
 
-void gui_window_position_frame(struct gui_window *w, int x0, int y0,
-			       int x1, int y1)
-{
-	if (w == NULL)
-		return;
-	LOG(("position frame %s: %d, %d, %d, %d", w->bw->name,
-	     x0, y0, x1, y1));
-	MoveWindow(w->drawingarea, x0, y0, x1-x0, y1-y0, true);
-}
-
 void gui_window_get_dimensions(struct gui_window *w, int *width, int *height,
 			       bool scaled)
 {
@@ -1733,7 +1698,7 @@ save_complete_gui_save(const char *path,
 		       const char *filename,
 		       size_t len,
 		       const char *sourcedata,
-		       content_type type)
+		       lwc_string *mime_type)
 {
 	return false;
 }
@@ -1764,25 +1729,11 @@ bool gui_window_box_scroll_start(struct gui_window *w,
 	return true;
 }
 
-bool gui_window_frame_resize_start(struct gui_window *w)
-{
-	LOG(("resize frame\n"));
-	return true;
-}
-
 void gui_window_save_link(struct gui_window *g, const char *url,
 			  const char *title)
 {
 }
 
-void gui_window_set_scale(struct gui_window *w, float scale)
-{
-	if (w == NULL)
-		return;
-	w->scale = scale;
-	LOG(("%.2f\n", scale));
-}
-
 void gui_drag_save_object(gui_save_type type, hlcache_handle *c,
 			  struct gui_window *w)
 {
@@ -1871,16 +1822,18 @@ gui_selection_traverse_handler(const char *text,
 
 bool gui_copy_to_clipboard(struct selection *s)
 {
-	if ((s->defined) && (s->bw != NULL) && (s->bw->window != NULL) &&
-	    (s->bw->window->main != NULL)) {
-		OpenClipboard(s->bw->window->main);
+	if (selection_defined(s)) {
+		/* TODO: Fix this, so it's not looking inside selection
+		 *       object */
+
+/*		OpenClipboard(s->bw->window->main);
 		EmptyClipboard();
 		if (selection_traverse(s, gui_selection_traverse_handler,
 				       NULL)) {
 			CloseClipboard();
 			return true;
 		}
-	}
+*/	}
 	return false;
 }
 
